package pwc.taxtech.atms.common.schedule;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import org.quartz.JobDataMap;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.quartz.QuartzJobBean;
import pwc.taxtech.atms.common.util.HttpUtil;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dao.RegionMapper;
import pwc.taxtech.atms.dto.organization.DDSyncOrgInfo;
import pwc.taxtech.atms.dto.organization.OrgSyncData;
import pwc.taxtech.atms.entity.Organization;
import pwc.taxtech.atms.entity.OrganizationExample;
import pwc.taxtech.atms.entity.Region;
import pwc.taxtech.atms.entity.RegionExample;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class OrgSyncJob extends QuartzJobBean {
    private static final Logger logger = LoggerFactory.getLogger(OrgSyncJob.class);

    @Resource
    private OrganizationMapper organizationMapper;

    @Resource
    private RegionMapper regionMapper;

    @Value("${org_sync_url}")
    private String orgSyncUrl;

    @Value("${org_sync_token}")
    private String token;

    @Override
    protected void executeInternal(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        JobDataMap dataMap = jobExecutionContext.getJobDetail().getJobDataMap();
        Map<String, String> headers = new HashMap<>();
        // todo 这里token需确认
        headers.put("token", token);
        headers.put("Content-Type", "application/x-www-form-urlencoded");
        headers.put("Idap", "eddie.wu_v");
        int pageSize = 1;
        List<OrgSyncData> failObjs = Lists.newArrayList();
        try {
            for (int i = 1; i <= pageSize; i++) {
                String response = HttpUtil.get(orgSyncUrl+"?currentPage="+i, headers);
                logger.info("机构信息同步记录:"+response);
                DDSyncOrgInfo ddSyncOrgInfo = JSONObject.parseObject(response, DDSyncOrgInfo.class);
                pageSize = ddSyncOrgInfo.getTotalPage();
                List<OrgSyncData> orgSyncDatas = ddSyncOrgInfo.getData();
                orgSyncDatas.forEach(osd -> {
                    OrganizationExample example = new OrganizationExample();
                    example.createCriteria().andNameEqualTo(osd.getNameCN());
                    Organization o = new Organization();
                    o.setClientCode(osd.getCode());
                    o.setCode(osd.getCode());
                    o.setEnterpriseAccountCode(String.valueOf(osd.getSobId()));
                    o.setEnterpriseAccountName(osd.getSobName());
                    o.setCurrencyCode(osd.getCurrencyCode());
                    o.setLegalEntity(osd.getLegalEntity());
                    o.setLegalPersonName(osd.getLegalRepresentative());
                    o.setAddress(osd.getAddress());
                    o.setCreateTime(osd.getGmtCreate());
                    o.setUpdateTime(osd.getGmtModified());
                    o.setPsCode(osd.getPsCode());
                    RegionExample regionExample = new RegionExample();
                    regionExample.createCriteria().andShortNameEqualTo(osd.getCompanyLocation());
                    List<Region> regions = regionMapper.selectByExample(regionExample);
                    if (regions.size() > 0) {
                        o.setRegionId(regions.get(0).getId());
                    }
                    if(organizationMapper.countByExample(example)<1){
                        failObjs.add(osd);
                    }
                    organizationMapper.updateByExampleSelective(o, example);
                });
            }
            logger.info("同步未匹配的机构记录:"+ JSON.toJSONString(failObjs));
        } catch (Exception e) {
            logger.error(String.format("机构信息同步异常:[%s]", e.getMessage()), e);
        }
    }

}
