package pwc.taxtech.atms.constant.enums;

public enum FormulaDataSourceType {
    /// <summary>
    /// 其他数据源，如公式中的税率、数字部分
    /// </summary>
    Other(0),

    /// <summary>
    /// 进项数据源
    /// </summary>
    InputInvoice(1),

    /// <summary>
    /// 进项明细数据源
    /// </summary>
    InputInvoiceDetail(2),

    /// <summary>
    /// 销项数据源
    /// </summary>
    OutputInvoice(3),

    /// <summary>
    /// 凭证数据源
    /// </summary>
    Voucher(4),

    /// <summary>
    /// 报表数据源
    /// </summary>
    Report(5),

    /// <summary>
    /// 条件判断数据源
    /// </summary>
    Judgment(6),

    /// <summary>
    /// Sap日报数据源
    /// </summary>
    SapDaily(7),


    /// <summary>
    /// 土地出让金数据源
    /// </summary>
    LandSell(8),

    /// <summary>
    /// 未开票数据源
    /// </summary>
    Unbilled(9),
    KeyInSource(10),
    //由于开会讨论，期末余额，期初余额等都是试算平衡数据源， code 11,12,14 所以删除他们统一改成试算平衡表
    //试算平衡数据源
    TrialBalanceSource(11),
    //QMYESource(12),
    ModelSource(13),
    WPSRSource(33),
    TrialBalance(20),
    AssetListSource(21),
    // FSESource(14),

    RSUMIF(22),

    CIT_TBAM(50); //针对试算平衡表

    private Integer code;

    FormulaDataSourceType(Integer code) {
        this.code = code;
    }

    public static FormulaDataSourceType getDSType(Integer dataSourceType) {
        switch (dataSourceType) {
            case 0:
                return FormulaDataSourceType.Other;
            case 1:
                return FormulaDataSourceType.InputInvoice;
            case 2:
                return FormulaDataSourceType.InputInvoiceDetail;
            case 3:
                return FormulaDataSourceType.OutputInvoice;
            case 4:
                return FormulaDataSourceType.Voucher;
            case 5:
                return FormulaDataSourceType.Report;
            case 6:
                return FormulaDataSourceType.Judgment;
            case 7:
                return FormulaDataSourceType.SapDaily;
            case 8:
                return FormulaDataSourceType.LandSell;
            case 9:
                return FormulaDataSourceType.Unbilled;
            case 10:
                return FormulaDataSourceType.KeyInSource;
            case 11:
                return FormulaDataSourceType.TrialBalanceSource;
            case 13:
                return FormulaDataSourceType.ModelSource;
            case 20:
                return FormulaDataSourceType.TrialBalance;
            case 21:
                return FormulaDataSourceType.AssetListSource;
            case 22:
                return FormulaDataSourceType.RSUMIF;
            case 33:
                return FormulaDataSourceType.WPSRSource;
            case 50:
                return FormulaDataSourceType.CIT_TBAM;

            default:
                return FormulaDataSourceType.Other;
        }
    }

    public Integer getCode() {
        return code;
    }
}
