package pwc.taxtech.atms.controller;

import com.github.pagehelper.PageInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.commons.CommonsMultipartFile;
import pwc.taxtech.atms.dto.ApiResultDto;
import pwc.taxtech.atms.dto.AssetDetailGroupDto;
import pwc.taxtech.atms.dto.AssetDetailGroupStringDto;
import pwc.taxtech.atms.entity.AssetDetailGroup;
import pwc.taxtech.atms.entity.AssetGroup;
import pwc.taxtech.atms.service.impl.AssetGroupServiceImpl;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description 资产相关Controller
 * @Author zhikai.z.wei
 */
@RestController
@RequestMapping("/api/v1/asset/")
public class AssetMappingController {
    private static Logger logger = LoggerFactory.getLogger(AssetMappingController.class);

    @Autowired
    private AssetGroupServiceImpl assetGroupService;

    /**
     * 获取各种资产（固定资产、长期待摊、无形资产）二级分类
     * @return ApiResultDto
     */
    @RequestMapping(value="getAllAssetDetailGroup",method= RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ApiResultDto getAllAssetDetailGroup(){
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            Map<String, List> assetsMap = new HashMap<String, List>();
            logger.info("获取各种类型资产二级分类");
            //获取资产一级分类相关数据
            List<AssetGroup> allAssetGroup = assetGroupService.getAllAssetGroup();
            //获取固定资产
//            List<AssetDetailGroup> fixedAssetDetailGroups = assetGroupService.getAllFixedAssetDetailGroup();
            //获取长期待摊
            List<AssetDetailGroup> longTermPendingDetailGroups = assetGroupService.getAllLongTermPendingDetailGroup();
            //获取无形资产
            List<AssetDetailGroup> intangibleAssetDetailGroups = assetGroupService.getAllIntangibleAssetsDetailGroup();

            assetsMap.put("allAssetGroup",allAssetGroup);
//            assetsMap.put("1",fixedAssetDetailGroups);
            assetsMap.put("2",longTermPendingDetailGroups);
            assetsMap.put("3",intangibleAssetDetailGroups);

            apiResultDto.setCode(1);
            apiResultDto.setMessage("获取成功");
            apiResultDto.setData(assetsMap);
            logger.info("获取资产二级分类成功");
            return apiResultDto;
        }catch(Exception e){
            logger.error("获取资产二级分类失败，错误信息如下：");
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("获取失败");
            return apiResultDto;
        }
    }

    /**
     * 获取固定资产二级分类
     * @return ApiResultDtoaddAssetDetailGroup
     */
    @RequestMapping(value="getFixedAssetDetailGroup",method= RequestMethod.POST)
    public  @ResponseBody
    ApiResultDto getAllFixedAssetDetailGroup(@RequestBody AssetDetailGroupDto assetDetailGroupDto){
        logger.info("获取固定资产二级分类,参数："+assetDetailGroupDto);
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            PageInfo<AssetDetailGroupStringDto> detailGroups = assetGroupService.getAllFixedAssetDetailGroup(assetDetailGroupDto);
            Long countAssetByType = assetGroupService.countAssetByType(assetDetailGroupDto.getAssetGroupType());
            Map map = new HashMap();
            map.put("detailGroup",detailGroups);
            map.put("totalItem",countAssetByType);
            apiResultDto.setCode(1);
            apiResultDto.setMessage("获取成功");
            apiResultDto.setData(map);
            logger.info("获取固定资产二级分类成功");
            return apiResultDto;
        }catch(Exception e){
            logger.error("获取固定资产二级分类失败，错误信息如下：");
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("获取失败");
            return apiResultDto;
        }
    }

    /**
     * 新增资产二级分类
     * @return
     */
    @RequestMapping(value="addAssetDetailGroup",method = RequestMethod.POST)
    public ApiResultDto addAssetDetailGroup(@RequestBody AssetDetailGroupDto assetDetailGroupDto){
        logger.info("新增资产二级分类");
        ApiResultDto apiResultDto = new ApiResultDto();
        if(assetDetailGroupDto ==null ){
            apiResultDto.setCode(0);
            apiResultDto.setMessage("传参为空！");
            return apiResultDto;
        }
        try{
            assetDetailGroupDto.setAssetGroupId(assetGroupService.getAssetGroupByType(assetDetailGroupDto.getAssetGroupType()).get(0).getId());
            int i = assetGroupService.addFixedAssetDetailGroup(assetDetailGroupDto);
            apiResultDto.setCode(1);
            apiResultDto.setMessage("新增成功");
            apiResultDto.setData(i);
            return apiResultDto;
        }catch(Exception e){
            logger.error("新增固定资产二级分类失败，错误信息如下：");
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("新增失败");
            return apiResultDto;
        }
    }
    /**
     * 编辑固定资产二级分类
     * @return
     */
    @RequestMapping(value="editAssetDetailGroup",method= RequestMethod.POST)
    public ApiResultDto editAssetDetailGroup(@RequestBody AssetDetailGroupDto assetDetailGroupDto){
        logger.info("编辑资产二级分类");
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            int i = assetGroupService.editFixedAssetDetailGroup(assetDetailGroupDto);
            apiResultDto.setCode(1);
            apiResultDto.setMessage("编辑成功");
            apiResultDto.setData(i);
            return apiResultDto;
        }catch(Exception e){
            logger.error("编辑资产二级分类失败，错误信息如下：");
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("编辑失败");
            return apiResultDto;
        }
    }
    /**
     * 删除固定资产二级分类
     * @return
     */
    @RequestMapping(value="deleteAssetDetailGroup",method= RequestMethod.DELETE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ApiResultDto deleteAssetDetailGroup(Long id){
        logger.info("删除资产二级分类");
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            int i = assetGroupService.deleteFixedAssetDetailGroup(id);
            apiResultDto.setCode(1);
            apiResultDto.setMessage("删除成功");
            apiResultDto.setData(i);
            return apiResultDto;
        }catch(Exception e){
            logger.error("删除资产二级分类失败，错误信息如下：");
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("删除失败");
            return apiResultDto;
        }
    }

}
