package pwc.taxtech.atms.controller;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.bind.annotation.*;
import pwc.taxtech.atms.dao.CitJournalEntryAdjustMapper;
import pwc.taxtech.atms.dao.CitTbamMapper;
import pwc.taxtech.atms.dao.OperationLogEntryLogMapper;
import pwc.taxtech.atms.dto.CitTbamDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.service.impl.DistributedIdService;
import pwc.taxtech.atms.vat.dao.PeriodFormulaBlockMapper;
import pwc.taxtech.atms.vat.entity.PeriodCellComment;
import pwc.taxtech.atms.vat.service.impl.CellCommentServiceImpl;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.locks.ReentrantLock;

@RestController
@RequestMapping(value = "api/v1/CellComment")
public class CellCommentController {
    @Autowired
    CellCommentServiceImpl cellCommentService;

    @RequestMapping(value = "List", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<List<PeriodCellComment>> getCellComments(@RequestParam("cellDataId") Optional<Long> cellDataId, @RequestHeader("from") String form) {
        String projectId = StringUtils.EMPTY;
        if (StringUtils.isNotBlank(form) && form.split("@").length > 0) {
            projectId = form.split("@")[0];
        }
        return cellCommentService.getCellComments(cellDataId, projectId);
    }

    @Autowired
    private CitTbamMapper citTbamMapper;
    @Autowired
    private CitJournalEntryAdjustMapper citJournalEntryAdjustMapper;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    //通过sql获取展示表格展示数据
    @RequestMapping(value = "getCellInformation")
    public OperationResultDto getCellInformation(String sql) {
        OperationResultDto operationResultDto = new OperationResultDto();
        int from = sql.indexOf("from");
        sql = sql.substring(from);
        List<CitTbam> citTbams = citTbamMapper.selectBySql(sql);
        List<CitTbamDto> citTbamDtos = new ArrayList<>();
        for (CitTbam citTbam : citTbams) {
            CitTbamDto citTbamDto = new CitTbamDto();
            BeanUtils.copyProperties(citTbam, citTbamDto);
            citTbamDtos.add(citTbamDto);
        }

        operationResultDto.setData(citTbamDtos);
        operationResultDto.setResultMsg("success");
        return operationResultDto;
    }
    //加载分录表格数据
    @RequestMapping("loadEntryListDataList")
    public OperationResultDto loadEntryListDataList(@RequestBody CitTbam citTbam) {
        OperationResultDto operationResultDto = new OperationResultDto();
        CitJournalEntryAdjust citJournalEntryAdjust = new CitJournalEntryAdjust();
        citJournalEntryAdjust.setSubjectCode(citTbam.getAccountCode());
        citJournalEntryAdjust.setProjectId(citTbam.getProjectId());
        List<CitJournalEntryAdjust> journalMerge = citJournalEntryAdjustMapper.getJournalMerge(citJournalEntryAdjust);
        operationResultDto.setData(journalMerge);
        CitTbamExample example = new CitTbamExample();
        CitTbamExample.Criteria criteria = example.createCriteria();
        criteria.andIdEqualTo(citTbam.getId());
        List<CitTbam> citTbams = citTbamMapper.selectByExample(example);
        operationResultDto.setResultMsg(citTbams.get(0).getDataType());
        return operationResultDto;
    }

    @Autowired
    private PeriodFormulaBlockMapper periodFormulaBlockMapper;

    @Autowired
    private OperationLogEntryLogMapper operationLogEntryLogMapper;
    @Autowired
    private DistributedIdService distributedIdService;

    //更新调整后金额
    @RequestMapping(value = "updateAdjust", method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto updateAdjust(@RequestBody List<CitTbamDto> citTbams) {
        OperationResultDto operationResultDto = new OperationResultDto();
        CitTbam citTbam = new CitTbam();
        for (CitTbamDto citTbamDto : citTbams) {
            BeanUtils.copyProperties(citTbamDto, citTbam);
            citTbam.setId(citTbam.getId());
            citTbam.setAdjustAccount(citTbam.getAdjustAccount());
            citTbamMapper.updateByPrimaryKeySelective(citTbam);
        }
        operationResultDto.setResultMsg("success");
        return operationResultDto;
    }

    @RequestMapping("selectEntryLog")
    public OperationResultDto selectEntryLog(String code, Long  id ) {
        OperationResultDto operationResultDto = new OperationResultDto();
        OperationLogEntryLogExample operationLogEntryLogExample = new OperationLogEntryLogExample();
        OperationLogEntryLogExample.Criteria criteria = operationLogEntryLogExample.createCriteria();
        criteria.andIdEqualTo(id);
        criteria.andSubjectCodeEqualTo(code);
        operationResultDto.setData(operationLogEntryLogMapper.selectByExample(operationLogEntryLogExample));
        return operationResultDto;
    }

    /**
     * 添加日志
     */
    @RequestMapping("addLog")
    public OperationResultDto addLog(@RequestBody List<OperationLogEntryLog> operationLogEntryLogs) {
        for (OperationLogEntryLog operationLogEntryLog : operationLogEntryLogs) {
            operationLogEntryLog.setMyId(distributedIdService.nextId());
            operationLogEntryLogMapper.insert(operationLogEntryLog);
        }
        OperationResultDto<Object> objectOperationResultDto = new OperationResultDto<>();
        objectOperationResultDto.setResultMsg("success");
        return objectOperationResultDto;
    }

}
