package pwc.taxtech.atms.controller;

import com.alibaba.fastjson.JSONArray;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.constant.enums.EnumCitImportType;
import pwc.taxtech.atms.dto.ApiResultDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.service.impl.CitImportExcelServiceImpl;

import java.io.InputStream;
import java.util.List;

import static pwc.taxtech.atms.constant.enums.EnumCitImportType.JournalAdjust;

/**
 * @Description CIT文件导入Controller
 * @Author zhikai.z.wei
 */
@RestController
@RequestMapping("/api/v1/citImport/")
public class CitImportExcelController {
    private static Logger logger = LoggerFactory.getLogger(CitImportExcelController.class);

    @Autowired
    private CitImportExcelServiceImpl citImportExcelService;

    /**
     * CIT文件导入
     * @param file
     * @param orgIds
     * @param periodDate
     * @param importType
     * @param importFileType
     * @return
     */
    @RequestMapping(value = "/citImportExcel", method = RequestMethod.POST)
    public @ResponseBody
    OperationResultDto citImportExcel(@RequestParam MultipartFile file, @RequestParam(required = false) String orgIds,
                                      @RequestParam String periodDate,
                                      @RequestParam Integer importType,
                                      @RequestParam Integer importFileType){
        logger.info("CIT文件导入");
        OperationResultDto opeResultDto = new OperationResultDto();
        try{
            if (file == null || file.getSize() <= 0) {
                opeResultDto.setResult(false);
                opeResultDto.setResultMsg("没有获取到文件");
                logger.warn("没有获取到文件");
                return opeResultDto;
            }
            logger.debug("file name: " + file.getOriginalFilename());
            List<String> orgList = (List<String>) JSONArray.parse(orgIds);
            opeResultDto = citImportExcelService.citImportExcel(file, orgList, file.getOriginalFilename(), periodDate, importType, importFileType);
            return opeResultDto;
        }catch (Exception e){
            logger.error("文件导入失败，错误信息如下：");
            e.printStackTrace();
            opeResultDto.setResult(false);
            opeResultDto.setResultMsg("文件导入失败");
            return opeResultDto;
        }
    }


    /**
     * CIT调整版日记账导入
     * @return
     */
//    @RequestMapping(value = "/journalAdjustImport", method = RequestMethod.POST)
//    public @ResponseBody
//    ApiResultDto journalAdjustImport(@RequestParam MultipartFile file,@RequestParam String periodDate,@RequestParam Integer importType){
//        logger.info("CIT调整版日记账导入");
//        ApiResultDto apiResultDto = new ApiResultDto();
//        try{
//            if (file == null || file.getSize() <= 0) {
//                apiResultDto.setCode(-1);
//                apiResultDto.setMessage("没有获取到文件");
//                logger.warn("没有获取到文件");
//                return apiResultDto;
//            }
//            logger.debug("file name: " + file.getOriginalFilename());
//            InputStream input = null;
//
//            input = file.getInputStream();
//            //调用资产导入业务逻辑
//            citImportExcelService.importJournalAdjust(input, file.getOriginalFilename(), periodDate, importType);
//
//            apiResultDto.setCode(1);
//            apiResultDto.setMessage("资产导入成功");
//            return apiResultDto;
//        }catch (Exception e){
//            logger.error("资产导入失败，错误信息如下：");
//            e.printStackTrace();
//            apiResultDto.setCode(0);
//            apiResultDto.setMessage("资产导入失败");
//            return apiResultDto;
//        }
//    }

    /**
     * 获取导入的调整日记账版本
     * @return
     */
    @RequestMapping(value = "/getJournalAdjust", method = RequestMethod.GET)
    public ApiResultDto getJournalAdjust(){
        logger.info("获取CIT调整版日记账");
        ApiResultDto apiResultDto = new ApiResultDto();
        apiResultDto.setCode(1);
        apiResultDto.setData(citImportExcelService.getJournalAdjust());
        return apiResultDto;
    }

    /**
     * 根据导入类型获取导入记录
     * @param type
     * @return
     */
    @RequestMapping(value = "/getCitDataImportLog", method = RequestMethod.GET)
    public ApiResultDto getCitDataImportLog(Integer type){
        logger.info("获取CIT调整版日记账");
        ApiResultDto apiResultDto = new ApiResultDto();
        apiResultDto.setCode(1);
        apiResultDto.setData(citImportExcelService.getCitDataImportLog(type));
        return apiResultDto;
    }


}
