package pwc.taxtech.atms.controller;

import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import pwc.taxtech.atms.dto.ReturnData;
import pwc.taxtech.atms.entity.OperationLogTaxDocument;
import pwc.taxtech.atms.service.impl.OperationLogTaxDocServiceImpl;
import pwc.taxtech.atms.thirdparty.ExcelUtil;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping("/api/v1/operLogTaxDoc")
public class OperationLogTaxDocController {

    @Autowired
    private OperationLogTaxDocServiceImpl operationLogTaxDocService;

    @RequestMapping("selectList")
    @ResponseBody
    public ReturnData selectTaxDocumentList() {
        List<OperationLogTaxDocument> operationLogTaxDocuments = operationLogTaxDocService.selectTaxDocumentList();
        ReturnData returnData = new ReturnData();
        returnData.setItems(operationLogTaxDocuments);
        returnData.setTotalCount(operationLogTaxDocuments.size());
        return returnData;
    }

    /**
     * 根据 id 数组来查询相关日志
     *
     * @param operationLogTaxDocument
     * @return
     */
    @RequestMapping("/selectListForLog")
    @ResponseBody
    public ReturnData selectListForLog(@RequestBody OperationLogTaxDocument operationLogTaxDocument) {
        List<String> ids = operationLogTaxDocument.getIds() == null ? Lists.newArrayList() : operationLogTaxDocument.getIds();
        List<OperationLogTaxDocument> operationLogTaxDocuments = operationLogTaxDocService.selectListForLog(ids);
        ReturnData returnData = new ReturnData();
        returnData.setItems(operationLogTaxDocuments);
        returnData.setTotalCount(operationLogTaxDocuments.size());
        return returnData;
    }

    @RequestMapping("add")
    @ResponseBody
    public boolean addTaxDocuments(OperationLogTaxDocument operationLogTaxDocument) {
        return operationLogTaxDocService.addTaxDocumentList(operationLogTaxDocument);
    }

    @RequestMapping("delete")
    @ResponseBody
    public boolean deleteTaxDocuments(String id) {
        return operationLogTaxDocService.deleteTaxDocument(id);
    }

    @RequestMapping("edit")
    @ResponseBody
    public boolean editTaxDocuments(OperationLogTaxDocument operationLogTaxDocument) {
        return operationLogTaxDocService.editFilesType(operationLogTaxDocument);
    }

    @RequestMapping("exportExcel")
    @ResponseBody
//    public void exportExcelFile(HttpServletResponse response, @RequestBody OperationLogTaxDocument operationLogTaxDocument) {
    public void exportExcelFile(HttpServletResponse response) {
        try {
            Map<String, String> headers = new LinkedHashMap<>();
            headers.put("id", "id");
//            headers.put("operation_content", "操作内容");
//            headers.put("module_name", "模块名称");
//            headers.put("operation_object", "操作对象");
            headers.put("operation_action", "操作action");
//            headers.put("original_state", "原始状态");
//            headers.put("update_state", "更新状态");
            headers.put("update_state", "操作内容");
            headers.put("operation_user", "操作者");
            headers.put("ip", "操作ip");
            headers.put("comment", "内容");
            headers.put("create_time", "创建时间");
            List<OperationLogTaxDocument> TaxDocuments = operationLogTaxDocService.selectTaxDocumentList();
//            List<String> ids = operationLogTaxDocument.getIds() == null ? Lists.newArrayList() : operationLogTaxDocument.getIds();
            response.setContentType("multipart/form-data");
            response.setHeader("Content-Disposition", "attachment;fileName=" + new String("档案列表日志".getBytes("GB2312"), "ISO-8859-1"));
            OutputStream ouputStream = response.getOutputStream();
            ExcelUtil.exportExcel(headers, TaxDocuments, ouputStream);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
