package pwc.taxtech.atms.controller;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.constant.enums.EnumServiceType;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dao.ProjectMapper;
import pwc.taxtech.atms.dpo.ReportDto;
import pwc.taxtech.atms.dto.*;
import pwc.taxtech.atms.dto.vatdto.*;
import pwc.taxtech.atms.entity.OrganizationExample;
import pwc.taxtech.atms.entity.Project;
import pwc.taxtech.atms.entity.ProjectExample;
import pwc.taxtech.atms.service.impl.DataImportService;
import pwc.taxtech.atms.vat.entity.*;
import pwc.taxtech.atms.vat.service.impl.ReportServiceImpl;

import java.util.List;
import java.util.Map;
import java.util.Optional;

@RestController
@RequestMapping(value = "api/v1/Report")
public class ReportController {
    @Autowired
    ReportServiceImpl reportService;

    @RequestMapping(value = "export", method = RequestMethod.POST)
    public ResponseEntity getExportFile(@RequestBody ReportExportDto report) {
        return ResponseEntity.ok(reportService.export(report.getReportData(), "~"));
    }

    @RequestMapping(value = "template/{projectId}/{serviceType}/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<List<ReportDto>> getTemplate(@PathVariable String projectId, @PathVariable int serviceType, @PathVariable Integer period) {
        return reportService.getReportTemplate(projectId, EnumServiceType.getEnumByCode(serviceType), period);
    }

    @RequestMapping(value = "filterTemplate/{projectId}/{serviceType}/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<List<ReportDto>> getFilterTemplate(@PathVariable String projectId, @PathVariable int serviceType, @PathVariable Integer period) {
        return reportService.getFilterReportTemplate(projectId, EnumServiceType.getEnumByCode(serviceType), period);
    }

    @RequestMapping(value = "generateByTotal/{projectId}/{mergeManual}/{period}", method = RequestMethod.POST,
            produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity generateAllData(@PathVariable String projectId, @PathVariable Integer period,
                                          @RequestParam Optional<String> generator, @PathVariable Boolean mergeManual) {
        return ResponseEntity.ok(reportService.generateData(projectId, EnumServiceType.VAT, mergeManual, period, null, generator));
    }

    @RequestMapping(value = "getRunningJob/{projectId}/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ResponseBody
    public PeriodJobDto getRunningJob(@PathVariable String projectId, @PathVariable Integer period) {
        PeriodJob job = reportService.getRunningJob(projectId, period);
        if (job != null) {
            return new PeriodJobDto().copyFromPeriodJob(job);
        } else {
            return null;
        }
    }

    @RequestMapping(value = "getJobStatus/{projectId}//{period}/{jobId}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ResponseBody
    public PeriodJobDto getJobStatus(@PathVariable String projectId, @PathVariable Integer period, @PathVariable String jobId) {
        return new PeriodJobDto().copyFromPeriodJob(reportService.getJobStatus(projectId, period, jobId));
    }

    @RequestMapping(value = "templateReferences/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public List<CellTemplateReferenceDto> getTemplateReferences(@PathVariable int period) {
        return reportService.getTemplateReferences(period);
    }

    @RequestMapping(value = "reportData/{reportId}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<ReportDataDto> getReportData(@PathVariable Long reportId, @RequestHeader String from) {
        OperationResultDto resultDto = new OperationResultDto();
        if (reportId == null || reportId == 0L) {
            resultDto.setResult(false);
            return resultDto;
        }
        return reportService.getCellData(reportId, from);
    }

    private static OperationResultDto<ReportDataDto> operationResultDto = null;

    @RequestMapping(value = "reportEbitData", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<ReportDataDto> reportEbitData(@RequestBody RequestParameterDto requestParameterDto) {
        /*OperationResultDto resultDto = new OperationResultDto();
      *//*  if (requestParameterDto.getProjectId() == null || requestParameterDto.getReportId() == null) {
            resultDto.setResult(false);
            return resultDto;
        }*/
        operationResultDto = reportService.getCellData(Long.parseLong(requestParameterDto.getReportId()), requestParameterDto.getOrgId(),requestParameterDto.getPeriod());
        return operationResultDto;
    }

    @Autowired
    private ProjectMapper projectMapper;

    private String getProjId(String orgId, Integer tmsPeriod) {
        ProjectExample pExample = new ProjectExample();
        pExample.createCriteria().andOrganizationIdEqualTo(orgId).andYearEqualTo(tmsPeriod / 100);
        List<Project> pList = projectMapper.selectByExample(pExample);
        if (!pList.isEmpty()) {
            return pList.get(0).getId();
        }
        return "";
    }

    public static void main(String[] args) {
        System.out.println(201833 % 100);
    }


    private DataImportService dataImportService;

    @RequestMapping(value = "saveAndRefresh", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<EbitDataDto> saveAndRefresh(@RequestParam(value ="orgId") String orgId, @RequestParam(value ="period")  Integer period, @RequestParam(value = "specialConsiderations", defaultValue = "0") Integer specialConsiderations, @RequestParam(value = "ebitRate", defaultValue = "1") String ebitRate) {
        OperationResultDto operationResultDto = new OperationResultDto();
        operationResultDto.setData(reportService.loadEbitData(orgId, period, specialConsiderations, ebitRate));
        reportService.saveDatasource(orgId, period, specialConsiderations, ebitRate, operationResultDto);
        return operationResultDto;
    }


    @RequestMapping(value = "report/{templateId}/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<ReportDto> getReportByTemplate(@PathVariable Long templateId, @PathVariable Integer period, @RequestHeader String from) {
        OperationResultDto resultDto = new OperationResultDto();
        if (templateId == null || templateId == 0L || period == null || period == 0) {
            resultDto.setResult(false);
            resultDto.setResultMsg("templateId or period is invalid");
            return resultDto;
        }
        return reportService.getReportByTemplate(templateId, period, from);
    }






    @RequestMapping(value = "getCellTemplateConfig/{reportTemplateId}/{period}/{rowIndex}/{columnIndex}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<PeriodCellTemplateConfig> getCellTemplateConfig(@PathVariable Long reportTemplateId,
                                                                              @PathVariable Integer period,
                                                                              @PathVariable int rowIndex,
                                                                              @PathVariable int columnIndex,
                                                                              @RequestHeader("from") String projectId) {
        return reportService.getCellTemplateConfig(reportTemplateId, period, rowIndex, columnIndex, projectId);
    }

    @RequestMapping(value = "getStdAccountByIndustry/{industryId}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<List<VatEnterpriseAccount>> getStdAccountByIndustry(@PathVariable String industryId, @RequestHeader String from) {
        String projectId = StringUtils.EMPTY;
        if (StringUtils.isNotBlank(from) && from.split("@").length > 0) {
            projectId = from.split("@")[0];
        }
        return reportService.getStdAccountByIndustry(industryId, projectId);
    }

    @RequestMapping(value = "getCellAccountRange/{reportTemplateId}/{period}/{rowIndex}/{columnIndex}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<List<VatEnterpriseAccountResultDto>> getCellAccountRange(@PathVariable Long reportTemplateId, @PathVariable Integer period, @PathVariable int rowIndex, @PathVariable int columnIndex, @RequestHeader String from) {
        String projectId = StringUtils.EMPTY;
        if (StringUtils.isNotBlank(from) && from.split("@").length > 0) {
            projectId = from.split("@")[0];
        }
        return reportService.getCellAccountRange(reportTemplateId, period, rowIndex, columnIndex, projectId);
    }

    @RequestMapping(value = "addCellManualData", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity addCellManualDataSource(@RequestBody ManualDataSourceDto data, @RequestHeader String from) {
        String projectId = StringUtils.EMPTY;
        if (StringUtils.isNotBlank(from) && from.split("@").length > 0) {
            projectId = from.split("@")[0];
        }
        return ResponseEntity.ok(reportService.addCellManualDataSource(data, from));
    }

    @RequestMapping(value = "addDataSource", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<String> addDataSource(@RequestBody DataSourceDto datasource) {
        return reportService.addDataSource(datasource);
    }

    @RequestMapping(value = "getAllDataItems/{dataSourceType}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<String> getAllDataItems(@PathVariable Integer dataSourceType) {
        return reportService.getAllDataItems(dataSourceType);
    }


    @RequestMapping(value = "addDataSourceItems", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto addDataSourceItems(@RequestBody DataSourceDto dataSource) {
        return reportService.addDataSourceItems(dataSource);
    }

    @RequestMapping(value = "getDataSourceDetailList/{dataSourceId}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<String> getDataSourceDetailList(@PathVariable Long dataSourceId) {
        return reportService.getDataSourceDetailList(dataSourceId);
    }

    @RequestMapping(value = "hasManualDataSource/{projectId}/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Boolean hasManualDataSource(@PathVariable String projectId, @PathVariable Integer period) {
        return reportService.hasManualDataSource(projectId, period);
    }


    @RequestMapping(value = "doUpload", method = RequestMethod.POST)
    public OperationResultDto doUploadAttach(@RequestParam("fileName") String fileName, MultipartFile file, String remarks, @RequestParam("activeCol") Long activeCol, @RequestParam("activeRow") Long activeRow, @RequestParam("activeTemplateId") String activeTemplateId) {
        System.out.println(activeCol + "----" + activeRow + "-----" + activeTemplateId);
        OperationResultDto operationResultDto = reportService.doUploadAttach(file, remarks);
        if (!"error".equals(operationResultDto.getResultMsg())) {//上传成功绑定
            reportService.bindPwcAttach(activeCol, activeRow, activeTemplateId, (FileDto) operationResultDto.getData());
        }
        return operationResultDto;
    }

    @RequestMapping("loadAttachList")
    public List<PwcReportAttach> loadAttachList(@RequestBody ReportAttachDto reportAttachDto) {
        return reportService.loadAttachList(reportAttachDto);
    }

    @RequestMapping("deleteAttach")
    public OperationResultDto deleteAttach(Long id) {
        return reportService.deleteAttach(id);
    }

    @Autowired
    private OrganizationMapper organizationMapper;

    @RequestMapping("getOrgLists")
    public OperationResultDto getOrgLists() {
        OperationResultDto operationResultDto = new OperationResultDto();
        OrganizationExample organizationExample = new OrganizationExample();
        operationResultDto.setData(organizationMapper.selectByExample(organizationExample));
        return operationResultDto;
    }

    @RequestMapping(value = "getReportByTemplateEbit/{templateId}/{period}/{orgId}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public OperationResultDto<PeriodReportDto> getReportByTemplateEbit(@PathVariable String templateId, @PathVariable Integer period, @PathVariable String orgId) {
        OperationResultDto resultDto = new OperationResultDto();
        if (templateId == null || period == null || period == 0) {
            resultDto.setResult(false);
            resultDto.setResultMsg("templateId or period is invalid");
            return resultDto;
        }
        return reportService.getReportByTemplateEbit(Long.parseLong(templateId), period % 100, getProjId(orgId, period));
    }


    /**
     *批量导出利润表
     */
    @RequestMapping("manyExport")
    public OperationResultDto manyExport(@RequestBody RequestParameterDto requestParameterDto){
        OperationResultDto operationResultDto = new OperationResultDto();

        operationResultDto.setResult(null);
        return  operationResultDto;
    }

}