package pwc.taxtech.atms.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import net.sf.json.JSONNull;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.DateUtil;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.PageResultVo;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.dto.TaxDocumentDto;
import pwc.taxtech.atms.entity.TaxDocument;
import pwc.taxtech.atms.service.impl.TaxDocumentServiceImpl;
import pwc.taxtech.atms.thirdparty.ExcelUtil;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.text.SimpleDateFormat;
import java.util.*;

@Controller
@RequestMapping("/api/v1/taxDoc")
public class TaxDocumentController {

    @Autowired
    private TaxDocumentServiceImpl taxDocumentService;

    @PostMapping("selectList")
    @ResponseBody
    public PageResultVo<TaxDocument> selectTaxDocumentList(@RequestBody TaxDocumentDto taxDocumentDto) {
        Page<TaxDocument> page = PageHelper.startPage(taxDocumentDto.getCurrentPage(), taxDocumentDto.getPageSize());
        taxDocumentService.selectTaxDocumentList(taxDocumentDto);
        PageInfo<TaxDocument> taxDocumentPageInfo = page.toPageInfo();
        List<TaxDocument> list = taxDocumentPageInfo.getList();
        return PageResultVo.getPageResultVo(taxDocumentPageInfo, list);
    }

    @PostMapping("/queryWhetherData")
    @ResponseBody
    public boolean queryWhetherData(@RequestBody TaxDocument taxDocument) {
        return taxDocumentService.queryWhetherData(taxDocument);
    }

    @PostMapping("add")
    @ResponseBody
    public boolean addTaxDocument(TaxDocument taxDocument,
                                  @RequestParam("file") MultipartFile file) {
        return taxDocumentService.addTaxDocumentList(file,taxDocument);
    }

    @PostMapping("delete")
    @ResponseBody
    public boolean deleteTaxDocument(@RequestBody TaxDocument taxDocument) {
        return taxDocumentService.deleteTaxDocument(taxDocument.getId());
    }

    @PostMapping("/batchDelete")
    @ResponseBody
    public boolean batchDelete(@RequestBody TaxDocumentDto taxDocumentDto) {
        return taxDocumentService.batchDelete(taxDocumentDto.getIds());
    }

    @PostMapping("edit")
    @ResponseBody
    public boolean editTaxDocument( TaxDocument taxDocument) {
        return taxDocumentService.editFilesType(taxDocument);
    }


    @RequestMapping("exportExcel")
    @ResponseBody
    public void exportExcelFile(HttpServletResponse response, @RequestBody TaxDocumentDto taxDocumentDto) {
        try {
            Map<String, String> headers = new LinkedHashMap<>();
            headers.put("Id", "序号");
            headers.put("FileAttr", "档案属性");
            headers.put("FileType", "档案类型");
            headers.put("FileName", "档案名称");
            headers.put("BusinessLine", "业务线");
            headers.put("CompanyName", "公司名称");
            headers.put("TaxType", "税种");
            headers.put("FileTime", "文件生效日期");
            headers.put("OwnTime", "所属期间");
            headers.put("EffectiveTime", "到期日");
            headers.put("AuditStatus", "审批状态");
            headers.put("PhysicalIndexNumber", "实物索引号");
            headers.put("StorageArea", "实物存放地点");
            headers.put("Keeper", "保管人");
            headers.put("UploadTime", "上传日期");
            headers.put("Creator", "创建人");
            headers.put("Remark", "档案备注");
            List<TaxDocument> TaxDocument = taxDocumentService.selectTaxDocumentList(taxDocumentDto);
            response.setContentType("multipart/form-data");
            response.setHeader("Content-Disposition", "attachment;fileName=" + new String("taxDocument.xlsx".getBytes("GB2312"), "ISO-8859-1"));
            OutputStream ouputStream = response.getOutputStream();
            ExcelUtil.exportExcel(headers, TaxDocument, ouputStream);
            ouputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 文件上传接口 createByZhangzezheng
     *
     * @param picture 图片文件
     * @param modual  模块名
     * @return
     */
    @RequestMapping("upload")
    @ResponseBody
    public String upload(@RequestPart("file") MultipartFile picture, @RequestParam(required = false) String modual) {
        return getUploadUrl(picture, modual);
    }

    /**
     * 生成上传url
     *
     * @param file
     * @param modual
     * @return
     */
    private String getUploadUrl(MultipartFile file, String modual) {
        String fileName = file.getOriginalFilename();
        String pictureName = UUID.randomUUID().toString() + fileName.substring(fileName.lastIndexOf("."));
        String dir = DateUtils.getStringDateShort();
        String typePath = "";
        try {
            String fileSavePath = File.separator + "images";
            if (StringUtils.isBlank(modual)) {
                modual = "default";
            }
            if (StringUtils.isNotBlank(modual)) {
                typePath = modual + File.separator + dir;
            }
            File basePath = new File(fileSavePath + File.separator + typePath);
            if (!basePath.exists()) {
                basePath.mkdirs();
            }
            file.transferTo(new File(fileSavePath + File.separator + typePath + File.separator + pictureName));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "images" + File.separator + typePath + File.separator + pictureName;
    }


    /**
     * 读取Excel转换成 Json
     *
     */
    @PostMapping("/previewExcelToJson")
    @ResponseBody
    public String previewExcel(@RequestBody TaxDocumentDto taxDocumentDto) {
        try {
            JSONArray dataArray = new JSONArray();
            //根据url地址 获取文件输入流
            InputStream is = getInputStreamByUrl(taxDocumentDto.getPath());
            InputStream inStream =is;
            XSSFWorkbook xssfWorkbook = new XSSFWorkbook(inStream);
            // 循环工作表Sheet 将数据解析后 存入json对象
            workForSheet(dataArray, xssfWorkbook);
            return dataArray.toString();
        } catch (Exception e) {
            e.printStackTrace();
            return JSONNull.getInstance().toString();
        }
    }

    /**
     * 下载全部附件
     */
    @PostMapping(value = "/downloadAllFile")
    public void  downloadAllFile(HttpServletResponse response, @RequestBody TaxDocumentDto taxDocumentDto) {
        taxDocumentService.downloadAllFile(response,taxDocumentDto.getIds());
    }

    /**
     * 解析json
     * @param cell
     * @return
     */
    private static Object toString(XSSFCell cell) {
        switch (cell.getCellTypeEnum()) {
            case _NONE:
                cell.setCellType(CellType.STRING);
                return "";
            case NUMERIC:
                if (DateUtil.isCellDateFormatted(cell)) {
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
                    return sdf.format(cell.getDateCellValue());
                }
                cell.setCellType(CellType.STRING);
                return cell.getStringCellValue();
            case STRING:
                String val = cell.getStringCellValue();
                if ("无".equalsIgnoreCase(val)) return "";
                //将其中的map格式和数组格式的字符串，转化为相应的数据类型
                if (val.indexOf("{") > -1) {
//                    JSONObject jsonObject = JSONObject.parseObject(val);
                    Map<String, Integer> mapJson = JSONObject.parseObject(val,HashMap.class);
                    return mapJson;
                }
                if (val.indexOf("[") > -1) {
                    val = val.substring(1, val.length() - 1);
                    String[] array = val.split(",");
                    return array;
                }
                return val;
            case FORMULA:
                return cell.getCellFormula();
            case BLANK:
                return "";
            case BOOLEAN:
                return cell.getBooleanCellValue() + "";
            case ERROR:
                return "非法字符";
            default:
                return "未知字符";
        }
    }

    /**
     * 输出数据
     */
    private static JSONArray shuffleData(JSONArray sheetArr) {
        JSONArray array = new JSONArray();
        for (int i = 0; i < sheetArr.size(); i++) {
            JSONObject object = sheetArr.getJSONObject(i);
            int count = 0;
            int length = 0;
            for (Object key : object.keySet()) {
                Object o = object.get((String) key);
                length++;
                boolean b = StringUtils.isEmpty(o.toString());
                if (b) {
                    count++;
                }
            }
            if (count != length) {
                array.add(object);
            }
        }
        return array;
    }

    /**
     * 读取sheet数据 添加到json数据中
     * @param dataArray
     * @param xssfWorkbook
     */
    private void workForSheet(JSONArray dataArray, XSSFWorkbook xssfWorkbook) {
        for (int numSheet = 0; numSheet < xssfWorkbook.getNumberOfSheets(); numSheet++) {
            XSSFSheet xssfSheet = xssfWorkbook.getSheetAt(numSheet);
            String sheetName = xssfSheet.getSheetName();
            if (xssfSheet == null) {
                continue;
            }
            //当前sheet的json文件
            JSONObject sheetJson = new JSONObject();
            //当前sheet的array，作为sheetJson 的value值
            JSONArray sheetArr = new JSONArray();
            //sheet的第一行，获取作为json的key值
            JSONArray key = new JSONArray();
            int xssfLastRowNum = xssfSheet.getLastRowNum();
            // 循环行Row
            for (int rowNum = 0; rowNum <= xssfLastRowNum; rowNum++) {
                XSSFRow xssfRow = xssfSheet.getRow(rowNum);
                if (xssfRow == null) {
                    continue;
                }

                // 循环列Cell，在这里组合json文件
                int firstCellNum = xssfRow.getFirstCellNum();
                int lastCellNum = xssfRow.getLastCellNum();
                JSONObject rowJson = new JSONObject();
                for (int cellNum = firstCellNum; cellNum < lastCellNum; cellNum++) {
                    XSSFCell cell = null;
                    try {
                        cell = xssfRow.getCell(cellNum);
                        if (cell == null) {
                            rowJson.put(key.getString(cellNum), "");
                            continue;
                        }
                        if (rowNum == 0)
                            key.add(toString(cell));
                        else {
                            //若是列号超过了key的大小，则跳过
                            if (cellNum >= key.size()) continue;
                            rowJson.put(key.getString(cellNum), toString(cell));
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                if (rowJson.keySet().size() > 0)
                    sheetArr.add(rowJson);
            }
            sheetJson.put(sheetName, shuffleData(sheetArr));
            dataArray.add(sheetJson);
        }
    }

    /**
     * 根据url地址 获取输入流
     * @param url
     * @return
     * @throws IOException
     */
    private InputStream getInputStreamByUrl(String url) throws IOException {
        URL httpurl=new URL(URLDecoder.decode(url, "UTF-8"));
        InputStream is;
        HttpURLConnection httpConn=(HttpURLConnection)httpurl.openConnection();
        httpConn.setDoOutput(true);// 使用 URL 连接进行输出
        httpConn.setDoInput(true);// 使用 URL 连接进行输入
        httpConn.setUseCaches(false);// 忽略缓存
        httpConn.setRequestMethod("GET");// 设置URL请求方法
        //可设置请求头
        httpConn.setRequestProperty("Content-Type", "application/octet-stream");
        httpConn.setRequestProperty("Connection", "Keep-Alive");// 维持长连接
        httpConn.setRequestProperty("Charset", "UTF-8");
        httpConn.connect();
        if (httpConn.getResponseCode() >= 400 ) {
            is = httpConn.getErrorStream();
        }
        else{
            is = httpConn.getInputStream();
        }
        return is;
    }
}
