package pwc.taxtech.atms.dto.vatdto;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.EvaluationException;
import org.apache.poi.ss.formula.eval.ValueEval;

import java.util.ArrayList;
import java.util.List;

import static pwc.taxtech.atms.common.util.FormulaUtil.resolverInteger;
import static pwc.taxtech.atms.common.util.FormulaUtil.resolverString;
import static pwc.taxtech.atms.exception.Exceptions.BAD_BBVO_PARAMS;

public class WPNAMEParasBo {
    private String reportCode;
    private Integer columnIndex;
    private Integer rowColumnIndex;
    private String rowName;
    private String formulaExpression;
    private List<PeriodCellDataTemplate> expressionData = new ArrayList<>();

    public static class PeriodCellDataTemplate {
        Integer period;
        Long cellTemplateId;

        @Override
        public String toString() {
            return period + ":" + cellTemplateId;
        }

        public PeriodCellDataTemplate(Integer period, Long cellTemplateId) {
            this.period = period;
            this.cellTemplateId = cellTemplateId;
        }

    }

    public void putPeriodCellTempate(Integer period, Long cellTemplateId) {
        if (period < 1 || period > 12 || cellTemplateId < 0) throw BAD_BBVO_PARAMS;
        expressionData.add(new PeriodCellDataTemplate(period, cellTemplateId));
    }

    public WPNAMEParasBo(ValueEval[] args, OperationEvaluationContext ec) throws EvaluationException {
        StringBuilder expression = new StringBuilder("");
        begin(expression);
        reportCode = resolverString(args, ec, 0);
        concatPara(expression, reportCode);
        try {
            rowColumnIndex = resolverInteger(args, ec, 1);
            split(expression);
            concatPara(expression, rowColumnIndex);
        } catch (Exception e) {
            String columnStr = resolverString(args, ec, 1);
            split(expression);
            concatPara(expression, columnStr);
            int rtn = 0;
            columnStr = columnStr.toUpperCase();
            char[] excelCol = columnStr.toCharArray();
            for (int i = columnStr.length() - 1; i >= 0; i--) {
                if (excelCol[i] >= 'A' && excelCol[i] <= 'Z') {
                    rtn += ((int) Math.pow(26, excelCol.length - 1 - i) * (excelCol[i] - 64));
                } else {
                    rtn = -1;
                }
            }
            rowColumnIndex = rtn;
        }
        rowName = resolverString(args, ec, 2);
        split(expression);
        concatPara(expression, rowName);
        try {
            columnIndex = resolverInteger(args, ec, 3);
            split(expression);
            concatPara(expression, columnIndex);
        } catch (Exception e) {
            String columnStr = resolverString(args, ec, 3);
            split(expression);
            concatPara(expression, columnStr);
            int rtn = 0;
            columnStr = columnStr.toUpperCase();
            char[] excelCol = columnStr.toCharArray();
            for (int i = columnStr.length() - 1; i >= 0; i--) {
                if (excelCol[i] >= 'A' && excelCol[i] <= 'Z') {
                    rtn += ((int) Math.pow(26, excelCol.length - 1 - i) * (excelCol[i] - 64));
                } else {
                    rtn = -1;
                }
            }
            columnIndex = rtn;
        }
        end(expression);
        formulaExpression = expression.toString();
    }

    private StringBuilder begin(StringBuilder expression) {
        return expression.append("WPNAME(");
    }

    private StringBuilder end(StringBuilder expression) {
        return expression.append(")");
    }

    private StringBuilder split(StringBuilder expression) {
        return expression.append(",");
    }

    private StringBuilder concatPara(StringBuilder expression, String para) {
        return expression.append("\"").append(para).append("\"");
    }

    private StringBuilder concatPara(StringBuilder expression, Integer para) {
        return expression.append(para);
    }

    public WPNAMEParasBo(WPNAMEParasBo otherBo) {
        this.reportCode = otherBo.reportCode;
        this.columnIndex = otherBo.getColumnIndex();
        this.rowName = otherBo.rowName;
        this.formulaExpression = otherBo.formulaExpression;
    }

    public String expression() {
        return formulaExpression;
    }

    public String getReportCode() {
        return this.reportCode;
    }

    public void setReportCode(String reportCode) {
        this.reportCode = reportCode;
    }

    public Integer getColumnIndex() {
        return this.columnIndex;
    }

    public void setColumnIndex(Integer columnIndex) {
        this.columnIndex = columnIndex;
    }

    public String getRowName() {
        return rowName;
    }

    public void setRowName(String rowName) {
        this.rowName = rowName;
    }

    public Integer getRowColumnIndex() {
        return rowColumnIndex;
    }

    public void setRowColumnIndex(Integer rowColumnIndex) {
        this.rowColumnIndex = rowColumnIndex;
    }


    public String getFormulaExpression() {
        return this.formulaExpression;
    }

    public void setFormulaExpression(String formulaExpression) {
        this.formulaExpression = formulaExpression;
    }

    public List<PeriodCellDataTemplate> getExpressionData() {
        return this.expressionData;
    }

    public void setExpressionData(List<PeriodCellDataTemplate> expressionData) {
        this.expressionData = expressionData;
    }

    @Override
    public String toString() {
        return "WPNAMEParasBo{" +
                "reportCode='" + reportCode + '\'' +
                ", columnIndex=" + columnIndex +
                ", rowColumnIndex=" + rowColumnIndex +
                ", rowName=" + rowName +
                '}';
    }
}
