package pwc.taxtech.atms.dto.vatdto;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.EvaluationException;
import org.apache.poi.ss.formula.eval.MissingArgEval;
import org.apache.poi.ss.formula.eval.ValueEval;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import static pwc.taxtech.atms.common.util.FormulaUtil.*;
import static pwc.taxtech.atms.exception.Exceptions.BAD_BBVO_PARAMS;

public class WPTYPEParasBo {
    private String reportCode;
    private Integer columnIndex;
    private Integer rowColumnIndex;
    private BigDecimal taxRate;
    private Integer revenueType;
    private Integer taxType;
    private String rowName;
    private String formulaExpression;
    private List<PeriodCellDataTemplate> expressionData = new ArrayList<>();

    public static class PeriodCellDataTemplate {
        Integer period;
        Long cellTemplateId;

        @Override
        public String toString() {
            return period + ":" + cellTemplateId;
        }

        public PeriodCellDataTemplate(Integer period, Long cellTemplateId) {
            this.period = period;
            this.cellTemplateId = cellTemplateId;
        }

    }

    public void putPeriodCellTempate(Integer period, Long cellTemplateId) {
        if (period < 1 || period > 12 || cellTemplateId < 0) throw BAD_BBVO_PARAMS;
        expressionData.add(new PeriodCellDataTemplate(period, cellTemplateId));
    }

    public WPTYPEParasBo(ValueEval[] args, OperationEvaluationContext ec) throws EvaluationException {
        StringBuilder expression = new StringBuilder("");
        begin(expression);
        reportCode = resolverString(args, ec, 0);
        concatPara(expression, reportCode);
        try {
            rowColumnIndex = resolverInteger(args, ec, 1);
            split(expression);
            concatPara(expression, rowColumnIndex);
        } catch (Exception e) {
            String columnStr = resolverString(args, ec, 1);
            split(expression);
            concatPara(expression, columnStr);
            int rtn = 0;
            columnStr = columnStr.toUpperCase();
            char[] excelCol = columnStr.toCharArray();
            for (int i = columnStr.length() - 1; i >= 0; i--) {
                if (excelCol[i] >= 'A' && excelCol[i] <= 'Z') {
                    rtn += ((int) Math.pow(26, excelCol.length - 1 - i) * (excelCol[i] - 64));
                } else {
                    rtn = -1;
                }
            }
            rowColumnIndex = rtn;
        }
        if(args[2] instanceof MissingArgEval){
            split(expression);
        }else{
            taxRate = new BigDecimal(resolverString(args, ec, 2));
            split(expression);
            concatPara(expression, String.valueOf(taxRate));
        }
        if(args[3] instanceof MissingArgEval){
            split(expression);
        }else{
            revenueType = resolverInteger(args, ec, 3);
            split(expression);
            concatPara(expression, revenueType);
        }
        if(args[4] instanceof MissingArgEval){
            split(expression);
        }else{
            taxType = resolverInteger(args, ec, 4);
            split(expression);
            concatPara(expression, taxType);
        }
        try {
            columnIndex = resolverInteger(args, ec, 5);
            split(expression);
            concatPara(expression, columnIndex);
        } catch (Exception e) {
            String columnStr = resolverString(args, ec, 5);
            split(expression);
            concatPara(expression, columnStr);
            int rtn = 0;
            columnStr = columnStr.toUpperCase();
            char[] excelCol = columnStr.toCharArray();
            for (int i = columnStr.length() - 1; i >= 0; i--) {
                if (excelCol[i] >= 'A' && excelCol[i] <= 'Z') {
                    rtn += ((int) Math.pow(26, excelCol.length - 1 - i) * (excelCol[i] - 64));
                } else {
                    rtn = -1;
                }
            }
            columnIndex = rtn;
        }
        end(expression);
        formulaExpression = expression.toString();
    }

    private StringBuilder begin(StringBuilder expression) {
        return expression.append("WPTYPE(");
    }

    private StringBuilder end(StringBuilder expression) {
        return expression.append(")");
    }

    private StringBuilder split(StringBuilder expression) {
        return expression.append(",");
    }

    private StringBuilder concatPara(StringBuilder expression, String para) {
        return expression.append("\"").append(para).append("\"");
    }

    private StringBuilder concatPara(StringBuilder expression, Integer para) {
        return expression.append(para);
    }

    public BigDecimal getTaxRate() {
        return taxRate;
    }

    public void setTaxRate(BigDecimal taxRate) {
        this.taxRate = taxRate;
    }

    public Integer getRevenueType() {
        return revenueType;
    }

    public void setRevenueType(Integer revenueType) {
        this.revenueType = revenueType;
    }

    public Integer getTaxType() {
        return taxType;
    }

    public void setTaxType(Integer taxType) {
        this.taxType = taxType;
    }

    public String expression() {
        return formulaExpression;
    }

    public String getReportCode() {
        return this.reportCode;
    }

    public void setReportCode(String reportCode) {
        this.reportCode = reportCode;
    }

    public Integer getColumnIndex() {
        return this.columnIndex;
    }

    public void setColumnIndex(Integer columnIndex) {
        this.columnIndex = columnIndex;
    }

    public String getRowName() {
        return rowName;
    }

    public void setRowName(String rowName) {
        this.rowName = rowName;
    }

    public Integer getRowColumnIndex() {
        return rowColumnIndex;
    }

    public void setRowColumnIndex(Integer rowColumnIndex) {
        this.rowColumnIndex = rowColumnIndex;
    }


    public String getFormulaExpression() {
        return this.formulaExpression;
    }

    public void setFormulaExpression(String formulaExpression) {
        this.formulaExpression = formulaExpression;
    }

    public List<PeriodCellDataTemplate> getExpressionData() {
        return this.expressionData;
    }

    public void setExpressionData(List<PeriodCellDataTemplate> expressionData) {
        this.expressionData = expressionData;
    }

    @Override
    public String toString() {
        return "WPTYPEParasBo{" +
                "reportCode='" + reportCode + '\'' +
                ", columnIndex=" + columnIndex +
                ", rowColumnIndex=" + rowColumnIndex +
                ", taxRate=" + taxRate +
                ", revenueType=" + revenueType +
                ", taxType=" + taxType +
                ", rowName=" + rowName +
                '}';
    }
}
