package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.dao.AssetDetailGroupMapper;
import pwc.taxtech.atms.dao.AssetGroupMapper;
import pwc.taxtech.atms.dto.AssetDetailGroupDto;
import pwc.taxtech.atms.dto.AssetDetailGroupStringDto;
import pwc.taxtech.atms.entity.AssetDetailGroup;
import pwc.taxtech.atms.entity.AssetDetailGroupExample;
import pwc.taxtech.atms.entity.AssetGroup;
import pwc.taxtech.atms.entity.AssetGroupExample;

import java.util.*;

/**
 * @Author zhikai.z.wei
 * 资产分类service
 */
@Service
public class AssetGroupServiceImpl extends BaseService {
    private static Logger logger = LoggerFactory.getLogger(AssetGroupServiceImpl.class);

    @Autowired
    private AssetDetailGroupMapper assetDetailGroupMapper;

    @Autowired
    private AssetGroupMapper assetGroupMapper;

    @Autowired
    private FileService fileService;

    /**
   获取资产一级分类相关数据
    */
    public List<AssetGroup> getAllAssetGroup() throws Exception {
        logger.debug("获取资产一级分类相关数据");
        AssetGroupExample assetGroupExample = new AssetGroupExample();
        return assetGroupMapper.selectByExample(assetGroupExample);
    }

    /**
   获取固定资产全部二级分类
    */
    public PageInfo<AssetDetailGroupStringDto> getAllFixedAssetDetailGroup(AssetDetailGroupDto assetDetailGroupDto) throws Exception {
        logger.debug("获取固定资产全部二级分类");
        AssetDetailGroupExample assetDetailGroupExample = new AssetDetailGroupExample();
        AssetDetailGroupExample.Criteria criteria = assetDetailGroupExample.createCriteria();
        //detail_group_name 表中asset_group_type 1：固定资产
        criteria.andAssetGroupTypeEqualTo(assetDetailGroupDto.getAssetGroupType());
        assetDetailGroupExample.setOrderByClause("id asc");
        PageHelper.startPage(assetDetailGroupDto.getPageIndex(), assetDetailGroupDto.getPageSize());
        List<AssetDetailGroup> assetDetailGroups = assetDetailGroupMapper.selectByExample(assetDetailGroupExample);

        //当时用分页包装数据后，不能对Long类型的字段进行序列化，仍会导致到前台精度缺失，
        // 创建一个全为String的实体，并进行复制
        List<AssetDetailGroupStringDto> assetDetailGroupDtos = new ArrayList<>();
        for (AssetDetailGroup assetDetailGroup:assetDetailGroups){
            AssetDetailGroupStringDto stringDto = new AssetDetailGroupStringDto();
            //相同类型及字段进行copy
            BeanUtils.copyProperties(assetDetailGroup,stringDto);
            //把Long类型数据转为String并赋给新的实体
            stringDto.setId(assetDetailGroup.getId().toString());
            stringDto.setAssetGroupId(assetDetailGroup.getAssetGroupId().toString());
            assetDetailGroupDtos.add(stringDto);
        }

//       assetDetailGroups.stream().forEach(assetDetailGroup -> {
//            AssetDetailGroupStringDto stringDto = new AssetDetailGroupStringDto();
//            //相同类型及字段进行copy
//            BeanUtils.copyProperties(assetDetailGroup,stringDto);
//            //把Long类型数据转为String并赋给新的实体
//            stringDto.setId(assetDetailGroup.getId().toString());
//            stringDto.setAssetGroupId(assetDetailGroup.getAssetGroupId().toString());
//            assetDetailGroupDtos.add(stringDto);
//        });


//        BeanUtils.copyProperties(assetDetailGroups,assetDetailGroupDtos);

//        PageInfo<AssetDetailGroup> pageInfo = new PageInfo<>(assetDetailGroups);
            PageInfo<AssetDetailGroupStringDto> pageInfoStringDto = new PageInfo<AssetDetailGroupStringDto>(assetDetailGroupDtos);

//        BeanUtils.copyProperties(pageInfo,pageInfoStringDto);
        return pageInfoStringDto;
    }

    /**
   获取长期待摊全部二级分类
    */
    public List<AssetDetailGroup> getAllLongTermPendingDetailGroup() throws Exception {
        logger.debug("获取长期待摊全部二级分类");
        AssetDetailGroupExample assetDetailGroupExample = new AssetDetailGroupExample();
        AssetDetailGroupExample.Criteria criteria = assetDetailGroupExample.createCriteria();
        //detail_group_name 表中asset_group_type 2：长期待摊
        criteria.andAssetGroupTypeEqualTo(2);
        return assetDetailGroupMapper.selectByExample(assetDetailGroupExample);
    }

    /**
   获取无形资产全部二级分类
    */
    public List<AssetDetailGroup> getAllIntangibleAssetsDetailGroup() throws Exception {
        logger.debug("获取无形资产全部二级分类");
        AssetDetailGroupExample assetDetailGroupExample = new AssetDetailGroupExample();
        AssetDetailGroupExample.Criteria criteria = assetDetailGroupExample.createCriteria();
        //detail_group_name 表中asset_group_type 3：无形资产
        criteria.andAssetGroupTypeEqualTo(3);
        return assetDetailGroupMapper.selectByExample(assetDetailGroupExample);
    }

    /**
     根据资产类型得到其总数
     */
    public Long countAssetByType(Integer assetGroupType) throws Exception {
        return assetDetailGroupMapper.countAssetByType(assetGroupType);
    }

    /**
   通过一级分类的类型获取一级分类的ID
    */
    public List<AssetGroup> getAssetGroupByType(Integer type) throws Exception {
        logger.debug("通过一级分类的类型获取一级分类的ID");
        AssetGroupExample assetGroupExample = new AssetGroupExample();
        AssetGroupExample.Criteria criteria = assetGroupExample.createCriteria();
        criteria.andAssetGroupTypeEqualTo(type);
        List<AssetGroup> assetGroups = assetGroupMapper.selectByExample(assetGroupExample);
        //判断查询出的List是否为null或者为空，
        // 若为其中一种，则创建对象并add进入list中，一面controller取用报空指针异常
        if(assetGroups == null || assetGroups.size()==0){
            AssetGroup assetGroup = new AssetGroup();
            assetGroups = new ArrayList<>();
            assetGroups.add(assetGroup);
        }
        return assetGroups;

    }

    /**
    新增固定资产二级分类
     */
    public int addFixedAssetDetailGroup(AssetDetailGroupDto assetDetailGroupDto) throws Exception {
        logger.debug("新增固定资产二级分类");
        AssetDetailGroup assetDetailGroup = new AssetDetailGroup();
        //把页面Dto的值赋给entity
        BeanUtils.copyProperties(assetDetailGroupDto,assetDetailGroup);
        assetDetailGroup.setId(idService.nextId());
        assetDetailGroup.setCreateTime(new Date());
        assetDetailGroup.setUpdateTime(new Date());
        return assetDetailGroupMapper.insertSelective(assetDetailGroup);
    }

    /**
    编辑固定资产二级分类
     */
    public int editFixedAssetDetailGroup(AssetDetailGroupDto assetDetailGroupDto) throws Exception {
        logger.debug("编辑固定资产二级分类");
        AssetDetailGroup assetDetailGroup = new AssetDetailGroup();
        //把页面Dto的值赋给entity
        BeanUtils.copyProperties(assetDetailGroupDto,assetDetailGroup);
        //81407534426099710     81407534426099710
        assetDetailGroup.setUpdateTime(new Date());
        return assetDetailGroupMapper.updateByPrimaryKeySelective(assetDetailGroup);
    }

    /**
    新增固定资产二级分类
     */
    public int deleteFixedAssetDetailGroup(Long id) throws Exception {
        logger.debug("删除固定资产二级分类");
        return assetDetailGroupMapper.deleteByPrimaryKey(id);
    }

}
