package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.message.ErrorMessage;
import pwc.taxtech.atms.common.util.JxlsUtils;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.CountTypeConstant;
import pwc.taxtech.atms.constant.ExportTemplatePathConstant;
import pwc.taxtech.atms.dao.CitJournalEntryAdjustMapper;
import pwc.taxtech.atms.dao.CitTrialBalanceMapper;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dto.CitJournalAdjustDto;
import pwc.taxtech.atms.dpo.CitTrialBalanceDto;
import pwc.taxtech.atms.dto.CitTrialBalanceExportDto;
import pwc.taxtech.atms.dto.JournalMergeExportDto;
import pwc.taxtech.atms.dto.export.ExportDto;
import pwc.taxtech.atms.dto.vatdto.*;
import pwc.taxtech.atms.dto.vatdto.dd.TrialBalanceDto;
import pwc.taxtech.atms.dto.vatdto.dd.*;
import pwc.taxtech.atms.dto.vatdto.excelheader.CashFlowHeader;
import pwc.taxtech.atms.dto.vatdto.excelheader.CertifiedInvoicesListHeader;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.exception.ServiceException;
import pwc.taxtech.atms.thirdparty.ExcelUtil;
import pwc.taxtech.atms.vat.dao.*;
import pwc.taxtech.atms.vat.dpo.TrialBalanceCondition;
import pwc.taxtech.atms.vat.dpo.*;
import pwc.taxtech.atms.vat.entity.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * CIT数据预览功能模块
 * @author zhikai.z.wei
 * @Date: 02/03/2019
 * @Description:
 */
@Service
public class CitDataPreviewServiceImpl extends BaseService {

    @Resource
    private CitJournalEntryAdjustMapper citJournalMapper;
    @Resource
    private CitTrialBalanceMapper citTbMapper;

    /**
     * 获取日记账合并版
     * @param citJournalAdjustDto
     * @return
     */
    public PageInfo<CitJournalAdjustDto> getJournalMergeData(CitJournalAdjustDto citJournalAdjustDto) {
        CitJournalEntryAdjust citJournalEntryAdjust = beanUtil.copyProperties(citJournalAdjustDto, new CitJournalEntryAdjust());
        Page page = PageHelper.startPage(citJournalAdjustDto.getPageInfo().getPageIndex(), citJournalAdjustDto.getPageInfo().getPageSize());
        List<CitJournalEntryAdjust> journalMerges = citJournalMapper.getJournalMerge(citJournalEntryAdjust);
        List<CitJournalAdjustDto> journalAdjustDtos = Lists.newArrayList();
        journalMerges.forEach(journal -> {
            CitJournalAdjustDto citJournalDto = new CitJournalAdjustDto();
            beanUtil.copyProperties(journal, citJournalDto);
            journalAdjustDtos.add(citJournalDto);
        });

        PageInfo<CitJournalAdjustDto> pageInfo =new PageInfo<>(journalAdjustDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(citJournalAdjustDto.getPageInfo().getPageIndex());
        return pageInfo;
    }

    /**
     * 日记账导出(第一种方式)--暂时不使用
     * @param citJournalAdjustDto
     * @param os
     * @return
     */
    public int exportJournalMergeData(CitJournalAdjustDto citJournalAdjustDto, OutputStream os) {
        try {
            CitJournalEntryAdjust citJournalEntryAdjust = beanUtil.copyProperties(citJournalAdjustDto, new CitJournalEntryAdjust());
            List<CitJournalEntryAdjust> journalMerges = citJournalMapper.getJournalMerge(citJournalEntryAdjust);

            Map<String, String> header = generalJMHeader();
            List<JournalMergeExportDto> cellList = new ArrayList<>();
            journalMerges.forEach(tb -> {
                JournalMergeExportDto d = new JournalMergeExportDto();
                d = beanUtil.copyProperties(tb, d);
                cellList.add(d);
            });
            ExcelUtil.exportExcel(header, cellList, os);
            return cellList.size();
        } catch (Exception e) {
            logger.error("日记账合并版导出转换Excel异常: %s", e.getMessage());
            return 0;
        }
    }

    /**
     * 日记账导出(第二种方式)--正在使用
     * @param citJournalAdjustDto
     * @param response
     * @return
     */
    public int exportJournalMergeData2(CitJournalAdjustDto citJournalAdjustDto, HttpServletResponse response){
        CitJournalEntryAdjust citJournalEntryAdjust = beanUtil.copyProperties(citJournalAdjustDto, new CitJournalEntryAdjust());
        List<CitJournalEntryAdjust> journalMerges = citJournalMapper.getJournalMerge(citJournalEntryAdjust);
        if(journalMerges.size()==0){
            return  0;
        }
        ExportDto exportDto = new ExportDto();
        exportDto.setFileName("日记账合并版");
        exportDto.setTemplateUrl(Constant.citTemplateUrl +  "/citJournalEntryAdjust.xlsx");
        exportDto.setResponse(response);
        exportDto.setList(journalMerges);
        exportDto.setRelation(null);
        JxlsUtils.export(exportDto);
        return 1;
    }

    /**
     * 日记账导出(第一种方式)所用到的获取Excel表头
     * @return
     */
    private Map<String, String> generalJMHeader() {
        Map<String, String> header = new LinkedHashMap<>();
        header.put("ApprovalStatus","审批状态");
        header.put("PostedStatus","过账");

        header.put("AccountingPeriod","会计期间");
        header.put("AccountingDate","凭证日期");
        header.put("JournalSource","日记账来源");
        header.put("Category","日记账类别");
        header.put("Name","日记账名称");
        header.put("VoucherNum","凭证编号");
        header.put("Description","摘要");
        header.put("OrgCode","主体代码");
        header.put("SubjectCode","科目代码");
        header.put("OrgName","主体代码");
        header.put("SubjectName","科目代码");

        header.put("AccountedDr","借方金额");
        header.put("AccountedCr","贷方金额");
        header.put("CreatedBy","创建人");
        header.put("CreatedDate","创建日期");
        header.put("LateUpdatedBy","最后更新人");
        header.put("LateUpdatedDate","最后更新日期");
        header.put("Period","税务系统期间");
        return header;
    }

    /**
     * 获取自动生成的试算平衡表数据
     * @param citTrialBalanceDto
     * @return
     */
    public PageInfo<CitTrialBalanceDto> getTbGeneVerData(CitTrialBalanceDto citTrialBalanceDto) {
        Page page = PageHelper.startPage(citTrialBalanceDto.getPageInfo().getPageIndex(), citTrialBalanceDto.getPageInfo().getPageSize());
        CitTrialBalanceExample citTbExample = new CitTrialBalanceExample();
        CitTrialBalanceExample.Criteria citTbExampleCriteria = citTbExample.createCriteria();
        citTbExampleCriteria.andProjectIdEqualTo(citTrialBalanceDto.getProjectId());
        List<CitTrialBalance> citTbList = citTbMapper.selectByExample(citTbExample);
        List<CitTrialBalanceDto> citTbDtos = Lists.newArrayList();
        citTbList.forEach(tb -> {
            CitTrialBalanceDto citTrialBalanceDtoTemp = new CitTrialBalanceDto();
            beanUtil.copyProperties(tb, citTrialBalanceDtoTemp);
            citTbDtos.add(citTrialBalanceDtoTemp);
        });

        PageInfo<CitTrialBalanceDto> pageInfo =new PageInfo<>(citTbDtos);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(citTrialBalanceDto.getPageInfo().getPageIndex());
        return pageInfo;
    }

    /**
     * 试算平衡表生成版的导出(第一种方式)
     * @param citTrialBalanceDto
     * @param os
     * @return
     */
    public int exportTbGeneVerData(CitTrialBalanceDto citTrialBalanceDto, OutputStream os) {
        try {
            CitTrialBalanceExample citTbExample = new CitTrialBalanceExample();
            CitTrialBalanceExample.Criteria citTbExampleCriteria = citTbExample.createCriteria();
            citTbExampleCriteria.andProjectIdEqualTo(citTrialBalanceDto.getProjectId());
            List<CitTrialBalance> citTbList = citTbMapper.selectByExample(citTbExample);

            Map<String, String> header = generalTbGeneVerHeader();
            List<CitTrialBalanceExportDto> cellList = new ArrayList<>();
            citTbList.forEach(tb -> {
                CitTrialBalanceExportDto d = new CitTrialBalanceExportDto();
                d = beanUtil.copyProperties(tb, d);
                cellList.add(d);
            });
            ExcelUtil.exportExcel(header, cellList, os);
            return cellList.size();
        } catch (Exception e) {
            logger.error("试算平衡表生成版导出转换Excel异常: %s", e.getMessage());
            return 0;
        }
    }

    /**
     * 试算平衡表生成版的导出(第二种导出方式)
     * @param citTrialBalanceDto
     * @param response
     * @return
     */
    public int exportTbGeneVerData2(CitTrialBalanceDto citTrialBalanceDto, HttpServletResponse response){
        CitTrialBalanceExample citTbExample = new CitTrialBalanceExample();
        CitTrialBalanceExample.Criteria citTbExampleCriteria = citTbExample.createCriteria();
//            citTbExampleCriteria.andProjectIdEqualTo(citTrialBalanceDto.getProjectId());
        List<CitTrialBalance> citTbList = citTbMapper.selectByExample(citTbExample);
        if(citTbList.size()==0){
            return  0;
        }
        ExportDto exportDto = new ExportDto();
        exportDto.setFileName("试算平衡表生成版");
        exportDto.setTemplateUrl(Constant.citTemplateUrl +  "/citTbGeneVer.xlsx");
        exportDto.setResponse(response);
        exportDto.setList(citTbList);
        exportDto.setRelation(null);
        JxlsUtils.export(exportDto);
        return 1;
    }

    /**
     * 试算平衡表生成版的导出(第一种方式)所用到的生成Excel的表头
     * @return
     */
    private Map<String, String> generalTbGeneVerHeader() {
        Map<String, String> header = new LinkedHashMap<>();
        header.put("AccountCode","科目代码");
        header.put("AccountDescription","科目说明");

        header.put("AccountingPeriod","期间");
        header.put("DebitAmount","借方发生额");
        header.put("CreditAmount","贷方发生额");
        header.put("BeginningBalance","期初余额");
        header.put("EndingBalance","期末余额");
        return header;
    }

    /**
     * 获取试算平衡表Mapping版的数据
     * @param citTrialBalanceDto
     * @return
     */
    public PageInfo<CitTrialBalanceDto> getTbMappingVerData(CitTrialBalanceDto citTrialBalanceDto) {
        Page page = PageHelper.startPage(citTrialBalanceDto.getPageInfo().getPageIndex(), citTrialBalanceDto.getPageInfo().getPageSize());
        List<CitTrialBalanceDto> citTbList = citTbMapper.getTbMappingData(citTrialBalanceDto);
//        List<CitTrialBalanceDto> citTbDtos = Lists.newArrayList();
//        citTbList.forEach(tb -> {
//            CitTrialBalanceDto citTrialBalanceDtoTemp = new CitTrialBalanceDto();
//            beanUtil.copyProperties(tb, citTrialBalanceDtoTemp);
//            citTbDtos.add(citTrialBalanceDtoTemp);
//        });

        PageInfo<CitTrialBalanceDto> pageInfo =new PageInfo<>(citTbList);
        pageInfo.setTotal(page.getTotal());
        pageInfo.setPageNum(citTrialBalanceDto.getPageInfo().getPageIndex());
        return pageInfo;
    }

    /**
     * 试算平衡表Mapping版的导出（第一种方式）
     * @param citTrialBalanceDto
     * @param os
     * @return
     */
    public int exportTbMappingVerData(CitTrialBalanceDto citTrialBalanceDto, OutputStream os) {
        try {
            List<CitTrialBalanceDto> citTbList = citTbMapper.getTbMappingData(citTrialBalanceDto);

            Map<String, String> header = generalTbGeneVerHeader();
            header.put("Attribute","小类");
            List<CitTrialBalanceExportDto> cellList = new ArrayList<>();
            citTbList.forEach(tb -> {
                CitTrialBalanceExportDto d = new CitTrialBalanceExportDto();
                d = beanUtil.copyProperties(tb, d);
                cellList.add(d);
            });
            ExcelUtil.exportExcel(header, cellList, os);
            return cellList.size();
        } catch (Exception e) {
            logger.error("试算平衡表Mapping版导出转换Excel异常: %s", e.getMessage());
            return 0;
        }
    }

    /**
     * 试算平衡表Mapping版的导出(第二种导出方式)
     * @param citTrialBalanceDto
     * @param response
     * @return
     */
    public int exportTbMappingVerData2(CitTrialBalanceDto citTrialBalanceDto, HttpServletResponse response){
        List<CitTrialBalanceDto> citTbList = citTbMapper.getTbMappingData(citTrialBalanceDto);
        if(citTbList.size()==0){
            return  0;
        }
        ExportDto exportDto = new ExportDto();
        exportDto.setFileName("试算平衡表Mapping版");
        exportDto.setTemplateUrl(Constant.citTemplateUrl +  "/citTbGeneMapping.xlsx");
        exportDto.setResponse(response);
        exportDto.setList(citTbList);
        exportDto.setRelation(citTbList.get(0));
        JxlsUtils.export(exportDto);
        return 1;
    }

}
