package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.util.JxlsUtils;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.dao.CitEAMAssetsDisposalMapper;
import pwc.taxtech.atms.dao.CitSalaryAdvanceMapper;
import pwc.taxtech.atms.dto.export.ExportDto;
import pwc.taxtech.atms.dto.previewData.CitEAMAssetsDisposalDto;
import pwc.taxtech.atms.dto.previewData.CitSalaryDataDto;
import pwc.taxtech.atms.entity.CitEAMAssetsDisposal;
import pwc.taxtech.atms.entity.CitEAMAssetsDisposalExample;
import pwc.taxtech.atms.entity.CitSalaryAdvance;
import pwc.taxtech.atms.entity.CitSalaryAdvanceExample;
import javax.servlet.http.HttpServletResponse;
import java.util.*;
@Service
public class CitPreviewDataServiceImpl extends  BaseService {

    @Autowired
    private CitSalaryAdvanceMapper citSalaryAdvanceMapper;

    public PageInfo<CitSalaryAdvance> getCitSalaryAdvanceDataList(CitSalaryDataDto citSalaryDataDto ){
        CitSalaryAdvanceExample example = new CitSalaryAdvanceExample();
        CitSalaryAdvanceExample.Criteria criteria = example.createCriteria();
        if(!"".equals(citSalaryDataDto.getPoSubjectName()) && citSalaryDataDto.getPoSubjectName() !=  null){//根据PO主体名称进行查询
            criteria.andPoSubjectNameEqualTo(citSalaryDataDto.getPoSubjectName());
        }
        Integer periodStart = null;
        Integer periodEnd = null;
        if(citSalaryDataDto.getPeriodStart() != null && !"".equals(citSalaryDataDto.getPeriodStart())){
            periodStart = citSalaryDataDto.getPeriodStart();
        }
        if(citSalaryDataDto.getPeriodEnd() != null &&!"".equals(citSalaryDataDto.getPeriodEnd())){
            periodEnd = citSalaryDataDto.getPeriodEnd();
        }
        if(periodEnd != null && periodStart!= null){
            criteria.andPeriodBetween(periodStart, periodEnd);
        }
        if(periodStart == null && periodEnd!= null){
            criteria.andPeriodLessThanOrEqualTo(periodEnd);
        }
        if(periodStart != null  && periodEnd == null ){
            criteria.andPeriodGreaterThanOrEqualTo(periodStart);
        }
        Page page = PageHelper.startPage(citSalaryDataDto.getPageInfo().getPageIndex(), citSalaryDataDto.getPageInfo().getPageSize());
        List<CitSalaryAdvance> citSalaryAdvances = citSalaryAdvanceMapper.selectByExample(example);
        PageInfo<CitSalaryAdvance> pageInfo =new PageInfo<CitSalaryAdvance>(citSalaryAdvances);
        pageInfo.setTotal(page.getTotal());
        return pageInfo;
    }

    @Autowired
    private CitEAMAssetsDisposalMapper citEAMAssetsDisposalMapper;

    public PageInfo<CitEAMAssetsDisposal> getCitPreviewEamDisposalDataList(CitEAMAssetsDisposalDto citEAMAssetsDisposalDto) {
        CitEAMAssetsDisposalExample example = new CitEAMAssetsDisposalExample();
        CitEAMAssetsDisposalExample.Criteria criteria = example.createCriteria();
        Integer periodStart = null;
        Integer periodEnd = null;
        if(citEAMAssetsDisposalDto.getPeriodStart() != null && !"".equals(citEAMAssetsDisposalDto.getPeriodStart())){
            periodStart = Integer.parseInt(citEAMAssetsDisposalDto.getPeriodStart());
        }
        if(citEAMAssetsDisposalDto.getPeriodEnd() != null &&!"".equals(citEAMAssetsDisposalDto.getPeriodEnd())){
            periodEnd = Integer.parseInt(citEAMAssetsDisposalDto.getPeriodEnd());
        }

        if(citEAMAssetsDisposalDto.getAssetLabelNumber() != null && !"".equals(citEAMAssetsDisposalDto.getAssetLabelNumber())){
            criteria.andAssetLabelNumberEqualTo(citEAMAssetsDisposalDto.getAssetLabelNumber());
        }

        if(citEAMAssetsDisposalDto.getCompensationSaleAmount() != null  && !"".equals(citEAMAssetsDisposalDto.getCompensationSaleAmount())){
            criteria.andCompensationSaleAmountEqualTo(citEAMAssetsDisposalDto.getCompensationSaleAmount());
        }

        if(periodEnd != null && periodStart!= null){
            criteria.andPeriodBetween(periodStart, periodEnd);
        }
        if(periodStart == null && periodEnd!= null){
            criteria.andPeriodLessThanOrEqualTo(periodEnd);
        }
        if(periodStart != null  && periodEnd == null ){
            criteria.andPeriodGreaterThanOrEqualTo(periodStart);
        }

        /*if(citEAMAssetsDisposalDto.getOrganizationId() != null){
            criteria.andOrganizationIdEqualTo(citEAMAssetsDisposalDto.getOrganizationId());
        }
*/
        if(citEAMAssetsDisposalDto.getPeriod() != null && !"".equals(citEAMAssetsDisposalDto.getPeriod())){
            criteria.andPeriodEqualTo(citEAMAssetsDisposalDto.getPeriod());
        }
        Page page = PageHelper.startPage(citEAMAssetsDisposalDto.getPageInfo().getPageIndex(), citEAMAssetsDisposalDto.getPageInfo().getPageSize());
        List<CitEAMAssetsDisposal> citSalaryAdvances = citEAMAssetsDisposalMapper.selectByExample(example);
        PageInfo<CitEAMAssetsDisposal> pageInfo =new PageInfo<CitEAMAssetsDisposal>(citSalaryAdvances);
        pageInfo.setTotal(page.getTotal());
        return  pageInfo;
    }

    private Map<String, String> generalEAMHeader() {
        Map<String, String> header = new LinkedHashMap<>();
        /*header.put("Date","数据日期");*/
        header.put("Source","来源");
        header.put("Period","期间");
        header.put("OccurPeriod","发生期间");
        header.put("AssetLabelNumber","资产标签号");
        header.put("CompensationSaleAmount","赔偿/变卖金额");
        header.put("CreatedBy","创建人");
        return header;
    }
    public int exportDataEAM(CitEAMAssetsDisposalDto citEAMAssetsDisposalDto, HttpServletResponse response) {
      /*  try {
            CitEAMAssetsDisposalExample example = new CitEAMAssetsDisposalExample();
            List<CitEAMAssetsDisposal> list = citEAMAssetsDisposalMapper.selectByCondition(citEAMAssetsDisposalDto);
            Map<String, String> header = generalEAMHeader();

            List<CitEAMAssetsDisposalExport> cellList = new ArrayList<>();
            list.forEach(tb -> {
                CitEAMAssetsDisposalExport d = new CitEAMAssetsDisposalExport();
                d = beanUtil.copyProperties(tb, d);
                cellList.add(d);
            });
            ExcelUtil.exportExcel(header, cellList, response);
            return list.size();
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("EAM资产处理金额记录表: %s", e.getMessage());
            return 0;
        }*/

      //第二种导出方式
        CitEAMAssetsDisposalExample example = new CitEAMAssetsDisposalExample();
        List<CitEAMAssetsDisposal> list = citEAMAssetsDisposalMapper.selectByCondition(citEAMAssetsDisposalDto);
        if(list.size()==0){
            return  0;
        }
        ExportDto exportDto = new ExportDto();
        exportDto.setFileName("EAM资产处置金额");
        exportDto.setTemplateUrl(Constant.citTemplateUrl +  "/eamDispoal.xlsx");
        exportDto.setResponse(response);
        exportDto.setList(list);
        exportDto.setRelation(null);
        JxlsUtils.export(exportDto);
        return  1;
    }

    public int exportDataSalaryAdvance(CitSalaryDataDto citSalaryDataDto , HttpServletResponse response) {
        CitEAMAssetsDisposalExample example = new CitEAMAssetsDisposalExample();
        List<CitSalaryAdvance> list = citSalaryAdvanceMapper.selectByCondition(citSalaryDataDto);
        if(list.size()==0){
            return  0;
        }
        ExportDto exportDto = new ExportDto();
        exportDto.setFileName("预提重分类数据源");
        exportDto.setTemplateUrl(Constant.citTemplateUrl +  "/salaryAdvance.xlsx");
        exportDto.setResponse(response);
        exportDto.setList(list);
        exportDto.setRelation(null);
        JxlsUtils.export(exportDto);
        return  1;
    }
}
