package pwc.taxtech.atms.service.impl;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.OperateLogType;
import pwc.taxtech.atms.common.OperationAction;
import pwc.taxtech.atms.common.OperationModule;
import pwc.taxtech.atms.common.message.LogMessage;
import pwc.taxtech.atms.common.util.CommonUtil;
import pwc.taxtech.atms.dao.EquityInformationHistoryMapper;
import pwc.taxtech.atms.dao.EquityInformationMapper;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dto.equity.EquityInfoDto;
import pwc.taxtech.atms.dto.OperationLogDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.exception.ApplicationException;

import javax.annotation.Resource;
import java.util.*;

import static pwc.taxtech.atms.common.CommonConstants.CommonFail;
import static pwc.taxtech.atms.common.CommonConstants.SystemError;

@Service
public class EquityServiceImpl extends BaseService{

    @Autowired
    private OperationLogServiceImpl operationLogServiceImpl;

    @Resource
    private EquityInformationMapper equityInformationMapper;

    @Resource
    private EquityInformationHistoryMapper equityInformationHistoryMapper;

    @Resource
    private OrganizationMapper organizationMapper;

    public List<EquityInfoDto> getEquityListByOrgId(String orgId) {
        List<EquityInfoDto> equityInfoDtos = new ArrayList<>();
        try {
            EquityInformationExample equityInformationExample = new EquityInformationExample();
            equityInformationExample.createCriteria().andOrganizationIdEqualTo(orgId);
            List<EquityInformation> equityInformations = equityInformationMapper.selectByExample(equityInformationExample);
            equityInformations.forEach(equityInformation -> {
                        EquityInfoDto equityInfoDto = new EquityInfoDto();
                        // 可能需要补充一些处理
                        BeanUtils.copyProperties(equityInformation, equityInfoDto);
                        equityInfoDtos.add(equityInfoDto);
                    }
            );
        } catch (Exception e) {
            logger.error(String.format("根据机构ID获取全部股东信息异常:%s", e.getMessage()));
            return null;
        }
        return equityInfoDtos;
    }

    public OperationResultDto<Object> insertAll(List<EquityInfoDto> equityInfoDtos) {
        try{
            // todo opt insertAll
            if(equityInfoDtos.get(0).getId()==null){
                equityInfoDtos.get(0).setId(idService.nextId());
                equityInfoDtos.get(0).seteNum(1);
            }
            for (EquityInfoDto equityInfoDto : equityInfoDtos) {
                EquityInformation equityInformation = new EquityInformation();
                BeanUtils.copyProperties(equityInfoDto,equityInformation);
                equityInformationMapper.insert(equityInformation);
            }
            AddOrgEquityLog(OperationAction.New.value(),LogMessage.AddOrganizationEquity,equityInfoDtos.get(0).getOrganizationId(),OperationModule.Equity.value());
            return new OperationResultDto(true);
        }catch (Exception e){
            logger.error(String.format("写入全部股东信息异常:%s", e.getMessage()));
            return new OperationResultDto(false, CommonFail + SystemError);
        }
    }

    public OperationResultDto<Object> update(EquityInfoDto equityInfoDto) {
        try{
            EquityInformation equityInformation = new EquityInformation();
            BeanUtils.copyProperties(equityInfoDto,equityInformation);
            EquityInformationExample example = new EquityInformationExample();
            example.createCriteria().andIdEqualTo(equityInformation.getId()).andENumEqualTo(equityInformation.geteNum());
            equityInformationMapper.updateByExampleSelective(equityInformation,example);
            AddOrgEquityLog(OperationAction.Update.value(),LogMessage.UpdateOrganizationEquity,equityInfoDto.getOrganizationId(),OperationModule.Equity.value());
            return new OperationResultDto(true);
        }catch (Exception e){
            logger.error(String.format("更新股东信息异常:%s", e.getMessage()));
            return new OperationResultDto(false, CommonFail + SystemError);
        }
    }

    public OperationResultDto<Object> delete(Long id, List<Integer> eNums) {
        try {
            if (null == id) {
                return new OperationResultDto(false, CommonFail + SystemError);
            }
            EquityInformationExample example = new EquityInformationExample();
            example.createCriteria().andIdEqualTo(id).andENumIn(eNums);
            int delRes = equityInformationMapper.deleteByExample(example);
            if (delRes > 0) {
                refreshEquityList(id);
            }
            AddOrgEquityLog(OperationAction.Delete.value(),LogMessage.DeleteOrganizationEquity,String.valueOf(id),OperationModule.Equity.value());
            return new OperationResultDto(true);
        } catch (Exception e) {
            logger.error(String.format("删除股东信息异常:%s", e.getMessage()));
            return new OperationResultDto(false, CommonFail + e.getMessage());
        }
    }

    /**
     * 17/01/2019 15:51
     * 变更当前股权信息时，刷新股权信息的序号
     * [id]
     * @author Gary J Li
     * @return
     */
    private void refreshEquityList(Long id) {
        EquityInformationExample exampleForENumUp = new EquityInformationExample();
        exampleForENumUp.createCriteria().andIdEqualTo(id);
        exampleForENumUp.setOrderByClause(" e_num ASC");
        List<EquityInformation> equityInformations = equityInformationMapper.selectByExample(exampleForENumUp);
        if (equityInformations.size() == 0) {
            return;
        }
        List<EquityInformation> resList = new ArrayList<>();
        int i =1;
        for (EquityInformation record : equityInformations) {
            if(record.geteNum()!=i){
                record.seteNum(i);
            }
            resList.add(record);
            i++;
        }
        EquityInformationExample delAllExample = new EquityInformationExample();
        delAllExample.createCriteria().andIdEqualTo(id);
        int delRes2 = equityInformationMapper.deleteByExample(delAllExample);
        if (delRes2 > 0) {
            // todo opt insertAll
            for (EquityInformation equityInfo : resList) {
                equityInformationMapper.insert(equityInfo);
            }
        }
    }

    /**
     * 17/01/2019 20:18 股权变更
     *  1、根据id（原始）查出旧数据 equity_infomation
     *  2、旧数据插入历史表 equity_infomation_history
     *  3、equityInfoDtos 生成新id、createTime、updateTime 插入equity_infomation
     *  4、根据id（原始）删除旧数据 equity_infomation
     *  5、记录日志  id（原始）
     *
     *  remark:
     *  1、无股权记录时，为初始变更只进行3、5
     *  2、// todo 事务过长并发时，可能出现问题
     * [equityInfoDtos]
     * @author Gary J Li
     * @return OperationResultDto<Object>
     */
    public OperationResultDto<Object> change(String orgName, String comment, List<EquityInfoDto> equityInfoDtos) {

        EquityInformationExample exampleForOldData = new EquityInformationExample();
        Long oldId = equityInfoDtos.get(0).getId();
        Long newId = idService.nextId();

        // remark-1 无股权记录时，初始变更为新增
        String orgId = equityInfoDtos.get(0).getOrganizationId();

        Organization org = organizationMapper.selectByPrimaryKey(orgId);

        List<EquityInformation> oldData = null;
        if (null != oldId) {
            // 1、根据id（原始）查出旧数据 equity_infomation
            exampleForOldData.createCriteria().andIdEqualTo(oldId);
            oldData = equityInformationMapper.selectByExample(exampleForOldData);
            // 2、旧数据插入历史表 equity_infomation_history
            for (EquityInformation equityInformation : oldData) {
                int insOldRes = equityInformationHistoryMapper.insert(equityInformation);
                if (insOldRes < 1) {
                    throw new ApplicationException("变更股权-2 写入历史数据异常");
                }
            }
        }
        // 3、equityInfoDtos 生成新id、createTime、updateTime 插入equity_infomation
        int i = 1;
        List<EquityInformation> newDataList = new ArrayList<>();
        for (EquityInfoDto equityInfoDto : equityInfoDtos) {
            EquityInformation newData = new EquityInformation();
            BeanUtils.copyProperties(equityInfoDto, newData);
            newData.setId(newId);
            newData.seteNum(i);
            float investRadio = 0L;
            if(CommonUtil.hasDigit(org.getRegistrationCapital())&&null!=newData.getInvestmentAmount()){
                investRadio = (float)newData.getInvestmentAmount()/(float)CommonUtil.getNum(org.getRegistrationCapital());
            }
            newData.setInvestmentRadio(investRadio);
            newData.setOrganizationId(orgId);
            Date now = new Date();
            newData.setCreateTime(now);
            newData.setUpdateTime(now);
            int insNewRes = equityInformationMapper.insert(newData);
            if (insNewRes < 1) {
                throw new ApplicationException("变更股权-3 写入新数据异常");
            }
            newDataList.add(newData);
            i++;
        }
        // 4、根据id（原始）删除旧数据 equity_infomation
        if (null != oldId) {
            EquityInformationExample example = new EquityInformationExample();
            example.createCriteria().andIdEqualTo(oldId);
            int delRes = equityInformationMapper.deleteByExample(example);
            if (delRes < 1) {
                throw new ApplicationException("变更股权-4 删除旧数据异常");
            }
        }
        // todo 5、记录日志  id（原始） 补充变更日期
        OperationLogDto opLog = new OperationLogDto();
        opLog.setId(String.valueOf(newId));
        if (null != oldId) {
            opLog.setAction(OperationAction.NewEquity.value());
        } else {
            opLog.setAction(OperationAction.ChangeEquity.value());
        }
        opLog.setOperationContent(LogMessage.ChangeOrganizationEquity);
        opLog.setOperationObject(orgName);
        opLog.setOriginalState(oldId == null ? "" : String.valueOf(oldId));
        opLog.setUpdateState(String.valueOf(newId));
        opLog.setIp(authUserHelper.getClientIp());
        opLog.setModule(OperationModule.ChangeEquity.value());
        opLog.setComment(comment);
        opLog.setLogType(OperateLogType.OperationLogEquity.value());
        opLog.setEquityLog(true);
        operationLogService.addOperationLog(opLog);
        AddOrgEquityLog(OperationAction.Update.value(),LogMessage.ChangeOrganizationEquity,orgName,OperationModule.Equity.value());
        return new OperationResultDto(true, "变更成功!", oldId);
    }

    /**
     * 18/01/2019 14:03
     * 根据id查询出所有日志，时间排序
     * [ids]
     * @author Gary J Li
     * @return List<OperationLogBasicData>
     */
    public List<OperationLogBasicData> getChangeLogByOrgName(String orgName) {
        List<OperationLogBasicData> logListByIds = null;
        try {
            logListByIds = operationLogServiceImpl.getEquityChangeLogByOrgName(orgName);
        } catch (Exception e) {
            logger.error(String.format("根据ID获取股权日志信息异常:%s", e.getMessage()));
            return null;
        }
        return logListByIds;
    }

    /**
     * 18/01/2019 13:58 撤销变更
     * 1、根据oldId历史数据查出来equityInfomations  equity_infomation_history
     * 2、equityInfomations存入新表   equity_infomation
     * 3、根据newId删除 equity_infomation
     * 4、根据oldId删除历史数据   equity_infomation_history
     * 5、根据oldId删除操作日志   operation_log_equity
     * remark : 最初的记录撤销变更仅做3、5
     * [id]
     * @author Gary J Li
     * @return OperationResultDto<Object>
     */
    public OperationResultDto<Object> cancelChange(Long oldId, Long newId) {
        if (null != oldId) {
            //        1、根据oldId历史数据查出来equityInfomations  equity_infomation_history
            EquityInformationExample exampleForOldData = new EquityInformationExample();
            exampleForOldData.createCriteria().andIdEqualTo(oldId);
            List<EquityInformation> equityInformations = equityInformationHistoryMapper.selectByExample(exampleForOldData);
            //        2、equityInfomations存入新表   equity_infomation
            int insRes = 0;
            for (EquityInformation equityInformation : equityInformations) {
                insRes += equityInformationMapper.insert(equityInformation);
            }
            if (insRes < 1) {
                logger.warn(String.format("撤销变更-2 表equity_infomation 存入新表数据异常.id: [ %s ]", newId));
                throw new ApplicationException("撤销变更-2 存入新表数据异常");
            }
        }
//        3、根据newId删除 equity_infomation
        EquityInformationExample exampleDeleteOld = new EquityInformationExample();
        exampleDeleteOld.createCriteria().andIdEqualTo(newId);
        if (equityInformationMapper.deleteByExample(exampleDeleteOld) < 1) {
            logger.warn(String.format("撤销变更-3 表equity_infomation 删除股权数据异常.id: [ %s ]", newId));
            throw new ApplicationException("撤销变更-3 删除股权数据异常");
        }
        if (null != oldId) {
//        4、根据oldId删除历史数据   equity_infomation_history
            EquityInformationExample exampleDeleteNew = new EquityInformationExample();
            exampleDeleteNew.createCriteria().andIdEqualTo(oldId);
            if (equityInformationHistoryMapper.deleteByExample(exampleDeleteNew) < 1) {
                logger.warn(String.format("撤销变更-3 表equity_infomation_history 删除股权数据异常.id: [ %s ]", oldId));
                throw new ApplicationException("撤销变更-4 删除历史数据异常");
            }
        }
//        5、根据oldId删除操作日志   operation_log_equity
        if (operationLogServiceImpl.deleteById(newId) < 1) {
            logger.warn(String.format("撤销变更-3 表operation_log_equity 删除操作日志异常.id: [ %s ]", oldId));
            throw new ApplicationException("撤销变更-5 删除操作日志异常");
        }
        AddOrgEquityLog(OperationAction.Update.value(),LogMessage.CancelChangeUpdateOrganizationEquity,String.valueOf(newId),OperationModule.Equity.value());
        return new OperationResultDto(true);
    }

    public List<EquityInfoDto> getEquityListById(Long id) {
        List<EquityInfoDto> equityInfoDtos = new ArrayList<>();
        try {
            EquityInformationExample equityInformationExample = new EquityInformationExample();
            equityInformationExample.createCriteria().andIdEqualTo(id);
            List<EquityInformation> equityInformations = equityInformationMapper.selectByExample(equityInformationExample);
            equityInformations.forEach(equityInformation -> {
                        EquityInfoDto equityInfoDto = new EquityInfoDto();
                        // 可能需要补充一些处理
                        BeanUtils.copyProperties(equityInformation, equityInfoDto);
                        equityInfoDtos.add(equityInfoDto);
                    }
            );
        } catch (Exception e) {
            logger.error(String.format("根据ID获取全部股东信息异常:%s", e.getMessage()));
            return null;
        }
        return equityInfoDtos;
    }

    public List<EquityInfoDto> getEquityHisListById(Long id) {
        List<EquityInfoDto> equityInfoDtos = new ArrayList<>();
        try {
            EquityInformationExample equityInformationExample = new EquityInformationExample();
            equityInformationExample.createCriteria().andIdEqualTo(id);
            List<EquityInformation> equityInformations = equityInformationHistoryMapper.selectByExample(equityInformationExample);
            equityInformations.forEach(equityInformation -> {
                        EquityInfoDto equityInfoDto = new EquityInfoDto();
                        // 可能需要补充一些处理
                        BeanUtils.copyProperties(equityInformation, equityInfoDto);
                        equityInfoDtos.add(equityInfoDto);
                    }
            );
        } catch (Exception e) {
            logger.error(String.format("根据ID获取历史全部股东信息异常:%s", e.getMessage()));
            return null;
        }
        return equityInfoDtos;
    }

    /**
     * 25/01/2019 14:39
     * [增删改查，操作描述，操作对象，操作模块] 更新、删除需补充oldData、newData
     * [actionValue, content, object, module]
     * @author Gary J Li
     * @return
     */
    private void AddOrgEquityLog(int actionValue,String content,String object,Integer module) {
        OperationLogDto opLog = new OperationLogDto();
        opLog.setAction(actionValue);
        opLog.setOperationContent(content);
        opLog.setOperationObject(object);
        opLog.setOriginalState("");
        opLog.setUpdateState("");
        opLog.setIp("");
        opLog.setModule(module);
        opLog.setComment("");
        opLog.setLogType(OperateLogType.OperationLogOrganization.value());
        operationLogService.addOperationLog(opLog);
    }
}
