package pwc.taxtech.atms.service.impl;

import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.dao.FormulaConfigDao;
import pwc.taxtech.atms.dao.FormulaParamConfigDao;
import pwc.taxtech.atms.dao.FormulaParamMappingDao;
import pwc.taxtech.atms.dao.FormulaParamOptionDao;
import pwc.taxtech.atms.dto.formula.FormulaParamConfigDto;
import pwc.taxtech.atms.dto.formula.FormulaParamMappingDto;
import pwc.taxtech.atms.dto.formula.FormulaParamOptionDto;
import pwc.taxtech.atms.entity.FormulaConfig;
import pwc.taxtech.atms.entity.FormulaParamConfig;
import pwc.taxtech.atms.entity.FormulaParamMapping;
import pwc.taxtech.atms.entity.FormulaParamOption;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Service
public class FormulaServiceImpl extends BaseService {

    @Autowired
    FormulaConfigDao formulaConfigDao;
    @Autowired
    FormulaParamOptionDao formulaParamOptionDao;
    @Autowired
    FormulaParamConfigDao paramConfigDao;
    @Autowired
    FormulaParamMappingDao paramMappingDao;

    public List<FormulaConfig> getAllFormulaList() {
        return formulaConfigDao.getAll();
    }

    public List<FormulaParamConfigDto> getAllFormulaParamList() {
        List<FormulaParamOption> paramOpts = formulaParamOptionDao.getAll();
        List<FormulaParamOptionDto> paramOptions = paramOpts.stream().map(x -> {
            FormulaParamOptionDto dto = new FormulaParamOptionDto();
            return CommonUtils.copyProperties(x, dto);
        }).collect(Collectors.toList());
        List<FormulaParamConfig> paramCfg = paramConfigDao.getAll();
        List<FormulaParamConfigDto> list = paramCfg.stream().map(x -> {
            FormulaParamConfigDto dto = new FormulaParamConfigDto();
            return CommonUtils.copyProperties(x, dto);
        }).collect(Collectors.toList());
        Set<String> optionFormulaIds = paramOptions.stream().map(FormulaParamOptionDto::getFormulaParamId).collect(Collectors.toSet());

        list.forEach(x -> {
            x.setOptions(Lists.newArrayList());
            if (optionFormulaIds.contains(x.getId())) {
                x.getOptions().addAll(paramOptions.stream().filter(y -> StringUtils.equals(y.getFormulaParamId(),
                        x.getId())).collect(Collectors.toList()));
            }
        });
        return list;
    }

    public List<FormulaParamMappingDto> getAllFormulaParamMappingList() {
        List<FormulaParamMapping> paramMap = paramMappingDao.getAll();
        return paramMap.stream().map(x -> {
            FormulaParamMappingDto dto = new FormulaParamMappingDto();
            return CommonUtils.copyProperties(x, dto);
        }).collect(Collectors.toList());
    }
}
