package pwc.taxtech.atms.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.OperateLogType;
import pwc.taxtech.atms.common.OperationAction;
import pwc.taxtech.atms.common.OperationModule;
import pwc.taxtech.atms.common.message.LogMessage;
import pwc.taxtech.atms.dao.*;
import pwc.taxtech.atms.dto.OperationLogDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.organization.*;
import pwc.taxtech.atms.entity.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
@Service
public class OrganizationExtraServiceImpl extends BaseService{

    @Resource
    private OrganizationInvoiceMapper organizationInvoiceMapper;

    @Resource
    private OrganizationAccountingRateMapper organizationAccountingRateMapper;

    @Resource
    private OrganizationReturnRateMapper organizationReturnRateMapper;

    @Resource
    private OrganizationApprovedLevyInfoMapper organizationApprovedLevyInfoMapper;

    @Resource
    private OrganizationTaxOfficerMapper organizationTaxOfficerMapper;

    @Resource
    private OrganizationTaxRuleMapper organizationTaxRuleMapper;

    @Resource
    private OrganizationEmployeeMapper organizationEmployeeMapper;

    @Resource
    private OrganizationTaxpayerQualificationMapper organizationTaxpayerQualificationMapper;

    public List<OrganizationInvoiceDto> getInvoiceListByOrgId(String orgId) {
        List<OrganizationInvoiceDto> res = new ArrayList<>();
        OrganizationInvoiceExample example = new OrganizationInvoiceExample();
        example.createCriteria().andOrganizationIdEqualTo(orgId);
        List<OrganizationInvoice> organizationInvoices= organizationInvoiceMapper.selectByExample(example);
        organizationInvoices.forEach(oi->{
            OrganizationInvoiceDto organizationInvoiceDto = new OrganizationInvoiceDto();
            beanUtil.copyProperties(oi,organizationInvoiceDto);
            res.add(organizationInvoiceDto);
        });
        return res;
    }

    public OperationResultDto<Object> insertAllInvoice(List<OrganizationInvoiceDto> organizationInvoiceDtos) {
        List<String> taxGroupList = new ArrayList<>();
        organizationInvoiceDtos.forEach(oid->{
            OrganizationInvoice organizationInvoice = new OrganizationInvoice();
            beanUtil.copyProperties(oid,organizationInvoice);
            organizationInvoice.setId(idService.nextId());
            organizationInvoiceMapper.insertSelective(organizationInvoice);
            taxGroupList.add(oid.getInvoiceTypeName());
        });
        AddOrgExtraLog(OperationAction.New.value(),LogMessage.AddOrganizationInvoice,organizationInvoiceDtos.get(0).getOrganizationId(),OperationModule.OrganizationInvoice.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> updateInvoice(OrganizationInvoiceDto organizationInvoiceDto) {
        OrganizationInvoiceExample example = new OrganizationInvoiceExample();
        example.createCriteria().andIdEqualTo(organizationInvoiceDto.getId());
        OrganizationInvoice organizationInvoice = new OrganizationInvoice();
        beanUtil.copyProperties(organizationInvoiceDto,organizationInvoice);
        organizationInvoiceMapper.updateByExampleSelective(organizationInvoice,example);
        AddOrgExtraLog(OperationAction.Update.value(),LogMessage.UpdateOrganizationInvoice,organizationInvoiceDto.getOrganizationId(),OperationModule.OrganizationInvoice.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> deleteInvoice(Long id) {
        OrganizationInvoiceExample example = new OrganizationInvoiceExample();
        example.createCriteria().andIdEqualTo(id);
        organizationInvoiceMapper.deleteByExample(example);
        AddOrgExtraLog(OperationAction.Delete.value(),LogMessage.DeleteOrganizationInvoice,String.valueOf(id),OperationModule.OrganizationInvoice.value());
        return new OperationResultDto(true);
    }

    public List<OrganizationAccountingRateDto> getAccountingRateListByOrgId(String orgId) {
        List<OrganizationAccountingRateDto> res = new ArrayList<>();
        OrganizationAccountingRateExample example = new OrganizationAccountingRateExample();
        example.createCriteria().andOrganizationIdEqualTo(orgId);
        List<OrganizationAccountingRate> organizationAccountingRates = organizationAccountingRateMapper.selectByExample(example);
        organizationAccountingRates.forEach(oar -> {
                    OrganizationAccountingRateDto organizationAccountingRateDto = new OrganizationAccountingRateDto();
                    beanUtil.copyProperties(oar, organizationAccountingRateDto);
                    res.add(organizationAccountingRateDto);
                }
        );
        return res;
    }

    public OperationResultDto<Object> insertAllAccountingRate(List<OrganizationAccountingRateDto> organizationAccountingRateDtos) {
        organizationAccountingRateDtos.forEach(oard->{
            OrganizationAccountingRate organizationAccountingRate = new OrganizationAccountingRate();
            beanUtil.copyProperties(oard,organizationAccountingRate);
            organizationAccountingRate.setId(idService.nextId());
            organizationAccountingRateMapper.insertSelective(organizationAccountingRate);
        });
        AddOrgExtraLog(OperationAction.New.value(),LogMessage.AddOrganizationAccountingRate,organizationAccountingRateDtos.get(0).getOrganizationId(),OperationModule.OrganizationAccountingRate.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> updateAccountingRate(OrganizationAccountingRateDto organizationAccountingRateDto) {
        OrganizationAccountingRateExample example = new OrganizationAccountingRateExample();
        example.createCriteria().andIdEqualTo(organizationAccountingRateDto.getId());
        OrganizationAccountingRate organizationAccountingRate = new OrganizationAccountingRate();
        beanUtil.copyProperties(organizationAccountingRateDto,organizationAccountingRate);
        organizationAccountingRateMapper.updateByExampleSelective(organizationAccountingRate,example);
        AddOrgExtraLog(OperationAction.Update.value(),LogMessage.UpdateOrganizationAccountingRate,organizationAccountingRateDto.getOrganizationId(),OperationModule.OrganizationAccountingRate.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> deleteAccountingRate(Long id) {
        OrganizationAccountingRateExample example = new OrganizationAccountingRateExample();
        example.createCriteria().andIdEqualTo(id);
        organizationAccountingRateMapper.deleteByExample(example);
        AddOrgExtraLog(OperationAction.Delete.value(),LogMessage.DeleteOrganizationAccountingRate,String.valueOf(id),OperationModule.OrganizationAccountingRate.value());
        return new OperationResultDto(true);
    }

    public List<OrganizationReturnRateDto> getReturnRateListByOrgId(String orgId) {
        List<OrganizationReturnRateDto> res = new ArrayList<>();
        OrganizationReturnRateExample example = new OrganizationReturnRateExample();
        example.createCriteria().andOrganizationIdEqualTo(orgId);
        List<OrganizationReturnRate> OrganizationReturnRates = organizationReturnRateMapper.selectByExample(example);
        OrganizationReturnRates.forEach(oar -> {
                    OrganizationReturnRateDto OrganizationReturnRateDto = new OrganizationReturnRateDto();
                    beanUtil.copyProperties(oar, OrganizationReturnRateDto);
                    res.add(OrganizationReturnRateDto);
                }
        );
        return res;
    }

    public OperationResultDto<Object> insertAllReturnRate(List<OrganizationReturnRateDto> organizationReturnRateDtos) {
        organizationReturnRateDtos.forEach(orrd->{
            OrganizationReturnRate organizationReturnRate = new OrganizationReturnRate();
            beanUtil.copyProperties(orrd,organizationReturnRate);
            organizationReturnRate.setId(idService.nextId());
            organizationReturnRateMapper.insertSelective(organizationReturnRate);
        });
        AddOrgExtraLog(OperationAction.New.value(),LogMessage.AddOrganizationReturnRate,organizationReturnRateDtos.get(0).getOrganizationId(),OperationModule.OrganizationReturnRate.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> updateReturnRate(OrganizationReturnRateDto organizationReturnRateDto) {
        OrganizationReturnRateExample example = new OrganizationReturnRateExample();
        example.createCriteria().andIdEqualTo(organizationReturnRateDto.getId());
        OrganizationReturnRate organizationReturnRate = new OrganizationReturnRate();
        beanUtil.copyProperties(organizationReturnRateDto,organizationReturnRate);
        organizationReturnRateMapper.updateByExampleSelective(organizationReturnRate,example);
        AddOrgExtraLog(OperationAction.Update.value(),LogMessage.UpdateOrganizationReturnRate,organizationReturnRateDto.getOrganizationId(),OperationModule.OrganizationReturnRate.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> deleteReturnRate(Long id) {
        OrganizationReturnRateExample example = new OrganizationReturnRateExample();
        example.createCriteria().andIdEqualTo(id);
        organizationReturnRateMapper.deleteByExample(example);
        AddOrgExtraLog(OperationAction.Delete.value(),LogMessage.DeleteOrganizationReturnRate,String.valueOf(id),OperationModule.OrganizationReturnRate.value());
        return new OperationResultDto(true);
    }

    public List<OrganizationApprovedLevyInfoDto> getApprovedLevyInfoListByOrgId(String orgId) {
        List<OrganizationApprovedLevyInfoDto> res = new ArrayList<>();
        OrganizationApprovedLevyInfoExample example = new OrganizationApprovedLevyInfoExample();
        example.createCriteria().andOrganizationIdEqualTo(orgId);
        List<OrganizationApprovedLevyInfo> organizationApprovedLevyInfos = organizationApprovedLevyInfoMapper.selectByExample(example);
        organizationApprovedLevyInfos.forEach(oali -> {
                    OrganizationApprovedLevyInfoDto organizationApprovedLevyInfoDto = new OrganizationApprovedLevyInfoDto();
                    beanUtil.copyProperties(oali, organizationApprovedLevyInfoDto);
                    res.add(organizationApprovedLevyInfoDto);
                }
        );
        return res;
    }

    public OperationResultDto<Object> insertAllApprovedLevyInfo(List<OrganizationApprovedLevyInfoDto> organizationApprovedLevyInfoDtos) {
        organizationApprovedLevyInfoDtos.forEach(oali->{
            OrganizationApprovedLevyInfo organizationApprovedLevyInfo = new OrganizationApprovedLevyInfo();
            beanUtil.copyProperties(oali,organizationApprovedLevyInfo);
            organizationApprovedLevyInfo.setId(idService.nextId());
            organizationApprovedLevyInfoMapper.insertSelective(organizationApprovedLevyInfo);
        });
        AddOrgExtraLog(OperationAction.New.value(),LogMessage.AddOrganizationApprovedLevyInfo,organizationApprovedLevyInfoDtos.get(0).getOrganizationId(),OperationModule.OrganizationApprovedLevyInfo.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> updateApprovedLevyInfo(OrganizationApprovedLevyInfoDto organizationApprovedLevyInfoDto) {
        OrganizationApprovedLevyInfoExample example = new OrganizationApprovedLevyInfoExample();
        example.createCriteria().andIdEqualTo(organizationApprovedLevyInfoDto.getId());
        OrganizationApprovedLevyInfo organizationApprovedLevyInfo = new OrganizationApprovedLevyInfo();
        beanUtil.copyProperties(organizationApprovedLevyInfoDto,organizationApprovedLevyInfo);
        organizationApprovedLevyInfoMapper.updateByExampleSelective(organizationApprovedLevyInfo,example);
        AddOrgExtraLog(OperationAction.Update.value(),LogMessage.UpdateOrganizationApprovedLevyInfo,organizationApprovedLevyInfoDto.getOrganizationId(),OperationModule.OrganizationApprovedLevyInfo.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> deleteApprovedLevyInfo(Long id) {
        OrganizationApprovedLevyInfoExample example = new OrganizationApprovedLevyInfoExample();
        example.createCriteria().andIdEqualTo(id);
        organizationApprovedLevyInfoMapper.deleteByExample(example);
        AddOrgExtraLog(OperationAction.Delete.value(),LogMessage.DeleteOrganizationApprovedLevyInfo,String.valueOf(id),OperationModule.OrganizationApprovedLevyInfo.value());
        return new OperationResultDto(true);
    }

    public List<OrganizationTaxOfficerDto> getTaxOfficerListByOrgId(String orgId) {
        List<OrganizationTaxOfficerDto> res = new ArrayList<>();
        OrganizationTaxOfficerExample example = new OrganizationTaxOfficerExample();
        example.createCriteria().andOrganizationIdEqualTo(orgId);
        List<OrganizationTaxOfficer> organizationTaxOfficers = organizationTaxOfficerMapper.selectByExample(example);
        organizationTaxOfficers.forEach(oto -> {
                    OrganizationTaxOfficerDto organizationTaxOfficerDto = new OrganizationTaxOfficerDto();
                    beanUtil.copyProperties(oto, organizationTaxOfficerDto);
                    res.add(organizationTaxOfficerDto);
                }
        );
        return res;
    }

    public OperationResultDto<Object> insertAllTaxOfficer(List<OrganizationTaxOfficerDto> organizationTaxOfficerDtos) {
        organizationTaxOfficerDtos.forEach(oali->{
            OrganizationTaxOfficer organizationTaxOfficer = new OrganizationTaxOfficer();
            beanUtil.copyProperties(oali,organizationTaxOfficer);
            organizationTaxOfficer.setId(idService.nextId());
            organizationTaxOfficerMapper.insertSelective(organizationTaxOfficer);
        });
        AddOrgExtraLog(OperationAction.New.value(),LogMessage.AddOrganizationTaxOfficer,organizationTaxOfficerDtos.get(0).getOrganizationId(),OperationModule.OrganizationTaxOfficer.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> updateTaxOfficer(OrganizationTaxOfficerDto organizationTaxOfficerDto) {
        OrganizationTaxOfficerExample example = new OrganizationTaxOfficerExample();
        example.createCriteria().andIdEqualTo(organizationTaxOfficerDto.getId());
        OrganizationTaxOfficer organizationTaxOfficer = new OrganizationTaxOfficer();
        beanUtil.copyProperties(organizationTaxOfficerDto,organizationTaxOfficer);
        organizationTaxOfficerMapper.updateByExampleSelective(organizationTaxOfficer,example);
        AddOrgExtraLog(OperationAction.Update.value(),LogMessage.UpdateOrganizationTaxOfficer,organizationTaxOfficerDto.getOrganizationId(),OperationModule.OrganizationTaxOfficer.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> deleteTaxOfficer(Long id) {
        OrganizationTaxOfficerExample example = new OrganizationTaxOfficerExample();
        example.createCriteria().andIdEqualTo(id);
        organizationTaxOfficerMapper.deleteByExample(example);
        AddOrgExtraLog(OperationAction.Delete.value(),LogMessage.DeleteOrganizationTaxOfficer,String.valueOf(id),OperationModule.OrganizationTaxOfficer.value());
        return new OperationResultDto(true);
    }

    public List<OrganizationTaxRuleDto> getTaxRuleListByOrgId(String orgId) {
        List<OrganizationTaxRuleDto> res = new ArrayList<>();
        OrganizationTaxRuleExample example = new OrganizationTaxRuleExample();
        example.createCriteria().andOrganizationIdEqualTo(orgId);
        List<OrganizationTaxRule> organizationTaxRules = organizationTaxRuleMapper.selectByExample(example);
        organizationTaxRules.forEach(otr -> {
                    OrganizationTaxRuleDto organizationTaxRuleDto = new OrganizationTaxRuleDto();
                    beanUtil.copyProperties(otr, organizationTaxRuleDto);
                    res.add(organizationTaxRuleDto);
                }
        );
        return res;
    }

    public OperationResultDto<Object> insertAllTaxRule(List<OrganizationTaxRuleDto> organizationTaxRuleDtos) {
        organizationTaxRuleDtos.forEach(otrd->{
            OrganizationTaxRule organizationTaxRule = new OrganizationTaxRule();
            beanUtil.copyProperties(otrd,organizationTaxRule);
            organizationTaxRule.setId(idService.nextId());
            organizationTaxRuleMapper.insertSelective(organizationTaxRule);
        });
        AddOrgExtraLog(OperationAction.New.value(),LogMessage.AddOrganizationTaxRule,organizationTaxRuleDtos.get(0).getOrganizationId(),OperationModule.OrganizationTaxRule.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> updateTaxRule(OrganizationTaxRuleDto organizationTaxRuleDto) {
        OrganizationTaxRuleExample example = new OrganizationTaxRuleExample();
        example.createCriteria().andIdEqualTo(organizationTaxRuleDto.getId());
        OrganizationTaxRule organizationTaxRule = new OrganizationTaxRule();
        beanUtil.copyProperties(organizationTaxRuleDto,organizationTaxRule);
        organizationTaxRuleMapper.updateByExampleSelective(organizationTaxRule,example);
        AddOrgExtraLog(OperationAction.Update.value(),LogMessage.UpdateOrganizationTaxRule,organizationTaxRuleDto.getOrganizationId(),OperationModule.OrganizationTaxRule.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> deleteTaxRule(Long id) {
        OrganizationTaxRuleExample example = new OrganizationTaxRuleExample();
        example.createCriteria().andIdEqualTo(id);
        organizationTaxRuleMapper.deleteByExample(example);
        AddOrgExtraLog(OperationAction.Delete.value(),LogMessage.DeleteOrganizationTaxRule,String.valueOf(id),OperationModule.OrganizationTaxRule.value());
        return new OperationResultDto(true);
    }

    public List<OrganizationEmployeeDto> getEmployeeListByOrgId(String orgId) {
        List<OrganizationEmployeeDto> res = new ArrayList<>();
        OrganizationEmployeeExample example = new OrganizationEmployeeExample();
        example.createCriteria().andOrganizationIdEqualTo(orgId);
        List<OrganizationEmployee> organizationEmployees = organizationEmployeeMapper.selectByExample(example);
        organizationEmployees.forEach(oe -> {
                OrganizationEmployeeDto organizationEmployeeDto = new OrganizationEmployeeDto();
                    beanUtil.copyProperties(oe, organizationEmployeeDto);
                    res.add(organizationEmployeeDto);
                }
        );
        return res;
    }

    public OperationResultDto<Object> insertAllEmployee(List<OrganizationEmployeeDto> organizationEmployeeDtos) {
        organizationEmployeeDtos.forEach(oed->{
            OrganizationEmployee organizationEmployee = new OrganizationEmployee();
            beanUtil.copyProperties(oed,organizationEmployee);
            organizationEmployee.setId(idService.nextId());
            organizationEmployeeMapper.insertSelective(organizationEmployee);
        });
        AddOrgExtraLog(OperationAction.New.value(),LogMessage.AddOrganizationEmployee,organizationEmployeeDtos.get(0).getOrganizationId(),OperationModule.OrganizationEmployee.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> updateEmployee(OrganizationEmployeeDto organizationEmployeeDto) {
        OrganizationEmployeeExample example = new OrganizationEmployeeExample();
        example.createCriteria().andIdEqualTo(organizationEmployeeDto.getId());
        OrganizationEmployee organizationEmployee = new OrganizationEmployee();
        beanUtil.copyProperties(organizationEmployeeDto,organizationEmployee);
        organizationEmployeeMapper.updateByExampleSelective(organizationEmployee,example);
        AddOrgExtraLog(OperationAction.Update.value(),LogMessage.UpdateOrganizationEmployee,organizationEmployeeDto.getOrganizationId(),OperationModule.OrganizationEmployee.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> deleteEmployee(Long id) {
        OrganizationEmployeeExample example = new OrganizationEmployeeExample();
        example.createCriteria().andIdEqualTo(id);
        organizationEmployeeMapper.deleteByExample(example);
        AddOrgExtraLog(OperationAction.Delete.value(),LogMessage.DeleteOrganizationEmployee,String.valueOf(id),OperationModule.OrganizationEmployee.value());
        return new OperationResultDto(true);
    }

    public List<OrganizationTaxpayerQualificationDto> getTaxpayerQualificationListByOrgId(String orgId) {
        List<OrganizationTaxpayerQualificationDto> res = new ArrayList<>();
        OrganizationTaxpayerQualificationExample example = new OrganizationTaxpayerQualificationExample();
        example.createCriteria().andOrganizationIdEqualTo(orgId);
        List<OrganizationTaxpayerQualification> organizationTaxpayerQualifications = organizationTaxpayerQualificationMapper.selectByExample(example);
        organizationTaxpayerQualifications.forEach(oe -> {
                    OrganizationTaxpayerQualificationDto organizationTaxpayerQualificationDto = new OrganizationTaxpayerQualificationDto();
                    beanUtil.copyProperties(oe, organizationTaxpayerQualificationDto);
                    res.add(organizationTaxpayerQualificationDto);
                }
        );
        return res;
    }

    public OperationResultDto<Object> insertAllTaxpayerQualification(List<OrganizationTaxpayerQualificationDto> organizationTaxpayerQualificationDtos) {
        organizationTaxpayerQualificationDtos.forEach(oed->{
            OrganizationTaxpayerQualification organizationTaxpayerQualification = new OrganizationTaxpayerQualification();
            beanUtil.copyProperties(oed,organizationTaxpayerQualification);
            organizationTaxpayerQualification.setId(idService.nextId());
            organizationTaxpayerQualificationMapper.insertSelective(organizationTaxpayerQualification);
        });
        AddOrgExtraLog(OperationAction.New.value(),LogMessage.AddOrganizationTaxpayerQualification,organizationTaxpayerQualificationDtos.get(0).getOrganizationId(),OperationModule.OrganizationTaxpayerQualification.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> updateTaxpayerQualification(OrganizationTaxpayerQualificationDto organizationTaxpayerQualificationDto) {
        OrganizationTaxpayerQualificationExample example = new OrganizationTaxpayerQualificationExample();
        example.createCriteria().andIdEqualTo(organizationTaxpayerQualificationDto.getId());
        OrganizationTaxpayerQualification organizationTaxpayerQualification = new OrganizationTaxpayerQualification();
        beanUtil.copyProperties(organizationTaxpayerQualificationDto,organizationTaxpayerQualification);
        organizationTaxpayerQualificationMapper.updateByExampleSelective(organizationTaxpayerQualification,example);
        AddOrgExtraLog(OperationAction.Update.value(),LogMessage.UpdateOrganizationTaxpayerQualification,organizationTaxpayerQualificationDto.getOrganizationId(),OperationModule.OrganizationTaxpayerQualification.value());
        return new OperationResultDto(true);
    }

    public OperationResultDto<Object> deleteTaxpayerQualification(Long id) {
        OrganizationTaxpayerQualificationExample example = new OrganizationTaxpayerQualificationExample();
        example.createCriteria().andIdEqualTo(id);
        organizationTaxpayerQualificationMapper.deleteByExample(example);
        AddOrgExtraLog(OperationAction.Delete.value(),LogMessage.DeleteOrganizationTaxpayerQualification,String.valueOf(id),OperationModule.OrganizationTaxpayerQualification.value());
        return new OperationResultDto(true);
    }


    /**
     * 24/01/2019 10:41
     * [增删改查，操作描述，操作对象，操作模块] 更新、删除需补充oldData、newData
     * [actionValue, content, object, module]
     * @author Gary J Li
     * @return
     */
    private void AddOrgExtraLog(int actionValue,String content,String object,Integer module) {
        OperationLogDto opLog = new OperationLogDto();
        opLog.setAction(actionValue);
        opLog.setOperationContent(content);
        opLog.setOperationObject(object);
        opLog.setOriginalState("");
        opLog.setUpdateState("");
        opLog.setIp("");
        opLog.setModule(module);
        opLog.setComment("");
        opLog.setLogType(OperateLogType.OperationLogOrganization.value());
        operationLogService.addOperationLog(opLog);
    }
}
