package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.constant.enums.FileUploadEnum;
import pwc.taxtech.atms.constant.enums.ReportFileUploadEnum;
import pwc.taxtech.atms.dao.TaxDocumentMapper;
import pwc.taxtech.atms.dto.TaxDocumentDto;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileIUploadParam;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileUploadDetailResult;
import pwc.taxtech.atms.entity.OperationLogTaxDocument;
import pwc.taxtech.atms.entity.TaxDocument;
import pwc.taxtech.atms.entity.TaxDocumentExample;
import pwc.taxtech.atms.vat.entity.FileUpload;
import pwc.taxtech.atms.vat.entity.ReportFileUpload;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.*;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * 查询
 */
@Slf4j
@Service
public class TaxDocumentServiceImpl {

    @Resource
    private TaxDocumentMapper taxDocumentMapper;

    @Autowired
    private AuthUserHelper authUserHelper;

    @Autowired
    private OperationLogTaxDocServiceImpl operationLogTaxDocService;

    @Autowired
    ReportFileUploadService reportFileUploadService;

    @Autowired
    DidiFileUploadService didiFileUploadService;


    @Autowired
    private OrganizationServiceImpl organizationService;

    public List<TaxDocument> selectTaxDocumentList(TaxDocumentDto taxDocumentDto) {
        List<TaxDocument> dataList = taxDocumentMapper.selectByExample(getExample(taxDocumentDto));
        DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
        fileParam.setUuids(dataList.stream()
                .map(o -> o.getFileUploadId()).collect(Collectors.toList()));
        PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
        Map<String,String> urlMap = null;
        if(CollectionUtils.isNotEmpty(uploadDetail.getList())){
            urlMap = uploadDetail.getList().stream().collect(Collectors.toMap(DidiFileUploadDetailResult::getUid, didiFileUploadDetailResult -> didiFileUploadDetailResult.getViewHttpUrl()));
        }
        if(urlMap!=null){
            for(TaxDocument data:dataList){
                data.setFilePositionUrl(urlMap.get(data.getFileUploadId()));
            }
        }


        return dataList;
    }


    /**
     * 根据条件查询
     *
     * @param taxDocumentDto
     * @return
     */
    private TaxDocumentExample getExample(TaxDocumentDto taxDocumentDto) {
        TaxDocumentExample example = new TaxDocumentExample();
        TaxDocumentExample.Criteria criteria = example.createCriteria();
        //档案属性 fileAttr
        if (StringUtils.isNotBlank(taxDocumentDto.getFileAttr())) {
            criteria.andFileAttrEqualTo(taxDocumentDto.getFileAttr());
        }
        //档案类型 fileType
        if (StringUtils.isNotBlank(taxDocumentDto.getFileType())) {
            criteria.andFileTypeEqualTo(taxDocumentDto.getFileType());
        }
        //文件生效起始日期 fileBeginTime
        if (null != taxDocumentDto.getFileBeginTime()) {
            criteria.andFileTimeGreaterThanOrEqualTo(taxDocumentDto.getFileBeginTime());
        }
        //文件生效到期日期 fileEndTTime
        if (null != taxDocumentDto.getFileEndTTime()) {
            criteria.andFileTimeLessThanOrEqualTo(taxDocumentDto.getFileEndTTime());
        }
        //所属时间 ownTime
        if (null != taxDocumentDto.getOwnTime()) {
            criteria.andOwnTimeEqualTo(taxDocumentDto.getOwnTime());
        }
        //档案名称 fileName
        if (StringUtils.isNotBlank(taxDocumentDto.getFileName())) {
            criteria.andFileNameLike("%" + taxDocumentDto.getFileName() + "%");
        }
        //业务线 businessLine
        if (StringUtils.isNotBlank(taxDocumentDto.getBusinessLine())) {
            criteria.andBusinessLineEqualTo(taxDocumentDto.getBusinessLine());
        }
        //公司名称 companyName
        if (StringUtils.isNotBlank(taxDocumentDto.getCompanyName())) {
            criteria.andCompanyNameEqualTo(taxDocumentDto.getCompanyName());
        }
        //到期起始日 effectiveBeginTime
        if (null != taxDocumentDto.getEffectiveBeginTime()) {
            criteria.andEffectiveTimeGreaterThanOrEqualTo(taxDocumentDto.getEffectiveBeginTime());
        }
        //到期结束日 effectiveEndTime
        if (null != taxDocumentDto.getEffectiveEndTime()) {
            criteria.andEffectiveTimeLessThanOrEqualTo(taxDocumentDto.getEffectiveEndTime());
        }
        //税种 taxType
        if (StringUtils.isNotBlank(taxDocumentDto.getTaxType())) {
            criteria.andTaxTypeEqualTo(taxDocumentDto.getTaxType());
        }
        //实物索引号 physicalIndexNumber
        if (StringUtils.isNotBlank(taxDocumentDto.getPhysicalIndexNumber())) {
            criteria.andPhysicalIndexNumberLike("%" + taxDocumentDto.getPhysicalIndexNumber() + "%");
        }
        //实物存放地点 storageArea
        if (StringUtils.isNotBlank(taxDocumentDto.getStorageArea())) {
            criteria.andStorageAreaLike("%" + taxDocumentDto.getStorageArea() + "%");
        }
        //实物保管人 keeper
        if (StringUtils.isNotBlank(taxDocumentDto.getKeeper())) {
            criteria.andKeeperLike("%" + taxDocumentDto.getKeeper() + "%");
        }
        //审批状态 auditStatus
        if (null != taxDocumentDto.getAuditStatus()) {
            criteria.andAuditStatusEqualTo(taxDocumentDto.getAuditStatus());
        }
        //上传开始日期 uploadBeginTime
        if (null != taxDocumentDto.getUploadBeginTime()) {
            criteria.andUploadTimeGreaterThanOrEqualTo(taxDocumentDto.getUploadBeginTime());
        }
        //上传结束日期 uploadEndTime
        if (null != taxDocumentDto.getUploadEndTime()) {
            criteria.andUploadTimeLessThanOrEqualTo(taxDocumentDto.getUploadEndTime());
        }
        //创建人 creator
        if (StringUtils.isNotBlank(taxDocumentDto.getCreator())) {
            criteria.andCreatorLike("%" + taxDocumentDto.getCreator() + "%");
        }
        return example;
    }

    @Transactional
    public synchronized boolean addTaxDocumentList(MultipartFile file, TaxDocument taxDocument) {
        try {
            //上传文件
            if(StringUtils.isBlank(taxDocument.getFileUploadId())){

                if(ReportFileUploadEnum.ReportType.MAPPING.containsKey(taxDocument.getFileType())){
                    //重新命名
                    String fileName = taxDocument.getCompanyName()+"_"+taxDocument.getFileType()+"_"+taxDocument.getOwnTime();
                    fileName+=file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf("."));
                    FileUpload fileUpload = didiFileUploadService.uploadFile(file,fileName, FileUploadEnum.BizSource.RECORD_UPLOAD.name());
                    taxDocument.setFileUploadId(fileUpload.getUid());
                    taxDocument.setFilePositionUrl(fileUpload.getViewHttpUrl());
                    taxDocument.setFileName(fileName);
                    ReportFileUpload reportFileUpload = new ReportFileUpload();
                    reportFileUpload.setOrgId(taxDocument.getCompanyId());
                    reportFileUpload.setSourceType(ReportFileUploadEnum.SuorceType.RECORD.name());
                    String period = String.valueOf(taxDocument.getOwnTime()).substring(0,6);
                    reportFileUpload.setPeriod(Integer.valueOf(period));
                    reportFileUpload.setFileUploadId(fileUpload.getUid());
                    reportFileUpload.setReportType(taxDocument.getFileType());
                    reportFileUpload.setReportFileName(fileName);
                    reportFileUploadService.saveData(file,reportFileUpload);
                }else{
                    FileUpload fileUpload = didiFileUploadService.uploadFile(file,file.getOriginalFilename(), FileUploadEnum.BizSource.RECORD_UPLOAD.name());
                    taxDocument.setFileUploadId(fileUpload.getUid());
                    taxDocument.setFilePositionUrl(fileUpload.getViewHttpUrl());
                }
            }
            //设置创建人 创建时间信息 设置年份区分
            taxDocument.setCreateTime(new Date());
            taxDocument.setUpdateTime(new Date());
            taxDocument.setCreator(authUserHelper.getCurrentAuditor().get());
            taxDocument.setCreatorId(authUserHelper.getCurrentUserId());
            taxDocument.setUploadTime(new Date());
            taxDocument.setYearRedundancy(Calendar.getInstance().get(Calendar.YEAR));
            int num = taxDocumentMapper.insert(taxDocument);
            //根据公司Id 设置业务线
            String businessLine = organizationService.queryBusinessByCompanyId(taxDocument.getCompanyId());
            if (StringUtils.isNotBlank(businessLine)) {
                taxDocument.setBusinessLine(businessLine);
            }
            if (num > 0) {
                OperationLogTaxDocument actionEntity = buildOperationLogTaxDocument();
                actionEntity.setId(taxDocument.getId().toString());
                actionEntity.setOperationAction("新增");
                actionEntity.setUpdateState(taxDocument.toString());
                boolean result = operationLogTaxDocService.addTaxDocumentList(actionEntity);
                if (result) {
                    return true;
                } else {
                    throw new RuntimeException("TaxDocumentServiceImpl addTaxDocument log error");
                }
            } else {
                return false;
            }
        } catch (Exception e) {
//            log.error("TaxDocumentServiceImpl addTaxDocument error : " + e.getMessage());
            return false;
        }
    }

    public boolean deleteTaxDocument(Long id) {
        try {
            //逻辑删除
            int num = taxDocumentMapper.updateEnableToF(id);
            if (num > 0) {
                OperationLogTaxDocument actionEntity = buildOperationLogTaxDocument();
                actionEntity.setOperationAction("删除");
                actionEntity.setId(id.toString());
                boolean result = operationLogTaxDocService.addTaxDocumentList(actionEntity);
                if (result) {
                    return true;
                } else {
                    throw new RuntimeException("TaxDocumentServiceImpl deleteTaxDocument log error");
                }
            } else {
                return false;
            }
        } catch (Exception e) {
//            log.error("TaxDocumentServiceImpl deleteTaxDocument error : " + e.getMessage());
            return false;
        }
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @Transactional
    public boolean batchDelete(List<Long> ids) {
        try {
            if (null == ids || ids.size() < 1) {
                return false;
            }
            int num = taxDocumentMapper.batchDelete(ids);
            if (num > 0) {
                for (Long id : ids) {
                    OperationLogTaxDocument actionEntity = buildOperationLogTaxDocument();
                    actionEntity.setOperationAction("删除");
                    actionEntity.setId(id.toString());
                    boolean result = operationLogTaxDocService.addTaxDocumentList(actionEntity);
                    if (!result) {
                        throw new RuntimeException("TaxDocumentServiceImpl batchDeleteTaxDocument log error");
                    }
                }
            } else {
                return false;
            }
            return true;
        } catch (Exception e) {
//            log.error("TaxDocumentServiceImpl batchDelete error : " + e.getMessage());
            return false;
        }
    }

    public boolean editFilesType(TaxDocument taxDocument) {
        try {
            taxDocument.setUpdateTime(new Date());
            int num = taxDocumentMapper.updateByPrimaryKey(taxDocument);
            if (num > 0) {
                OperationLogTaxDocument actionEntity = buildOperationLogTaxDocument();
                actionEntity.setOperationAction("修改");
                actionEntity.setId(taxDocument.getId().toString());
                //设置更新值
                actionEntity.setUpdateState(taxDocument.toString());
                boolean result = operationLogTaxDocService.addTaxDocumentList(actionEntity);
                if (result) {
                    return true;
                } else {
                    throw new RuntimeException("TaxDocumentServiceImpl updateTaxDocument log error");
                }
            } else {
                return false;
            }
        } catch (Exception e) {
//            log.error("TaxDocumentServiceImpl updateTaxDocument error : " + e.getMessage());
            return false;
        }
    }

    /**
     * 构建日志对象
     *
     * @return
     */
    private OperationLogTaxDocument buildOperationLogTaxDocument() {
        OperationLogTaxDocument actionEntity = new OperationLogTaxDocument();
        actionEntity.setOperationContent("主页");
        actionEntity.setModuleName("档案管理");
        actionEntity.setOperationObject("税务档案");
        actionEntity.setOperationUser(authUserHelper.getCurrentAuditor().get());
        actionEntity.setIp(authUserHelper.getClientIp());
        actionEntity.setCreateTime(new Date());
        return actionEntity;
    }

    /**
     * 根据档案属性 档案类型 公司名称 档案名称 查询是否已存在数据
     * 不存在则返回 true
     * 已存在则返回 false
     *
     * @param taxDocument
     * @return
     */
    public boolean queryWhetherData(TaxDocument taxDocument) {
        try {
            //简单参数校验
       /* if (StringUtils.isAnyBlank(taxDocumentDto.getFileAttr(), taxDocumentDto.getFileName(),
                taxDocumentDto.getFileType(), taxDocumentDto.getCompanyName())){
            return false;
        }*/
            TaxDocumentExample taxDocumentExample = new TaxDocumentExample();
            TaxDocumentExample.Criteria criteria = taxDocumentExample.createCriteria();
            criteria.andFileAttrEqualTo(taxDocument.getFileAttr());
            criteria.andFileTypeEqualTo(taxDocument.getFileType());
            criteria.andFileNameEqualTo(taxDocument.getFileName());
            criteria.andCompanyNameEqualTo(taxDocument.getCompanyName());
            criteria.andOwnTimeEqualTo(taxDocument.getOwnTime());
            List<TaxDocument> taxDocuments = taxDocumentMapper.selectByExample(taxDocumentExample);
            if (null != taxDocuments && taxDocuments.size() > 0) {
                return false;
            }
            return true;
        } catch (Exception e) {
//            log.error("TaxDocumentServiceImpl queryWhetherData error :" + e.getMessage());
            return false;
        }
    }
    public void downloadAllFile(HttpServletResponse response, List<Long> ids) {
        //如果只选择了一个附件,则不打包
        if (null != ids && ids.size() == 1) {
            TaxDocumentExample example = new TaxDocumentExample();
            TaxDocumentExample.Criteria criteria = example.createCriteria();
            criteria.andIdIn(ids);
            TaxDocument taxDocument = taxDocumentMapper.selectByExample(example).get(0);
            DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
            fileParam.setUuids(Arrays.asList(taxDocument.getFileUploadId()));
            PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
            String urlPath = uploadDetail.getList().get(0).getViewHttpUrl();
            //如果url为null或空字符串而抛出异常
            if (StringUtils.isBlank(urlPath)) {
                throw new RuntimeException("文件url为空,id为:" + taxDocument.getId());
            }
            //文件名称（带后缀） 文件名前+7位uuid截取码以防文件名相同
            String fileName = StringUtils.isBlank(uploadDetail.getList().get(0).getFileName())
                    ? "未知文件(请修改后缀名).xlsx"
                    : uploadDetail.getList().get(0).getFileName();//设置输出流信息
            try {
                response.setContentType("multipart/form-data");
                response.setHeader("Content-Disposition", "attachment;fileName=" + URLEncoder.encode(fileName, "UTF-8"));
            } catch (UnsupportedEncodingException e) {
                throw new RuntimeException("单个下载文件名编码时出现错误.", e);
            }
            //获取需要下载的文件流
            InputStream is = null;
            BufferedInputStream in = null;
            byte[] buffer = new byte[1024];
            int len;
            OutputStream out = null;
            try {
                URL httpurl = new URL(URLDecoder.decode(urlPath, "UTF-8"));
                HttpURLConnection httpConn = (HttpURLConnection) httpurl.openConnection();
                httpConn.setDoOutput(true);// 使用 URL 连接进行输出
                httpConn.setDoInput(true);// 使用 URL 连接进行输入
                httpConn.setUseCaches(false);// 忽略缓存
                httpConn.setRequestMethod("GET");// 设置URL请求方法
                //可设置请求头
                httpConn.setRequestProperty("Content-Type", "application/octet-stream");
                httpConn.setRequestProperty("Connection", "Keep-Alive");// 维持长连接
                httpConn.setRequestProperty("Charset", "UTF-8");
                httpConn.connect();
                if (httpConn.getResponseCode() >= 400) {
                    is = httpConn.getErrorStream();
                } else {
                    is = httpConn.getInputStream();
                }
                in = new BufferedInputStream(is);
                out = response.getOutputStream();
                //文件流循环写入ZipOutputStream
                while ((len = in.read(buffer)) != -1) {
                    out.write(buffer, 0, len);
                }
            } catch (IOException e) {
//                log.error("单个附件下载异常:" + e.getMessage());
            } finally {
                if (null != out) {
                    try {
                        out.close();
                    } catch (IOException e) {
//                        log.error("关闭输出流错误:" + e.getMessage());
                    }
                }
                if (null != is) {
                    try {
                        is.close();
                    } catch (IOException e) {
//                        log.error("关闭输入流错误:" + e.getMessage());
                    }
                }
                if (null != in) {
                    try {
                        in.close();
                    } catch (IOException e) {
//                        log.error("关闭缓存输入流错误:" + e.getMessage());
                    }
                }
            }
            return;
        }
        //如果选择了多个附件
        String downloadName = "多选附件.zip";
        try {
            response.setContentType("multipart/form-data");
            response.setHeader("Content-Disposition", "attachment;fileName=" + URLEncoder.encode(downloadName, "UTF-8"));
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("下载文件名编码时出现错误.", e);
        }
        OutputStream outputStream = null;
        ZipOutputStream zos = null;
        try {
            outputStream = response.getOutputStream();
            zos = new ZipOutputStream(outputStream);
            // 将文件流写入zip中
            downloadTolocal(zos, ids);

        } catch (IOException e) {
//            log.error("downloadAllFile-xxx下载全部附件失败，ids=[{}],错误信息=[{}]", ids, e);
        } finally {
            if (zos != null) {
                try {
                    zos.close();
                } catch (Exception e2) {
//                    log.info("关闭输入流时出现错误", e2);
                }
            }
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (Exception e2) {
//                    log.info("关闭输入流时出现错误", e2);
                }
            }

        }

    }

    /**
     * // 将文件流写入zip中
     *
     * @param zos
     * @param ids
     */
    private void downloadTolocal(ZipOutputStream zos, List<Long> ids) {
        TaxDocumentExample example = new TaxDocumentExample();
        TaxDocumentExample.Criteria criteria = example.createCriteria();
        if (null == ids || ids.size() < 1) {
            throw new RuntimeException("传入参数错误:空参数数组");
        }
        criteria.andIdIn(ids);
        //查询出根据条件过滤后需要下载的附件数据信息
        List<TaxDocument> taxDocuments = taxDocumentMapper.selectByExample(example);
        DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
        fileParam.setUuids(taxDocuments.stream()
                .map(o -> o.getFileUploadId()).collect(Collectors.toList()));
        PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);

        //新建一个空集合,记录下载的名字,并用于判断是否重名,重名则修改名字为(n)
        List<String> fileNameList = new ArrayList<>();
        for (DidiFileUploadDetailResult item : uploadDetail.getList()) {
            //文件url
            String urlPath = item.getViewHttpUrl();
//            //如果url为null或空字符串而抛出异常
//            if (StringUtils.isBlank(urlPath)) {
//                throw new RuntimeException("文件url为空,id为:" + item.getId());
//            }
            //检查文件名是否有重复
            int fileNameRepeatTimes = 0;
            if (fileNameList.size()>0){
                fileNameRepeatTimes = Collections.frequency(fileNameList, item.getFileName());
            }
            String fileName;
            if (fileNameRepeatTimes > 0) {
                fileName = StringUtils.isBlank(item.getFileName())
                        ? "未知文件(请修改后缀名).xlsx"
                        : "(" + fileNameRepeatTimes + ")" + item.getFileName();
            } else {
                //文件名称（带后缀
                fileName = StringUtils.isBlank(item.getFileName())
                        ? "未知文件(请修改后缀名).xlsx"
                        : item.getFileName();
            }
            fileNameList.add(item.getFileName());
            InputStream is = null;
            BufferedInputStream in = null;
            byte[] buffer = new byte[1024];
            int len;
            //创建zip实体（一个文件对应一个ZipEntry）
            ZipEntry entry = new ZipEntry(fileName);
            try {
                //获取需要下载的文件流
                URL httpurl = new URL(URLDecoder.decode(urlPath, "UTF-8"));
                HttpURLConnection httpConn = (HttpURLConnection) httpurl.openConnection();
                httpConn.setDoOutput(true);// 使用 URL 连接进行输出
                httpConn.setDoInput(true);// 使用 URL 连接进行输入
                httpConn.setUseCaches(false);// 忽略缓存
                httpConn.setRequestMethod("GET");// 设置URL请求方法
                //可设置请求头
                httpConn.setRequestProperty("Content-Type", "application/octet-stream");
                httpConn.setRequestProperty("Connection", "Keep-Alive");// 维持长连接
                httpConn.setRequestProperty("Charset", "UTF-8");
                httpConn.connect();
                if (httpConn.getResponseCode() >= 400) {
                    is = httpConn.getErrorStream();
                } else {
                    is = httpConn.getInputStream();
                }
                in = new BufferedInputStream(is);
                zos.putNextEntry(entry);
                //文件流循环写入ZipOutputStream
                while ((len = in.read(buffer)) != -1) {
                    zos.write(buffer, 0, len);
                }
            } catch (Exception e) {
//                log.info("xxx--下载全部附件--压缩文件出错", e);
            } finally {
                if (entry != null) {
                    try {
                        zos.closeEntry();
                    } catch (Exception e2) {
//                        log.info("xxx下载全部附件--zip实体关闭失败", e2);
                    }
                }
                if (in != null) {
                    try {
                        in.close();
                    } catch (Exception e2) {
//                        log.info("xxx下载全部附件--文件输入流关闭失败", e2);
                    }
                }
                if (is != null) {
                    try {
                        is.close();
                    } catch (Exception e) {
//                        log.info("xxx下载全部附件--输入缓冲流关闭失败", e);
                    }
                }


            }

        }
    }
}
