package pwc.taxtech.atms.vat.service.impl;

import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Attachment;
import org.activiti.engine.task.Task;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.dao.TemplateMapper;
import pwc.taxtech.atms.dto.approval.ApprovalDto;
import pwc.taxtech.atms.entity.Template;
import pwc.taxtech.atms.exception.BadParameterException;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.vat.dao.PeriodApproveMapper;
import pwc.taxtech.atms.vat.dao.PeriodReportMapper;
import pwc.taxtech.atms.vat.dao.PeriodTemplateMapper;
import pwc.taxtech.atms.vat.dpo.ApprovalTaskInfo;
import pwc.taxtech.atms.vat.entity.*;

import java.util.*;

import static pwc.taxtech.atms.common.util.SpringContextUtil.periodTemplateMapper;

@Service
public class ApprovalService {
    private static Logger logger = LoggerFactory.getLogger(ApprovalService.class);
    @Autowired
    RuntimeService runtimeService;
    @Autowired
    TaskService taskService;
    @Autowired
    PeriodApproveMapper periodApproveMapper;
    @Autowired
    PeriodReportMapper reportMapper;
    @Autowired
    AuthUserHelper authUserHelper;
    @Autowired
    TemplateMapper templateMapper;


    @Transactional
    public void startInstanceAndAssignee(ApprovalDto dto) {
        PeriodApprove pa = new PeriodApprove();
        startInstance(dto, pa);
        startAssignee(pa);
        createAttache(pa);
    }


    public List<ApprovalTaskInfo> getTask() {
        return periodApproveMapper.queryApprovalList();
    }

    private void startInstance(ApprovalDto dto, PeriodApprove pa) {
        List<PeriodReport> currentReports = reportMapper.selectOrderReportIds(dto.getProjectId(),dto.getPeriod());
        MyAsserts.assertNotEmpty(currentReports, Exceptions.NOT_FOUND_REPORT_EXCEPTION);

        pa.setId(UUID.randomUUID().toString());
        pa.setPeriod(dto.getPeriod());
        pa.setYear(dto.getYear());

        StringBuilder reportIds = new StringBuilder("");
        StringBuilder reportTemplateIds = new StringBuilder("");
        currentReports.forEach(m -> {
            reportIds.append(m.getId()).append(",");
            reportTemplateIds.append(m.getTemplateId()).append(",");
        });

        PeriodApproveExample pae = new PeriodApproveExample();
        pae.createCriteria().andProjectIdEqualTo(dto.getProjectId()).andPeriodEqualTo(dto.getPeriod())
                .andYearEqualTo(dto.getYear()).andReportIdsEqualTo(reportIds.toString())
                .andTemplateIdsEqualTo(reportTemplateIds.toString()).andStatusEqualTo(Constant.APPROVAL_COMMITTED);

        List<PeriodApprove> pas = periodApproveMapper.selectByExample(pae);
        MyAsserts.assertEmpty(pas, Exceptions.REPORT_HAS_COMMIT_EXCEPTION);

        ProcessInstance pi = runtimeService.startProcessInstanceByKey("approvalProcess");

        dto.setInstanceId(pi.getId());
        pa.setInstanceId(pi.getId());
        pa.setReportIds(reportIds.toString());
        pa.setTemplateIds(reportTemplateIds.toString());
        pa.setStatus(Constant.APPROVAL_COMMITTED);
        pa.setProjectId(dto.getProjectId());
        pa.setCreateBy(authUserHelper.getCurrentUserId() == null ? "Admin" : authUserHelper.getCurrentUserId());
        pa.setCreateTime(new Date());
        periodApproveMapper.insert(pa);
    }

    private void startAssignee(PeriodApprove pa) {
        List<Task> tasks = taskService.createTaskQuery().taskAssignee(Constant.ASSIGNEE_ACCOUNTANT).processInstanceId(
                pa.getInstanceId()).list();
        if (tasks != null && tasks.size() == 1) {
            Task task = tasks.get(0);

            Map<String, Object> map = new HashMap<>();
            map.put("committed", 0);

            taskService.complete(task.getId(), map);
        } else {
            logger.warn("task must not null or size eq 1");
        }
    }

    private void createAttache(PeriodApprove pa) {
        List<Task> tasks = taskService.createTaskQuery().taskAssignee(Constant.ASSIGNEE_MANAGER).processInstanceId(
                pa.getInstanceId()).list();
        if (tasks != null && tasks.size() == 1) {
            Task task = tasks.get(0);
            taskService.createAttachment("java.lang.String", task.getId(), task.getProcessInstanceId(),
                    "period_approval_uuid", pa.getId(), pa.getId());
        } else {
            logger.warn("task must not null or size eq 1");
        }
    }


    @Transactional
    public void checkTask(String projectId,Integer period, String decide,String comment) {
        PeriodApprove approve= periodApproveMapper.getInstanceIdByProjectIdAndPeriod(projectId,period);
        MyAsserts.assertTrue(approve!=null&&approve.getStatus().equals("committed"),Exceptions.NOT_FOUND_INSTANCE_EXCEPTION);
        String instanceId = approve.getInstanceId();
        Map<String, Object> map = new HashMap<>();
        PeriodApprove pa = new PeriodApprove();
        switch (decide) {
            case Constant.APPROVAL_AGREED:
                pa.setStatus(Constant.APPROVAL_AGREED);
                map.put("decide", 1);
                break;
            case Constant.APPROVAL_DISAGREED:
                pa.setStatus(Constant.APPROVAL_DISAGREED);
                map.put("decide", 0);
                break;
            default:
                throw new BadParameterException("not support decide param type");
        }
        List<Task> tasks = taskService.createTaskQuery().taskAssignee(Constant.ASSIGNEE_MANAGER).processInstanceId(
                instanceId).list();

        if (tasks != null && tasks.size() == 1) {
            Task task = tasks.get(0);
            List<Attachment> attachments = taskService.getTaskAttachments(task.getId());

            if (attachments != null && attachments.size() == 1) {
                String uuid = attachments.get(0).getDescription();
                taskService.complete(task.getId(), map);

                pa.setId(uuid);
                pa.setApprovalBy(authUserHelper.getCurrentUserId() == null ? "Admin" : authUserHelper.getCurrentUserId());
                pa.setApprovalTime(new Date());
                pa.setApprovalResualt(comment);
                periodApproveMapper.updateByPrimaryKeySelective(pa);
            } else {
                logger.warn("task must not null or size gt 1");
            }

        } else {
            logger.warn("task must not null or size eq 1");
        }




    }

    public Template getTemplateInfo(Long templateId) {
       return templateMapper.selectByPrimaryKey(templateId);
    }

    public String getApprovalStatus(String projectId, Integer period) {
        return periodApproveMapper.getStatusByProjectIdAndPeriod(projectId,period);
    }
}
