package pwc.taxtech.atms.vat.service.impl.report.functions;

import com.alibaba.fastjson.JSON;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.WorkbookEvaluator;
import org.apache.poi.ss.formula.eval.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceType;
import pwc.taxtech.atms.vat.entity.*;
import pwc.taxtech.atms.vat.service.impl.FormulaAgent;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.*;

public class FunctionBase {
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());
    public static final BigDecimal[] defaultTaxRates = {new BigDecimal("0.17"), new BigDecimal("0.13")
            , new BigDecimal("0.11"), new BigDecimal("0.06"), new BigDecimal("0.05"), new BigDecimal("0.03")};
    protected FormulaContext formulaContext;
    private static Logger LOGGER = LoggerFactory.getLogger(FunctionBase.class);
    final FormulaAgent agent;

    public FunctionBase(FormulaContext formulaContext) {
        this.formulaContext = formulaContext;
        this.agent = formulaContext.getFormulaAgent();
    }

    public String getStringParam(ValueEval valueEval, OperationEvaluationContext ec) {
        try {
            ValueEval v1 = OperandResolver.getSingleValue(valueEval,
                    ec.getRowIndex(),
                    ec.getColumnIndex());
            return OperandResolver.coerceValueToString(v1);
        } catch (EvaluationException e) {
            e.printStackTrace();
            return null;
        }
    }

    public Integer getIntParam(ValueEval valueEval, OperationEvaluationContext ec) {
        try {
            ValueEval v1 = OperandResolver.getSingleValue(valueEval,
                    ec.getRowIndex(),
                    ec.getColumnIndex());
            return OperandResolver.coerceValueToInt(v1);
        } catch (EvaluationException e) {
            e.printStackTrace();
            return null;
        }
    }

    public Long getCellTemplateId(int period, OperationEvaluationContext ec) {
        return FormulaHelper
                .getCellTemplateIdWithTemplateCodeAndColumnIndexAndRowIndex(ec.getWorkbook().getSheetName(ec.getSheetIndex()), ec.getColumnIndex(), ec.getRowIndex(), period, formulaContext);
    }

    public int getYear(int parameterYear) {
        if (parameterYear <= -1) {
            return parameterYear + formulaContext.getYear();
        } else if (parameterYear == 0) {
            return formulaContext.getYear();
        } else {
            return parameterYear;
        }
    }

    public int getPeriod(int parameterPeriod, Integer yearOffset) {

        yearOffset = 0;
        if (parameterPeriod == -99) {
            return parameterPeriod;
        } else if (parameterPeriod <= -1) {
            int period = parameterPeriod + formulaContext.getPeriod();
            while (period <= 0) {
                yearOffset--;
                period += 12;
            }
            return period;
        } else if (parameterPeriod == 0) {
            return formulaContext.getPeriod();
        } else {
            return parameterPeriod;
        }
    }

    public Long saveDataSource(OperationEvaluationContext ec, List<Object> dataSourceList,FormulaDataSourceType sourceType,
                               FormulaDataSourceDetailType formulaDataSourceDetailType, BigDecimal val, int period, Long templateGroupId, String projectId) {
        return saveDataSource(ec, dataSourceList, sourceType,formulaDataSourceDetailType, val, period, templateGroupId, null, null, projectId);
    }
    public Long saveDataSource(OperationEvaluationContext ec, List<Object> dataSourceList,
                               FormulaDataSourceDetailType formulaDataSourceDetailType, BigDecimal val, int period, Long templateGroupId, String projectId) {
        return saveDataSource(ec, dataSourceList, formulaDataSourceDetailType, val, period, templateGroupId, null, null, projectId);
    }

    public Long saveDataSource(OperationEvaluationContext ec, List<Object> dataSourceList,FormulaDataSourceType sourceType,
                               FormulaDataSourceDetailType formulaDataSourceDetailType, BigDecimal val,
                               int period, Long templateGroupId, Integer colNumP, Integer rowNumP, String projectId) {

        return saveDataSource(ec, dataSourceList, sourceType, formulaDataSourceDetailType, val, period,
                templateGroupId, colNumP, rowNumP, projectId, null);
    }

    public Long saveDataSource(OperationEvaluationContext ec, List<Object> dataSourceList,
                               FormulaDataSourceDetailType formulaDataSourceDetailType, BigDecimal val,
                               int period, Long templateGroupId, Integer colNumP, Integer rowNumP, String projectId) {

        return saveDataSource(ec, dataSourceList, FormulaDataSourceType.Report, formulaDataSourceDetailType, val, period,
                templateGroupId, colNumP, rowNumP, projectId, null);
    }

    public Long saveDataRSUMIFSource(OperationEvaluationContext ec, List<Object> dataSourceList,
                               FormulaDataSourceDetailType formulaDataSourceDetailType, BigDecimal val,
                               int period, Long templateGroupId, Integer colNumP, Integer rowNumP, String projectId, FormulaDataSourceType type, String sql) {

        return saveDataSource(ec, dataSourceList, type, formulaDataSourceDetailType, val, period,
                templateGroupId, colNumP, rowNumP, projectId, sql);
    }

    public Long saveDataSource(OperationEvaluationContext ec, List<Object> dataSourceList, FormulaDataSourceType dataSourceType,
                               FormulaDataSourceDetailType formulaDataSourceDetailType, BigDecimal val,
                               int period, Long templateGroupId, Integer colNumP, Integer rowNumP, String projectId, String sql ) {
        String reportCode = ec.getWorkbook().getSheetName(ec.getSheetIndex());
        int colNum = ec.getColumnIndex();
        int rowNum = ec.getRowIndex();
        //根据这三个值还有period获取 cell_template_Id的值
        Map<String, Object> map = new HashMap<>();
        map.put("code", reportCode);
        map.put("templateGroupId", templateGroupId);
        map.put("period", period);
        map.put("columnIndex", colNum);
        map.put("rowIndex", rowNum);
        map.put("projectId", projectId);
        Long periodCellTemplateId = SpringContextUtil.periodCellTemplateMapper.getCellTemplateIdWithMap(map);

        Date creatime = new Date();
        PeriodDataSource dataSource = new PeriodDataSource();
        Long dataSourceId = SpringContextUtil.distributedIdService.nextId();
        dataSource.setId(dataSourceId);
        dataSource.setType(dataSourceType.getCode());
        dataSource.setName("ReportDataSource");
        dataSource.setAmount(val.setScale(0, BigDecimal.ROUND_HALF_DOWN));
        dataSource.setDescription("描述");
        dataSource.setCreateBy("Admin");
        dataSource.setUpdateBy("Admin");
        dataSource.setCreateTime(creatime);
        dataSource.setUpdateTime(creatime);
        dataSource.setRowIndex(rowNumP == null ? ec.getRowIndex() : rowNumP);
        dataSource.setColumnIndex(colNumP == null ? ec.getColumnIndex() : colNumP);
        dataSource.setRowName("行名");
        dataSource.setColumnName("列名");
        dataSource.setCellTemplateId(periodCellTemplateId);
        dataSource.setPeriod(period);
        dataSource.setProjectId(projectId);
        if(sql != null){
            dataSource.setRelSql(sql);
        }
        SpringContextUtil.periodDataSourceMapper.insertSelective(dataSource);
        if (CollectionUtils.isNotEmpty(dataSourceList)) {
            for (Object obj : dataSourceList) {
                if (obj.getClass() == java.util.ArrayList.class) {
                    for (Object obj2 : (ArrayList<Object>) obj) {
                        PeriodDataSourceDetail dataSourceDetail = new PeriodDataSourceDetail();
                        dataSourceDetail.setId(SpringContextUtil.distributedIdService.nextId());
                        dataSourceDetail.setDataSourceId(dataSourceId);
                        dataSourceDetail.setDataSourceType(formulaDataSourceDetailType.getCode());
                        dataSourceDetail.setItemValue(JSON.toJSONString(obj2));
                        dataSourceDetail.setPeriod(period);
                        dataSourceDetail.setProjectId(projectId);
                        SpringContextUtil.periodDataSourceDetailMapper.insertSelective(dataSourceDetail);
                    }
                } else {
                    PeriodDataSourceDetail dataSourceDetail = new PeriodDataSourceDetail();
                    dataSourceDetail.setId(SpringContextUtil.distributedIdService.nextId());
                    dataSourceDetail.setDataSourceId(dataSourceId);
                    dataSourceDetail.setDataSourceType(formulaDataSourceDetailType.getCode());
                    dataSourceDetail.setItemValue(JSON.toJSONString(obj));
                    dataSourceDetail.setPeriod(period);
                    dataSourceDetail.setProjectId(projectId);
                    SpringContextUtil.periodDataSourceDetailMapper.insertSelective(dataSourceDetail);
                }
            }
        } else {
            PeriodDataSourceDetail dataSourceDetail = new PeriodDataSourceDetail();
            dataSourceDetail.setId(SpringContextUtil.distributedIdService.nextId());
            dataSourceDetail.setDataSourceId(dataSourceId);
            dataSourceDetail.setDataSourceType(formulaDataSourceDetailType.getCode());
            dataSourceDetail.setItemValue("{}");
            dataSourceDetail.setPeriod(period);
            dataSourceDetail.setProjectId(projectId);
            SpringContextUtil.periodDataSourceDetailMapper.insertSelective(dataSourceDetail);
        }

        return dataSourceId;
    }

    public void saveFormulaBlock(int period, OperationEvaluationContext ec,
                                 String formulaExpression, BigDecimal val, Long dataSourceId, String projectId) {
        Long cellTemplateId = getCellTemplateId(period, ec);
        Date creatime = new Date();
        PeriodFormulaBlock periodFormulaBlock = new PeriodFormulaBlock();
        periodFormulaBlock.setId(SpringContextUtil.distributedIdService.nextId());
        periodFormulaBlock.setProjectId(projectId);
        periodFormulaBlock.setPeriod(period);
        periodFormulaBlock.setReportId(0L);
        periodFormulaBlock.setCellTemplateId(cellTemplateId);
        periodFormulaBlock.setFormulaExpression(formulaExpression);
        periodFormulaBlock.setData(val.setScale(2, BigDecimal.ROUND_HALF_UP).toString());
        periodFormulaBlock.setDataSourceId(dataSourceId);
        periodFormulaBlock.setCreateBy("Admin");
        periodFormulaBlock.setCreateTime(creatime);
        periodFormulaBlock.setUpdateBy("Admin");
        periodFormulaBlock.setUpdateTime(creatime);
        SpringContextUtil.periodFormulaBlockMapper.insertSelective(periodFormulaBlock);
    }

    public static BigDecimal getCellValue(int index, OperationEvaluationContext ec, FormulaContext formulaContext, FormulaAgent agent,
                                          int rowIndex, int columnIndex, Long cellTemplateId) throws NoSuchFieldException, IllegalAccessException {
        Field evaluatorField = OperationEvaluationContext.class.getDeclaredField("_bookEvaluator");
        evaluatorField.setAccessible(true);
        WorkbookEvaluator evaluator = (WorkbookEvaluator) evaluatorField.get(ec);
        ValueEval eval = evaluator.evaluate(ec.getWorkbook().getSheet(index).getCell(rowIndex, columnIndex));

        List<PeriodDataSource> dss = agent.queryManualDataSource(cellTemplateId, formulaContext.getProjectId(), formulaContext.getPeriod());

        if (eval instanceof ErrorEval || eval == null || eval instanceof BlankEval) {
            if(eval !=null && eval instanceof ErrorEval){
                LOGGER.warn("[Formula_Exception] error eval for cell {} and error code {} and error String {}", cellTemplateId,
                        ((ErrorEval) eval).getErrorCode(), ((ErrorEval) eval).getErrorString());
            }
            if (!dss.isEmpty()) return dss.get(0).getAmount();
        } else {
            String evalStr = OperandResolver.coerceValueToString(eval);
            if(StringUtils.isBlank(evalStr)){
                evalStr = "0";
            }
            LOGGER.debug("[Formula_debug] eval other cell value {}", evalStr);
            if(evalStr.endsWith("%")){
                evalStr = evalStr.replace("%","");
                try {
                    BigDecimal bigDecimal = new BigDecimal(evalStr);
                    bigDecimal = bigDecimal.divide(new BigDecimal(100));
                    if (!dss.isEmpty()) {
                        return bigDecimal.add(dss.get(0).getAmount());
                    } else {
                        return bigDecimal;
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    if (!dss.isEmpty()){
                        return dss.get(0).getAmount();}
                    else {return new BigDecimal(0);}

                }
            }else{
                try {
                    BigDecimal bigDecimal = new BigDecimal(evalStr);
                    if (!dss.isEmpty()) {
                        return bigDecimal.add(dss.get(0).getAmount());
                    } else {
                        return bigDecimal;
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    if (!dss.isEmpty()){
                        return dss.get(0).getAmount();}
                    else {return new BigDecimal(0);}
                }
            }



        }

        return new BigDecimal(0);
    }

    public String StringBuddler(String target, String design) {
        if (target.equals("")) {
            target = design;
        } else {
            target = target + ", " + design;
        }
        return target;
    }

    public Long getSUM2Data(Long reportTemplateId, Integer period, Map map) {
        String bufferString = "";
        Long refValue = null;
        PeriodCellTemplateExample example = new PeriodCellTemplateExample();
        PeriodCellTemplateExample.Criteria criteria = example.createCriteria();
        criteria.andReportTemplateIdEqualTo(reportTemplateId);
        criteria.andPeriodEqualTo(period);
        criteria.andColumnIndexEqualTo(Integer.parseInt(map.get("colIndex").toString()));
        criteria.andRowIndexBetween(Integer.parseInt(map.get("rowIndexBegin").toString()), Integer.parseInt(map.get("rowIndexEnd").toString()));
        List<PeriodCellTemplate> periodCellTemplates = SpringContextUtil.periodCellTemplateMapper.selectByExample(example);
        for (PeriodCellTemplate p : periodCellTemplates) {
            StringBuddler(bufferString, p.getCellTemplateId().toString());
        }
        List<PeriodCellData> list = SpringContextUtil.periodCellDataMapper.selectDataByCellTemplateIdAround(bufferString);
        for (PeriodCellData periodCellData : list) {
            if (refValue == null) refValue = Long.parseLong(periodCellData.getData());
            refValue += Long.parseLong(periodCellData.getData());
        }
        return refValue;
    }

}
