package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.EvaluationException;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceType;
import pwc.taxtech.atms.dto.TableRule;
import pwc.taxtech.atms.dto.vatdto.RSUMIFParasBo;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.dto.vatdto.ReportCellTableSUMIFDataSourceDto;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.vat.service.impl.FormulaAgent;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static pwc.taxtech.atms.common.util.FormulaUtil.resolverInteger;
import static pwc.taxtech.atms.common.util.FormulaUtil.resolverString;

/**
 * @description 计算指定科目期末余额，目前不考虑科目类型，统一为企业科目
 * @author ZhiKai Z Wei
 */
public class QMYE extends FunctionBase implements FreeRefFunction {
    private String tableName = "CIT_TBAM";

    public QMYE(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        //科目类型
        Integer subjectType = 0;
        //科目代码
        String subjectCode = "";
        Integer period = 0;
        Integer year = 0;
        //参数验证
        parameterCheck(args, ec);
        BigDecimal cellValue = BigDecimal.ZERO;
        List<Object> dataSource = new ArrayList<>();
        Map map = new HashMap<String, String>();
        ReportCellTableSUMIFDataSourceDto reportCellTableSUMIFDataSourceDto = new ReportCellTableSUMIFDataSourceDto();
        reportCellTableSUMIFDataSourceDto.insertQMYEData(ec, formulaContext);
        dataSource.add(reportCellTableSUMIFDataSourceDto);
        try {
            //科目类型，固定写为企业科目
            subjectType = resolverInteger(args, ec, 0);
            //科目代码
            subjectCode = resolverString(args, ec, 1);
            year = resolverInteger(args, ec, 2);//年份
            period = resolverInteger(args, ec, 3);//期间
            cellValue = agent.getQMYE(subjectType, subjectCode, year, period, formulaContext);
            return new NumberEval(cellValue.doubleValue());
        } catch (EvaluationException e) {
            e.printStackTrace();
            return new NumberEval(0.00);
        } finally {
            Long dataSourceId = null;
            try {
                dataSourceId = saveDataRSUMIFSource(ec, dataSource, FormulaDataSourceDetailType.ReportCellTableSUMIFDataSourceDto,
                        cellValue, formulaContext.getPeriod(),
                        formulaContext.getReportTemplateGroupId(), ec.getColumnIndex(), ec.getRowIndex(),
                        formulaContext.getProjectId(), selectShow(tableName), null);
            } catch (Exception e) {
                e.printStackTrace();
            }
            try {
                saveFormulaBlock(formulaContext.getPeriod(), ec,
                        new RSUMIFParasBo().getQMYEExpression(subjectType, subjectCode, year, period, ec), cellValue, dataSourceId, formulaContext.getProjectId());
            } catch (EvaluationException e) {
                e.printStackTrace();
            }
        }
    }

    //进行参数验证
    public void parameterCheck(ValueEval[] args, OperationEvaluationContext ec) {
        Integer argsLength = args.length;
        if (argsLength != 4) {
            //参数小于4当做异常处理
            throw Exceptions.parameterError;
        }
    }

    //解决不同表结构关联不同的展示方式
    public FormulaDataSourceType selectShow(String tableName) {
        if ("CIT_TBAM".equals(tableName)) {
            return FormulaDataSourceType.CIT_TBAM;
        }
        return null;
    }

    public static String getSql (){
        return "";
    }

}
