package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.enums.EnumOperationType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.KeyValueConfigResultType;
import pwc.taxtech.atms.dpo.CitAssetDetailResultDto;
import pwc.taxtech.atms.dto.vatdto.AssetDetailDataSourceDto;
import pwc.taxtech.atms.vat.dpo.AssetDetailResultDto;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/// <summary>
/// 名称：资产
/// 功能：从已经分类的资产数据中获取对应的数据
/// </summary>
/// <param name="assetType">资产类别：1，固定资产；2，长期待摊；3，无形资产</param>
/// <param name="assetDetailType">资产二级分类：见数据库AssetDetailGroup表DetailGroupType字段</param>
/// <param name="dataType">
/// 取值列：1，原值（财务）；2，本年折旧额（财务）；3，累计折旧额（财务）；4，每月折旧额（财务）；5，本年年终剩余价值（财务）
/// 6，每月折旧额（税务）；7，截止本年累计折旧额（税务）；8，本年折旧额（税务）；9，累计差异（税务），10，本年差异（税务）
/// </param>
/// <param name="year">年份：0，当年；-1，上一年；1，下一年（以此类推到10）</param>
/// <returns></returns>
public class ZC extends FunctionBase implements FreeRefFunction {

    public ZC(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length < 4) {
            return NumberEval.ZERO;
        }

        int assetType = getIntParam(args[0], ec);
        int assetDetailType = getIntParam(args[1], ec);
        int dataType = getIntParam(args[2], ec);
        int year = getIntParam(args[3], ec);

        String formulaExpression = "ZC(" + assetType + "," + assetDetailType + ","
                + dataType + "," + year + ")";
        logger.debug(formulaExpression);

        int curYear = formulaContext.getYear() + year;
//        String dbName =
//                SpringContextUtil.projectDao.getDbNameWithYearAndOrgId(formulaContext.getOrganizationId(), curYear);
        List<AssetDetailDataSourceDto> dataSource = new ArrayList<>();
        BigDecimal totalData = new BigDecimal("0");
        try {
            List<CitAssetDetailResultDto> assetDetailResultDtos =
                    SpringContextUtil.assetsListMapper.getCitAssetDetialResult(assetType, assetDetailType);
if(assetDetailResultDtos != null&&assetDetailResultDtos.size()>0){
    System.out.println(assetDetailResultDtos.size());
}
            for (CitAssetDetailResultDto x : assetDetailResultDtos) {
                AssetDetailDataSourceDto assetDetailDataSourceDto = new AssetDetailDataSourceDto();
                assetDetailDataSourceDto.setResultType(KeyValueConfigResultType.Accounting.getCode());
                assetDetailDataSourceDto.setAssetNumber(x.getAssetsList().getAssetGroupName());
                assetDetailDataSourceDto.setAssetGroupName(x.getAssetDetailGroup().getDetailGroupName());
                assetDetailDataSourceDto.setAssetDetailGroupName(x.getAssetDetailGroup().getDetailGroupName());
                assetDetailDataSourceDto.setAccountAcquisitionValue(FormulaHelper.roundValue(x.getAssetsList().getAccountAcquisitionValue(), KeyValueConfigResultType.Accounting, null, formulaContext));
                assetDetailDataSourceDto.setAccountMonthDepreciationAmount(FormulaHelper.roundValue(x.getAssetsList().getAccountMonthDepreciationAmount(), KeyValueConfigResultType.Accounting, null, formulaContext));
                assetDetailDataSourceDto.setAccountYearDepreciationAmount(FormulaHelper.roundValue(x.getAssetsList().getYearDepreciationAmount(), KeyValueConfigResultType.Accounting, null, formulaContext));
                assetDetailDataSourceDto.setAccountTotalepreciationAmount(FormulaHelper.roundValue(x.getAssetsList().getAccountTotalDepreciationAmount(), KeyValueConfigResultType.Accounting, null, formulaContext));
                assetDetailDataSourceDto.setYearEndValue(FormulaHelper.roundValue(x.getAssetsList().getYearEndValue(), KeyValueConfigResultType.Accounting, null, formulaContext));
                assetDetailDataSourceDto.setTaxMonthDepreciationAmount(FormulaHelper.roundValue(x.getAssetsList().getTaxMonthDepreciationAmount(), KeyValueConfigResultType.Accounting, null, formulaContext));
                assetDetailDataSourceDto.setTaxToCurrentYearDepreciationAmount(FormulaHelper.roundValue(x.getAssetsList().getTaxToCurrentYearDepreciationAmount(), KeyValueConfigResultType.Accounting, null, formulaContext));
                assetDetailDataSourceDto.setTaxCurrentYearDepreciationAmount(FormulaHelper.roundValue(x.getAssetsList().getTaxCurrentYearDepreciationAmount(), KeyValueConfigResultType.Accounting, null, formulaContext));
                assetDetailDataSourceDto.setTotalDifferenceAmount(FormulaHelper.roundValue(x.getAssetsList().getTotalDifferenceAmount(), KeyValueConfigResultType.Accounting, null, formulaContext));
                assetDetailDataSourceDto.setYearDifferenceAmount(FormulaHelper.roundValue(x.getAssetsList().getYearDifferenceAmount(), KeyValueConfigResultType.Accounting, null, formulaContext));
                assetDetailDataSourceDto.setDepreciationPeriod(x.getAssetsList().getDepreciationPeriod());
                assetDetailDataSourceDto.setTaxDepreciationPeriod(x.getAssetsList().getTaxDepreciationPeriod());
                assetDetailDataSourceDto.setIsRetain(x.getAssetsList().getIsRetain()==0?false:true);
                assetDetailDataSourceDto.setDataType(dataType);
                assetDetailDataSourceDto.setName(Constant.DataSourceName.AssetListDataSource);
                assetDetailDataSourceDto.setOperationType(EnumOperationType.Single.getCode());
                dataSource.add(assetDetailDataSourceDto);
            }

            dataSource = dataSource.stream()
                    .sorted(Comparator.comparing(AssetDetailDataSourceDto::getAssetGroupName)
                            .thenComparing(AssetDetailDataSourceDto::getAssetDetailGroupName))
                    .collect(Collectors.toList());

            switch (dataType) {
                case 1:
                    totalData = new BigDecimal(dataSource.stream().mapToDouble(a -> a.getAccountAcquisitionValue().doubleValue()).sum());
                    break;
                case 2:
                    totalData = new BigDecimal(dataSource.stream().mapToDouble(a -> a.getAccountYearDepreciationAmount().doubleValue()).sum());
                    break;
                case 3:
                    totalData = new BigDecimal(dataSource.stream().mapToDouble(a -> a.getAccountTotalepreciationAmount().doubleValue()).sum());
                    break;
                case 4:
                    totalData = new BigDecimal(dataSource.stream().mapToDouble(a -> a.getAccountMonthDepreciationAmount().doubleValue()).sum());
                    break;
                case 5:
                    totalData = new BigDecimal(dataSource.stream().mapToDouble(a -> a.getYearEndValue().doubleValue()).sum());
                    break;
                case 6:
                    totalData = new BigDecimal(dataSource.stream().mapToDouble(a -> a.getTaxMonthDepreciationAmount().doubleValue()).sum());
                    break;
                case 7:
                    totalData = new BigDecimal(dataSource.stream().mapToDouble(a -> a.getTaxToCurrentYearDepreciationAmount().doubleValue()).sum());
                    break;
                case 8:
                    totalData = new BigDecimal(dataSource.stream().mapToDouble(a -> a.getTaxCurrentYearDepreciationAmount().doubleValue()).sum());
                    break;
                case 9:
                    dataSource = dataSource.stream().filter(a -> a.getIsRetain() != null && a.getIsRetain()).collect(Collectors.toList());
                    totalData = new BigDecimal(dataSource.stream().mapToDouble(a -> a.getTotalDifferenceAmount().doubleValue()).sum());
                    break;
                case 10:
                    dataSource = dataSource.stream().filter(a -> a.getIsRetain() != null && a.getIsRetain()).collect(Collectors.toList());
                    totalData = new BigDecimal(dataSource.stream().mapToDouble(a -> a.getYearDifferenceAmount().doubleValue()).sum());
                    break;
                default:
                    break;
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("计算ZC公式报错");
            return new NumberEval(0);
        }finally {
            Long dataSourceId = saveDataSource(ec, Collections.singletonList(dataSource),
                    FormulaDataSourceDetailType.AssetDetailDataSourceDto,
                    totalData, formulaContext.getPeriod(), formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
            saveFormulaBlock(0, ec, formulaExpression, totalData, dataSourceId, formulaContext.getProjectId());
        }

        return new NumberEval(totalData.doubleValue());
    }
}
