package pwc.taxtech.atms.dao;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import pwc.taxtech.atms.MyMapper;
import pwc.taxtech.atms.dpo.CellTemplatePerGroupDto;
import pwc.taxtech.atms.dpo.GroupId;
import pwc.taxtech.atms.entity.Project;

import java.math.BigDecimal;
import java.util.List;

@Mapper
public interface FormulaAdminMapper extends MyMapper {

    @Select("" +
            "SELECT  " +
            "  cell_template.cell_template_id AS cellTemplateID,  " +
            "  template.template_id AS reportTemplateID,  " +
            "  template.template_group_id AS reportTemplateGroupID,  " +
            "  cell_template.row_index AS rowIndex,  " +
            "  cell_template.column_index AS columnIndex,  " +
            "  template.code AS reportCode,  " +
            "  cell_template.column_name AS columnName,  " +
            "  cell_template.row_name AS rowName,  " +
            "  cell_template.data_type AS resultType   " +
            "FROM  " +
            "  period_template template  " +
            "  JOIN period_cell_template cell_template ON template.template_id = cell_template.report_template_id   " +
            "WHERE  " +
            "  template.template_id IN ( SELECT template_id FROM period_template WHERE template_group_id = #{templateGroupId} " +
            "  AND is_active_association = 1 AND code = #{code} )   " +
            "  AND row_index = #{rowIndex}   " +
            "  AND column_index = #{columnIndex}   " +
            "  AND template.project_id = #{projectId} " +
            "  AND template.period = #{period}   " +
            "  AND cell_template.project_id = #{projectId}   " +
            "  AND cell_template.period = #{period}" +
            "")
    List<CellTemplatePerGroupDto> getCellTemplatePerGroupDto(@Param("templateGroupId") Long templateGroupId,
                                                             @Param("code") String code,
                                                             @Param("rowIndex") Integer rowIndex,
                                                             @Param("columnIndex") Integer columnIndex,
                                                             @Param("projectId") String projectId,
                                                             @Param("period") Integer period);


    @Select("SELECT  " +
            "    rule.template_group_id as groupId, rule.is_default as isDefault " +
            "FROM " +
            "    project p, " +
            "    tax_payer_report_rule rule " +
            "WHERE " +
            "    (p.organization_id = rule.organization_id " +
            "        OR rule.is_default = 1) " +
            "        AND p.id = #{projectId}")
    List<GroupId> getTemplateGroupId(@Param("projectId") String projectId);


    @Select("SELECT  " +
            "    p.id " +
            "FROM " +
            "    project p " +
            "        JOIN " +
            "    project_service_type ps ON p.id = ps.project_id " +
            "WHERE " +
            "    ps.service_type_id IN (\"2\" , \"3\") " +
            "        AND p.year = #{year} " +
            "        AND p.organization_id = #{orgId} " +
            "        AND p.is_active = 1 " +
            // todo 19/12/2018 gary 需修改rownum
//            " AND ROWNUM=1"+
            "ORDER BY ps.service_type_id ")
    String getPastProjectId(@Param("year") int year, @Param("orgId") String organizationId);

    @Select("" +
            "SELECT  " +
            "  p.id AS id,  " +
            "  p.code AS code,  " +
            "  p.name AS name,  " +
            "  p.organization_id AS organizationId   " +
            "FROM  " +
            "  project p  " +
            "  JOIN ( SELECT code, organization_id FROM project WHERE id = #{projectId}  ) t ON p.code = t.code   " +
            "  AND p.organization_id = t.organization_id AND p.year = #{year}" +
            "")
    Project getFixedProject(@Param("projectId") String projectId, @Param("year") Integer year);


    @Select("select " +
            " detail.*, " +
            " source_cell.cell_template_id AS cellTemplateID," +
            " source_cell.column_index     AS columnIndex," +
            " source_cell.column_name      AS columnName," +
            " source_cell.row_name         AS rowName," +
            " source_cell.data_type        AS resultType " +
            "from (" +
            " select " +
            " template.template_id           AS reportTemplateID," +
            " template.template_group_id     AS reportTemplateGroupID," +
            " template.code                  AS reportCode," +
            " cell_template.row_index        AS rowIndex " +
            " from period_template as template " +
            " inner join period_cell_template as cell_template" +
            " on template.template_id = cell_template.report_template_id" +
            " where template.template_group_id = #{templateGroupId}" +
            " and template.is_active_association = 1" +
            " and template.code = #{code}" +
            " and template.project_id = #{projectId}" +
            " and template.period = #{period}" +
            " and cell_template.project_id = #{projectId}" +
            " and cell_template.period = #{period}" +
            " and cell_template.column_index = #{rowColumnIndex}" +
            " and cell_template.row_name = #{rowName}) as detail," +
            " period_cell_template as source_cell" +
            " where source_cell.project_id = #{projectId}" +
            " and source_cell.period = #{period}" +
            " and detail.reportTemplateID = source_cell.report_template_id" +
            " and detail.rowIndex = source_cell.row_index" +
            " and source_cell.column_index = #{columnIndex}"
    )
    List<CellTemplatePerGroupDto> getCellTemplateByNameAndIndex(
            @Param("templateGroupId") Long templateGroupId,
            @Param("code") String code,
            @Param("rowColumnIndex") Integer rowColumnIndex,
            @Param("rowName") String rowName,
            @Param("columnIndex") Integer columnIndex,
            @Param("projectId") String projectId,
            @Param("period") Integer period);


    List<CellTemplatePerGroupDto> getCellTemplateByTypeAndIndex(
            @Param("templateGroupId") Long templateGroupId,
            @Param("projectId") String projectId,
            @Param("code") String code,
            @Param("rowColumnIndex") Integer rowColumnIndex,
            @Param("taxRate") BigDecimal taxRate,
            @Param("revenueType") Integer revenueType,
            @Param("taxType") Integer taxType,
            @Param("columnIndex") Integer columnIndex,
            @Param("period") Integer period,
            @Param("orgId") String orgId,
            @Param("startDate") String startDate,
            @Param("endDate") String endDate);


    @Select("" +
            "SELECT  " +
            "  cell_template.cell_template_id AS cellTemplateID,  " +
            "  template.template_id AS reportTemplateID,  " +
            "  template.template_group_id AS reportTemplateGroupID,  " +
            "  cell_template.row_index AS rowIndex,  " +
            "  cell_template.column_index AS columnIndex,  " +
            "  template.code AS reportCode,  " +
            "  cell_template.column_name AS columnName,  " +
            "  cell_template.row_name AS rowName,  " +
            "  cell_template.data_type AS resultType   " +
            "d.data AS content" +
            "FROM  " +
            "  period_template template  " +
            "  LEFT JOIN period_cell_template cell_template ON template.template_id = cell_template.report_template_id   " +
            "  LEFT JOIN period_cell_data  d on d.cell_template_id = cell_template.cell_template_id" +
            "WHERE  " +
            "  template.template_id IN ( SELECT template_id FROM period_template WHERE template_group_id = #{templateGroupId} " +
            "  AND is_active_association = 1 AND code = #{code} )   " +
            "  AND row_index = #{cellRow}   " +
            "  AND column_index = #{cellCol}   " +
            "  AND template.project_id = #{projectId} " +
            "  AND template.period = #{period}   " +
            "  AND cell_template.project_id = #{projectId}   " +
            "  AND cell_template.period = #{period}" +
            "")
    List<CellTemplatePerGroupDto> getCellDataByPos(@Param("templateId") Long templateId, @Param("period") Integer period, @Param("cellRow") String cellRow, @Param("cellCol") String cellCol, @Param("projectId") String projectId);


    @Select("" +
            "SELECT  " +
            "${getField} as data" +
            "FROM ${tableName}  " +
            "inner join project pro on pro.organization_id = t.organization_id   " +
            " and ${selectFilter}" +
            " and pro.year = #{year}" +
            "")
    List<CellTemplatePerGroupDto> getTableData(@Param("tableName") String tableName,
                                               @Param("getField") String getField,
                                               @Param("selectFilter") String selectFilter,
                                               @Param("year") String year,
                                               @Param("selectPeriod") String selectPeriod);
}
