﻿basicDataModule
    .controller('orangizationStructureManageController', ['$scope', '$log', 'SweetAlert', 'organizationStructureService', '$translate', 'uiGridConstants', 'Upload', 'apiInterceptor', '$interval',
        function($scope, $log, SweetAlert, organizationStructureService, $translate, uiGridConstants, Upload, apiInterceptor, $interval) {
            'use strict';

            // 模态框的编辑状态 true 编辑状态,false 创建状态
            $scope.isEdit = false;

            var formValidator = null;

            //当前编辑对象
            $scope.editingObject = { ID: "", name: "", IsActive: true };

            var editModalSelector = ".orangization-structure #edit-modal";

            //显示指定消息编码对应的翻译文本信息
            var showError = function(messageCode) {
                var errMsg = $translate.instant(messageCode);
                SweetAlert.warning(errMsg);
            };

            //清空当前编辑字段值
            $scope.resetEditingObject = function() {
                $scope.editingObject = { ID: "", name: "", IsActive: true };

                var elms = $(editModalSelector).find("form:first div[data-for]");
                elms.html("");
                elms.attr("class", "validate-success");
            }

            //创建(添加)新事业部,使用模态框实现编辑入口
            $scope.createNewData = function() {
                $scope.isEdit = false;

                $scope.resetEditingObject();

                $(editModalSelector).modal('show');
            };

            //编辑焦点行数据
            $scope.editFocusedRow = function(data) {
                $scope.isEdit = true;

                $scope.editingObject = { ID: data.id, name: data.name, IsActive: data.isActive };

                $(editModalSelector).modal('show');
            };

            //保存当前正在编辑数据行,经过界面初步校验后才会进入本功能
            $scope.saveEditRow = function() {

                if ($scope.editingObject == null) {
                    showError("OrganizationStructureNoSelected");
                    return;
                }
                if ($scope.editingObject && $scope.editingObject.name.trim().length == 0) {
                    showError("OrganizationStructureEmptyNode");
                    return;
                }

                if (!($(editModalSelector).find("#edit-form").valid())) {
                    return;
                }

                var osArray = [];
                osArray.push($scope.editingObject);

                var successedFun = function() {
                    $scope.refreshData();

                    $(editModalSelector).modal('hide');

                    if ($scope.isEdit) {
                        SweetAlert.success($translate.instant("OrganizationStructureEditSuccess"));
                    } else {
                        SweetAlert.success($translate.instant("OrganizationStructureAddSuccess"));
                    }
                }

                if ($scope.isEdit) {
                    organizationStructureService.updateOrganizationStructure(osArray).success(function (data) {
                        if (data.code===0) {
                            successedFun();
                        }else {
                            SweetAlert.error(data.message);
                        }
                    });
                } else {
                    organizationStructureService.addOrganizationStructure(osArray).success(function (data) {
                        if (data.code===0) {
                            successedFun();
                        }else {
                            SweetAlert.error(data.message);
                        }
                    });
                }
            };

            //取消保存当前编辑行
            $scope.cancelEditOS = function() {
                $(editModalSelector).modal('hide');
            };

            //停用或启用当前事业部
            $scope.activeOS = function(data) {

                data.isActive = !data.isActive;
                var osArray = [];
                osArray.push(data);

                organizationStructureService.updateOrganizationStructure(osArray).success(function(rst) {
                    $scope.refreshData();
                }).error(function(){
                    $scope.refreshData();
                    SweetAlert.warning("修改失败！");
                });
            };

            //获取事业部数据
            $scope.refreshData = function() {
                organizationStructureService.getOrganizationStructureList().success(function(data) {
                    $scope.gridOptions.data = data;
                });
            };

            $scope.getGridHeight = function() {
                return { height: $(".orangization-structure-containers .list-container").height() + "px" };
            }

            function intiUiGrid() {

                var operation = "";
                if ($scope.hasEditPermission){
                    operation = '<div class="orangization-structure-gridcell-operation">' +
                            '<a ng-click="grid.appScope.editFocusedRow(row.entity)">' +
                            '    <i class="material-icons" style="margin-right: 10px;">&#xE254;</i>' +
                            '    <span>{{"OrganizationStructureEdit"|translate}}</span>' +
                            '</a>' +
                            '<a ng-click="grid.appScope.activeOS(row.entity)" ng-if="row.entity.isActive">' +
                            '    <i class="material-icons" style="color:#3f3f40">&#xE15C;</i>' +
                            '    <span>{{"OrganizationStructureDisabledNode"|translate}}</span>' +
                            '</a>' +
                            '<a ng-click="grid.appScope.activeOS(row.entity)" ng-if="!row.entity.isActive">' +
                            '    <i class="material-icons" style="color:#e64400">&#xE038;</i>' +
                            '    <span>{{"OrganizationStructureEnableNode"|translate}}</span>' +
                            '</a>' +
                            '</div>'
                } else{
                    operation = '<div class="orangization-structure-gridcell-operation ' + constant.noPermissionClass + '">' +
                            '<a enabled="false" >' +
                            '    <i class="material-icons" style="margin-right: 10px;">&#xE254;</i>' +
                            '    <span>{{"OrganizationStructureEdit"|translate}}</span>' +
                            '</a>' +
                            '<a enabled="false" ng-if="row.entity.isActive">' +
                            '    <i class="material-icons" style="color:#3f3f40">&#xE15C;</i>' +
                            '    <span>{{"OrganizationStructureDisabledNode"|translate}}</span>' +
                            '</a>' +
                            '<a enabled="false" ng-if="!row.entity.isActive">' +
                            '    <i class="material-icons" style="color:#e64400">&#xE038;</i>' +
                            '    <span>{{"OrganizationStructureEnableNode"|translate}}</span>' +
                            '</a>' +
                            '</div>'
                }

                $scope.gridOptions = {
                    rowHeight: constant.UIGrid.rowHeight,
                    selectionRowHeaderWidth: constant.UIGrid.selectionRowHeaderWidth,
                    enableSorting: true,
                    enableColumnMenus: false,
                    enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
                    enableRowSelection: true,
                    enableFullRowSelection: true,
                    enableRowHeaderSelection: false,
                    multiSelect: false,
                    columnDefs: [{
                        field: 'name',
                        name: $translate.instant('OrganizationStructureName'),
                        width: '50%',
                        cellTemplate: '<div class="orangization-structure-gridcell-name" style="text-align: center"><span>{{row.entity.name}}</span><span ng-hide="true">{{row.entity.id}}</span></div>'
                    }, {
                        field: 'isActive',
                        name: $translate.instant('OrganizationStructureStatus'),
                        width: '20%',
                        cellTemplate: '<div class="orangization-structure-gridcell-isactive">' +
                            '<span ng-if="row.entity.isActive">{{"OrganizationStructureActive"|translate}}</span>' +
                            '<span ng-if="!row.entity.isActive">{{"OrganizationStructureDisabled"|translate}}</span>' +
                            '</div>'
                    }, {
                        field: 'Operation',
                        name: $translate.instant('Operation'),
                        width: '30%',
                        cellTemplate: operation
                    }],

                    onRegisterApi: function(gridApi) {
                        $scope.gridApi = gridApi;
                        $scope.gridApi.grid.registerRowsProcessor($scope.singleFilter, 200);
                        $interval(function() {
                            $scope.gridApi.core.handleWindowResize();
                        }, 500, 60 * 60 * 8);
                    }
                };

                $scope.singleFilter = function(renderableRows) {

                    var matcher = new RegExp($scope.searchEASText);
                    renderableRows.forEach(function(row) {

                        try {
                            var match = false;
                            ['code', 'name'].forEach(function(field) {
                                if (row.entity[field].match(matcher)) {
                                    match = true;
                                }
                            });
                            if (!match) {
                                row.visible = false;
                            }
                        } catch (e) {
                            $log.debug('singleFilter-match:' + JSON.stringify(row.entity));
                        }
                    });
                    return renderableRows;
                };
            }

            function intiValidate() {

                $.validator.addMethod("OSRepeated", function(value, element, param) {
                    var objects = $.grep($scope.gridOptions.data, function(n, i) {
                        return $scope.editingObject.ID != n.id && $.trim(n.name) == $.trim(value);
                    });

                    return objects.length < 1;
                });

                formValidator = $(editModalSelector).find("#edit-form").validate({
                    debug: true,

                    rules: {
                        name: {
                            required: true,
                            minlength: 2,
                            maxlength: 50,
                            OSRepeated: true
                        },
                        IsActive: {
                            required: "input[name='IsActive']:checked"
                        }
                    },
                    messages: {
                        name: {
                            required: $translate.instant('OrganizationStructureEmptyNode'),
                            minlength: $translate.instant('OrganizationStructureEmptyNode'),
                            maxlength: $translate.instant('OrganizationStructureOutOfLengthNode'),
                            OSRepeated: $translate.instant('OrganizationStructureDuplicateNode')
                        },
                        IsActive: $translate.instant('OrganizationStructureStatusUnsureness')
                    },
                    errorPlacement: function(error, element) {
                        var elm = $(element).parents("form:first");
                        elm = elm.find("div[data-for='" + element.attr("name") + "']");
                        elm.html(error);
                        elm.attr("class", "validate-fail");
                    }
                });
            }

            var havePermission = function() {
                $scope.$root.checkUserPermission(constant.adminPermission.basicData.orangizationStructure.editCode).success(function(data) {
                    $scope.hasEditPermission = data;

                    intiUiGrid();

                    $scope.refreshData();

                    intiValidate();
                });
            };

            (function initialize() {
                $log.debug('orangizationStructureManageController.ctor()...');
                $scope.hasEditPermission = false;

                havePermission();
            })();
        }
    ]);
