﻿systemConfigurationModule.controller('ModelConfigurationController', [
    '$scope', '$log', 'uiGridConstants', 'SweetAlert', '$translate', 'stdAccountService', 'enterpriseAccountService', '$interval', 'orgService', '$timeout', '$q', '$uibModal', 'modelConfigurationService', 'enums', "projectService", 'KeyValueConfigService', '$location', "dxDataGridService",
    function ($scope, $log, uiGridConstants, SweetAlert, $translate, stdAccountService, enterpriseAccountService, $interval, orgService, $timeout, $q, $uibModal, modelConfigurationService, enums, projectService, KeyValueConfigService, $location, dxDataGridService) {
        'use strict';


        //模型编辑状态枚举
        $scope.editStatusEnum = {
            view: 1,
            edit: 2,
            add: 3
        };

        //模型类型枚举
        $scope.modelTypeEnum = {
            systemCommon: {
                id: 1,
                name: "系统公共模型",
                typeAbbr: "COM"
            },
            organizationCommon: {
                id: 2,
                name: "机构公共模型",
                typeAbbr: "COM"
            },
            organizationCustomize: {
                id: 3,
                name: "机构自定义模型",
                typeAbbr: "UDF"
            }
        };

        //模型特性枚举
        $scope.modelFeatureEnum = {
            indexAnalysis: {
                id: 1,
                name: $translate.instant('IndexAnalysis'),
                abbr: "IA"
            },
            entriesCheck: {
                id: 2,
                name: $translate.instant('JournalCompare'),
                abbr: "EC"
            }
        };

        $scope.queryParams = {
            //查询模板
            listQueryCondition: [],
            //机构ID
            organizationID: "",
            organizationName: "",
            modelID: "",
            parentID: ""
        };
        $scope.selectNode = {};
        $scope.filterData = {
            modelName: "",
            isSystemModel: false,
            isCommonModel: false,
            isCustomModel: false,
            isEnabledModel: false,
            isDisableModel: false
        };

        $scope.buttonShow = {
            editBtnShow: true,
            saveAsBtnShow: true,
            defaultSetBtnShow: false,
            saveBtnShow: false,
            cancelBtnShow: false
        };

        $scope.buttonDisabled = {
            editBtnShow: true,
            saveAsBtnShow: true,
            defaultSetBtnShow: true,
            saveBtnShow: true,
            cancelBtnShow: true,
            previeBtnShow: true
        };
        $scope.haveModelTreeNode = false;
        $scope.isSaveClick = false;
        $scope.dataList = [];
        //模型树
        $scope.modelTreeList = [];
        //模型信息复制
        $scope.copyModelDetail = {};
        $scope.events = [];

        $scope.validationRules = {
            modelName: [
                {
                    type: 'required',
                    message: '请输入模型名称'
                },
                {
                    type: 'stringLength',
                    min: 4,
                    max: 6,
                    message: '请输入 4 到 50 字的模型名称'
                },
                {
                    type: 'pattern',
                    pattern: '^[a-zA-Z]+$',
                    message: '请输入不包含特殊字符的模型名称'
                }
            ]
        };

        $scope.editDetailBtnDisabled = false;
        $scope.addDetailBtnDisabled = true;
        $scope.deleteDetailBtnDisabled = false;
        $scope.indexBtnShow = false;
        $scope.entriesBtnShow = false;
        $scope.gridColumns = [];
        //指标分析编辑行计算顺序
        $scope.currentRowSequence = -1;
        $scope.currentRowIndexShow = -1;

        //视图/编辑模式下使用的 Model
        $scope.viewEditModel = {
            basicModel: {},
            detailModel: {
                featureId: 1,
                dataList: [],
                formulaCriterion: {}
            },
            editStatus: 1,
            showType: 1,
            validationGroupName: "viewEditGroup",
            serviceList: []
        };

        $scope.serviceList = [];
        //创建模式下使用的 Model
        $scope.createModel = {
            basicModel: {},
            detailModel: {
                featureId: 1,
                dataList: [],
                formulaCriterion: {}
            },
            editStatus: 3,
            showType: 2,
            validationGroupName: "createGroup",
            serviceList: []
        };

        //查看默认设置模式下使用的 Model
        $scope.seeDefaultModel = {
            basicModel: {},
            detailModel: {
                featureId: 1,
                dataList: [],
                formulaCriterion: {}
            },
            editStatus: 1,
            showType: 1,
            validationGroupName: "seeDefaultGroup",
            serviceList: []
        };
        $scope.keyValueDataSource = [];
        $scope.industryInfo = {};
        $scope.currentEditOrAddModelCode = '';
        $scope.dxControlsOptions = {
            name: {
                ControlOption: {
                    bindingOptions: {
                        value: "viewEditModel.basicModel.name"
                    }
                },
                validatorOption: {
                    validationRules: [
                        {
                            type: 'required',
                            message: '请输入模型名称'
                        },
                        {
                            type: 'stringLength',
                            max: 50,
                            message: '请输入不大于 50 字的模型名称'
                        }
                    ]
                }
            }
        };

        $scope.isShowKeyValue = false;
        var showSelectKeyValuePop = function () {

        };
        $scope.keyValueSelectorApi = {
            onSelectorConfirm: function () {
                var selection = this.getSelection();
                if (!_.isEmpty(selection)) {
                    $scope.editModel.formula = $.trim($scope.editModel.formula) + ' @' + selection.keyCode + ' ';
                }
            }
        };
        $scope.keyValueTitle = $translate.instant('AddKeyValue');
        $scope.keyValueSelectorOptions = {
            top: 120
        };

        //设置Button的显示
        var setButtonShow = function (index) {

            if (index == 1 || index == 2) {
                $scope.buttonShow.editBtnShow = false;
                $scope.buttonShow.saveAsBtnShow = false;
                $scope.buttonShow.defaultSetBtnShow = false;
                $scope.buttonShow.saveBtnShow = true;
                $scope.buttonShow.cancelBtnShow = true;
                $scope.editDetailBtnDisabled = true;
                $scope.deleteDetailBtnDisabled = true;
                $scope.addDetailBtnDisabled = false;
            }
            if (index == 4 || index == 5) {
                $scope.buttonShow.editBtnShow = true;
                $scope.buttonShow.saveAsBtnShow = true;
                $scope.buttonShow.defaultSetBtnShow = false;
                $scope.buttonShow.saveBtnShow = false;
                $scope.buttonShow.cancelBtnShow = false;
                $scope.editDetailBtnDisabled = false;
                $scope.deleteDetailBtnDisabled = false;
                $scope.addDetailBtnDisabled = true;
            }
        };

        var setButtonDisabled = function (disabled, modelType) {
            if (modelType === 3) {
                $scope.buttonDisabled.editBtnShow = (!disabled && $scope.hasMaintainCustomPermission) ? false : true;
                $scope.buttonDisabled.defaultSetBtnShow = (!disabled && $scope.hasMaintainCustomPermission) ? false : true;
            } else {
                $scope.buttonDisabled.editBtnShow = (!disabled && $scope.hasEditPermission) ? false : true;
                $scope.buttonDisabled.defaultSetBtnShow = (!disabled && $scope.hasEditPermission) ? false : true;
            }
            $scope.buttonDisabled.saveAsBtnShow = (!disabled && $scope.hasMaintainCustomPermission) ? false : true;
            $scope.buttonDisabled.saveBtnShow = disabled;
            $scope.buttonDisabled.cancelBtnShow = disabled;
            $scope.buttonDisabled.previeBtnShow = disabled;
        };

        //自动生成模型Code
        var getNewModelCode = function (ruleId, feature, modelType) {
            var newCode = "";
            if (ruleId < 10) {
                newCode = "0" + ruleId;
            } else {
                newCode = ruleId + "";
            }
            if (feature == $scope.modelFeatureEnum.indexAnalysis.id) {
                newCode += "." + $scope.modelFeatureEnum.indexAnalysis.abbr;
            }
            if (feature == $scope.modelFeatureEnum.entriesCheck.id) {
                newCode += "." + $scope.modelFeatureEnum.entriesCheck.abbr;
            }
            if (modelType == $scope.modelTypeEnum.systemCommon.id || modelType == $scope.modelTypeEnum.organizationCommon.id) {
                newCode += "." + $scope.modelTypeEnum.systemCommon.typeAbbr + "." + randomWord(true, 5);
            }
            if (modelType == $scope.modelTypeEnum.organizationCustomize.id) {
                newCode += "." + $scope.modelTypeEnum.organizationCustomize.typeAbbr + "." + randomWord(false, 5);
            }
            return newCode;
        };
        //生成随机字符串
        var randomWord = function (allNumber, stringLength) {
            var str = "";
            var arr = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"];
            var arrNum = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"];
            for (var i = 0; i < stringLength; i++) {
                if (allNumber) {
                    var pos = Math.round(Math.random() * (arrNum.length - 1));
                    str += arrNum[pos];
                } else {
                    var pos = Math.round(Math.random() * (arr.length - 1));
                    str += arr[pos];
                }
            }

            return str;
        };

        $scope.doDataFilter = function () {
            $scope.queryParams.listQueryCondition = [];
            if (PWC.isNullOrEmpty($scope.queryParams.organizationID)) {
                return;
            }
            if (!PWC.isNullOrEmpty($scope.filterData.modelName)) {
                var modelNameMto = {};
                modelNameMto.conditionName = "模型名称";
                modelNameMto.searchValue = $scope.filterData.modelName;
                $scope.queryParams.listQueryCondition.push(modelNameMto);
            }
            if (!$scope.filterData.isSystemModel || !$scope.filterData.isCommonModel || !$scope.filterData.isCustomModel) {
                if ($scope.filterData.isSystemModel) {
                    var systemMto = {};
                    systemMto.conditionName = "模型类型";
                    systemMto.searchValue = 1;
                    $scope.queryParams.listQueryCondition.push(systemMto);
                }
                if ($scope.filterData.isCommonModel) {
                    var commonMto = {};
                    commonMto.conditionName = "模型类型";
                    commonMto.searchValue = 2;
                    $scope.queryParams.listQueryCondition.push(commonMto);
                }
                if ($scope.filterData.isCustomModel) {
                    var customMto = {};
                    customMto.conditionName = "模型类型";
                    customMto.searchValue = 3;
                    $scope.queryParams.listQueryCondition.push(customMto);
                }
            }
            if (!$scope.filterData.isEnabledModel || !$scope.filterData.isDisableModel) {
                if ($scope.filterData.isEnabledModel) {
                    var enabledMto = {};
                    enabledMto.conditionName = "使用状态";
                    enabledMto.searchValue = 1;
                    $scope.queryParams.listQueryCondition.push(enabledMto);
                }
                if ($scope.filterData.isDisableModel) {
                    var disableMto = {};
                    disableMto.conditionName = "使用状态";
                    disableMto.searchValue = 0;
                    $scope.queryParams.listQueryCondition.push(disableMto);
                }
            }
            loadModelConfigurationTree();
            $('.filter-button').popover("hide");
        };

        $scope.doDataFilterReset = function () {
            $scope.queryParams.listQueryCondition = [];
            $scope.filterData = {
                modelName: "",
                isSystemModel: false,
                isCommonModel: false,
                isCustomModel: false,
                isEnabledModel: false,
                isDisableModel: false
            };
            loadModelConfigurationTree();
            $('.filter-button').popover("hide");
        };

        var getFirstMoelInTheTree = function (node) {
            if (!node)
                return null;

            if (!node.isCategory)
                return node;

            if (node.isCategory) {

                for (var i = 0; i < node.nodes.length; i++) {
                    var subNode = node.nodes[i];
                    var mNode = getFirstMoelInTheTree(subNode);
                    if (mNode)
                        return mNode;
                }
            }

            return null;
        };

        var getModelInTheTreeByModelCode = function (modelCode, list) {
            if (PWC.isNullOrEmpty(modelCode) || !list) {
                return null;
            }
            for (var i = 0; i < list.length; i++) {
                var node = list[i]
                if (!node.isCategory && node.modelCode === modelCode) {
                    return node;
                }
                if (node.nodes.length > 0) {
                    var temp = getModelInTheTreeByModelCode(modelCode, node.nodes);
                    if (temp) {
                        return temp;
                    }
                }
            }
            return null;
        };

        var loadModelConfigurationTree = function () {
            if (!PWC.isNullOrEmpty($scope.queryParams.organizationID)) {
                modelConfigurationService.getModelList($scope.queryParams.organizationID, $scope.queryParams.listQueryCondition).success(function (data) {

                    var dic = {};
                    for (var ix = 0; ix < data.length; ix++) {
                        var item = data[ix];
                        dic[item.modelID] = {
                            id: item.modelID,
                            title: item.modelName,
                            modelCode: item.modelCode,
                            isCategory: item.isCategory,
                            quantity: 0,
                            parentID: item.parentID,
                            modelType: item.modelType,
                            parentNode: null,
                            sort: item.sort,
                            data: item,
                            nodes: [],
                            isEdit: false,
                            isAdd: false
                        };
                    }

                    //查找每个节点是否存在父节点,并维护父节点与本节点的关系
                    for (var key in dic) {
                        if (!dic.hasOwnProperty(key))
                            continue;
                        var item = dic[key];
                        var parentId = item.parentID;
                        if (!parentId || !dic.hasOwnProperty(parentId))
                            continue;

                        var p = dic[parentId];
                        if (!p && p === item)
                            continue;

                        item.parentNode = p;
                        p.nodes.push(item);
                    }

                    var srcData = [];
                    for (var key in dic) {
                        if (!dic.hasOwnProperty(key))
                            continue;

                        var item = dic[key];
                        if (!item || item.parentNode)
                            continue;

                        srcData.push(item);
                    }
                    $scope.modelTreeList = srcData;
                    refreshModelTreeList();


                    var mNode = null;
                    if (!PWC.isNullOrEmpty($scope.currentEditOrAddModelCode)) {
                        mNode = getModelInTheTreeByModelCode($scope.currentEditOrAddModelCode, $scope.modelTreeList);
                        $scope.currentEditOrAddModelCode = '';
                    }
                    if (mNode) {
                        selectModel(mNode);
                    } else {
                        //完成树形控件初始化后,默认加载第一个节点,若不存在任何模型则将右侧详细信息置空                       
                        for (var i = 0; i < $scope.modelTreeList.length; i++) {
                            mNode = getFirstMoelInTheTree($scope.modelTreeList[i]);
                            if (mNode) {
                                selectModel(mNode);
                                break;
                            }
                        }
                        if (!mNode) {
                            $scope.selectNode = {};
                            $(".content-container .right-container > div").hide();
                        }
                    }
                });
            }

        };

        $scope.propOrders = {
            "isCategory": "desc",
            "sort": "asc",
            "modelType": "asc",
            "modelCode": "asc"
        };

        var sortByProps = function (item1, item2) {
            "use strict";
            var props = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                props[_i - 2] = arguments[_i];
            }

            var cps = []; // 存储排序属性比较结果。
            // 如果未指定排序属性，则按照全属性升序排序。    
            var asc = true;
            if (props.length < 1) {
                for (var p in item1) {
                    if (item1[p] > item2[p]) {
                        cps.push(1);
                        break; // 大于时跳出循环。
                    } else if (item1[p] === item2[p]) {
                        cps.push(0);
                    } else {
                        cps.push(-1);
                        break; // 小于时跳出循环。
                    }
                }
            } else {
                for (var i = 0; i < props.length; i++) {
                    var prop = props[i];
                    for (var o in prop) {
                        asc = prop[o] === "asc";
                        if (item1[o] > item2[o]) {
                            cps.push(asc ? 1 : -1);
                            break; // 大于时跳出循环。
                        } else if (item1[o] === item2[o]) {
                            cps.push(0);
                        } else {
                            cps.push(asc ? -1 : 1);
                            break; // 小于时跳出循环。
                        }
                    }
                }
            }

            for (var j = 0; j < cps.length; j++) {
                if (cps[j] === 1 || cps[j] === -1) {
                    return cps[j];
                }
            }
            return 0;
        }

        var listOrder = function (propOrders, list) {
            list.sort(function (a, b) {
                return sortByProps(a, b, propOrders);
            });
        };
        //递归排序
        var recursionOrder = function (propOrders, list) {
            listOrder(propOrders, list);
            list.forEach(function (node) {
                if (node.nodes.length > 0) {
                    recursionOrder(propOrders, node.nodes);
                }
            });
        };

        var countModel = function (list) {
            list.forEach(function (node) {
                if (node.isCategory) {
                    node.quantity = recursionOrderCount(node);
                }
                if (node.nodes.length > 0) {
                    countModel(node.nodes);
                }
            });
        };
        //排序、统计
        var refreshModelTreeList = function () {
            recursionOrder($scope.propOrders, $scope.modelTreeList);
            countModel($scope.modelTreeList);
        };

        var recursionOrderCount = function (node) {
            var m = 0;
            node.nodes.forEach(function (row) {
                if (!row.isCategory) {
                    m++;
                } else {
                    m = m + recursionOrderCount(row);
                }
            });
            return m;
        };

        var updateSortByTree = function (list) {
            var index = 1;
            list.forEach(function (node) {
                if (node.isCategory) {
                    node.sort = index;
                    index++;
                    if (node.nodes.length > 0) {
                        updateSortByTree(node.nodes);
                    }
                }
            });
        };

        var loadModelData = function (orgId, modelId, categoryId) {
            if (!PWC.isNullOrEmpty(modelId) && !PWC.isNullOrEmpty(orgId)) {
                modelConfigurationService.getModelSingle(orgId, modelId, categoryId).success(function (data) {

                    $scope.viewEditModel.basicModel = data;
                    var serviceTypeID = "";
                    if (data.serviceTypeID != null && data.serviceTypeID != undefined) {
                        serviceTypeID = data.serviceTypeID;
                    }
                    $scope.viewEditModel.basicModel.serviceTypeID = $.map($.grep(serviceTypeID.split(","), function (x) {
                        return x.trim() != ""
                    }), function (x) {
                        return x.trim()
                    });

                    $scope.viewEditModel.detailModel.featureId = data.feature;
                    $scope.viewEditModel.detailModel.dataList = data.feature == 1 ? data.indexAnalysisDetaileList : data.entriesCheckDetailList;
                    $scope.viewEditModel.detailModel.formulaCriterion = data;

                    if ($scope.viewEditModel.basicModel.type == $scope.modelTypeEnum.organizationCommon.id) {
                        $scope.buttonShow.defaultSetBtnShow = true;
                    } else {
                        $scope.buttonShow.defaultSetBtnShow = false;
                    }
                });
            }
        };

        var loadKeyValueData = function () {
            if ($scope.queryParams.organizationID) {
                return KeyValueConfigService.getByOrgID($scope.queryParams.organizationID).then(function (res) {
                    $scope.keyValueDataSource.splice(0, $scope.keyValueDataSource.length);
                    res.data.forEach(function (e) {
                        $scope.keyValueDataSource.push(angular.copy(e));
                    });
                    return $q.when($scope.keyValueDataSource);
                }, function (response) {
                    return $q.reject($translate.instant("DataLoadingError"));
                });
            }
        };
        var loadIndustryData = function () {
            if ($scope.queryParams.organizationID) {
                return modelConfigurationService.getIndustry($scope.queryParams.organizationID).success(function (data) {
                    $scope.industryInfo = data;
                });
            }
        };
        var loadServiceList = function (orgId, modelId, isAdd) {
            //初始化加载服务类型列表数据
            modelConfigurationService.getServiceList(orgId, modelId, isAdd).success(function (serviceListData) {
                if (isAdd) {
                    $scope.createModel.serviceList.splice(0, $scope.createModel.serviceList.length);
                    for (var i = 0; i < serviceListData.length; i++) {
                        $scope.createModel.serviceList.push(serviceListData[i]);
                    }
                } else {
                    $scope.viewEditModel.serviceList.splice(0, $scope.viewEditModel.serviceList.length);
                    for (var i = 0; i < serviceListData.length; i++) {
                        $scope.viewEditModel.serviceList.push(serviceListData[i]);
                    }
                }
            });
        }

        $scope.$watch("queryParams.organizationID", function (newValue, oldValue) {
            if (newValue !== oldValue) {
                loadModelConfigurationTree();
                getStdAccountList();
                loadKeyValueData();
                loadIndustryData();
                getProject();
            }
        });

        //选择模型
        var selectModel = function (node) {
            $scope.selectNode = node;

            setButtonDisabled(false, node.modelType);
            setButtonShow(4);
            cancelEditGrid();
            $scope.queryParams.modelID = node.id;
            $scope.queryParams.parentID = node.parentID;
            loadServiceList($scope.queryParams.organizationID, node.id, false);
            loadModelData($scope.queryParams.organizationID, node.id, node.parentID);
            $scope.viewEditModel.editStatus = 1;

            $(".content-container .right-container > div").show();
        };
        var selectTreeNode = function (node) {
            $scope.haveModelTreeNode = false;
            judgeHaveModel(node.nodes);
            $scope.selectNode = node;
            $scope.queryParams.modelID = node.id;
            $scope.queryParams.parentID = node.parentID;
            $scope.selectOldName = node.title;
        };

        var editTreeNodeName = function ($event, node) {
            if (!$scope.hasMaintainModelTreePermission) {
                return;
            }
            $scope.selectNode = node;
            node.isEdit = true;
            $timeout(function () {
                $("#tree-root input:visible").focus();
            }, 100);
        };
        //模型树
        $scope.treeOptions = {
            beforeDrop: function (e) {
                $scope.selectNode = {};
                //$scope.queryParams.modelID = "";
                var nodesource = e.source.nodeScope.$modelValue;
                var oldParentId = nodesource.parentID;
                var newParent = e.dest.nodesScope.$parent.$modelValue;
                //模型移动
                if (!nodesource.isCategory) {
                    if (!newParent || !newParent.isCategory) {
                        return $q.reject();
                    }
                    if (oldParentId != newParent.id) {
                        return modelConfigurationService.updateModelConfig(nodesource.id, oldParentId, newParent.id).success(function (opResult) {
                            //保存失败时撤销移动
                            if (!opResult.result) {
                                $q.reject();
                            } else {
                                //成功后更新结点信息
                                e.source.nodeScope.$modelValue.parentID = newParent.id;
                                e.source.nodeScope.$modelValue.data.categoryID = newParent.id;
                            }
                        });
                    }
                } else {
                    var newParentId = '';
                    if (newParent) {
                        if (!newParent.isCategory) {
                            return $q.reject();
                        }
                        newParentId = newParent.id;
                    } else {
                        newParentId = $scope.queryParams.organizationID;
                    }
                    //节点移动
                    var sortCurrent = e.dest.index + 1;
                    return modelConfigurationService.updateCategoryWhenMove(nodesource.id, oldParentId, newParentId, sortCurrent).success(function (opResult) {
                        //保存失败时撤销移动
                        if (!opResult.result) {
                            $q.reject();
                        } else {
                            //成功后更新结点信息
                            e.source.nodeScope.$modelValue.parentID = newParentId;
                            e.source.nodeScope.$modelValue.data.categoryID = newParentId;
                        }
                    });
                }
            },
            dragStop: function (e) {
                updateSortByTree($scope.modelTreeList);
                refreshModelTreeList();
                loadModelData($scope.queryParams.organizationID, $scope.queryParams.modelID, "");
                $scope.viewEditModel.editStatus = 1;
                $(".content-container .right-container > div").show();

            }
        };

        var setValidation = function (isValid, info, setFocus) {
            $timeout(function () {
                var nameInstance = $(".categoryBox:visible").dxTextBox("instance");
                if (nameInstance) {
                    $(".categoryBox:visible").dxTextBox("instance").option("isValid", isValid);
                    $(".categoryBox:visible").dxTextBox("instance").option("validationError", {
                        message: info
                    })
                    if (setFocus) {
                        nameInstance.focus();
                    }
                }
            }, 100);
        }

        //失去焦点
        var saveEditName = function () {
            if (PWC.isNullOrEmpty($scope.selectNode.title)) {
                setValidation(false, "分类名称不能为空", true);
                return;
            } else {
                setValidation(true, "", false)
            }

            modelConfigurationService.uniqueCategoryName($scope.selectNode.title, $scope.queryParams.organizationID, $scope.selectNode.id).success(function (data) {
                if (data.result) {

                    var mto = {};
                    mto.modelID = $scope.selectNode.id;
                    mto.modelName = $scope.selectNode.title;
                    mto.parentID = $scope.selectNode.parentID;
                    mto.organizationID = $scope.queryParams.organizationID;
                    mto.sort = $scope.selectNode.sort;
                    modelConfigurationService.updateCategory(mto).success(function (opResult) {
                        if (opResult.result) {
                            setValidation(true, "", false);
                            //重新给节点赋值(部分缺失字段)
                            $scope.selectNode.id = opResult.data.modelID;
                            $scope.selectNode.title = opResult.data.modelName;
                            $scope.selectNode.parentID = opResult.data.parentID;
                            $scope.selectNode.organizationID = opResult.data.organizationID;
                            $scope.selectNode.sort = opResult.data.sort;
                            $scope.selectNode.isEdit = false;
                        } else {
                            SweetAlert.warning("保存失败:" + opResult.resultMsg);
                            $timeout(function () {
                                nameInstance.focus();
                            }, 100);
                        }
                    });
                } else {
                    setValidation(false, "分类名称重复,请重新输入...", true)
                }
            });

        };

        //编辑模型
        var editModel = function () {
            setButtonShow(1);
            $scope.copyModelDetail = angular.copy($scope.viewEditModel.basicModel);
            if ($scope.viewEditModel.basicModel.type == $scope.modelTypeEnum.systemCommon.id) {
                $scope.viewEditModel.editStatus = $scope.editStatusEnum.add;
                $scope.viewEditModel.basicModel.type = $scope.modelTypeEnum.organizationCommon.id;
                $scope.viewEditModel.basicModel.modelTypeName = $scope.modelTypeEnum.organizationCommon.name;
                $scope.viewEditModel.basicModel.oldType = $scope.modelTypeEnum.systemCommon.id;
            } else {
                $scope.viewEditModel.editStatus = $scope.editStatusEnum.edit;
            }
            $timeout(function () {
                var nameInstance = $("#nameValidation").dxTextBox("instance");
                if (nameInstance) {
                    nameInstance.option("isValid", true);
                    nameInstance.option("validationError", {
                        message: ''
                    });
                }
            }, 100);
        };

        $scope.validateAndSubmit = function (params) {
            var result = params.validationGroup.validate();
            if (result.isValid) {
                DevExpress.ui.notify({
                    message: "You are registered",
                    position: {
                        my: "left top",
                        at: "left top",
                        offset: 15
                    }
                }, "success", 3000);
            }
        };


        //保存
        var saveModel = function (index, modalInstance) {
            if (index === 1) {
                $scope.isSaveClick = true;
            }
            var viewContainerClass = ".right-container";
            var creatContainerClass = ".model-configuration-manage-create-model-tree-node-modal";
            var showFirstbrokenRuleMessage = function (brokenRules) {
                if (!brokenRules || brokenRules.length < 1)
                    return;

                var firstResult = brokenRules["0"];
                SweetAlert.warning(firstResult.message);
                firstResult.validator.element().focusin();
            }

            // 验证基础信息关键字段(包括 modelName 字段)
            var validationGroupName = index == 1 ? "viewEditGroup" : "createGroup";
            var fieldValidateResult = DevExpress.validationEngine.getGroupConfig(validationGroupName).validate();
            if (!fieldValidateResult.isValid) {
                showFirstbrokenRuleMessage(fieldValidateResult.brokenRules);
                return;
            }

            //等待模型名称远程验证结束,再继续后续的验证以及保存操作
            var waitRemoteValidationComplete = function () {
                if ($scope.remoteValidationCompleted) {
                    var selectorStr = (index == 1 ? viewContainerClass : creatContainerClass) + " [dx-validator='modelNameValidationRules']";
                    var modelNameValidateRst = $(selectorStr).dxValidator("instance").validate();
                    if (!modelNameValidateRst.isValid) {
                        showFirstbrokenRuleMessage([modelNameValidateRst.brokenRule]);
                        return;
                    }

                    //验证明细列表信息
                    selectorStr = (index == 1 ? viewContainerClass : creatContainerClass) + " [dx-data-grid]";
                    var dataGrid = $(selectorStr).dxDataGrid('instance');
                    dataGrid.saveEditData();
                    if (dataGrid.hasEditData()) {
                        return;
                    }
                    var dataGridValidateRst = $scope.validateDataGrid(dataGrid);
                    if (!dataGridValidateRst.isValid) {
                        SweetAlert.warning("模型明细存在不符合要求的数据," + (dataGridValidateRst.message ? dataGridValidateRst.message : "请仔细检查后提交"));
                        return;
                    }

                    $scope.isSaveClick = false;

                    if (index === 1) {
                        var model = angular.copy($scope.viewEditModel.basicModel);
                        model.serviceTypeID = model.serviceTypeID.join(",");
                        if ((model.indexAnalysisDetaileList == null || model.indexAnalysisDetaileList.length === 0) && (model.entriesCheckDetailList == null || model.entriesCheckDetailList.length === 0)) {
                            SweetAlert.warning("请添加模型明细");
                            return;
                        }
                        if ($scope.viewEditModel.editStatus === $scope.editStatusEnum.add) {
                            modelConfigurationService.addModel(model).success(function (opResult) {
                                if (opResult.result) {
                                    SweetAlert.warning("保存成功");
                                    $scope.currentEditOrAddModelCode = model.code;
                                    refreshData(opResult.data, model.type);
                                } else {
                                    SweetAlert.warning("保存失败:" + opResult.resultMsg);
                                }
                            });
                        } else if ($scope.viewEditModel.editStatus === $scope.editStatusEnum.edit) {
                            modelConfigurationService.updateModel(model).success(function (opResult) {
                                if (opResult.result) {
                                    SweetAlert.warning("保存成功");
                                    $scope.currentEditOrAddModelCode = model.code;
                                    setButtonDisabled(false, model.type);
                                    setButtonShow(4);
                                    cancelEditGrid();
                                    $scope.viewEditModel.editStatus = 1;
                                    loadModelData($scope.queryParams.organizationID, $scope.queryParams.modelID, "");
                                } else {
                                    SweetAlert.warning("保存失败:" + opResult.resultMsg);
                                }
                            });
                        }
                    }
                    if (index === 2) {
                        var model = angular.copy($scope.createModel.detailModel);
                        if (model.dataList == null || model.dataList.length === 0) {
                            SweetAlert.warning("请添加模型明细");
                            return;
                        }
                        var newModel = angular.copy($scope.createModel.basicModel);
                        newModel.orgId = $scope.queryParams.organizationID;
                        newModel.ruleType = 2;
                        newModel.isStatus = 1;
                        newModel.feature = model.featureId;
                        if (newModel.feature == 1) {
                            newModel.indexAnalysisDetaileList = model.dataList;
                            newModel.monthJudge = model.formulaCriterion.monthJudge;
                            newModel.monthExplain = model.formulaCriterion.monthExplain;
                            newModel.yearJudge = model.formulaCriterion.yearJudge;
                            newModel.yearExplain = model.formulaCriterion.yearExplain;
                        }
                        if (newModel.feature == 2) {
                            newModel.entriesCheckDetailList = model.dataList;
                        }
                        newModel.serviceTypeID = newModel.serviceTypeID.join(",");
                        modelConfigurationService.addModel(newModel).success(function (opResult) {
                            if (opResult.result) {
                                $scope.currentEditOrAddModelCode = newModel.code;
                                //loadModelConfigurationTree();
                                SweetAlert.warning("保存成功");
                                modalInstance.close();
                                refreshData(opResult.data, newModel.type);
                            } else {
                                SweetAlert.warning("保存失败:" + opResult.resultMsg);
                            }
                        });
                    }

                    return;
                }

                $timeout(waitRemoteValidationComplete, 100);
            };

            waitRemoteValidationComplete();
        };

        //取消
        var cancelEdit = function () {
            setButtonShow(4);
            $scope.viewEditModel.basicModel = angular.copy($scope.copyModelDetail);
            if ($scope.viewEditModel.basicModel.type == $scope.modelTypeEnum.organizationCommon.id) {
                $scope.buttonShow.defaultSetBtnShow = true;
            } else {
                $scope.buttonShow.defaultSetBtnShow = false;
            }
            if ($scope.viewEditModel.detailModel.featureId == $scope.modelFeatureEnum.indexAnalysis.id) {
                $scope.viewEditModel.detailModel.dataList = $scope.viewEditModel.basicModel.indexAnalysisDetaileList;
            }
            if ($scope.viewEditModel.detailModel.featureId == $scope.modelFeatureEnum.entriesCheck.id) {
                $scope.viewEditModel.detailModel.dataList = $scope.viewEditModel.basicModel.entriesCheckDetailList;
            }
            $scope.viewEditModel.detailModel.formulaCriterion = $scope.viewEditModel.basicModel;
            $scope.viewEditModel.editStatus = 1;
        };

        //另存为自定义
        var saveASModel = function () {
            setButtonShow(1);
            $scope.copyModelDetail = angular.copy($scope.viewEditModel.basicModel);
            $scope.viewEditModel.basicModel.name = "";
            $scope.viewEditModel.basicModel.id = "";
            $scope.viewEditModel.editStatus = $scope.editStatusEnum.add;
            $scope.viewEditModel.basicModel.type = $scope.modelTypeEnum.organizationCustomize.id;
            $scope.viewEditModel.basicModel.modelTypeName = $scope.modelTypeEnum.organizationCustomize.name;
            $scope.viewEditModel.basicModel.code = getNewModelCode(2, $scope.viewEditModel.basicModel.feature, $scope.viewEditModel.basicModel.type);
        };

        var cancelEditGrid = function () {
            var dataGrid = $('.model-configuration-manage-detail-info [dx-data-grid]').dxDataGrid('instance');
            if (dataGrid) {
                dataGrid.cancelEditData();
            }
        };

        $scope.selectedStandardList = [];
        $scope.standardAccountList = [];
        var getStdAccountList = function () {
            stdAccountService.getStdAccountByOrgID($scope.queryParams.organizationID).success(function (data) {
                _.each(data, function (account) {
                    account.directionName = $translate.instant(enums.standardAccountDirection[account.direction]);
                });
                $scope.standardAccountList.splice(0, $scope.standardAccountList.length);
                if (data) {
                    data.forEach(function (e) {
                        $scope.standardAccountList.push(angular.copy(e));
                    });
                }
            });
        };

        //#region 左侧导航模块操作按钮 functions 

        // 参数 nodeType 为 1 时,表示期望创建"模型分类根节点"
        // 参数 nodeType 为 2 时,表示期望创建"模型分类"
        // 参数 nodeType 为 3 时,表示期望创建"机构自定义模型"
        $scope.createModelTreeNode = function ($event, nodeType) {
            $scope.createModel = {
                basicModel: {},
                detailModel: {
                    featureId: 1,
                    dataList: [],
                    formulaCriterion: {}
                },
                editStatus: 3,
                showType: 2,
                validationGroupName: "createGroup",
                serviceList: []
            };

            switch (nodeType) {
                case 1:
                    var dic = {};
                    dic.title = ""
                    dic.isCategory = true;
                    dic.isEdit = true;
                    dic.isAdd = true;
                    dic.nodes = [];
                    $scope.modelTreeList.push(dic);
                    $timeout(function () {
                        $("#tree-root input:visible").focus();
                    }, 100);
                    break;
                case 2:
                    if ($scope.selectNode && $scope.selectNode.isCategory) {
                        var dic = {};
                        dic.title = ""
                        dic.parentID = $scope.selectNode.id;
                        dic.parentNode = $scope.selectNode;
                        dic.isCategory = true;
                        dic.isEdit = true;
                        dic.isAdd = true;
                        dic.nodes = [];
                        $scope.selectNode.nodes.push(dic);
                        $timeout(function () {
                            $("#tree-root input:visible").focus();
                        }, 100);
                    }
                    else
                        SweetAlert.warning("请选择模型需要添加分类的模型分类");
                    break;
                case 3:
                    if ($scope.selectNode && $scope.selectNode.isCategory) {
                        $scope.createModel.detailModel.featureId = 1;
                        $scope.createModel.basicModel.type = $scope.modelTypeEnum.organizationCustomize.id;
                        $scope.createModel.basicModel.modelTypeName = $scope.modelTypeEnum.organizationCustomize.name;
                        $scope.createModel.basicModel.industyId = $scope.industryInfo.id;
                        $scope.createModel.basicModel.industryName = $scope.industryInfo.name;
                        $scope.createModel.basicModel.organizationName = $scope.queryParams.organizationName;
                        $scope.createModel.basicModel.organizationID = $scope.queryParams.organizationID;
                        $scope.createModel.basicModel.code = getNewModelCode(2, $scope.createModel.detailModel.featureId, $scope.createModel.basicModel.type);
                        $scope.createModel.basicModel.isStatus = 1;
                        $scope.createModel.basicModel.isFilterShow = true;
                        $scope.createModel.basicModel.categoryID = $scope.selectNode.id;
                        //$scope.createModel.basicModel.categoryString = $scope.modelFeatureEnum.indexAnalysis.name;
                        // editStatus 由 1 变为 3 的目的是:使 dataGrid 变为可编辑状态
                        $scope.createModel.editStatus = 1;
                        modelConfigurationService.getModelCategoryString($scope.queryParams.organizationID, $scope.selectNode.id).success(function (cateString) {
                            $scope.createModel.basicModel.categoryString = cateString;
                        });
                        loadServiceList($scope.queryParams.organizationID, "", true);
                        var modalInstance = $uibModal.open({
                            animation: false,
                            ariaLabelledBy: 'modal-title',
                            ariaDescribedBy: 'modal-body',
                            templateUrl: 'model-config-manage-create-model-tree-node-modal.html',
                            scope: $scope,
                            windowClass: "model-configuration-manage-create-model-tree-node-modal"
                        });
                        $scope.creatModelSave = function () {
                            saveModel(2, modalInstance);
                        };
                        $scope.saveClose = function () {
                            modalInstance.dismiss('cancel');
                        };
                        $timeout(function () {
                            $scope.createModel.editStatus = 3;
                        }, 500);
                    }
                    else
                        SweetAlert.warning("请选择模型需要添加模型的模型分类");
                    break;
                default:
                    SweetAlert.warning("不受支持的 nodeType 枚举值");
                    return;
            }

        };

        var removeNode = function (id, list) {
            var index = 0;
            list.forEach(function (row) {
                if (row.id == id) {
                    list.splice(index, 1);
                }
                if (row.nodes.length > 0) {
                    removeNode(id, row.nodes);
                }
                index++;
            });
        };

        //删除模型分类或模型
        $scope.deleteModelTreeNode = function () {
            if ($scope.selectNode) {
                if ($scope.selectNode.isCategory) {
                    if (!$scope.hasMaintainModelTreePermission) {
                        SweetAlert.warning("你没有权限删除该分类");
                        return;
                    }
                    //判断该分类下是否有模型  
                    $scope.tmpValue = false;
                    judgeHaveModel($scope.selectNode.nodes);
                    if ($scope.tmpValue) {
                        SweetAlert.warning("该分类或子分类下含有模型，无法删除");
                    } else {
                        modelConfigurationService.deleteCategory($scope.selectNode.id, $scope.queryParams.organizationID).success(function (opResult) {
                            if (opResult.result) {
                                removeNode($scope.selectNode.id, $scope.modelTreeList);
                                updateSortByTree($scope.modelTreeList);
                                refreshModelTreeList();
                            }
                        });

                    }
                } else {
                    if (!$scope.hasMaintainCustomPermission) {
                        SweetAlert.warning("你没有权限删除该模型");
                        return;
                    }
                    if ($scope.selectNode.modelType == $scope.modelTypeEnum.organizationCustomize.id) {
                        SweetAlert.swal({
                            title: "你确定要删除该机构自定义模型吗？",
                            type: "warning",
                            showCancelButton: true,
                            confirmButtonColor: "#DD6B55",
                            confirmButtonText: $translate.instant('Confirm'),
                            cancelButtonText: $translate.instant('Cancel'),
                            closeOnConfirm: true,
                            closeOnCancel: true
                        },
                            function (isConfirm) {
                                if (isConfirm) {
                                    modelConfigurationService.deleteModel($scope.selectNode.id).success(function (opResult) {
                                        if (opResult.result) {
                                            removeNode($scope.selectNode.id, $scope.modelTreeList);
                                            refreshModelTreeList();
                                        }
                                    });
                                }
                            });

                    } else {
                        SweetAlert.warning("非机构自定义模型，无法删除");
                    }
                }

            }
        };
        $scope.tmpValue = false;
        var judgeHaveModel = function (nodes) {
            if (_.find(nodes, function (num) {
                return num.isCategory == false
            })) {
                $scope.tmpValue = true;
                $scope.haveModelTreeNode = true;
            } else {
                nodes.forEach(function (mc) {
                    judgeHaveModel(mc.nodes);
                });
            }
        };


        //#endregion

        //#region 右侧基本信息按钮 functions


        var loadModelDefaultData = function (orgId, modelId, categoryId) {
            if (!PWC.isNullOrEmpty(modelId) && !PWC.isNullOrEmpty(orgId)) {
                modelConfigurationService.getModelDefaultSetting(orgId, modelId, categoryId).success(function (data) {

                    $scope.seeDefaultModel.basicModel = data;

                    $scope.seeDefaultModel.detailModel.featureId = data.feature;
                    $scope.seeDefaultModel.detailModel.dataList = data.feature == 1 ? data.indexAnalysisDetaileList : data.entriesCheckDetailList;
                    $scope.seeDefaultModel.detailModel.formulaCriterion = data;
                });
            }
        };
        $scope.seeDefault = function () {
            loadModelDefaultData($scope.queryParams.organizationID, $scope.queryParams.modelID, $scope.queryParams.parentID);

            var modalInstance = $uibModal.open({
                animation: false,
                ariaLabelledBy: 'modal-title',
                ariaDescribedBy: 'modal-body',
                templateUrl: 'model-config-manage-defalut-settings-modal.html',
                scope: $scope,
                windowClass: "model-configuration-manage-defalut-settings-modal"
            });
            $scope.resetModel = function () {
                modelConfigurationService.resetOrgModelSetting($scope.queryParams.modelID, $scope.queryParams.organizationID).success(function (data) {
                    if (data.result) {
                        modalInstance.close();
                        $scope.currentEditOrAddModelCode = $scope.seeDefaultModel.basicModel.code;
                        refreshData(data.data, 1)
                    }
                });
            };
            $scope.resetClose = function () {
                modalInstance.dismiss('cancel');
            };
        };

        var refreshData = function (modelId, modelType) {
            $scope.viewEditModel.editStatus = 1;
            loadModelConfigurationTree();
            $scope.queryParams.modelID = modelId;
            setButtonDisabled(false, modelType);
            setButtonShow(4);
            cancelEditGrid();
            loadModelData($scope.queryParams.organizationID, $scope.queryParams.modelID, "");
        };

        //#endregion
        $scope.textBox = {
            modelName: {
                bindingOptions: {
                    value: "viewEditModel.basicModel.name",
                    visible: "viewEditModel.editStatus != 1",
                    readOnly: "viewEditModel.basicModel.type!=3"
                }
            }
        };
        $scope.loadNodes = function (node) {
            return {
                maxLength: 10,
                onFocusIn: function () {
                    selectTreeNode(node);
                },
                onFocusOut: function (params) {
                    if (!(params.component.option("value") || "").trim()) {
                        var nodeScope = params.element.parents("[ui-tree-node]:first").data().$uiTreeNodeController.scope;
                        var modelValue = nodeScope.$modelValue;
                        //取消添加:将当前节点从树节点中移除,并将当前节点的父节点设置为 选中节点
                        if (modelValue.isAdd) {
                            selectTreeNode(nodeScope.$parentNodeScope.$modelValue);
                            nodeScope.remove();
                            return;
                        }
                        //取消编辑:将测试对显示字段的编辑
                        if (modelValue.isEdit) {
                            modelValue.title = modelValue.data.modelName;
                            modelValue.isEdit = false;
                            return;
                        }
                    }

                    saveEditName();
                },
                bindingOptions: {
                    value: 'node.title',
                    visible: 'node.isEdit'
                }
            };
        };

        $scope.remoteValidationCompleted = false;
        $scope.remoteValidateModelName = function (params, modelName, modelId, organizationId, flag) {
            $scope.remoteValidationCompleted = false;
            var remoteRst = modelConfigurationService.uniqueModelName(modelName, modelId, organizationId, flag);
            remoteRst.success(function (data) {
                params.rule.isValid = data.result;
                params.validator.validate();
                $scope.remoteValidationCompleted = true;
            });
        };

        $scope.modelNameValidationRules = {
            validationRules: [
                {
                    type: "required",
                    message: "模型名称不能为空"
                },
                {
                    type: 'stringLength',
                    max: 50,
                    message: '请输入不超过 50 字的模型名称'
                },
                {
                    type: "custom",
                    validationCallback: function (params) {
                        if ($scope.viewEditModel.editStatus == 1)
                            return true;

                        var modelName = $scope.viewEditModel.basicModel.name;
                        var modelId = $scope.viewEditModel.basicModel.id;
                        var organizationId = $scope.queryParams.organizationID;
                        var flag = $scope.viewEditModel.editStatus === 3;
                        $scope.remoteValidateModelName(params, modelName, modelId, organizationId, flag);
                        return true;
                    },
                    message: "名称重复,请重新输入..."
                }
            ],
            validationGroup: "viewEditGroup"
        };

        $("#saveButton").dxButton({
            text: '保存',
            onClick: register
        });

        $scope.saveButtonOptions = {
            text: '保存',
            type: 'normal',
            validationGroup: "viewEditGroup",
            onClick: function (e) {
                var result = e.validationGroup.validate();
                if (result.isValid) {
                    DevExpress.ui.notify({
                        message: "You are logged in！",
                        position: {
                            my: "left top",
                            at: "left top"
                        }
                    }, "success", 3000);
                }
            }
        };
        var register = function (params) {
            var result = params.validationGroup.validate();
            if (result.isValid) {
                DevExpress.ui.notify({
                    message: "You are logged in！",
                    position: {
                        my: "left top",
                        at: "left top"
                    }
                }, "success", 3000);
            }
        };
        //根据 dataGrid 的 Option 设置验证数据源的有效性,并返回验证结果,isValid:true 验证通过,isValid:false 验证无效,message 验证信息
        //数据源为空则当做验证通过
        $scope.validateDataGrid = function (gridInstance) {
            var data = gridInstance.option().dataSource;
            var columns = gridInstance.option().columns;
            var validateRst = {
                isValid: true,
                message: ""
            };

            if (!data || data.length < 1)
                return validateRst;

            for (var i = 0; i < columns.length; i++) {
                var col = columns[i];
                // isMainValueShow 需要单独验证,因为在 dataGrid 中无法进行验证
                if (col.cellTemplate && col.cellTemplate === "isMainValueTemplate") {
                    //没有主要值的校验
                    var haveMainValue = false;
                    for (var j = 0; !haveMainValue && j < data.length; j++) {
                        haveMainValue = data[j].isMainValueShow;
                    }
                    if (!haveMainValue) {
                        validateRst.isValid = false;
                        validateRst.message = "请设置模型的主要值";
                        var colIndex = _.findIndex(gridInstance.getVisibleColumns(), function (item) {
                            return item.cellTemplate === col.cellTemplate;
                        });
                        for (var j = 0; j < data.length; j++) {
                            var key = gridInstance.keyOf(data[j]);
                            var rowIndex = gridInstance.getRowIndexByKey(key);
                            var cell = gridInstance.getCellElement(rowIndex, colIndex);
                            cell.addClass("dx-validator dx-editor-cell dx-datagrid-invalid dx-focused");
                            cell.find("> div").addClass("dx-highlight-outline dx-texteditor-input");
                        }
                    }
                }

                var rules = col.validationRules;
                if (!rules || rules.length < 1) {
                    continue;
                }

                var haveRepeatIndexName = null;
                for (var j = 0; j < data.length; j++) {
                    var value = data[j][col.dataField];
                    //特殊逻辑验证
                    if ((col.dataField === "acctDirection" || col.dataField === "standardCode" || col.dataField === "entriesComparison" || col.dataField === "thresholdValue") && data[j]["acctJudge"] === "N/A") {
                        if (value) {
                            console.log("代码异常,字段" + col.dataField + " :应该为 null,实际值为: " + value);
                        }
                        continue;
                    }

                    rules = angular.copy(rules);
                    _.each(rules, function (item) {
                        item.value = null;
                        item.isValid = false;
                    });
                    var rst = DevExpress.validationEngine.validate(value, rules, col.dataField);
                    if (!rst.isValid) {
                        validateRst.isValid = false;
                        if (col.dataField === "indexName" && rst.brokenRule.type === "custom" && (!haveRepeatIndexName || !haveRepeatIndexName.isValid)) {
                            haveRepeatIndexName = {
                                isValid: rst.brokenRule.isValid,
                                message: rst.brokenRule.message
                            };
                        }
                        var key = gridInstance.keyOf(data[j]);
                        var rowIndex = gridInstance.getRowIndexByKey(key);
                        var cell = gridInstance.getCellElement(rowIndex, col.dataField);
                        cell.addClass("dx-validator dx-editor-cell dx-datagrid-invalid dx-focused");
                        var divContainer = cell.find("> div");
                        if (divContainer.length < 1) {
                            cell.wrapInner("<div></div>");
                            divContainer = cell.find("> div");
                        }

                        divContainer.addClass("dx-highlight-outline");
                        if (cell.find(".dx-texteditor-input").length < 1) {
                            if (cell.find("input:first") > 1) {
                                cell.find("input:first").addClass("dx-texteditor-input");
                            } else {
                                divContainer.addClass("dx-texteditor-input");
                            }
                        }
                    }
                }
                if (haveRepeatIndexName && !haveRepeatIndexName.isValid) {
                    validateRst.message = haveRepeatIndexName.message;
                }
            }
            return validateRst;
        };

        //模型预览
        $scope.projectYearList = [];
        $scope.previewYear = [];
        $scope.enterpriseList = [];
        $scope.enterpriseSelectShow = false;
        $scope.popoverYearSelect = 520;
        $scope.selectProjectDto = {};
        $scope.selectedYear = {};
        $scope.selectedEnterprise = {};
        var getProject = function () {
            if (!PWC.isNullOrEmpty($scope.queryParams.organizationID)) {
                modelConfigurationService.getProject($scope.queryParams.organizationID).success(function (data) {
                    $scope.projectYearList = data;
                    $scope.projectYearList.forEach(function (row) {
                        if (!_.find($scope.previewYear, function (num) {
                            return num === row.year
                        })) {
                            $scope.previewYear.push(row.year);
                        }
                    })
                    $scope.selectedYear = $scope.previewYear[0];
                    $scope.enterpriseList = $scope.projectYearList.filter(function (element) {
                        return element.year === $scope.selectedYear;
                    });
                    $scope.selectedEnterprise = $scope.enterpriseList[0];
                });
            }
        };

        $scope.resultPreview = function (isAdd) {
            $scope.isAddPreview = isAdd;
            var model = $scope.isAddPreview ? angular.copy($scope.createModel.basicModel) : angular.copy($scope.viewEditModel.basicModel)

            if ((!model.indexAnalysisDetaileList && !model.entriesCheckDetailList) ||
                (model.indexAnalysisDetaileList && model.indexAnalysisDetaileList.length === 0) ||
                (model.entriesCheckDetailList && model.entriesCheckDetailList.length === 0)
            ) {
                SweetAlert.warning("请添加模型明细后再预览");
                return;
            } else {

                if (PWC.isNullOrEmpty($scope.queryParams.modelID) && !isAdd) {
                    return;
                }
                if ($scope.projectYearList.length == 0) {
                    SweetAlert.warning("界面卡片必须显示“进行中”，才能查看预览结果");
                    return;
                }
                getPopoverInstance().show();
            }
        };

        $scope.yearSelectOptions = {
            //target: "#previewModel",
            //showEvent: "dxclick",
            position: "bottom",
            //shading: true,
            //shadingColor: "rgba(0, 0, 0, 0.5)",
            visible: false,
            bindingOptions: {
                width: "popoverYearSelect",
                target: "isAddPreview?'#addPreviewModel':'#previewModel'"
            }
        };
        $scope.projectYearSelect = {
            width: 80,
            bindingOptions: {
                dataSource: "previewYear",
                value: 'selectedYear'
            },
            onSelectionChanged: function (e) {
                $scope.enterpriseList = $scope.projectYearList.filter(function (element) {
                    return element.year === e.selectedItem;
                });
                if ($scope.enterpriseList.length > 1) {
                    $scope.enterpriseSelectShow = true;
                    $scope.popoverYearSelect = 650;
                } else {
                    //$scope.enterpriseSelectShow = false;
                    //$scope.popoverYearSelect = 520;
                    $scope.enterpriseSelectShow = true;
                    $scope.popoverYearSelect = 650;
                }
                $scope.selectProjectDto = e.selectedItem;
                $scope.selectedEnterprise = $scope.enterpriseList[0];
            }
        };
        $scope.enterpriseSetSelect = {
            width: 130,
            displayExpr: 'enterpriseAccountSetName',
            bindingOptions: {
                dataSource: "enterpriseList",
                visible: "enterpriseSelectShow",
                value: 'selectedEnterprise'
            },
            onSelectionChanged: function (e) {
                $scope.selectProjectDto = e.selectedItem;
            }
        };
        var getPopoverInstance = function () {
            var popover = $("#popoverYearSelect").dxPopover('instance');
            return popover;
        };
        $scope.yearSelectClose = function () {
            getPopoverInstance().hide();
        };
        $scope.indexGridColumns = [];
        $scope.indexResultDataList = [];
        $scope.indexResultData = {};
        $scope.enterCheckData = {};
        $scope.enterCheckDetailDataList = [];
        $scope.startPreview = function () {
            var model = $scope.isAddPreview ? angular.copy($scope.createModel.basicModel) : angular.copy($scope.viewEditModel.basicModel)
            if ($scope.selectProjectDto && model) {
                if ($scope.selectProjectDto.haveMappedAccountByPeriod === false) {
                    SweetAlert.warning("请完成科目对应操作，再查看预览结果。");
                    return;
                }
                ;
                if (model.serviceTypeID) {
                    model.serviceTypeID = model.serviceTypeID.join(",");
                }
                //用Array解决参数传递的问题
                var modelList = [];
                modelList.push(model);
                var projectList = [];
                projectList.push($scope.selectProjectDto);
                modelConfigurationService.getModelPreviewResult(modelList, projectList).success(function (data) {
                    if (data.result) {
                        var modalInstance = null;
                        if (model.feature === 1) {
                            //基础信息
                            $scope.indexResultData.code = data.data.code;
                            $scope.indexResultData.name = data.data.name;
                            $scope.indexResultData.description = data.data.description;
                            $scope.indexResultData.monthExplain = data.data.monthExplain;
                            $scope.indexResultData.yearExplain = data.data.yearExplain;
                            $scope.indexResultData.year = $scope.selectProjectDto.year;
                            $scope.indexResultData.yearException = false;
                            $scope.indexResultData.mainValue = '';
                            $scope.indexGridColumns = [];
                            var isShowMonthJudge = false;
                            if (model.monthJudge && model.monthExplain) {
                                isShowMonthJudge = true;
                            }
                            var maxSqence = _.max(data.data.indexAnalysisDetailResults, function (element) {
                                return element.sequence;
                            });
                            $scope.indexResultDataList = [];
                            var searchData = data.data.indexAnalysisDetailResults;
                            $scope.indexGridColumns.push({
                                dataField: "period",
                                caption: "期间",
                                width: "4em",
                                alignment: "center"
                            });
                            for (var i = 1; i <= 13; i++) {
                                var rowObj = {};
                                if (i === 13) {
                                    rowObj.period = "年度";
                                } else {
                                    rowObj.period = i;
                                }
                                for (var m = 1; m <= maxSqence.sequence; m++) {
                                    var cellData = _.first(_.filter(searchData, function (element) {
                                        return (element.period === i && element.sequence === m);
                                    }));
                                    if (cellData) {
                                        if (cellData.displayType === 1 || cellData.displayType === 2) {
                                            rowObj["colmun" + m] = PWC.isNullOrEmpty(cellData.indexResult) ? 0 : parseFloat(cellData.indexResult);
                                        } else if (cellData.displayType === 3) {
                                            rowObj["colmun" + m] = cellData.indexResult === "1" ? '是' : '否';
                                        } else {
                                            rowObj["colmun" + m] = cellData.indexResult;
                                        }
                                        //是否添加月度判断
                                        if (isShowMonthJudge === true && cellData.isMainValue === 1) {
                                            if (i !== 13)
                                                rowObj["monthJudge"] = cellData.isMonExp === 1 ? true : false;
                                            else
                                                rowObj["monthJudge"] = cellData.isYearExp === 1 ? true : false;
                                        }

                                        //动态添加列
                                        if (i === 1) {
                                            var colmun = {};
                                            colmun.dataField = "colmun" + m;
                                            colmun.caption = cellData.indexName;
                                            colmun.width = colmun.caption.length + "em";
                                            if (cellData.displayType === 1) {
                                                colmun.format = {
                                                    type: "fixedPoint",
                                                    precision: 2
                                                };
                                            }
                                            if (cellData.displayType === 2) {
                                                colmun.format = {
                                                    type: "percent",
                                                    precision: 2
                                                };
                                            }
                                            if (cellData.displayType === 3) {
                                                colmun.dataType = "string";

                                            }
                                            if (cellData.isMainValue === 1)
                                                $scope.indexResultData.mainColmunName = cellData.indexName;
                                            $scope.indexGridColumns.push(colmun)
                                        }
                                        //设置基础信息
                                        if (i === 13) {
                                            if (cellData.isMainValue === 1) {
                                                if (cellData.displayType === 1 || cellData.displayType === 2) {
                                                    if (cellData.displayType === 1) {
                                                        $scope.indexResultData.mainValue = PWC.isNullOrEmpty(cellData.indexResult) ?
                                                            PWC.round(0, 2) : PWC.round(cellData.indexResult, 2);
                                                    } else {
                                                        $scope.indexResultData.mainValue = PWC.isNullOrEmpty(cellData.indexResult) ?
                                                            PWC.round(0, 2) + "%" : PWC.round(cellData.indexResult * 100, 2) + "%";
                                                    }

                                                } else {
                                                    $scope.indexResultData.mainValue = cellData.indexResult;
                                                }
                                                $scope.indexResultData.yearException = cellData.isYearExp === 1 ? true : false;
                                            }
                                        }
                                    }
                                }
                                $scope.indexResultDataList.push(rowObj);
                            }
                            //是否添加月度判断
                            if (isShowMonthJudge === true) {
                                var colmunJudge = {};
                                colmunJudge.dataField = "monthJudge";
                                colmunJudge.caption = "异常判断";
                                colmunJudge.dataType = "boolean";
                                colmunJudge.alignment = 'left';
                                colmunJudge.width = "6em";
                                $scope.indexGridColumns.push(colmunJudge)
                            }

                            var visibleFlag = _.reduce($scope.indexGridColumns, function (sum, item) {
                                return sum + parseInt(item.width);
                            }, 0);

                            $scope.indexGridColumns.push({
                                caption: "TRTest",
                                width: "100%",
                                alignment: "center",
                                visible: visibleFlag > 40,
                                visibleIndex: -1
                            });

                            getPopoverInstance().hide();
                            modalInstance = $uibModal.open({
                                animation: false,
                                ariaLabelledBy: 'modal-title',
                                ariaDescribedBy: 'modal-body',
                                templateUrl: 'model-config-manage-result-index-analysis-preview-modal.html',
                                scope: $scope,
                                windowClass: "model-configuration-manage-result-preview-modal index-analysis"
                            });
                        }

                        if (model.feature === 2) {
                            $scope.enterCheckData.code = data.data.code;
                            $scope.enterCheckData.name = data.data.name;
                            $scope.enterCheckData.description = data.data.description;
                            $scope.enterCheckData.year = $scope.selectProjectDto.year;
                            $scope.enterCheckData.doubts = data.data.voucherDoubts;
                            $scope.enterCheckData.debitRelevantAmt = PWC.round(data.data.debitRelevantAmt, 2);
                            $scope.enterCheckData.creditRelevantAmt = PWC.round(data.data.creditRelevantAmt, 2);
                            $scope.enterCheckData.voucherNum = data.data.voucherNum;
                            $scope.enterCheckData.entriesNum = data.data.entriesNum;
                            $scope.enterCheckDetailDataList = data.data.entriesCheckDetailResults;
                            listOrder({
                                "period": "asc",
                                "date": "asc",
                                "group": "asc",
                                "vid": "asc"
                            }, $scope.enterCheckDetailDataList);
                            var index = 1;
                            $scope.enterCheckDetailDataList.forEach(function (row) {
                                row.indexShow = index;
                                //row.debit = PWC.round(row.debit, 2);
                                //row.credit = PWC.round(row.credit, 2);
                                index++;
                            });
                            getPopoverInstance().hide();
                            modalInstance = $uibModal.open({
                                animation: false,
                                ariaLabelledBy: 'modal-title',
                                ariaDescribedBy: 'modal-body',
                                templateUrl: 'model-config-manage-result-entries-check-preview-modal.html',
                                scope: $scope,
                                windowClass: "model-configuration-manage-result-preview-modal entries-check"
                            });
                        }

                        $scope.resultPreviewClose = function () {
                            if (modalInstance == null)
                                return;

                            modalInstance.dismiss('cancel');
                        };
                    } else {
                        SweetAlert.warning("预览计算失败");
                    }
                });

            }
        };

        var getModelConfigurationPermission = function () {

            var list = [];

            var modelManageTemp = constant.adminPermission.systemConfiguration.modelConfiguration;
            list.push(modelManageTemp.queryCode);
            list.push(modelManageTemp.editCode);
            list.push(modelManageTemp.maintainCustomCode);
            list.push(modelManageTemp.maintainModelTreeCode);

            $scope.$root.checkUserPermissionList(list).success(function (data) {
                $scope.hasQueryPermission = data[modelManageTemp.queryCode];
                $scope.hasEditPermission = data[modelManageTemp.editCode];
                $scope.hasMaintainCustomPermission = data[modelManageTemp.maintainCustomCode];
                $scope.hasMaintainModelTreePermission = data[modelManageTemp.maintainModelTreeCode];
            });
        };
        $scope.showOperateLogPop = function () {
            /**
             * 重新设置日志列宽
             */
            var grid = $(".operate-log-wrapper #operateLogGridOptions").dxDataGrid("instance")
            grid.beginUpdate();
            grid.columnOption("originalState", "visible", false);
            grid.columnOption("updateState", "visible", false);
            grid.columnOption("comment", "visible", false);
            grid.columnOption("id", "width", "5em");
            grid.columnOption("createTime", "width", "12em");
            grid.columnOption("operationObject", "width", "12em");
            grid.columnOption("action", "width", "5em");
            grid.columnOption("operationContent", "width", "100%");
            grid.columnOption("operationUser", "width", "10em");
            grid.endUpdate();

            $scope.isShowLog = true;
        };
        (function initializa() {
            $log.debug("ModelConfigurationController.ctor()...");
            $scope.hasQueryPermission = false;
            $scope.hasEditPermission = false;
            $scope.hasMaintainCustomPermission = false;
            $scope.hasMaintainModelTreePermission = false;

            getModelConfigurationPermission();

            $scope.loadModelConfigurationTree = loadModelConfigurationTree();
            $scope.selectModel = selectModel;
            $scope.selectTreeNode = selectTreeNode;
            $scope.editTreeNodeName = editTreeNodeName;
            $scope.editModel = editModel;
            $scope.saveModel = saveModel;
            $scope.saveEditName = saveEditName;
            $scope.cancelEdit = cancelEdit;
            $scope.saveASModel = saveASModel;
            setButtonDisabled(true);
            orgService.getOrgList(0).success(function (data) {
                if ($scope.selectNoOption) {
                    data.splice(0, 0, NoOptions);
                }
                //关键数据配置跳转设置默认选中机构
                if ($scope.organizationId && _.find(data, function (org) {
                    return org.id == $scope.organizationId
                })) {
                    var orgSelected = _.find(data, function (org) {
                        return org.id == $scope.organizationId
                    });
                    $scope.queryParams.organizationName = orgSelected.name;
                    $scope.queryParams.organizationID = orgSelected.id;
                }
                else {
                    $scope.queryParams.organizationName = data[0].name;
                    $scope.queryParams.organizationID = data[0].id;
                }
            });
            //关键数据配置跳转设置默认选中模型
            if ($scope.selectedModelCode) {
                $scope.currentEditOrAddModelCode = $scope.selectedModelCode;
            }
            loadModelConfigurationTree();
            loadIndustryData();
            getStdAccountList();
            //loadStdAccountDatagrid();
            loadKeyValueData();
            getProject();

        })();

        $scope.analysisResultPreviewDataGridOptions = {
            bindingOptions: {
                dataSource: "indexResultDataList",
                columns: "indexGridColumns"
            },
            paging: {
                enabled: false
            },
            selection: {
                mode: "single"
            },
            allowColumnResizing: true,
            sorting: false,
            showBorders: true,
            showRowLines: true,
            onCellPrepared: function (e) {
                if (e.rowType === "data") {
                    if (e.column.dataType === "boolean") {
                        e.cellElement.empty();
                        if (e.value === true)
                            e.cellElement.append($('<span class="horizontal-center"><i class="fa fa-exclamation-circle"></i>异常</span>'));
                    }
                }
            },
            onRowPrepared: function (e) {
                if (e.rowType != 'header' && e.data && e.data.monthJudge && e.data.monthJudge === true)
                    e.rowElement.addClass('entriesHighLight');
            },
            onInitialized: function (e) {
                //var year = "年度";
                //var data = $scope.analysisResultPreviewData;
                //var row = $.grep(data, function (n, i) { return n.month == year });
                //if (row.length > 0)
                //    return;

                //row = {
                //    month: year,
                //    avgValue: data.map(function (x) { return x.avgValue; }).reduce(function (acc, val) { return acc + val; }, 0),
                //    deprecation: data.map(function (x) { return x.deprecation; }).reduce(function (acc, val) { return acc + val; }, 0),
                //    compositePercent: data.map(function (x) { return x.compositePercent; }).reduce(function (acc, val) { return acc + val; }, 0),
                //    exception: data.map(function (x) { return x.exception; }).reduce(function (acc, val) { return acc || val; }, false)
                //}
                //data.push(row);
                //e.component.refresh();
                dxDataGridService.registerAutoSize(e.component);
            }
        };

        $scope.checkResultPreviewDataGridOptions = {
            columns: [
                {
                    dataField: "indexShow",
                    dataType: 'string',
                    caption: "序号",
                    width: 45,
                    alignment: "center"
                },
                {
                    dataField: "period",
                    dataType: 'string',
                    caption: "期间",
                    width: 45,
                    alignment: "center"
                },
                {
                    dataField: "date",
                    dataType: 'date',
                    caption: "凭证日期",
                    width: 90,
                    alignment: "center",
                    format: "yyyy-MM-dd"
                },
                {
                    dataField: "group",
                    dataType: 'string',
                    caption: "凭证类型",
                    width: 90,
                    alignment: "left"
                },
                {
                    dataField: "vid",
                    dataType: 'string',
                    caption: "凭证编号",
                    width: "33%",
                    alignment: "left"
                },
                {
                    dataField: "summary",
                    dataType: 'string',
                    caption: "摘要",
                    width: "34%",
                    alignment: "left"
                },
                {
                    dataField: "acctCode",
                    dataType: 'string',
                    caption: "会计科目",
                    width: "33%",
                    alignment: "left"
                },
                {
                    dataField: "debit",
                    dataType: 'decimal',
                    caption: "借方发生额",
                    format: {
                        type: 'fixedPoint',
                        precision: 2
                    },
                    width: 85,
                    alignment: "right"
                },
                {
                    dataField: "credit",
                    dataType: 'decimal',
                    caption: "贷方发生额",
                    format: {
                        type: 'fixedPoint',
                        precision: 2
                    },
                    width: 85,
                    alignment: "right"
                }
            ],
            bindingOptions: {
                dataSource: "enterCheckDetailDataList"
            },
            paging: {
                enabled: false
            },
            selection: {
                mode: "single"
            },
            scrolling: {
                mode: "virtual",
                preloadEnabled: true
            },
            summary: {
                totalItems: [
                    {
                        column: "debit",
                        displayFormat: "{0}",
                        valueFormat: {
                            type: 'fixedPoint',
                            precision: 2
                        },
                        summaryType: "sum"
                    }, {
                        column: "credit",
                        displayFormat: "{0}",
                        valueFormat: {
                            type: 'fixedPoint',
                            precision: 2
                        },
                        summaryType: "sum"
                    }, {
                        showInColumn: "acctCode",
                        displayFormat: "合计"
                    }
                ]
            },
            allowColumnResizing: true,
            sorting: false,
            showBorders: true,
            showRowLines: true,
            onInitialized: function (e) {
                dxDataGridService.registerAutoSize(e.component);
            },
            onRowPrepared: function (e) {
                if (e.rowType != 'header' && e.data && e.data.isDoubt === 1)
                    e.rowElement.addClass('entriesHighLight');
            }
        };
    }
]);
