citModule.controller('citModelAnalysisController', ['$scope', '$translate', '$uibModal', 'vatSessionService', 'citSessionService', 'modelConfigurationService', '$compile', '$timeout', '$q', 'SweetAlert', 'enums',
    function ($scope, $translate, $uibModal, vatSessionService, citSessionService, modelConfigurationService, $compile, $timeout, $q, SweetAlert, enums) {
        'use strict';

        //由 project 决定的在本页面中固定的变量
        $scope.industryId = vatSessionService.project.industryID;
        $scope.organizationId = vatSessionService.project.organizationID;
        $scope.projectId = vatSessionService.project.id;
        $scope.projectYear = vatSessionService.project.year;
        $scope.serviceTypeId = vatSessionService.project.serviceTypeID;

        /************* ViewModels BEGIN *************/
        $scope.logOptions = {
            isShown: false,
            period: citSessionService.month,
            moduleId: "26"  //VAT 中的 Id
        };

        $scope.modelType = 1;
        $scope.periodFrom = 1;
        $scope.periodTo = 12;
        $scope.modelTreeList = [];

        $scope.selectCategoryId = null;

        $scope.resizableOptions = {
                handles: "right",
                Width: 250,
                minWidth: 200,
                maxWidth: 350
        };

        $scope.dirtyDataProcess = {
            hasCheckedFinished: false,
            intervalId: null,
            progressBarValue: 0,
            processBarMax: 100,
            processBarModalInstance: null,
            progressBarOptions: {
                min: 0,
                width: "90%",
                bindingOptions: {
                    value: 'dirtyDataProcess.progressBarValue',
                    max: 'dirtyDataProcess.processBarMax'
                },
                statusFormat: function (value) {
                    return "计算进度: " + (value * 100).toFixed(0) + "%";
                },
                onComplete: function (e) {
                    $scope.dirtyDataProcess.processBarModalInstance.dismiss({$value: 'cancel'});
                    $scope.dirtyDataProcess.processBarModalInstance = null;

                    clearInterval($scope.dirtyDataProcess.intervalId);
                    $scope.dirtyDataProcess.intervalId = null;

                    loadTreeData();
            }
                }
        };
        /************* ViewModels END *************/

        var loadTreeData = function () {
            var apiFunc = modelConfigurationService.getModelCategory;
            apiFunc($scope.organizationId, $scope.industryId, $scope.serviceTypeId).success(function (data) {
                var dic = {};
                for (var ix = 0; ix < data.length; ix++) {
                    var item = data[ix];
                    dic[item.categoryID] = {
                        id: item.modelID,
                        title: item.modelName,
                        isCategory: item.isCategory,
                        questionCount: item.questionCount,
                        totalCount: item.modelTotalCount,
                        categoryId: item.categoryID,
                        parentID: item.parentID,
                        modelType: item.modelType,
                        parentNode: null,
                        sort: item.sort,
                        data: item,
                        feature: item.feature,
                        nodes: [],
                    };
                }

                //查找每个节点是否存在父节点,并维护父节点与本节点的关系
                for (var key in dic) {
                    if (!dic.hasOwnProperty(key))
                        continue;
                    var item = dic[key];
                    var parentId = item.parentID;
                    if (!parentId || !dic.hasOwnProperty(parentId))
                        continue;

                    var p = dic[parentId];
                    if (!p && p === item)
                        continue;

                    item.parentNode = p;
                    p.nodes.push(item);
                }

                var srcData = [];
                for (var key in dic) {
                    if (!dic.hasOwnProperty(key))
                        continue;

                    var item = dic[key];
                    if (!item || item.parentNode)
                        continue;

                    srcData.push(item);
                }
                $scope.modelTreeList = srcData;
                var mNode = null;
                //完成树形控件初始化后,默认加载第一个模型不为空的节点                      
                for (var i = 0; i < $scope.modelTreeList.length; i++) {
                    mNode = getFirstNodeInTheTree($scope.modelTreeList[i]);
                    if (mNode) {
                        $scope.nodeSelected(mNode);
                        break;
                    }
                }
            });
        };

        var getFirstNodeInTheTree = function (node) {
            if (!node)
                return null;

            if (node.totalCount>0)
                return node;

            if (node.nodes.length>0) {

                for (var i = 0; i < node.nodes.length; i++) {
                    var subNode = node.nodes[i];
                    var mNode = getFirstNodeInTheTree(subNode);
                    if (mNode)
                        return mNode;
                }
            }

            return null;
        };

        var getModelAnalysisPermission = function () {
            var list = [];
            var modelManageTemp = constant.citPermission.dataAnalysis.modelAnalysis;
            list.push(modelManageTemp.queryCode);
            list.push(modelManageTemp.filterCode);
            $scope.hasQueryPermission = false;
            $scope.hasFilterIssuePermission = false;
            $scope.$root.checkUserOrganizationPermissionList(list).success(function (data) {
                $scope.hasQueryPermission = data[modelManageTemp.queryCode];
                $scope.hasFilterIssuePermission = data[modelManageTemp.filterCode];
            });
        };

        $scope.nodeSelected = function (node) {
            $scope.selectCategoryId = node.categoryId;
            $scope.modelType = node.feature;
        };

        (function () {
            getModelAnalysisPermission();

            //检查是否有脏数据
            modelConfigurationService.projectHasDirtyData($scope.projectId, $scope.serviceTypeId).success(function (result) {
                if (!result) {
                    loadTreeData();
                    return;
                    }

                SweetAlert.swal({
                    title: $translate.instant('OrganizationsHasDirtyData'),
                    text: '',
                    type: "warning",
                    showCancelButton: true,
                    confirmButtonColor: "#DD6B55",
                    confirmButtonText: $translate.instant('Confirm'),
                    cancelButtonText: $translate.instant('Cancel'),
                    closeOnConfirm: true
                }, function (isConfirm) {
                    if (!isConfirm) {
                        loadTreeData();
                        return;
            }

                    $scope.dirtyDataProcess.processBarModalInstance = $uibModal.open({
                    ariaLabelledBy: 'modal-title',
                    ariaDescribedBy: 'modal-body',
                        templateUrl: "cit-model-analysis-dirty-data-process-bar.html",
                        backdrop: 'static',
                    scope: $scope,
                        windowClass: "cit-model-analysis-dirty-data-process-bar"
                });

                    $scope.dirtyDataProcess.intervalId = setInterval(function () {
                        $scope.dirtyDataProcess.progressBarValue++;
                        if (!$scope.dirtyDataProcess.hasCheckedFinished) {
                            $scope.dirtyDataProcess.processBarMax++;
                        }
            }, 100);

                    //重新计算模型数据
                    modelConfigurationService.updateProjectModelDirtyDataSingleCard($scope.projectId, $scope.serviceTypeId).success(function () {
                        $scope.dirtyDataProcess.hasCheckedFinished = true;
                });
            });
            });
        })();
    }
]);