﻿citModule.controller('citDocumentListTreeController', ['$scope', '$rootScope', '$log', '$q', '$timeout', '$filter', 'loginContext',
    '$translate', '$location', 'vatSessionService', 'vatReportService', '$state', '$stateParams', 'enums',
    function ($scope, $rootScope, $log, $q, $timeout, $filter, loginContext, $translate, $location, vatSessionService,
        vatReportService, $state, $stateParams, enums) {
        'use strict';
        $log.debug('citDocumentListTreeController.ctor()...');

        var getNodeLeavel = function (treeNode) {
            if (!(treeNode && treeNode.hasOwnProperty("parent")))
                throw "参数 treeNode 无效";

            if (null === treeNode.parent)
                return 1;

            return getNodeLeavel(treeNode.parent) + 1;
        }

        //var reportCode = {
        //    FinanceLostReport: 'CIT.CL004',
        //    CurrencyLost: 'CIT.CL00401',
        //    NonCurrencyLost: 'CIT.CL00402',
        //    InvestmentLost: 'CIT.CL00403',
        //    OtherLost: 'CIT.CL00404',
        //    ResearchCost: 'CIT.CL022'
        //};

        var taxTypeCode = {
            FinanceLost: '1100', //资产损失
            ResearchCost: '1250', //研发费用
            TaxAudit: '7050' //税务稽查
        };

        $scope.menuTreeOption = {
            bindingOptions: {
                dataSource: 'menuTreeDataSource'
            },
            dataStructure: "plain",
            selectByClick: true,
            selectionMode: "single",
            height: "100%",
            width: "100%",
            keyExpr: "id",
            expandAllEnabled: true,
            parentIdExpr: "parentId",
            itemTemplate: 'itemTemplate',
            onItemRendered: function (e) {
                if (e && e.node) {
                    if ((_.isEmpty($scope.reportParam) || _.isEmpty($scope.reportParam.id))
                        && !_.isEmpty(e.node.reportId)) {
                        $scope.reportParam = {
                            id: e.node.reportId,
                            templateid: e.node.templateId,
                            name: e.node.text
                        };
                    }

                    var leavel = getNodeLeavel(e.node.internalFields.publicNode);
                    e.itemElement.attr('title', e.node.text);
                    e.itemData.limitedText = $filter('limitString')(e.node.text, 32 - leavel * 2);
                    $scope.treeEle = e.element;
                }
            },
            onItemClick: function (e) {
                if (e && e.itemData) {
                    $scope.hasDocument = false;
                    $scope.selectedReportCode = e.itemData.code;
                    $scope.selectedReportItem = e.itemData;
                    $(e.itemElement).parent().find('.dx-treeview-toggle-item-visibility').click();
                    $timeout(function () {
                        if (e && e.itemData && e.itemData.templateId) {
                            $state.go('cit.importData.documentList.reportView', {
                                id: e.itemData.reportId,
                                templateid: e.itemData.templateId,
                                name: e.itemData.text
                            });
                        }
                    }, 300);

                    //$('#dfiframe').attr("src", baseDFFilePath + 'WorkingPaper1.html' + '?t_' + Math.random() + '&dataFileName=' + e.itemData.dataFileName);
                }
            }
        };

        var loadTemplateMenu = function () {
            var projectID = vatSessionService.project.id;
            $scope.reportParam = _.isEmpty($stateParams.id || $stateParams.docId) ? null : {
                id: $stateParams.id || $stateParams.docId,
                templateid: $stateParams.templateid || $stateParams.docTemplateId,
                name: $stateParams.name || $stateParams.docName
            };

            return vatReportService.citTryGenerateDocList(projectID, enums.serviceType.CIT, vatSessionService.logUser.ID).then(function (data) {
                if (data && data.data&& data.data.result) {
                    return $q.when(data.data);
                }

                return $q.reject();
            }).then(function () {
                return vatReportService.citGetTemplateTree(projectID).success(function (data) {
                    if (!data || !data.data) {
                        return;
                    }

                    var data = _.filter(data.data, function (item) {
                        return item.reportType === 4;
                    });

                    _.each(data, function (item, index) {
                        item.expanded = true;

                        var requiredItems = ['CIT.CL001', 'CIT.CL011', 'CIT.CL021', 'CIT.CL025', 'CIT.CL01502', 'CIT.CL01504', 'CIT.CL01506',
                        'CIT.CL01507', 'CIT.CL018', 'CIT.CL004', 'CIT.CL00401', 'CIT.CL00402', 'CIT.CL00403', 'CIT.CL00404', 'CIT.CL006',
                        'CIT.CL007', 'CIT.CL008', 'CIT.CL012', 'CIT.CL027', 'CIT.CL022', 'CIT.CL020', 'CIT.CL002', 'CIT.CL00201', 'CIT.CL003',
                        'CIT.PL001', 'CIT.CL010'];
                        if (requiredItems.indexOf(item.code) >= 0) {
                            item.isRequired = true;
                        }

                        item.text = item.code.replace('CIT.', '') + item.text;
                        //if (!item.parentId && item.reportType == 4) {
                            item.parentId = 'root';
                        //}
                    });

                    data.push({
                        id: 'root',
                        parentId: null,
                        text: '问卷调查项',
                        expanded: true,
                        reportId: null,
                        templateGrouptId: null
                    });
                    //data.push.apply(data, _.map(constant.citDocumentTypeList, function (x) {
                    //    return {
                    //        id: x.value.toString(),
                    //        parentId: 'root',
                    //        text: x.name,
                    //        expanded: false,
                    //        reportId: null,
                    //        templateGrouptId: null
                    //    };
                    //}));
                    $scope.menuTreeDataSource = data;

                    $timeout(function () {
                        if (!_.isEmpty($scope.reportParam)) {
                            if ($scope.treeEle) {
                                $scope.treeEle.dxTreeView("selectItem", $scope.reportParam.templateid);
                            }

                            $state.go('cit.importData.documentList.reportView', $scope.reportParam);
                        }
                    }, 300);
                });
            });
        };

        (function initialize() {
            loadTemplateMenu();
        })();
    }
]);