﻿commonModule.controller('reportCheckListModalController', ['$scope', '$log', '$translate', '$location', "$compile", '$timeout', '$interval', '$filter',
    'SweetAlert', '$uibModal', 'enums', 'checkListService', 'vatSessionService',
    function ($scope, $log, $translate, $location, $compile, $timeout, $interval, $filter, SweetAlert, $uibModal, enums, checkListService, vatSessionService) {

        $scope.stagesColors = {
            "0": {
                color: "white",
                icon: "white",
                text: $translate.instant('CheckListOTAll')
            },
            "1": {
                //无需填报
                color: "grey",
                icon: "fa-circle",
                text: $translate.instant('CheckListOTNotRequired')
            },
            "2": {
                //未填报
                color: "#c63",
                icon: "fa-circle-thin",
                text: $translate.instant('CheckListOTBlank')
            },
            "3": {
                //填报中
                color: "#c63",
                icon: "fa-adjust",
                text: $translate.instant('CheckListOTDraft')
            },
            "4": {
                //已提交
                color: "#40A010",
                icon: "fa-circle",
                text: $translate.instant('CheckListOTSubmitted')
            },
            "5": {
                //已完成
                color: "#3c763d",
                icon: "fa-check-circle",
                text: $translate.instant('CheckListOTCompleted')
            }
        };

        $scope.sheetList = [];
        $scope.modalInstance = null;

        $scope.filter = function (event, dataStages) {
            $scope.sheetList = $.grep($scope.listDataSource, function (x) {
                return (!dataStages) || x.reportStatus == dataStages;
            });

            var eventTarget = event.target;
            $(eventTarget).closest(".summary").find("button").removeClass("activated");
            $(eventTarget).addClass("activated");
        };

        $scope.open = function () {
            $.each($scope.listDataSource, function (i, x) {
                if (!!x.lastModifiedTime)
                    x.lastModifiedTime = new Date(x.lastModifiedTime);

                x.reportDisplayName = [x.reportCode.split(".").reverse().shift(), x.reportName].join("_");
                x.alertNumber = _.reduce(x.cellAlerts, function (memo, list) {
                    return memo + (list || []).length
                }, 0);

                var status = x.reportStatus.toString();
                x.statusIcon = $scope.stagesColors[status].icon;
                x.statusText = $scope.stagesColors[status].text;
                x.statusColor = {color: $scope.stagesColors[status].color};
                _.each(x.reportComments, function (item) {
                    if (item.createTime)
                        item.createTime = new Date(item.createTime);
                });


                //TODO:暂时简单的处理为有无异常,异常是否已处理就好,后期需要重新实现
                x.hasError = _.reduce(x.exceptions, function (memo, list) {
                    return memo + (list || []).length
                }, 0) > 0;
                x.hasErrorFixed = _.reduce(_.union(_.values(x.exceptions)), function (memo, x) {
                    return memo && x.hasOwnProperty("status") ? x.status == 1 : true
                }, true);
            });

            $scope.sheetList = $scope.listDataSource;

            $scope.modalInstance = $uibModal.open({
                animation: false,
                ariaLabelledBy: 'modal-title',
                ariaDescribedBy: 'modal-body',
                backdrop: "static",
                templateUrl: '/app/common/controls/check-list/check-list.html',
                scope: $scope,
                windowClass: "report-check-list-modal"
            });
        };

        $scope.cancel = function () {
            $scope.modalInstance.dismiss('cancel');
            $scope.modalInstance = null;
            $scope.isOpen = false;
        };

        $scope.close = function () {
            $scope.modalInstance.close();
            $scope.modalInstance = null;
            $scope.isOpen = false;
        };

        $scope.$watch('isOpen', function (newValue, oldValue) {
            if (newValue && !$scope.modalInstance)
                $scope.open();

            if (!newValue && $scope.modalInstance)
                $scope.close();
        });

        /*
            显示"模型异常","预警详情"和"校验异常"详细信息
        */
        $scope.modelExceptionModal = {
            title: "",
            modalInstance: null,
            dataSource: [],
            modelAnalysisColumns: [
                {
                    alignment: "right",
                    dataField: "sequence",
                    dataType: "string",
                    caption: null,
                    width: "2em"
                },
                {
                    alignment: "left",
                    dataField: "modelName",
                    dataType: "string",
                    caption: "模型名称",
                    width: "16em"
                },
                {
                    alignment: "right",
                    dataField: "amount",
                    dataType: "number",
                    caption: "金额",
                    width: "9em",
                    format: {
                        type: 'currency',
                        precision: 2
                    }
                },
                {
                    alignment: "left",
                    dataField: "advice",
                    dataType: "string",
                    caption: "建议"
                },
                {
                    alignment: "center",
                    dataField: "status",
                    dataType: "string",
                    caption: "状态",
                    width: "5em"
                }
            ],
            riskWarningColumns: [
                {
                    alignment: "right",
                    dataField: "sequence",
                    dataType: "string",
                    caption: null,
                    width: "2em"
                },
                {
                    alignment: "left",
                    dataField: "riskName",
                    dataType: "string",
                    caption: "预警明细",
                    width: "16em"
                },
                {
                    alignment: "left",
                    dataField: "advice",
                    dataType: "string",
                    caption: "建议"
                },
                {
                    alignment: "center",
                    dataField: "operation",
                    dataType: "string",
                    caption: "操作",
                    width: "6em"
                }
            ],
            dxDataGridInstance: null,
            dxDataGridOptions: {
                bindingOptions: {
                    dataSource: "modelExceptionModal.dataSource"
                },
                allowColumnResizing: true,
                columns: [],
                editing: {
                    allowAdding: false,
                    allowDeleting: false,
                    allowUpdating: false
                },
                focusStateEnabled: true,
                hoverStateEnabled: true,
                keyExpr: null,
                loadPanel: {
                    enabled: true
                },
                noDataText: $translate.instant('NoDataText'),
                paging: {
                    enabled: false
                },
                rowAlternationEnabled: false, //单双行颜色
                scrolling: {
                    mode: "infinite"
                },
                selection: {
                    allowSelectAll: false,
                    mode: "single"
                },
                showBorders: true,
                showColumnLines: true,
                showRowLines: true,
                sorting: {
                    mode: "none"
                },
                onInitialized: function (e) {
                    $scope.modelExcptionModal.dxDataGridInstance = e.component;
                },
                onCellPrepared: function (e) {
                    if (e.rowType !== "data" || e.column.dataField !== "operation")
                        return;

                    var btn = $("<button type='button' ng-click='modelExceptionModal.confirmRiskWarning($event)'>确认</button>");
                    e.cellElement.html($compile(btn)($scope));
                }
            },
            open: function (item) {
                if (!$.isArray(item.exceptions) || item.exceptions.length < 1)
                    return;

                var that = $scope.modelExcptionModal;
                var columns, templateUrl, title, windowClass;
                if ($scope.isModelAnalysis) {
                    columns = that.modelAnalysisColumns;
                    templateUrl = "/app/common/controls/check-list/check-list-model-analysis-modal.html";
                    title = "模型异常详情";
                    windowClass = "check-list-model-analysis-modal";
                }
                else {
                    columns = that.riskWarningColumns;
                    templateUrl = "/app/common/controls/check-list/check-list-risk-warning-modal.html";
                    title = "预警详情";
                    windowClass = "check-list-risk-warning-modal";
                }

                that.dxDataGridOptions.columns = columns;
                that.dataSource = item.exceptions;
                that.title = title;

                $scope.modelExcptionModal.modalInstance = $uibModal.open({
                    animation: false,
                    ariaLabelledBy: 'modal-title',
                    ariaDescribedBy: 'modal-body',
                    backdrop: "static",
                    templateUrl: templateUrl,
                    scope: $scope,
                    windowClass: windowClass
                });
            },
            close: function () {
                $scope.modelExcptionModal.modalInstance.close();
                $scope.modelExcptionModal.modalInstance = null;
            },
            confirmRiskWarning: function (item) {
                //TODO:自定义逻辑
                console.log(item);
            }
        };

        $scope.reportCommentsModal = {
            checkListItem: null,
            modalInstance: null,
            dataSource: [],
            dxDataGridInstance: null,
            dxDataGridOptions: {
                bindingOptions: {
                    dataSource: "reportCommentsModal.dataSource"
                },
                allowColumnResizing: true,
                columns: [
                    {
                        alignment: "left",
                        dataField: "content",
                        dataType: "string",
                        caption: "备注"
                    },
                    {
                        alignment: "left",
                        dataField: "createBy",
                        dataType: "string",
                        caption: "备注人",
                        width: "6em"
                    },
                    {
                        alignment: "center",
                        dataField: "createTime",
                        dataType: "datetime",
                        caption: "备注时间",
                        width: "11em",
                        format: "yyyy-MM-dd HH:mm:ss"
                    },
                    {
                        alignment: "center",
                        dataField: "operation",
                        dataType: "string",
                        caption: "",
                        width: "6em"
                    }
                ],
                editing: {
                    mode: "popup",
                    form: {
                        items: []
                    },
                    popup: {
                        title: "添加备注",
                        height: "260px",
                        width: "450px",
                        showTitle: true,
                        position: {
                            my: "center",
                            at: "center",
                            of: window
                        },
                        onContentReady: function (e) {
                            var textArea = $("<div/>").dxTextArea({height: "100%"});
                            e.component.content().html(textArea).css("padding-bottom", 0);
                            e.component.bottomToolbar().find("[dx-button]:eq(0)").data("dxButton").option("onClick", function () {
                                var text = textArea.data("dxTextArea").option("value");
                                $scope.reportCommentsModal.addComment(text).then(function (rsponse) {
                                    var rst = rsponse.data;
                                    if (!rst.result)
                                        return;

                                    rst.data = {
                                        id: rst.data,
                                        content: text,
                                        createBy: vatSessionService.logUser.UserName,
                                        createTime: new Date()
                                    };
                                    $scope.reportCommentsModal.dataSource.push(rst.data);
                                    e.component.hide();
                                });
                            });
                        }
                    }
                },
                focusStateEnabled: true,
                hoverStateEnabled: true,
                keyExpr: null,
                loadPanel: {
                    enabled: false
                },
                noDataText: $translate.instant('NoDataText'),
                paging: {
                    enabled: false
                },
                rowAlternationEnabled: false, //单双行颜色
                scrolling: {
                    mode: "infinite"
                },
                selection: {
                    allowSelectAll: false,
                    mode: "single"
                },
                showBorders: true,
                showColumnLines: true,
                showRowLines: true,
                sorting: {
                    mode: "none"
                },
                onInitialized: function (e) {
                    $scope.reportCommentsModal.dxDataGridInstance = e.component;
                },
                onCellPrepared: function (e) {
                    if (e.rowType !== "data" || e.column.dataField !== "operation")
                        return;

                    var btn = $("<button type='button' class='operation' ng-click='reportCommentsModal.deleteComment($event)'>删除</button>");
                    e.cellElement.html($compile(btn)($scope));
                }
            },
            open: function (item) {
                if (!item.reportId)
                    return;

                var that = $scope.reportCommentsModal;
                that.dataSource = item.reportComments;
                that.checkListItem = item;
                that.modalInstance = $uibModal.open({
                    animation: false,
                    ariaLabelledBy: 'modal-title',
                    ariaDescribedBy: 'modal-body',
                    backdrop: "static",
                    templateUrl: "/app/common/controls/check-list/check-list-report-comments-modal.html",
                    scope: $scope,
                    windowClass: "check-list-report-comments-modal"
                });
            },
            close: function () {
                $scope.reportCommentsModal.modalInstance.close();
                $scope.reportCommentsModal.modalInstance = null;
            },
            popEditForm: function () {
                $scope.reportCommentsModal.dxDataGridInstance.addRow();
            },
            addComment: function (content) {
                return checkListService.addReportComment($scope.reportCommentsModal.checkListItem.reportId, content);
            },
            deleteComment: function (e) {
                var options = $(e.target.closest("tr")).data("options");
                checkListService.removeReportComment(options.data.id).then(function (response) {
                    if (response.data && response.data.result) {
                        var index = $scope.reportCommentsModal.dataSource.findIndex(function (x) {
                            return x.id === options.data.id;
                        });
                        if (index !== -1) {
                            $scope.reportCommentsModal.dataSource.splice(index, 1);
                        }
                    }
                });
            }
        }
    }
]);
