﻿// Formula translator, show translated formula info
commonModule
    .factory('formulaTranslateUtil', ['$log', '$translate', 'enums',
    function ($log, $translate, enums) {
        'use strict';

        var getParamsFromStr = function (formulaParams) {
            var paramStrs = [];
            if (!_.isEmpty(formulaParams)) {
                formulaParams = formulaParams.trim();
                if (formulaParams.length <= 1 || formulaParams.substring(formulaParams.length - 1) !== ')') {
                    return paramStrs;
                }

                formulaParams = formulaParams.substring(0, formulaParams.length - 1);
                if (formulaParams.indexOf('(') >= 0 || formulaParams.indexOf(')') >= 0) {
                    return paramStrs;
                }

                var params = formulaParams.split(',');
                paramStrs = _.map(params, function (p) {
                    var param = { value: p.trim(), dataType: enums.paramDataType.undefined };
                    if (param.value.length > 0) {
                        var startChar = param.value.substring(0, 1);
                        var endChar = param.value.substring(param.value.length - 1);

                        try {
                            param.value = eval(param.value);
                            param.dataType = typeof (param.value) === 'string' ? enums.paramDataType.string
                                : (typeof (param.value) === 'number' ? enums.paramDataType.number : enums.paramDataType.undefined);
                        }
                        catch (ex) {
                            param.value = undefined;
                            param.dataType = enums.paramDataType.undefined;
                        }
                    }

                    return param;
                });
            }

            return paramStrs;
        };

        var translateFormula = function (formulaList, formulaName, formulaParams, includeOptional) {
            var rtn = '';
            if (!_.isEmpty(formulaList) && !_.isEmpty(formulaName)) {
                var formula = _.findWhere(formulaList, { code: formulaName });
                if (formula) {
                    rtn = formula.name + '(';
                    if(formulaName=='PSUM') return rtn +formulaParams;
                    var paramInfo = getParamsFromStr(formulaParams); // 括号内输入的参数
                    var knownParams = angular.copy(formula.params || []); // 公式已知的参数（数据库中与该公式关联的参数）
                    knownParams.forEach(function (p) {
                        if (paramInfo[p.index] && (p.name === 'ColumnNum' || paramInfo[p.index].dataType === p.paramDataType
                            || p.paramDataType === enums.paramDataType.accountCode && paramInfo[p.index].dataType === enums.paramDataType.string)) {
                            p.value = paramInfo[p.index].value;
                            if (!_.isString(p.value)) {
                                p.value = p.value.toString();
                            }
                        }
                        else {
                            p.value = null;
                        }
                    });

                    var knownParamIndexList = _.pluck(knownParams, 'index');
                    var extraParams = _.filter(paramInfo, function (p) {
                        return knownParamIndexList.indexOf(paramInfo.indexOf(p)) < 0;
                    });

                    _.each(extraParams, function (p) { // 将输入的参数合并入knownParams
                        knownParams.push({
                            index: paramInfo.indexOf(p),
                            value: p.value,
                            paramDataType: p.dataType,
                            paramType: enums.paramType.input
                        });
                    });

                    if (includeOptional == null || includeOptional == undefined) {
                        includeOptional = 'true';
                    }

                    rtn += getParamStrFromParamList(knownParams, formula.requiredParamNum, includeOptional);
                }
            }

            return rtn;
        };

        var getParamStrFromParamList = function (params, requiredParamNum, includeOptional) {
            var rtn = '';
            if (!_.isEmpty(params)) {
                for (var i = 0; i <= params[params.length - 1].index; i++) {
                    var param = _.findWhere(params, { index: i });
                    if (includeOptional === 'true' || i < requiredParamNum
                        || param.value !== undefined && param.value !== null && param.value !== '') {
                        rtn += $translate.instant(getParamDisplayName(param, includeOptional));
                        rtn += ',';
                    }
                }

                rtn = rtn.substring(0, rtn.length - 1);
            }
            rtn = rtn + ')';

            return rtn;
        };

        var getParamDisplayName = function (param, includeOptional) {
            if (!_.isEmpty(param)) {
                var paramStr = '';
                if (param.paramType === enums.paramType.input
                    || param.paramType === enums.paramType.dropdownNumber) {
                    if (param.value === undefined || param.value === null) {
                        if (includeOptional === 'true' && !_.isEmpty(param.name)) {
                            paramStr = param.name;
                        }
                    }
                    else {
                        // 如果输入型参数有options,这些options是特殊情况,按option.name显示；
                        // options里找不到再用displayFormat来格式化
                        var opt;
                        if (_.isEmpty(param.options)) {
                            opt = null;
                        }
                        else {
                            if (param.paramDataType === enums.paramDataType.string) {
                                opt = _.find(param.options, function (p) {
                                    return p.value.toLowerCase() === param.value.toLowerCase();
                                });
                            }
                            else if (param.paramDataType === enums.paramDataType.number) {
                                opt = _.find(param.options, function (p) {
                                    return p.value === param.value.toString();
                                });
                            }
                        }

                        if (param.name === 'ColumnNum') {
                            var val = param.value;
                            if (!isNaN(param.value)) {
                                val = PWC.numToLetter(val - 1);
                            }

                            if (!_.isEmpty(param.displayFormat)) {
                                paramStr = param.displayFormat.replace(/\{0\}/g, val);
                            }
                            else {
                                paramStr = '"' + param + '"';
                            }
                        }
                        else if (!_.isEmpty(opt)) {
                            if (param.paramDataType === enums.paramDataType.string) {
                                paramStr = '"' + opt.name + '"';
                            }
                            else if (param.paramDataType === enums.paramDataType.number) {
                                paramStr = opt.name;
                            }
                        }
                        else if (!_.isEmpty(param.displayFormat)) {
                            var val = param.value;
                            paramStr = param.displayFormat.replace(/\{0\}/g, val);
                        }
                        else {
                            if (param.paramDataType === enums.paramDataType.string) {
                                paramStr = '"' + param.value + '"';
                            }
                            else if (param.paramDataType === enums.paramDataType.number) {
                                var val = param.value;
                                paramStr = val.toString();
                            }
                        }
                    }
                }
                else if (param.paramType === enums.paramType.dropdown
                    || param.paramType === enums.paramType.accountCode) {
                    if (param.value === undefined || param.value === null) {
                        if (includeOptional === 'true' && !_.isEmpty(param.name)) {
                            paramStr = param.name;
                        }
                    }
                    else if (!_.isEmpty(param.options)) {
                        // 如果列表型参数有options,这些options是所有允许的值,将option值格式化后显示；
                        // 格式化字符串中,{0}: option.value, {1}: option.name, {2}: option.shortName
                        var opt;
                        if (param.paramDataType === enums.paramDataType.string) {
                            opt = _.find(param.options, function (p) {
                                return p.value.toLowerCase() === param.value.toLowerCase();
                            });
                        }
                        else if (param.paramDataType === enums.paramDataType.number) {
                            opt = _.find(param.options, function (p) {
                                return p.value === param.value.toString();
                            });
                        }

                        if (!_.isEmpty(opt)) {
                            if (!_.isEmpty(param.displayFormat)) {
                                paramStr = param.displayFormat
                                    .replace(/\{0\}/g, opt.value)
                                    .replace(/\{1\}/g, opt.name)
                                    .replace(/\{2\}/g, opt.shortName);
                            }
                            else {
                                if (param.paramDataType === enums.paramDataType.string) {
                                    paramStr = '"' + opt.value + '"';
                                }
                                else if (param.paramDataType === enums.paramDataType.number) {
                                    paramStr = opt.value;
                                }
                            }
                        }
                        else if (param.paramDataType === enums.paramDataType.string) {
                            paramStr = '"' + param.value + '"';
                        }
                        else if (param.paramDataType === enums.paramDataType.number) {
                            paramStr = param.value.toString();
                        }
                        //else if (!_.isEmpty(param.name)) {
                        //    paramStr = param.name;
                        //}
                    }
                    else if (!_.isEmpty(param.name)) {
                        paramStr = param.name;
                    }
                }

                return paramStr;
            }

            return '';
        };

        return {
            getParamsFromStr: getParamsFromStr,
            getParamDisplayName: getParamDisplayName,
            getParamStrFromParamList: getParamStrFromParamList,
            translateFormula: translateFormula
        };
    }])
    .directive('formulaTranslator', ['$log', '$translate', 'enums', 'formulaTranslateUtil',
    function ($log, $translate, enums, formulaTranslateUtil) {
        'use strict';
        $log.debug('formulaTranslator.ctor()...');

        return {
            restrict: 'E',
            templateUrl: '/app/common/controls/formula-translator/formula-translator.html' + '?_=' + Math.random(),
            replace: true,
            scope: {
                hideMode: '=?', // hide or notVisible
                formulaList: '=*',
                keyValueList: '=*',
                formulaName: '=?', // 公式名，必须传入formulaExp,或是同时传入formulaName & formulaParams
                formulaParams: '=?', // 公式参数
                formulaExp: '=?', // 公式表达式，包含了公式名与参数，支持同时包含多个公式
                accountDataSource: '=',
                includeOptional: '@?', // Default is true. 如果为false,可选参数未输入则不显示；如果为true,可选参数未输入需显示
                translatorFocus: '&',
                translatorBlur: '&'
            },
            link: function (scope, element, attr) {
                var splitExp = function (exp) {
                    var opExp = /[+\-*/]+/g;
                    var leftExp = /\(/g;
                    var rightExp = /\)/g;
                    var strs = [];
                    var rtn = [];
                    if (_.isEmpty(exp)) {
                        return rtn;
                    }

                    // B1 - (1 + BB("VAT001",1,1,0))*0.07 split to ['B1', '-', '(1', '+', 'BB("VAT001",1,1,0)', ')', '*', '0.07']
                    var lastIndex = 0;
                    var result = null;
                    while ((result = opExp.exec(exp)) != null) {
                        var lastExp = exp.substring(lastIndex, opExp.lastIndex - result[0].length);
                        if (!_.isEmpty(lastExp)) {
                            strs.push(lastExp);
                        }

                        strs.push(result[0]);

                        lastIndex = opExp.lastIndex;
                    }

                    strs.push(exp.substring(lastIndex));

                    var inFormula = false;
                    var diff = 0;
                    var startBlockIndex = 0;
                    var toHandleBlocks = [];
                    for (var i = 0; i < strs.length; i++) {
                        strs[i] = strs[i].trim();
                        var leftTime = strs[i].match(leftExp);
                        leftTime = leftTime ? leftTime.length : 0;
                        var rightTime = strs[i].match(rightExp);
                        rightTime = rightTime ? rightTime.length : 0;
                        if (leftTime !== rightTime) {
                            if (diff === 0) {
                                diff = leftTime - rightTime;
                                startBlockIndex = i;
                                if (strs[i][0] !== '(') {
                                    inFormula = true;
                                }
                                else {
                                    inFormula = false;
                                }
                            }
                            else {
                                diff = diff + leftTime - rightTime;
                                if (diff === 0) {
                                    toHandleBlocks.push({
                                        start: startBlockIndex,
                                        end: i,
                                        inFormula: inFormula
                                    });
                                }
                            }
                        }
                    }

                    for (var i = toHandleBlocks.length - 1; i >= 0; i--) {
                        // 如果括号不平衡的块不是formula，需要将括号与内容分离，并需递归处理内容
                        if (toHandleBlocks[i].inFormula) {
                            var newBlock = '';
                            for (var idx = toHandleBlocks[i].start; idx <= toHandleBlocks[i].end; idx++) {
                                newBlock = newBlock + strs[idx];
                            }

                            strs.splice(toHandleBlocks[i].start, toHandleBlocks[i].end - toHandleBlocks[i].start + 1, newBlock);
                        }
                        else {
                            var subBlock = '';
                            var lastBlock = '';

                            for (var j = toHandleBlocks[i].start; j <= toHandleBlocks[i].end; j++) {
                                var tmp = strs[j];
                                if (j === toHandleBlocks[i].start) {
                                    tmp = tmp.substring(1);
                                }
                                else if (j === toHandleBlocks[i].end) {
                                    var lastIndex = tmp.lastIndexOf(')');
                                    tmp = tmp.substring(0, lastIndex);
                                    lastBlock = tmp.substring(lastIndex);
                                }

                                subBlock = subBlock + tmp;
                            }

                            var tmpArr = splitExp(subBlock);
                            strs = _.flatten([strs.slice(0, toHandleBlocks[i].start), '(', tmpArr, ')', lastBlock,
                                strs.slice(toHandleBlocks[i].end + 1)]);
                        }
                    }

                    return strs;
                };

                var getFormulaInfo = function (exp) {
                    var arr = splitExp(exp);
                    return _.map(arr, function (x) {
                        var idx = x.indexOf('(');
                        if (idx > 0) {
                            return {
                                formulaName: x.substring(0, idx),
                                formulaParams: x.substring(idx + 1)
                            };
                        }
                        else {
                            return x;
                        }
                    });
                };

                var translateExp = function (exp) {
                    var formulaInfoList = getFormulaInfo(exp);
                    scope.translatedFormula = _.reduce(formulaInfoList, function (memo, i) {
                        if (!_.isEmpty(i.formulaName)) {
                            return memo + formulaTranslateUtil
                                .translateFormula(scope.formulaList, i.formulaName, i.formulaParams, scope.includeOptional);
                        }
                        else if (i[0] === '@') {
                            var keyValue = _.find(scope.keyValueList, function (x) {
                                return x.code.trim().toLowerCase() === i.toLowerCase();
                            });

                            if (!_.isEmpty(keyValue)) {
                                return keyValue.name;
                            }
                            else {
                                return i;
                            }
                        }
                        else {
                            return memo + i;
                        }
                    }, '');
                };

                scope.translatedFormula = '';

                scope.isHide = function () {
                    return _.isEmpty(scope.translatedFormula) && scope.hideMode === 'hide';
                };

                scope.isNotVisible = function () {
                    return _.isEmpty(scope.translatedFormula) && scope.hideMode !== 'hide';
                };

                scope.$watch('formulaName', function (newVal, oldValue) {
                    scope.translatedFormula = formulaTranslateUtil
                        .translateFormula(scope.formulaList, newVal, scope.formulaParams, scope.includeOptional);
                });

                scope.$watch('formulaParams', function (newVal, oldValue) {
                    scope.translatedFormula = formulaTranslateUtil
                        .translateFormula(scope.formulaList, scope.formulaName, newVal, scope.includeOptional);
                });

                scope.$watch('formulaExp', function (newVal, oldValue) {
                    translateExp(newVal);
                });

                scope.$watchCollection('formulaList', function (newVal, oldValue) {
                    newVal.forEach(function (f) {
                        var params = _.find(f.params, { paramType: enums.paramType.accountCode });
                        if (!_.isEmpty(params) && _.isEmpty(params.options) && !_.isEmpty(scope.accountDataSource)) {
                            params.options = _.map(scope.accountDataSource, function (o) {
                                return {
                                    value: o.code,
                                    name: o.fullName,
                                    shortName: o.name
                                };
                            });
                        }
                    });

                    if (scope.formulaExp) {
                        translateExp(scope.formulaExp);
                    }
                    else {
                        scope.translatedFormula = formulaTranslateUtil
                            .translateFormula(newVal, scope.formulaName, scope.formulaParams, scope.includeOptional);
                    }
                });

                scope.$watchCollection('keyValueList', function (newVal, oldValue) {
                    if (scope.formulaExp) {
                        translateExp(scope.formulaExp);
                    }
                    else {
                        scope.translatedFormula = formulaTranslateUtil
                            .translateFormula(newVal, scope.formulaName, scope.formulaParams, scope.includeOptional);
                    }
                });

                scope.$watchCollection('accountDataSource', function (newVal, oldValue) {
                    if (scope.formulaList) {
                        scope.formulaList.forEach(function (f) {
                            var params = _.find(f.params, { paramType: enums.paramType.accountCode });
                            if (!_.isEmpty(params) && _.isEmpty(params.options) && !_.isEmpty(newVal)) {
                                params.options = _.map(newVal, function (o) {
                                    return {
                                        value: o.code,
                                        name: o.fullName,
                                        shortName: o.name
                                    };
                                });
                            }
                        });
                    }
                    if (scope.formulaExp) {
                        translateExp(scope.formulaExp);
                    }
                    else {
                        scope.translatedFormula = formulaTranslateUtil
                            .translateFormula(newVal, scope.formulaName, scope.formulaParams, scope.includeOptional);
                    }
                });

                element.bind('focus', function (e) {
                    if (_.isFunction(scope.translatorFocus)) {
                        scope.translatorFocus({
                            '$event': e
                        });
                    }
                });

                element.bind('blur', function (e) {
                    if (_.isFunction(scope.translatorBlur)) {
                        scope.translatorBlur({
                            '$event': e
                        });
                    }
                });
            }
        };
    }
    ]);