﻿commonModule.controller('VatReportViewController', ['$scope', '$rootScope', '$log', '$translate', '$timeout', '$q', '$compile', '$state', '$stateParams',
    'apiInterceptor', 'vatExportService', 'SweetAlert', 'BSPLService', 'vatReportService', 'vatReportCacheService', 'vatSessionService',
    'loginContext', 'enums', 'vatCommonService', 'vatWorkflowService', 'projectService', '$uibModal', '$cookies', 'Upload', 'vatImportService', 'vatApproveService',
    function ($scope, $rootScope, $log, $translate, $timeout, $q, $compile, $state, $stateParams, apiInterceptor, vatExportService, SweetAlert, BSPLService,
              vatReportService, vatReportCacheService, vatSessionService, loginContext, enums, vatCommonService, vatWorkflowService, projectService,
              $uibModal, $cookies, Upload, vatImportService, vatApproveService) {
        'use strict';
        $log.debug('VatReportViewController.ctor()...');

        // 根据ServiceType判断当前实际Period
        var getActualPeriod = function () {
            return $scope.serviceType === enums.serviceType.VAT ? vatSessionService.month : 0;
        };

        //Voucher Comments:
        $scope.voucherComments = "";
        $scope.voucherFileName = "";
        $scope.voucherFileID = "";
        $scope.manualSpread = {};
        $scope.file = null;

        $scope.activeSheet = {
            activeCol: null,
            activeRow: null,
            activeTemplateId: null
        };


        //Notice: ************************************
        //$scope.templateCode, $scope.reportId, $scope.initRow, $scope.initCol等都是外部传递进来的数据。
        //$scope.templateCode对应的是TaxAdmin.Template中的Code字段
        //$scope.reportId对应的是项目数据库中的Report表的ID字段
        //$scope.initRow, $scope.initCol表示初始化时默认focus的单元格位置，非必填
        //Notice: ************************************

        //***************************************************************************************
        // File: Voucher Upload Declarations BEGIN:
        var uploadUrl = apiInterceptor.vatWebApiHostUrl + '/FileUpload/NewFile';
        var apiTokenObj = $cookies.getObject('AtmsApiToken');
        var webHost = apiTokenObj.api_host;
        var chunkSize = 100000;
        var resumable = true;
        var hasManualLoad = false;
        // File: Voucher Upload Declarations ENDED:
        //***************************************************************************************

        //写日志
        $scope.period = getActualPeriod();
        $scope.moduleid = enums.vatModuleEnum.Report_TaxDeclaration;
        $scope.userId = vatSessionService.logUser.id;
        //$scope.isShowUpdateCell = false;        
        var comment = vatSessionService.project.name + " " + vatSessionService.project.year + "年"
            + ($scope.serviceType === enums.serviceType.VAT ? (getActualPeriod() + "月") : "");

        var logDto = {
            ID: '',
            OperationName: '',
            ModuleID: $scope.moduleid,
            OperationObject: '',
            OperationType: '',
            OperationContent: '',
            OriginalState: '',
            UpdateState: '',
            CreatorID: vatSessionService.logUser.id,
            Comment: comment,
            IP: '',
            Period: $scope.period,
        };

        var setDetailPositionForBSPL = function () {
            //just the first time need to adjust the position of cell detail panel
            if ($scope.firstShowDetailForBSPL) {
                //var height = $('#reportContainer').css('height');
                ////this width is last sheet width.
                //var fullWidth = $('#reportContainer').css('width');
                //var wNum = fullWidth.substring(0, fullWidth.length - 2);
                //wNum = parseInt(wNum) * 0.6;
                ////var num = height.substring(0, height.length -2);
                ////num = parseInt(num) +40;
                ////$('#cell-detail').css({
                ////    left: $scope.width, top: '-' +num + 'px' });
                //$('#cell-detail').css({
                //    left: wNum + 'px', top: '-' + height
                //});
                //$timeout(function () {
                var h = $(window).height() / 2;
                var w = $(window).width() / 2;
                //$('#cell-detail').css({
                //    'height': '530px', 'width': '202px',
                //});
                //var h1 = $('#cell-detail').css('height') / -2;
                //var w1 = $('#cell-detail').css('width') / -2;
                $('#cell-detail').css({
                    'position': 'fixed',
                    'z-index': '2000',
                    left: w + 'px',
                    top: h + 'px',
                    'margin-left': '-150px',
                    'margin-top': '-260px',
                });
                $scope.firstShowDetailForBSPL = false;
                //}, 1000);
            }
        };

        //显示纳税申报表的单元格信息详情
        var showTaxReportCellDetail = function (isSingleClick) {
            if (!isSingleClick) {
                var h = $(window).height() / 2;
                var w = $(window).width() / 2;
                $('#tax-cell-detail').css({
                    'position': 'fixed',
                    'z-index': '1500',
                    left: w + 'px',
                    top: h + 'px',
                    'margin-left': '-475px',
                    'margin-top': '-324px',
                });
                $('#taxReportCellDetailModal').css('width', '950px').show('normal');
                return $q.when();
            }
        };

        //根据发票类型获取发票类型名称
        var getInvoiceTypeName = function (invoiceType) {
            for (var item in enums.invoiceType) {
                if (enums.invoiceType[item] === invoiceType) {
                    return $translate.instant(item);
                }
            }
            return invoiceType;
        }

        //根据销项发票类型获取销项发票类型名称
        var getOutputInvoiceTypeName = function (invoiceType) {
            for (var item in enums.outputInvoiceType) {
                if (enums.outputInvoiceType[item] === invoiceType) {
                    return $translate.instant(item);
                }
            }
            return invoiceType;
        }

        var getDataSourceModel = function (sourceData, type) {
            var obj = new Object();
            switch (type) {
                //进项数据源
                case enums.formulaDataSourceType.InputInvoice:
                    obj.period = sourceData.period;
                    obj.date = sourceData.certificationDate ? (new Date(sourceData.certificationDate)).dateTimeToString('yyyyMMdd') : "";
                    obj.saleId = sourceData.sellerTaxNumber;
                    obj.invoiceCode = sourceData.invoiceCode;
                    obj.invoiceNumber = sourceData.invoiceNumber;
                    obj.money = sourceData.amount;
                    obj.invoiceType = getInvoiceTypeName(sourceData.invoiceType);
                    obj.taxMoney = sourceData.taxAmount;
                    break;
                //进项明细数据源
                case enums.formulaDataSourceType.InputInvoiceDetail:
                    obj.period = sourceData.period;
                    obj.date = sourceData.invoiceDate ? (new Date(sourceData.invoiceDate)).dateTimeToString('yyyyMMdd') : "";
                    obj.invoiceCode = sourceData.invoiceCode;
                    obj.invoiceNumber = sourceData.invoiceNumber;
                    obj.money = sourceData.amount;
                    obj.taxRate = (sourceData.taxRate * 100).toFixed(2) + '%';
                    obj.taxMoney = sourceData.taxAmount;
                    obj.cargoName = sourceData.productionName;
                    break;

                //销项数据源
                case enums.formulaDataSourceType.OutputInvoice:
                    obj.period = sourceData.period;
                    obj.date = sourceData.invoiceDate ? (new Date(sourceData.invoiceDate)).dateTimeToString('yyyyMMdd') : "";
                    obj.acquiringEnterprise = sourceData.buyerName;
                    obj.invoiceCode = sourceData.invoiceCode;
                    obj.invoiceNumber = sourceData.invoiceNumber;
                    obj.money = sourceData.amount;
                    obj.taxRate = (sourceData.taxRate * 100).toFixed(2) + '%';
                    obj.taxMoney = sourceData.taxAmount;
                    obj.invoiceType = getOutputInvoiceTypeName(sourceData.invoiceType);
                    break;

                //凭证数据源
                case enums.formulaDataSourceType.Voucher:
                    obj.period = sourceData.period;
                    obj.voucherDate = sourceData.voucherDate ? (new Date(sourceData.voucherDate)).dateTimeToString('yyyyMMdd') : "";
                    obj.acctCode = sourceData.acctCode;
                    obj.voucherID = sourceData.voucherID;
                    obj.summary = sourceData.summary;
                    obj.debit = sourceData.debit;
                    obj.credit = sourceData.credit;
                    break;

                //报表数据源
                case enums.formulaDataSourceType.Report:
                    obj.period = sourceData.period;
                    obj.year = sourceData.year;
                    obj.reportName = sourceData.reportName;
                    obj.project = sourceData.rowName;
                    obj.reportColumn = sourceData.columnName;
                    obj.reportID = sourceData.reportID;
                    obj.reportTemplateID = sourceData.reportTemplateID;
                    obj.reportCode = sourceData.reportCode;
                    obj.cellDataID = sourceData.cellDataID;
                    obj.cellTemplateID = sourceData.cellTemplateID;
                    if (sourceData.name == 'ReportDataSource') {
                        obj.cellName = PWC.numToExcelChar(sourceData.rowIndex, sourceData.columnIndex);
                    } else {
                        obj.cellName = sourceData.name;
                    }

                    obj.cellValue = sourceData.amount;
                    if ($scope.taxCellDetail.summaryExp) {
                        $scope.taxCellDetail.summaryExp = $scope.taxCellDetail.summaryExp.replace(sourceData.internalCode, obj.cellName);
                        $scope.taxCellDetail.summaryValue = $scope.taxCellDetail.summaryValue.replace(sourceData.internalCode, obj.cellValue);
                    }
                    //单元格的数值是作为百分数显示的
                    if (sourceData.resultType === 2) {
                        obj.cellValue = (obj.cellValue * 100).toFixed(2) + '%';
                    }
                    break;
                //BSPL数据源
                case enums.formulaDataSourceType.BSPL:
                    obj.index = sourceData.index;
                    obj.account = sourceData.accountCode + '-' + sourceData.accountName;
                    obj.amount = sourceData.amount;
                    // TODO: 关联凭证 obj.relatedVoucher
                    if ($scope.taxCellDetail.summaryExp) {
                        $scope.taxCellDetail.summaryExp = $scope.taxCellDetail.summaryExp.replace(sourceData.internalCode, obj.account);
                        $scope.taxCellDetail.summaryValue = $scope.taxCellDetail.summaryValue.replace(sourceData.internalCode, obj.amount);
                    }
                    break;

                //条件判断数据源
                case enums.formulaDataSourceType.Judgment:
                    obj.cellIndex = sourceData.cellIndex;
                    obj.period = sourceData.period;
                    obj.reportName = sourceData.reportName;
                    obj.project = sourceData.rowName;
                    obj.column = sourceData.columnName;
                    obj.cellConditionName = PWC.numToExcelChar(sourceData.rowIndex, sourceData.columnIndex);
                    obj.cellConditionValue = sourceData.amount;

                    if ($scope.taxCellDetail.summaryExp) {
                        if ($scope.taxCellDetail.summaryExp.indexOf('IF(') >= 0) {
                            // TODO: 目前无法取得IF条件表达式，所以无法在前端计算条件判断
                            $scope.taxCellDetail.summaryExp = $translate.instant('Conclusion');
                            $scope.taxCellDetail.summaryValue = $scope.taxCellDetail.summaryValue.replace(sourceData.internalCode, '(' + obj.cellConditionValue + ')')
                                .replace('IF(True/False,', 'Math.min(');
                        } else {
                            $scope.taxCellDetail.summaryExp = $translate.instant('Conclusion') + '&nbsp;&nbsp;&nbsp;&nbsp;'
                                + $scope.taxCellDetail.summaryExp.replace(sourceData.internalCode, obj.cellIndex);
                            $scope.taxCellDetail.summaryValue = $scope.taxCellDetail.summaryValue.replace(sourceData.internalCode, '(' + obj.cellConditionValue + ')')
                                .replace('Min', 'Math.min').replace('Max', 'Math.max');
                        }
                    }
                    break;
                case enums.formulaDataSourceType.Special:
                    obj.money = sourceData.amount.formatAmount(2);
                    break;
                //凭证筛选
                case enums.formulaDataSourceType.VoucherFilter:
                    obj.id = sourceData.id;
                    obj.period = sourceData.period;
                    obj.date = sourceData.date ? (new Date(sourceData.date)).dateTimeToString('yyyyMMdd') : "";
                    obj.enterpriseAccounting = sourceData.accountSubject;
                    obj.voucherNumber = sourceData.voucherCode;
                    obj.voucherSammary = sourceData.voucherSummary;
                    obj.debitAmount = sourceData.debitAmount;
                    obj.creditAmount = sourceData.creditAmount;
                    obj.operationType = sourceData.operationType;
                    obj.direction = sourceData.direction ? sourceData.direction : 1;
                    break;
                case enums.formulaDataSourceType.InvoiceFilter:
                    break;
            }

            if (sourceData.type === 0 && sourceData.dataSourceType === enums.cellDataSourceType.RelatedModel) {
                // 模型数据源
                obj.period = sourceData.period;
                obj.name = sourceData.name;
                obj.amount = sourceData.amount;
            }

            return obj;
        }

        //解析发票筛选数据
        var translateInvoiceFilterDataSource = function (sourceData, invoiceType) {
            var obj = new Object();
            switch (invoiceType) {
                case 1:
                    obj.id = sourceData.id;
                    obj.period = sourceData.period;
                    obj.inputInvoiceDate = sourceData.invoiceDate ? (new Date(sourceData.invoiceDate)).dateTimeToString('yyyyMMdd') : "";
                    obj.invoiceCode = sourceData.invoiceCode;
                    obj.invoiceNumber = sourceData.invoiceNumber;
                    obj.amount = sourceData.amount;
                    obj.taxRate = (sourceData.rate * 100).toFixed(2) + '%';
                    obj.taxAmount = sourceData.taxAmount;
                    obj.productionName = sourceData.productionName;
                    obj.operationType = sourceData.operationType;
                    break;
                case 2:
                    obj.id = sourceData.id;
                    obj.period = sourceData.period;
                    obj.outputInvoiceDate = sourceData.invoiceDate ? sourceData.invoiceDate.substring(0, 10) : '';
                    obj.buyerName = sourceData.buyerName;
                    obj.invoiceCode = sourceData.invoiceCode;
                    obj.invoiceNumber = sourceData.invoiceNumber;
                    obj.amount = sourceData.amount;
                    obj.taxRate = (sourceData.taxRate * 100).toFixed(2) + '%';
                    obj.taxAmount = sourceData.taxAmount;
                    obj.invoiceType = getOutputInvoiceTypeName(sourceData.invoiceType);
                    obj.operationType = sourceData.operationType;
                    break;
                case 3:
                    obj.id = sourceData.customsId;
                    obj.period = sourceData.periodId;
                    obj.IssueDate = sourceData.IssueDate;
                    obj.amount = sourceData.InvoiceAmount;
                    obj.taxRate = sourceData.InvoiceTaxAmount;
                    obj.payNum = sourceData.payNum;
                    obj.auditResult = sourceData.auditResult;
                    obj.operationType = sourceData.operationType;
                    break;
            }
            return obj;
        }

        //计算凭证的数值
        function calculateVoucherValue(voucher) {
            return voucher.direction * ((voucher.debitAmount ? voucher.debitAmount : 0) - (voucher.creditAmount ? voucher.creditAmount : 0));
        }

        //计算单元格的除了凭证筛选和发票筛选外的数值
        function calculateCellSubValue(jsonTagInfo) {
            var datasourceAmountValue = 0;
            $scope.taxCellDetail.cellSubValue = 0;
            if (jsonTagInfo.dataVoucherList && jsonTagInfo.dataVoucherList.length > 0) {
                _.each(jsonTagInfo.dataVoucherList, function (voucher) {
                    if (voucher.operationType == 1) { //加法
                        datasourceAmountValue += calculateVoucherValue(voucher);
                    } else if (voucher.operationType == 2) { //减法
                        datasourceAmountValue -= calculateVoucherValue(voucher);
                    }
                });
            }

            if (jsonTagInfo.dataInvoiceList && jsonTagInfo.dataInvoiceList.length > 0) {
                _.each(jsonTagInfo.dataInvoiceList, function (invoice) {
                    if (invoice.operationType == 1) { //加法
                        if (jsonTagInfo.cellTemplateConfig.invoiceAmountType == enums.invoiceAmountType.amount) {
                            datasourceAmountValue += invoice.amount;
                        } else if (jsonTagInfo.cellTemplateConfig.invoiceAmountType == enums.invoiceAmountType.taxAmount) {
                            datasourceAmountValue += invoice.taxAmount;
                        } else if (jsonTagInfo.cellTemplateConfig.invoiceAmountType == enums.invoiceAmountType.quntity) {
                            datasourceAmountValue += 1;
                        }
                    } else if (invoice.operationType == 2) { //减法
                        if (jsonTagInfo.cellTemplateConfig.invoiceAmountType == enums.invoiceAmountType.amount) {
                            datasourceAmountValue -= invoice.amount;
                        } else if (jsonTagInfo.cellTemplateConfig.invoiceAmountType == enums.invoiceAmountType.taxAmount) {
                            datasourceAmountValue -= invoice.taxAmount;
                        } else if (jsonTagInfo.cellTemplateConfig.invoiceAmountType == enums.invoiceAmountType.quntity) {
                            datasourceAmountValue -= 1;
                        }
                    }
                });
            }

            $scope.taxCellDetail.cellSubValue = (jsonTagInfo.value ? parseFloat((jsonTagInfo.value + '').replace(/,/g, '')) : 0) - datasourceAmountValue;
        }

        var getCellInvoiceAndVoucher = function (jsonTagInfo, isSingleClick) {
            //获取凭证筛选数据
            if (jsonTagInfo.dataVoucherList && jsonTagInfo.dataVoucherList.length > 0) {
                _.each(jsonTagInfo.dataVoucherList, function (voucher) {

                    var filterItems = _.where($scope.taxCellDetail.items, {
                        id: voucher.dataSourceId,
                        name: voucher.dataSourceName
                    });
                    if (filterItems && filterItems.length > 0) {
                        $scope.taxCellDetail.items = _.map($scope.taxCellDetail.items, function (item) {
                            if (item.id === voucher.dataSourceId && item.name === voucher.dataSourceName) {
                                item.dataSource.push(getDataSourceModel(voucher, enums.formulaDataSourceType.Voucher));
                            }
                            return item;
                        });
                    } else {
                        if ($scope.taxCellDetail.cellType === 0) {
                            $scope.taxCellDetail.cellType = voucher.type;
                            $scope.taxCellDetail.cellTypeId = voucher.dataSourceId;
                        }
                        $scope.taxCellDetail.dataSourceCount++;
                        $scope.taxCellDetail.items.push({
                            type: voucher.type,
                            operationType: voucher.operationType,
                            amount: 0,
                            createTime: voucher.CreateTime,
                            name: voucher.dataSourceName,
                            id: voucher.dataSourceId,
                            dataSource: [getDataSourceModel(voucher, enums.formulaDataSourceType.Voucher)]
                        })
                    }
                });
            }

            //获取发票筛选数据
            if (jsonTagInfo.dataInvoiceList && jsonTagInfo.dataInvoiceList.length > 0) {
                _.each(jsonTagInfo.dataInvoiceList, function (invoice) {
                    var filterItems = _.where($scope.taxCellDetail.items, {
                        id: invoice.dataSourceId,
                        name: invoice.dataSourceName
                    });
                    if (filterItems && filterItems.length > 0) {
                        $scope.taxCellDetail.items = _.map($scope.taxCellDetail.items, function (item) {
                            if (item.id === invoice.dataSourceId && item.name === invoice.dataSourceName) {
                                item.dataSource.push(translateInvoiceFilterDataSource(invoice, jsonTagInfo.cellTemplateConfig.invoiceType));
                            }
                            return item;
                        });
                    } else {
                        if ($scope.taxCellDetail.cellType === 0) {
                            $scope.taxCellDetail.cellType = invoice.type;
                            $scope.taxCellDetail.cellTypeId = invoice.dataSourceId;
                        }
                        $scope.taxCellDetail.dataSourceCount++;
                        $scope.taxCellDetail.items.push({
                            type: invoice.type,
                            operationType: invoice.operationType,
                            amount: 0,
                            createTime: invoice.createTime,
                            name: invoice.dataSourceName,
                            id: invoice.dataSourceId,
                            dataSource: [translateInvoiceFilterDataSource(invoice, jsonTagInfo.cellTemplateConfig.invoiceType)]
                        });
                    }
                });
            }
            calculateCellSubValue(jsonTagInfo);

            showTaxReportCellDetail(isSingleClick);
        };


        var setTaxReportCellPopInfo = function (cellEvent, isSingleClick) {
            var row = cellEvent.args.row;
            var column = cellEvent.args.col;
            //纳税申报表
            var sheets = $scope.spread.sheets;
            $scope.activeSheet.activeTemplateId = cellEvent.templateId;
            $scope.activeSheet.activeCol = cellEvent.args.col;
            $scope.activeSheet.activeRow = cellEvent.args.row;
            if (sheets && sheets.length > 0) {
                var tagInfo = sheets[0].getTag(row, column);
                if (constant.regesterInformation.active) {
                    tagInfo = sheets[0].getTag(row, column);
                } else {
                    tagInfo = sheets[1].getTag(row, column);
                }
                if (tagInfo) {
                    var jsonTagInfo = JSON.parse(tagInfo);
                    if (jsonTagInfo.isReadOnly)
                        return;
                    var tmpValue;
                    if (_.isString(jsonTagInfo.value)) {
                        //如果后台的格式是会计格式，转换为普通格式，用于前端格式转换。
                        tmpValue = jsonTagInfo.value.replace(/,/g, '');

                        if (!isNaN(parseFloat(tmpValue))) {
                            jsonTagInfo.value = tmpValue;
                        }
                    }

                    var value = jsonTagInfo.value;
                    // 保证无论jsonTagInfo.value是数字或是字符串都能执行以下操作
                    if (jsonTagInfo.value || (jsonTagInfo.value + '').indexOf('0') > -1) {
                        if (jsonTagInfo.dataType === enums.cellDataType.Integer) {
                            jsonTagInfo.value = parseFloat(jsonTagInfo.value).formatAmount(0);
                        } else if (jsonTagInfo.dataType === enums.cellDataType.Percentage) {
                            jsonTagInfo.value = (parseFloat(jsonTagInfo.value) * 100).toFixed(2) + '%';
                        } else {
                            jsonTagInfo.value = parseFloat(jsonTagInfo.value).formatAmount(2);
                        }
                    }

                    $scope.taxCellDetail = {
                        cellType: 0,
                        dataSourceCount: 0,
                        items: [],
                        cellInfo: {
                            project: jsonTagInfo.rowName,
                            column: jsonTagInfo.columnName,
                            description: jsonTagInfo.cellTemplateConfig.formulaDescription,
                            money: jsonTagInfo.value,
                            value: value,
                            modelIdList: jsonTagInfo.cellTemplateConfig.modelIDs,
                            modifiedReportCell: jsonTagInfo.modifiedReportCell
                        },
                        dataType: jsonTagInfo.dataType,
                        reportTemplateId: jsonTagInfo.reportTemplateID,
                        rowIndex: jsonTagInfo.rowIndex,
                        columnIndex: jsonTagInfo.columnIndex,
                        cellTemplateId: jsonTagInfo.cellTemplateID,
                        inputValue: '',
                        inputMemo: '',
                        allowAddDatasource: false,
                        showFormulaSummary: true
                    };

                    //解析数据源
                    $scope.taxCellDetail.hasKeyIn = jsonTagInfo.cellTemplateConfig.hasKeyIn;
                    $scope.taxCellDetail.config = jsonTagInfo.cellTemplateConfig;
                    $scope.taxCellDetail.config.cellID = jsonTagInfo.cellID;
                    $scope.taxCellDetail.reportId = $scope.reportId;
                    $scope.taxCellDetail.modelWriteBackData = _.isArray(jsonTagInfo.modelWriteBackData) ? jsonTagInfo.modelWriteBackData : [];
                    $scope.taxCellDetail.hasModelError = jsonTagInfo.hasModelError;
                    $scope.taxCellDetail.validationErrorList = jsonTagInfo.validationErrorList;
                    if ($scope.taxCellDetail.hasKeyIn) {
                        $scope.taxCellDetail.dataSourceCount++;
                    }

                    //设置手工输入保存时需要的信息
                    $scope.handInputModel = {
                        cellID: jsonTagInfo.cellID,
                        cellTemplateID: jsonTagInfo.cellTemplateID,
                        reportID: jsonTagInfo.reportID
                    };

                    if (jsonTagInfo.dataSourceList && jsonTagInfo.dataSourceList.length > 0) {
                        //单元格计算表达式解析
                        if (!$scope.taxCellDetail.summaryExp) {
                            $scope.taxCellDetail.summaryExp = jsonTagInfo.dataSourceExp;
                            $scope.taxCellDetail.summaryValue = jsonTagInfo.dataSourceExp;
                        }

                        // 条件判断数据源需合并数据源列表
                        var judgmentDataSource = _.where(jsonTagInfo.dataSourceList, {type: enums.formulaDataSourceType.Judgment});

                        _.each(judgmentDataSource, function (ds) {
                            jsonTagInfo.dataSourceList.splice(jsonTagInfo.dataSourceList.indexOf(ds), 1);
                            var cellIndex = 0;
                            _.each(ds.trueDataSource, function (tds) {
                                var x = tds;
                                if (tds.type !== enums.formulaDataSourceType.Report) {
                                    x = {amount: tds.amount};
                                }
                                x.name = ds.name;
                                cellIndex++;
                                x.cellIndex = String.fromCharCode(64 + cellIndex);// Get Char Index for each cell data source
                                x.type = enums.formulaDataSourceType.Judgment;
                                jsonTagInfo.dataSourceList.push(x);
                            });
                            _.each(ds.falseDataSource, function (fds) {
                                var x = fds;
                                if (fds.type !== enums.formulaDataSourceType.Report) {
                                    x = {amount: fds.amount};
                                }
                                x.name = ds.name;
                                cellIndex++;
                                x.cellIndex = String.fromCharCode(64 + cellIndex);// Get Char Index for each cell data source
                                x.type = enums.formulaDataSourceType.Judgment;
                                jsonTagInfo.dataSourceList.push(x);
                            });
                        });

                        var bsplDataSource = _.where(jsonTagInfo.dataSourceList, {type: enums.formulaDataSourceType.BSPL});
                        var dsIndex = 0;
                        _.each(bsplDataSource, function (ds) {
                            dsIndex++;
                            ds.index = dsIndex;
                        });

                        $(jsonTagInfo.dataSourceList).each(function (index, element) {
                            if (element.dataSourceType === enums.cellDataSourceType.KeyIn) {
                                if (jsonTagInfo.cellTemplateConfig.hasKeyIn) {
                                    $scope.taxCellDetail.inputValue = element.amount;
                                    $scope.taxCellDetail.inputMemo = element.name;
                                    $scope.taxCellDetail.keyinData = element.keyinData;
                                }
                            } else if (element.dataSourceType === enums.cellDataSourceType.ModelDatasource) {
                                var modelItem = {
                                    type: element.dataSourceType,
                                    name: $translate.instant(element.name),
                                    operationType: element.operationType,
                                    createTime: element.CreateTime,
                                    id: element.id,
                                    dataSource: []
                                };
                                if (jsonTagInfo.modelWriteBackData && jsonTagInfo.modelWriteBackData.length > 0) {
                                    jsonTagInfo.modelWriteBackData.forEach(function (row) {
                                        var parseItem = {
                                            index: row.index,
                                            name: row.name,
                                            amount: row.value,
                                            keyinData: row.keyinData
                                        };
                                        modelItem.dataSource.push(parseItem);
                                    });
                                } else if (jsonTagInfo.entriesCheckResultList && jsonTagInfo.entriesCheckResultList.length > 0) {
                                    jsonTagInfo.entriesCheckResultList.forEach(function (row) {
                                        var parseItem = {
                                            index: row.index,
                                            name: row.name,
                                            keyinData: row.keyinData,
                                            amount: row.amount
                                        };
                                        modelItem.dataSource.push(parseItem);
                                    });
                                }
                                $scope.taxCellDetail.cellType = enums.cellDataSourceType.ModelDatasource;
                                $scope.taxCellDetail.items.push(modelItem);
                                $scope.taxCellDetail.dataSourceCount++;
                            } else if (element.dataSourceType === enums.cellDataSourceType.Voucher
                                || element.dataSourceType === enums.cellDataSourceType.InputInvoice
                                || element.dataSourceType === enums.cellDataSourceType.OutputInvoice) {
                                var flag = true;
                                //todo: according to element to get the detail data
                                var outItems;
                                var convert = false;
                                // if (element.items.length === 1) {
                                //     outItems = JSON.parse(element.items[0]);
                                // } else {
                                    outItems = element.items;
                                    convert = true;
                                // }

                                //todo: according to element to get the detail data
                                angular.forEach(outItems, function (jsonData) {
                                    var item;
                                    if (convert) {
                                        item = JSON.parse(jsonData);
                                    } else {
                                        item = jsonData;
                                    }
                                    var model = getDataSourceModel(item, element.type);
                                    $($scope.taxCellDetail.items).each(function (itemIndex, item) {
                                        if (item.type === element.type && item.name === element.name) {
                                            item.dataSource.push(model);
                                            flag = false;
                                            return;
                                        }
                                    });
                                    if (flag && element.type && element.type > 0) {
                                        $scope.taxCellDetail.items.push({
                                            type: element.type,
                                            operationType: element.operationType,
                                            createTime: element.createTime,
                                            name: element.name,
                                            id: element.id,
                                            dataSource: [model]
                                        });
                                        if ($scope.taxCellDetail.cellType === 0) {
                                            $scope.taxCellDetail.cellType = element.type;
                                            $scope.taxCellDetail.cellTypeId = undefined;
                                        }
                                        $scope.taxCellDetail.dataSourceCount++;
                                    }
                                });
                                // $scope.taxCellDetail.items = [_.first($scope.taxCellDetail.items)];
                            } else {
                                var flag = true;
                                var model = getDataSourceModel(element, element.type);
                                //var element = $scope.element;
                                //SAP日报数据源和未开票数据源需要设置需要显示的款项列
                                if ((enums.formulaDataSourceType.Unbilled === element.type
                                    || enums.formulaDataSourceType.SapDaily === element.type)
                                    && !$scope.taxCellDetail.visibleMoneys) {
                                    $scope.taxCellDetail.visibleMoneys = element.visibleMoneys;
                                }

                                if (element.type == enums.formulaDataSourceType.Report) {
                                    if (element.reportTemplateID != $scope.templateId || element.period != $scope.period) {
                                        $scope.taxCellDetail.showFormulaSummary = false;
                                    }
                                } else if (element.type == enums.formulaDataSourceType.BSPL
                                    || element.type == enums.formulaDataSourceType.Judgment) {
                                    $scope.taxCellDetail.showFormulaSummary = false;
                                }

                                model.description = element.description;
                                $($scope.taxCellDetail.items).each(function (itemIndex, item) {
                                    if (item.type === element.type && item.name === element.name) {
                                        item.dataSource.push(model);
                                        flag = false;
                                        return;
                                    }
                                });
                                if (flag && element.type && element.type > 0) {
                                    $scope.taxCellDetail.items.push({
                                        type: element.type,
                                        operationType: element.operationType,
                                        createTime: element.createTime,
                                        name: element.name,
                                        id: element.id,
                                        dataSource: [model]
                                    });
                                    if ($scope.taxCellDetail.cellType === 0) {
                                        $scope.taxCellDetail.cellType = element.type;
                                        $scope.taxCellDetail.cellTypeId = undefined;
                                    }
                                    $scope.taxCellDetail.dataSourceCount++;
                                }
                            }
                        });
                        //获取凭证筛选数据
                        if (jsonTagInfo.cellTemplateConfig.hasVoucher || jsonTagInfo.cellTemplateConfig.hasInvoice) {
                            $scope.taxCellDetail.allowAddDatasource = true;
                            if ($scope.taxCellDetail.summaryValue) {
                                $scope.taxCellDetail.summaryValue = $scope.taxCellDetail.summaryValue.replace(/DS\[\d+\]/g, "0");
                                $scope.taxCellDetail.summaryExp = $scope.taxCellDetail.summaryExp.replace(/\+DS\[\d+\]|DS\[\d+\]\+|-DS\[\d+\]|DS\[\d+\]-/g, "");
                            }
                            getCellInvoiceAndVoucher(jsonTagInfo);
                        } else {
                            $timeout(function () {
                                if ($scope.taxCellDetail.items.length > 0) {
                                    $scope.taxCellDetail.dataGridSource = $scope.taxCellDetail.items[0].dataSource;
                                } else {
                                    $scope.taxCellDetail.dataGridSource = [];
                                }
                                if ($scope.taxCellDetail.summaryValue) {
                                    $scope.taxCellDetail.summaryValue = $scope.taxCellDetail.summaryValue.replace(/DS\[\d+\]/g, "0");
                                    $scope.taxCellDetail.summaryExp = $scope.taxCellDetail.summaryExp.replace(/\+DS\[\d+\]|DS\[\d+\]\+|-DS\[\d+\]|DS\[\d+\]-/g, "");
                                }
                            }, 500);
                            //显示纳税申报表单元格详细信息
                            showTaxReportCellDetail(isSingleClick);
                        }
                    } else {
                        getCellInvoiceAndVoucher(jsonTagInfo, isSingleClick);
                    }
                }
            }
        };

        var cellClick = function ($event) {
            var row = $event.args.row;
            var column = $event.args.col;
            var display = $('#cellDetailPanel').is(':visible');
            var taxCellPopDisplay = $('#taxReportCellDetailModal').is(':visible');
            if ($scope.isBSPL && display) {
                if (!($event.data && $event.data.cellDataID)) {
                    BSPLService.GetCellInfo($scope.templateId, column, row).success(function (res) {
                        if (res === null) {
                            return;
                        }
                        //console.log(res);
                        $scope.cellDetail = {
                            rowIndex: row,
                            columnIndex: column,
                            cellPosition: PWC.numToExcelChar(row, column),
                            formulaDesc: res.formulaDescription,
                            dataSources: [],
                            cellValue: '',
                            keyName: res.keyName,
                            modelWriteBackData: []
                        };
                        //setDetailPositionForBSPL();
                        //$('#cellDetailPanel').show('normal');
                    });
                } else {
                    BSPLService.GetCellDetailInfo($event.data.cellDataID).success(function (res) {
                        var flag = false;
                        _.each(res, function (item, key) {
                            item.id = key + 1;
                            if (item.name === 'ReportDataSource') {
                                flag = true;
                                item.cellPosition = PWC.numToExcelChar(item.rowIndex, item.columnIndex);
                            }
                        });
                        $scope.isBSPLSpecial = flag ? -1 : 0;
                        $scope.cellDetail = {
                            rowIndex: row,
                            columnIndex: column,
                            cellPosition: PWC.numToExcelChar(row, column),
                            formulaDesc: $event.data.formulaDescription,
                            dataSources: res,
                            cellValue: $event.data.value,
                            keyName: $event.data.keyName,
                            modelWriteBackData: []
                        };
                        //setDetailPositionForBSPL();
                        //$('#cellDetailPanel').show('normal');
                    }).error(function (err) {
                        SweetAlert.error('', $translate.instant('CellInfoFail'));
                    });
                }
            } else if (taxCellPopDisplay) {
                setTaxReportCellPopInfo($event, true);
            }
        };

        var cellDoubleClick = function ($event) {
            var row = $event.args.row;
            var column = $event.args.col;
            if ($scope.isBSPL) {
                if (!($event.data && $event.data.cellDataID)) {
                    BSPLService.GetCellInfo($scope.templateId, column, row).success(function (res) {
                        if (res === null) {
                            return;
                        }
                        //console.log(res);
                        $scope.cellDetail = {
                            rowIndex: row,
                            columnIndex: column,
                            cellPosition: PWC.numToExcelChar(row, column),
                            formulaDesc: res.formulaDescription,
                            dataSources: [],
                            cellValue: '',
                            keyName: res.keyName,
                            modelWriteBackData: []
                        };
                        setDetailPositionForBSPL();
                        $('#cellDetailPanel').show('normal');
                        $scope.$broadcast("refreshCellPanel", {isShow: true});
                    });
                } else {
                    BSPLService.GetCellDetailInfo($event.data.cellDataID).success(function (res) {
                        var flag = false;
                        _.each(res, function (item, key) {
                            item.id = key + 1;
                            if (item.name === 'ReportDataSource') {
                                flag = true;
                                item.cellPosition = PWC.numToExcelChar(item.rowIndex, item.columnIndex);
                            }
                        });
                        $scope.isBSPLSpecial = flag ? -1 : 0;
                        $scope.cellDetail = {
                            rowIndex: row,
                            columnIndex: column,
                            cellPosition: PWC.numToExcelChar(row, column),
                            formulaDesc: $event.data.formulaDescription,
                            dataSources: res,
                            cellValue: $event.data.value,
                            keyName: $event.data.keyName,
                            modelWriteBackData: []
                        };
                        setDetailPositionForBSPL();
                        $('#cellDetailPanel').show('normal');
                        $scope.$broadcast("refreshCellPanel", {isShow: true});
                    }).error(function (err) {
                        SweetAlert.error('', $translate.instant('CellInfoFail'));
                    });
                }
            } else {
                setTaxReportCellPopInfo($event);
            }

            //

        };

        $scope.exportExcel = function () {
            if ($scope.spread) {
                //fix bug 11737 不能使用Excel本身自带的功能，比如选中几行数据，自动求和，求平均这种功能
                $scope.spread.getActiveSheet().setIsProtected(false);
                var dataObj = {
                    "spread": $scope.spread.toJSON(),
                    //'': $scope.spread.sheets[0]._name,
                    "exportFileType": "xlsx"
                };
                var content = JSON.stringify(dataObj);
                dataObj = null
                vatExportService.exportReport(content).success(function (data) {
                    window.open(loginContext.vatApiHost + '/' + data);
                });
            }
        };

        //////////////////////
        $scope.spread;
        $scope.evenType;
        $scope.spreads = [];
        $scope.exportReportData = [];
        $scope.templateIds = [];
        $scope.exportSheetNum = 0;

        $scope.exportBSPL = function () {
            var dataObj = {
                "spread": $scope.spread.toJSON(),
                "exportFileType": "xlsx"
            };

            var content = JSON.stringify(dataObj);
            dataObj = null;

            vatExportService.exportReport(content).success(function (data) {
                window.open(loginContext.vatApiHost + '/' + data);
            });
        };

        $scope.exportReportTreeOptions = {
            dataSource: [{id: '0', templateId: '0', name: '纳税申报表', items: [], expanded: true}],
            displayExpr: 'name',
            keyExpr: 'id',
            selectionMode: 'Multiple',
            showCheckBoxesMode: 'normal',
            dataStructure: "plain",
            selectByClick: true,
            selectNodesRecursive: true,
            expandAllEnabled: true,
            expandNodesRecursive: true,
            onSelectionChanged: function (args) {
                $scope.selectedReportIds = args.component.getSelectedNodesKeys();
            }
        };

        //打开导出文件弹出框
        $scope.openExportPop = function (evenType) {
            $scope.evenType = evenType;
            if ('export' == $scope.evenType) {
                $scope.viewTitle = $translate.instant('SelectExportReport');
            }
            if ('upload' == $scope.evenType) {
                $scope.viewTitle = $translate.instant('SelectUploadReport');
            }
            var grp = _.find($scope.$parent.$parent.groups, function (g) {
                return g.name == 'TaxReturnType';
            });

            if (!grp || !grp.children) {
                return;
            }

            $scope.exportReportTreeOptions.dataSource[0].items = [];
            $scope.exportReportTreeOptions.dataSource[0].items = _.map(grp.children, function (item) {
                return {id: item.reportId, templateId: item.id, name: item.name, parentId: '0'};
            });

            var parentElem = angular.element($('#exportReportFilesContainer'));
            $scope.exportReportInstance = $uibModal.open({
                ariaLabelledBy: 'modal-title',
                ariaDescribedBy: 'modal-body',
                templateUrl: 'exportReport.html',
                windowClass: 'data-table',
                appendTo: parentElem,
                backdrop: 'static',
                scope: $scope
            });
        }

        $scope.uploadReportTreeOptions = {
            dataSource: [{id: '0', templateId: '0', name: '纳税申报表', items: [], expanded: true}],
            displayExpr: 'name',
            keyExpr: 'id',
            selectionMode: 'Multiple',
            showCheckBoxesMode: 'normal',
            dataStructure: "plain",
            selectByClick: true,
            selectNodesRecursive: true,
            expandAllEnabled: true,
            expandNodesRecursive: true,
            onSelectionChanged: function (args) {
                $scope.selectedReportIds = args.component.getSelectedNodesKeys();
            }
        };

        //批量导出EXCEL
        $scope.export = function () {
            var grp = _.find($scope.$parent.$parent.groups, function (g) {
                return g.name == 'TaxReturnType';
            });

            if (!grp || !grp.children) {
                return;
            }

            if (!$scope.selectedReportIds || $scope.selectedReportIds.length == 0) {
                SweetAlert.warning($translate.instant('ExportReportRequiredCheck'));
                return;
            }

            //设置报表的排序号，用于对导出的报表sheet进行排序
            var orderIndex = 0;
            _.each(grp.children, function (item) {
                item.orderIndex = orderIndex;
                orderIndex++;
            });

            var reportArray = $scope.selectedReportIds;

            $scope.spreads = [];
            $scope.exportReportData = [];
            $('.export-container').html('');
            $('#export').html('');

            $scope.exportSheetNum = reportArray.length;
            var promiss = [];

            //报表数据源公式检查表达式。
            var reg = /^([A-Z]+[0-9]+){1}$|^([A-Z]+[0-9]+){1}([\+-]{1}[A-Z]+[0-9]+)+$/g;
            var exportReportData = [];
            angular.forEach(reportArray, function (currentVal) {
                var node = _.first(_.where(grp.children, {reportId: currentVal}));
                if (!node) {
                    return;
                }
                var reportPromiss = vatReportService.getReportData(node.reportId).success(function (reportData) {
                    if (reportData && reportData.data && reportData.data.cellData) {
                        _.each(reportData.data.cellData, function (x) {
                            x.value = x.cellValue;
                            if (x.formula && reg.test(x.formula)) {
                                x.isFormula = true;
                            }
                            x.isExportData = true;
                            delete x.cellValue;
                        });

                        // reportData.data.cellData.reportName = node.name;
                        reportData.data.cellData.orderIndex = node.orderIndex;
                        reportData.data.cellData.templateId = node.id;
                        exportReportData.push(reportData.data.cellData);
                    }
                });
                promiss.push(reportPromiss);
            });

            $q.all(promiss).then(function () {
                //重新排序报表
                $('#busy-indicator-container').show();
                exportReportData = _.sortBy(exportReportData, function (item) {
                    return item.orderIndex;
                });

                for (var j = 0; j < exportReportData.length; j++) {
                    var report = "<vat-report-sheet id='r_" + exportReportData[j].templateId + "' spread='spreads[" + j + "]' report-source = 'exportReportData[" + j + "]' template-id='\"" + exportReportData[j].templateId + "\"' is-export='true' service-type='6'></vat-report-sheet>";
                    var html = $compile(report)($scope);
                    $('.export-container').append(html);
                }
                timeOutExportSpread(exportReportData);
            });
            return;
        };

        var timeOutCount = 0;

        function timeOutExportSpread(exportReportData) {
            //20秒之内一直等待spreadjs初始化完成所有需要下载的报表，直到超时
            if ($scope.validateFileCount === $scope.spreads.length || timeOutCount === 20) {
                exportSpread(exportReportData);
                timeOutCount = 0;
            } else {
                $timeout(function () {
                    timeOutExportSpread(exportReportData);
                }, 1000);
            }
            timeOutCount++;
        }

        function exportSpread(exportReportData) {
            $scope.exportReportData = exportReportData;
            $timeout(function () {
                $('#busy-indicator-container').show();
                var mainSpread = new GC.Spread.Sheets.Workbook(document.getElementById("export"), {sheetCount: 1});
                mainSpread.isPaintSuspended(true);
                mainSpread.sheets.pop();
                for (var index = 0; index < $scope.spreads.length; index++) {

                    var currentSheet = $scope.spreads[index].getActiveSheet();
                    currentSheet.options.isProtected = false;
                    for (var rowIndex = 0; rowIndex < currentSheet.getRowCount(); rowIndex++) {
                        for (var columnIndex = 0; columnIndex < currentSheet.getColumnCount(); columnIndex++) {
                            var cellStyle = currentSheet.getActualStyle(rowIndex, columnIndex, GC.Spread.Sheets.SheetArea.viewport, true);
                            currentSheet.setStyle(rowIndex, columnIndex, cellStyle, GC.Spread.Sheets.SheetArea.viewport);
                        }
                    }

                    mainSpread.sheets.push(currentSheet);
                }


                var excelIo = new GC.Spread.Excel.IO();
                // here is excel IO API
                excelIo.save(mainSpread.toJSON(), function (blob) {
                    if ('export' == $scope.evenType) {
                        saveAs(blob, vatSessionService.project.name + '-' + vatSessionService.month + '-纳税申报.xlsx');
                        $('#busy-indicator-container').hide();
                    } else if ('upload' == $scope.evenType) {
                        var month = vatSessionService.month < 10 ? ("0" + vatSessionService.month) : vatSessionService.month;
                        Upload.upload({
                            url: apiInterceptor.webApiHostUrl + '/template/upload',
                            data: {
                                projectId: vatSessionService.project.id,
                                period: "" + vatSessionService.year + month
                            },
                            file: blob,
                            headers: {
                                'Access-Control-Allow-Origin': '*',
                                Authorization: apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken(),
                                withCredentials: true
                            },
                            withCredentials: true
                        }).then(function (resp) {
                            SweetAlert.success('', $translate.instant('FileUploadSuccess'));
                            $('#busy-indicator-container').hide();
                        }, function (resp) {
                            SweetAlert.error('', $translate.instant('FileUploadFailed'));
                            $('#busy-indicator-container').hide();
                        }, function (evt) {

                        });
                    }


                }, function (e) {
                    alert(e);
                });

//                vatExportService.exportReport(content).success(function (data) {
//                    window.open(loginContext.vatApiHost + '/' + data);
//                });
                $('.export-container').html('');
                $('#export').html('');

                $log.debug(mainSpread);
            }, 500);
        }


        ///////////////////////

        $scope.testGetReportData = function () {
            BSPLService.getReportData();
        };

        var getReportData = function () {
            vatReportService.getReportData($scope.reportId).success(function (reportData) {
                if (reportData && reportData.data && reportData.data.cellData) {
                    _.each(reportData.data.cellData, function (x) {
                        x.value = x.cellValue;
                        delete x.cellValue;
                    });

                    $scope.reportData = reportData.data.cellData;
                    $scope.formulaBlocks = reportData.data.formulaBlocks;
                    $scope.manualDataSources = reportData.data.manualDataSources;
                } else {
                    $scope.reportData = [];
                    $scope.formulaBlocks = [];
                    $scope.manualDataSources = [];
                }


                //if ($scope.isShowUpdateCell && $scope.taxCellDetail) {
                //    // 如果模型回填数据了，则刷新单元格数据
                //    var cellEvent = {
                //        args: {
                //            row: $scope.taxCellDetail.rowIndex,
                //            col: $scope.taxCellDetail.columnIndex
                //        }
                //    };

                //    $timeout(function() {
                //        setTaxReportCellPopInfo(cellEvent);
                //    }, 500);

                //    // 标志位清空
                //    $scope.isShowUpdateCell = false;
                //}
            });
        };

        var loadCellData = function (period) {
            if ($scope.isBSPL) {
                //if period has no value, get current month as period.
                //if (!period) {
                //    var currentTime = new Date();
                //    period= currentTime.getMonth()+1;
                //}
                var projectID = vatSessionService.project.id;
                //because project in overview is mock up, so I have to mock the project id from database,
                //otherwise the project not exist in database.
                //通用
                //projectID = 'd6dcfb47-a92c-4b0b-a063-354b63756bd0';
                //房地产
                //projectID = '9bc51587-e697-48bf-b136-d9a8e834869b';
                //call service, parmeter: $scope.templateId, projectID,period.
                BSPLService.getReportData(projectID, $scope.templateId, period, vatSessionService.logUser.ID).success(function (reportData) {
                    $scope.reportData = reportData;
                });
            } else {
                if ($scope.reportId === undefined) {
                    if ($scope.templateId !== undefined) {
                        //todo: according to templateId and period get reportId
                        vatReportService.getReportByTemplateId($scope.templateId, period).success(function (report) {
                            if (report && report.result && report.data) {
                                $scope.reportId = report.data.id;
                                getReportData();
                            } else if (!(report.result && report.data)) {
                                $scope.reportData = [];
                                $scope.formulaBlocks = [];
                                $scope.manualDataSources = [];
                            }
                        });
                    }
                } else {
                    getReportData();
                }
            }
        };
        $scope.periodChange = function (data) {
            if (data.value) {
                //call backend service to change BS/PL report
                //need report id and period to query data for each cell
                var period = data.value;
                loadCellData(period);
            }
        };

        $scope.showOperateLogPop = function () {
            $scope.isShowLog = true;
        };

        // 更新spreadjs计算所需datasource信息
        $scope.updateCellByManualChange = function (manualData) {
            var cellData = _.find($scope.reportData, function (x) {
                return x.cellTemplateID == manualData.cellTemplateId
                    || x.cellID === manualData.cellId;
            });
            if (!cellData) {
                return;
            }

            for (var i = 0; i < $scope.reportData.length; i++) {
                var reportD = $scope.reportData[i];
                if (manualData.keyinData && reportD.rowIndex === cellData.rowIndex
                    && reportD.columnIndex === cellData.columnIndex) {
                    reportD.keyinData = manualData.keyinData;
                }
                for (var j = 0; j < reportD.dataSourceList.length; j++) {
                    if (reportD.dataSourceList[j].rowIndex === cellData.rowIndex
                        && reportD.dataSourceList[j].columnIndex === cellData.columnIndex) {
                        reportD.dataSourceList[j].keyinData = manualData.keyinData;
                        reportD.dataSourceList[j].amount = manualData.amount;
                    }
                }
            }

            if (!manualData.cellTemplateId) {
                manualData.cellTemplateId = cellData.cellTemplateID;
            }

            if (!manualData.cellId) {
                manualData.cellId = cellData.cellID;
            }

            if (!cellData.cellID) {
                cellData.cellID = manualData.cellId;
            }

            // 更新单元格用于展示的数据源列表
            var manualDsList = _.where(cellData.dataSourceList, {dataSourceType: manualData.dataSourceType});
            if (!manualData.id) {
                manualDsList = _.where(manualDsList, {id: manualData.id});
            }

            var manualDs = null;
            if (manualDsList.length > 0) {
                manualDs = manualDsList[0];
            }

            if (manualDs) {
                manualDs.amount = manualData.amount;
                manualDs.name = manualData.name;
                manualDs.keyinData = manualData.keyinData;
                manualDs.description = manualData.description;
            } else {
                cellData.dataSourceList.push(manualData);
            }

            // 如果修改的是凭证、发票筛选数据源，需要更新对应的凭证列表或发票列表
            if (manualData.dataSourceType === enums.cellDataSourceType.OutputInvoice
                || manualData.dataSourceType === enums.cellDataSourceType.InputInvoice
                || manualData.dataSourceType === enums.cellDataSourceType.CustomInvoice) {
                var invoiceDs = manualData.extraItems && _.chain(manualData.extraItems).reject(function (x) {
                    return _.some(cellData.dataInvoiceList, {id: x.id});
                }).map(function (x) {
                    var rtn = angular.copy(x);
                    rtn.dataSourceId = manualData.id;
                    rtn.dataSourceName = manualData.name;
                    return rtn;
                }).value();

                if (!_.isEmpty(invoiceDs)) {
                    cellData.dataInvoiceList = cellData.dataInvoiceList.concat(invoiceDs);
                }
            } else if (manualData.dataSourceType === enums.cellDataSourceType.Voucher) {
                var voucherDs = manualData.extraItems && _.chain(manualData.extraItems).reject(function (x) {
                    return _.some(cellData.dataVoucherList, {id: x.id});
                }).map(function (x) {
                    var rtn = angular.copy(x);
                    rtn.dataSourceId = manualData.id;
                    rtn.dataSourceName = manualData.name;
                    return rtn;
                }).value();

                if (!_.isEmpty(voucherDs)) {
                    cellData.dataVoucherList = cellData.dataVoucherList.concat(voucherDs);
                }
            }

            delete manualData.extraItems;

            // 更新单元格用于计算_Manual公式的手工数据源列表（包括凭证筛选、发票筛选、手工输入等）
            var manualDataSourceList = _.filter($scope.manualDataSources, function (x) {
                return x.item1 == manualData.cellTemplateId
                    && x.item2.dataSourceType === manualData.dataSourceType;
            });
            if (!manualData.id) {
                manualDataSourceList = _.filter(manualDataSourceList, function (x) {
                    return x.item2.id === manualData.id;
                });
            }

            var manualDataSource = null;
            if (manualDataSourceList.length > 0) {
                manualDataSource = manualDataSourceList[0];
            }

            if (manualDataSource) {
                manualDataSource.item2.amount = manualData.amount;
                manualDataSource.item2.keyinData = manualData.keyinData;
                manualDataSource.item2.name = manualData.name;
                manualDataSource.item2.description = manualData.description;
            } else {
                $scope.manualDataSources.push({item1: manualData.cellTemplateId, item2: manualData});
            }

            // 刷新spreadsheet，获取操作数据源后最新的计算结果，并拿来与当前所有单元格value比较，得到value修改前后的值对比
            var updatedCells = $scope.reportApi.refreshReport();
            loadCellData(getActualPeriod());
        };

        $scope.updateCellBySourceDetailDelete = function (dataSource, detailId, opType) {
            // 获取包含对应发票或凭证的筛选数据源列表
            var prop = opType === 1 ? 'dataInvoiceList' : 'dataVoucherList';
            var detailLists = _.chain($scope.reportData)
                .filter(function (x) {
                    return !_.isEmpty(x[prop]);
                })
                .pluck(prop)
                .filter(function (x) {
                    return _.some(x, {id: detailId});
                }).value();

            // 删除dataInvoiceList或dataVoucherList中对应的发票或凭证
            detailLists.forEach(function (detailList) {
                var idx = _.findIndex(detailList, function (x) {
                    return x.id === detailId;
                });
                if (idx >= 0) {
                    detailList.splice(idx, 1);
                }
            });

            // 更新单元格用于展示的数据源列表
            var dataSourceLists = _.chain($scope.reportData)
                .pluck('dataSourceList')
                .filter(function (dataSourceList) {
                    return _.some(dataSourceList, {id: dataSource.id});
                }).value();
            dataSourceLists.forEach(function (dataSourceList) {
                var ds = _.find(dataSourceList, function (x) {
                    return x.id === dataSource.id;
                });
                if (!_.isEmpty(ds)) {
                    ds.amount = dataSource.amount;
                }
            });

            // 更新单元格用于计算_Manual公式的手工数据源列表（凭证筛选、发票筛选）
            var manualDataSource = _.find($scope.manualDataSources, function (x) {
                return x.item1 === dataSource.cellTemplateID
                    && x.item2.dataSourceType === dataSource.dataSourceType
                    && x.item2.id === dataSource.id;
            });

            if (manualDataSource) {
                manualDataSource.item2.amount = dataSource.amount;
            }

            // 刷新spreadsheet，获取操作数据源后最新的计算结果，并拿来与当前所有单元格value比较，得到value修改前后的值对比
            var updatedCells = $scope.reportApi.refreshReport();
        };

        $scope.updateCellByManualSourceDelete = function (dataSourceId, cellId) {
            var cellData = _.findWhere($scope.reportData, {cellID: cellId});

            // 删除dataInvoiceList或dataVoucherList或modelWriteBackData中对应的发票或凭证
            if (!_.isEmpty(cellData.dataInvoiceList)) {
                for (var i = cellData.dataInvoiceList.length - 1; i >= 0; i--) {
                    if (cellData.dataInvoiceList[i].dataSourceId === dataSourceId) {
                        cellData.dataInvoiceList.splice(i, 1);
                    }
                }
            }

            if (!_.isEmpty(cellData.dataVoucherList)) {
                for (var i = cellData.dataVoucherList.length - 1; i >= 0; i--) {
                    if (cellData.dataVoucherList[i].dataSourceId === dataSourceId) {
                        cellData.dataVoucherList.splice(i, 1);
                    }
                }
            }

            if (!_.isEmpty(cellData.modelWriteBackData)) {
                for (var i = cellData.modelWriteBackData.length - 1; i >= 0; i--) {
                    if (cellData.modelWriteBackData[i].dataSourceId === dataSourceId) {
                        cellData.modelWriteBackData.splice(i, 1);
                    }
                }
            }

            // 删除单元格用于展示的数据源列表中的对应数据源
            var manualDsIdx = _.findIndex(cellData.dataSourceList, {id: dataSourceId});
            if (manualDsIdx >= 0) {
                cellData.dataSourceList.splice(manualDsIdx, 1);
            }

            var manualDataSourceIdx = _.findIndex($scope.manualDataSources, function (x) {
                return x.item1 === cellData.cellTemplateID && x.item2.id === dataSourceId;
            });
            if (manualDataSourceIdx >= 0) {
                $scope.manualDataSources.splice(manualDataSourceIdx, 1);
            }

            // 刷新spreadsheet，获取操作数据源后最新的计算结果，并拿来与当前所有单元格value比较，得到value修改前后的值对比
            $scope.reportApi.refreshReport();

            // TODO: 本来报表单元格值更新后弹出框值就会自动更新，但模型回填相关操作很奇怪，不会自动更新
            if ($scope.taxCellDetail.cellType === enums.formulaDataSourceType.ModelDatasource) {
                $scope.taxCellDetail.cellInfo.money = Number(cellData.value).formatAmount();
            }
        };

        $scope.updateCellByModelFillBack = function (manualData) {
            var cellData = _.find($scope.reportData, function (x) {
                return !_.isEmpty(manualData.cellTemplateID) && x.cellTemplateID === manualData.cellTemplateID
                    || !_.isEmpty(manualData.cellDataID) && x.cellID === manualData.cellDataID;
            });
            if (_.isEmpty(cellData)) {
                return;
            }

            if (_.isEmpty(manualData.cellTemplateID)) {
                manualData.cellTemplateID = cellData.cellTemplateID;
            }

            if (_.isEmpty(manualData.cellDataID)) {
                manualData.cellDataID = cellData.cellID;
            }

            // 更新单元格用于展示的数据源列表
            var manualDsList = _.where(cellData.dataSourceList, {dataSourceType: manualData.dataSourceType});
            if (!_.isEmpty(manualData.id)) {
                manualDsList = _.where(manualDsList, {id: manualData.id});
            }

            var manualDs = null;
            if (manualDsList.length > 0) {
                manualDs = manualDsList[0];
            }

            if (manualDs) {
                manualDs.amount = manualData.amount;
                manualDs.name = manualData.name;
                manualDs.description = manualData.description;
            } else {
                cellData.dataSourceList.push(manualData);
            }

            var filterItems = _.where($scope.taxCellDetail.items, {id: manualData.id});
            if (filterItems && filterItems.length > 0) {
                $scope.taxCellDetail.items = _.each($scope.taxCellDetail.items, function (item) {
                    if (item.type === enums.formulaDataSourceType.ModelDatasource
                        && manualData.modelWriteBackData
                        && manualData.modelWriteBackData.length > 0) {
                        item.dataSource.length = 0;
                        _.each(manualData.modelWriteBackData, function (row) {
                            var parseItem = {
                                index: row.index,
                                name: row.name,
                                amount: row.value
                            }

                            item.dataSource.push(parseItem);
                        });
                    }
                });
            } else {
                var modelItem = {
                    type: manualData.dataSourceType,
                    name: $translate.instant(manualData.name),
                    operationType: manualData.operationType,
                    id: manualData.id,
                    amount: manualData.amount,
                    dataSource: []
                };

                if (manualData.modelWriteBackData && manualData.modelWriteBackData.length > 0) {
                    _.each(manualData.modelWriteBackData, function (row) {
                        var parseItem = {
                            index: row.index,
                            name: row.name,
                            amount: row.value
                        }

                        modelItem.dataSource.push(parseItem);
                    });
                }

                $scope.taxCellDetail.cellType = enums.formulaDataSourceType.ModelDatasource;
                $scope.taxCellDetail.items.push(modelItem);
                $scope.taxCellDetail.dataSourceCount++;
            }

            // 更新对应的模型回填数据
            if (!_.isEmpty(manualData.modelWriteBackData)) {
                cellData.modelWriteBackData = manualData.modelWriteBackData;
            }

            // 更新单元格用于计算_Manual公式的手工数据源列表（包括凭证筛选、发票筛选、手工输入等）
            var manualDataSourceList = _.filter($scope.manualDataSources, function (x) {
                return x.item1 === manualData.cellTemplateId
                    && x.item2.dataSourceType === manualData.dataSourceType;
            });
            if (!_.isEmpty(manualData.id)) {
                manualDataSourceList = _.filter(manualDataSourceList, function (x) {
                    return x.item2.id === manualData.id;
                });
            }

            var manualDataSource = null;
            if (manualDataSourceList.length > 0) {
                manualDataSource = manualDataSourceList[0];
            }

            if (manualDataSource) {
                manualDataSource.item2.amount = manualData.amount;
                manualDataSource.item2.name = manualData.name;
                manualDataSource.item2.description = manualData.description;
            } else {
                $scope.manualDataSources.push({item1: manualData.cellTemplateID, item2: manualData});
            }

            // 刷新spreadsheet，获取操作数据源后最新的计算结果，并拿来与当前所有单元格value比较，得到value修改前后的值对比
            var updatedCells = $scope.reportApi.refreshReport();

            // TODO: 本来报表单元格值更新后弹出框值就会自动更新，但这里很奇怪，不会自动更新
            $scope.taxCellDetail.cellInfo.money = Number(cellData.value).formatAmount();
        };

        //单元格详细信息点击确定时执行
        $scope.confirm = function () {
            if (vatSessionService.month)
                vatSessionService.project.period = vatSessionService.month;
            vatApproveService.approvalStatus(vatSessionService.project.id, vatSessionService.project.period).success(function (result) {
                if (result && result == 'committed') {
                    SweetAlert.error('报表提审中！');
                } else {
                    $scope.handInputModel.name = $scope.taxCellDetail.inputMemo;
                    var r = /^(-[1-9]\d*|[1-9]\d*|[0]{1,1})$/;
                    if ($scope.taxCellDetail.keyinData) {
                        $scope.handInputModel.keyinData = $scope.taxCellDetail.keyinData;
                    } else {
                        $scope.handInputModel.amount = $scope.taxCellDetail.inputValue;
                    }
                    $scope.handInputModel.description = $scope.taxCellDetail.inputMemo;
                    $scope.handInputModel.projectID = vatSessionService.project.id;
                    $scope.handInputModel.serviceTypeID = vatSessionService.project.serviceTypeID;
                    $scope.handInputModel.period = vatSessionService.month;

                    if ($scope.handInputModel.amount || $scope.handInputModel.name || $scope.handInputModel.keyinData) {
                        //日志对象
                        logDto.ID = PWC.newGuid();
                        logDto.CreateTime = new Date();
                        logDto.UpdateTime = new Date();
                        logDto.OperationContent = "Amount: " + $scope.handInputModel.amount
                            + "; Description: " + $scope.handInputModel.description + "keyinData:" + $scope.handInputModel.keyinData;
                        logDto.OperationName = $translate.instant('ManualInputDataSource');
                        logDto.Comment = comment + "(Cell ID:" + $scope.handInputModel.cellID
                            + ", Cell Template ID:" + $scope.handInputModel.cellTemplateID + ")";
                        logDto.OperationType = enums.vatLogOperationTypeEnum.RV_ManualInput;

                        // 前端保存数据
                        return vatReportService.addCellManualData($scope.handInputModel, logDto).then(function (manualData) {
                            var obj = manualData.data.data;
                            obj.dataSourceType = manualData.dataSourceType;
                            $scope.updateCellByManualChange(obj);
                            return $q.when();
                        });
                    }

                    return $q.reject();
                }
            });

        };

        $scope.saveReportCache = function () {
            if (!$scope.reportData) {
                return $q.reject();
            }

            var dirtyData = _.where($scope.reportData, {isDirty: true});
            if (!vatReportCacheService.hasReportCache() && _.isEmpty(dirtyData)) {
                return $q.reject();
            }

            _.each(dirtyData, function (x) {
                x.cellValue = x.value;
            });
            if (vatSessionService.project.projectStatusList[getActualPeriod()] >= constant.ProjectStatusEnum.AccountMapSubmitted) {
                swal({
                        title: "warning!",
                        text: $translate.instant('IsConfirmSaveReport').formatObj({status: vatCommonService.getProjectStautsEnumDesc(vatSessionService.project.projectStatusList[getActualPeriod()])}),
                        type: "warning",
                        showCancelButton: true,
                        confirmButtonColor: "#DD6B55",
                        confirmButtonText: $translate.instant('Yes'),
                        cancelButtonText: $translate.instant('No'),
                        closeOnConfirm: true,
                        closeOnCancel: true
                    },
                    function (isConfirm) {
                        if (isConfirm) {
                            return vatReportCacheService.saveReportCache(dirtyData, $scope.reportId).then(function (data) {
                                loadCellData(getActualPeriod());
                                SweetAlert.success($translate.instant('SaveSuccess'));
                            });
                        } else {
                            swal.close();
                            return $q.reject();
                        }
                    });
            } else {
                return vatReportCacheService.saveReportCache(dirtyData, $scope.reportId).then(function (data) {
                    loadCellData(getActualPeriod());
                    SweetAlert.success($translate.instant('SaveSuccess'));
                });
            }
        };

        //提交报表审核
        $scope.submitReport = function (operationTypeId) {
            if (_.isEqual(vatSessionService.project.projectStatusList[getActualPeriod()], constant.ProjectStatusEnum.Generated) ||
                _.isEqual(vatSessionService.project.projectStatusList[getActualPeriod()], constant.ProjectStatusEnum.ReportRejected)) {
                swal({
                        title: "warning!",
                        text: $translate.instant('IsComfirmToSubmitReport'),
                        type: "warning",
                        showCancelButton: true,
                        confirmButtonColor: "#DD6B55",
                        confirmButtonText: $translate.instant('Yes'),
                        cancelButtonText: $translate.instant('No'),
                        closeOnConfirm: true,
                        closeOnCancel: true
                    },
                    function (isConfirm) {
                        if (isConfirm) {
                            showApproveLogPopUp(operationTypeId);
                        } else {
                            swal.close();
                        }
                    });
            } else {
                SweetAlert.warning($translate.instant('SubmitReportStatusCheck'));
            }
        }

        //撤销审核
        $scope.withdrawReport = function (operationTypeId) {
            if (_.isEqual(vatSessionService.project.projectStatusList[getActualPeriod()], constant.ProjectStatusEnum.ReportSubmitted)) {
                swal({
                        title: "warning!",
                        text: $translate.instant('IsComfirmToWithdrawSubmitReport'),
                        type: "warning",
                        showCancelButton: true,
                        confirmButtonColor: "#DD6B55",
                        confirmButtonText: $translate.instant('Yes'),
                        cancelButtonText: $translate.instant('No'),
                        closeOnConfirm: true,
                        closeOnCancel: true
                    },
                    function (isConfirm) {
                        if (isConfirm) {
                            showApproveLogPopUp(operationTypeId);

                        } else {
                            swal.close();
                        }
                    });
            } else {
                SweetAlert.warning($translate.instant('WithdrawReportStatusCheck'));
            }
        }

        //审核通过
        $scope.approve = function (operationTypeId) {
            if (vatCommonService.isPreviousNodeFinished(constant.DictionaryDictKey.ApproveReport, vatSessionService.project.workflow) &&
                (
                    _.isEqual(vatSessionService.project.projectStatusList[getActualPeriod()], constant.ProjectStatusEnum.AccountMapSubmitted) ||
                    _.isEqual(vatSessionService.project.projectStatusList[getActualPeriod()], constant.ProjectStatusEnum.Generated) ||
                    _.isEqual(vatSessionService.project.projectStatusList[getActualPeriod()], constant.ProjectStatusEnum.ReportSubmitted) ||
                    _.isEqual(vatSessionService.project.projectStatusList[getActualPeriod()], constant.ProjectStatusEnum.ReportRejected) ||
                    vatCommonService.isReportLastLevelApproved($scope.approveLevel))) {
                swal({
                        title: "warning!",
                        text: $translate.instant('IsConfirmToApproveReport'),
                        type: "warning",
                        showCancelButton: true,
                        confirmButtonColor: "#DD6B55",
                        confirmButtonText: $translate.instant('Yes'),
                        cancelButtonText: $translate.instant('No'),
                        closeOnConfirm: true,
                        closeOnCancel: true
                    },
                    function (isConfirm) {
                        if (isConfirm) {
                            showApproveLogPopUp(operationTypeId);
                        } else {
                            swal.close();
                        }
                    });
            } else {
                SweetAlert.warning($translate.instant('ApproveReportStatusCheck'));
            }
        }

        //拒绝审核
        $scope.reject = function (operationTypeId) {
            swal({
                    title: "warning!",
                    text: $translate.instant('IsConfirmToRejectReport'),
                    type: "warning",
                    showCancelButton: true,
                    confirmButtonColor: "#DD6B55",
                    confirmButtonText: $translate.instant('Yes'),
                    cancelButtonText: $translate.instant('No'),
                    closeOnConfirm: true,
                    closeOnCancel: true
                },
                function (isConfirm) {
                    if (isConfirm) {
                        showApproveLogPopUp(operationTypeId);
                    } else {
                        swal.close();
                    }
                });
        }

        //申报完成
        $scope.declarationComplete = function (operationTypeId) {
            if (vatCommonService.isPreviousNodeFinished(constant.DictionaryDictKey.DeclarationComplete, vatSessionService.project.workflow) &&
                vatSessionService.project.projectStatusList[getActualPeriod()] >= constant.ProjectStatusEnum.Generated &&
                vatSessionService.project.projectStatusList[getActualPeriod()] !== constant.ProjectStatusEnum.ReportRejected) {
                swal({
                        title: "warning!",
                        text: $translate.instant('IsConfirmToCompleteDeclaration'),
                        type: "warning",
                        showCancelButton: true,
                        confirmButtonColor: "#DD6B55",
                        confirmButtonText: $translate.instant('Yes'),
                        cancelButtonText: $translate.instant('No'),
                        closeOnConfirm: true,
                        closeOnCancel: true
                    },
                    function (isConfirm) {
                        if (isConfirm) {
                            setApproveValue(operationTypeId);

                            projectService.setProjectStatus(vatSessionService.project.id, getActualPeriod(), constant.ProjectStatusEnum.Completed)
                                .success(function (r1) { //设置项目状态
                                    if (!r1.result) {
                                        return false;
                                    }
                                    $rootScope.$broadcast('statusChanged', {status: constant.ProjectStatusEnum.Completed});
                                    vatSessionService.project.projectStatusList[getActualPeriod()] = constant.ProjectStatusEnum.Completed;
                                    vatWorkflowService.setProcessLog(constant.ProjectStatusEnum.Completed, vatSessionService.project.id, getActualPeriod(), constant.DictionaryDictKey.WFDeclarationComplete, enums.FinishStatusEnum.Finished)
                                        .success(function (r2) { //更新WorkflowTaskLog记录
                                            if (!r2.result) {
                                                return false;
                                            }
                                            vatWorkflowService.addApproveReportLog($scope.approveReportLogDto)
                                                .success(function (r3) { //添加WorkflowApproveReportLog记录
                                                    if (!r3.result) {
                                                        return false;
                                                    }
                                                    vatWorkflowService.getWorkflowDetailByDb(vatSessionService.project.id, getActualPeriod())
                                                        .success(function (r4) { //获取Workflow详细内容
                                                            if (r4.result) {
                                                                vatSessionService.project.workflow = r4.data;
                                                                $log.debug("$scope.declarationComplete ： workflowData.result");
                                                                $log.debug(vatSessionService.project.workflow);
                                                                setCompleteBtShow();
                                                            }
                                                        })
                                                })
                                        })
                                });
                        } else {
                            swal.close();
                        }
                    });
            } else {
                SweetAlert.warning($translate.instant('CompleteDeclarationStatusCheck'));
            }
        }

        //上传凭证
        //todo: 有任意一个凭证上传即为完成
        $scope.uploadCertificate = function () {
            $("#showVoucherUploadPortal").modal("show");
            searchEvidence();
        }

        //点击弹出框中的确认按钮
        $scope.popUpOnConfirm = function () {
            $log.debug("start to popUpOnConfirm..........");

            var status = -1;
            var dictKey = '';
            var finishStatus = -1;
            $scope.approveReportLogDto.Remark = $("#txtRemarks").val();
            logDto.Comment = $("#txtRemarks").val();

            switch ($scope.currentOperationTypeId) {
                case enums.vatLogOperationTypeEnum.SubmitReport:
                    status = constant.ProjectStatusEnum.ReportSubmitted;
                    dictKey = constant.DictionaryDictKey.WFSubmitReport;
                    finishStatus = enums.FinishStatusEnum.Finished;
                    break;
                case enums.vatLogOperationTypeEnum.WithdrawReport:
                    status = constant.ProjectStatusEnum.Generated;
                    dictKey = constant.DictionaryDictKey.WFSubmitReport;
                    finishStatus = enums.FinishStatusEnum.NotFinished;
                    break;
                case enums.vatLogOperationTypeEnum.ApproveReport:
                    status = constant.ProjectStatusEnum.ReportApproved;
                    dictKey = constant.DictionaryDictKey.WFReportApproval;
                    finishStatus = enums.FinishStatusEnum.Finished;
                    var wf = vatSessionService.project.workflow;
                    //如果审批有多级任务时，所有层级审核完成后“审核报表”Task才完成
                    var approveReportNode = _.where(wf.workflowNodeList, {dictionaryKey: constant.DictionaryDictKey.ApproveReport})[0];
                    if (!_.isNull(approveReportNode) && !_.isUndefined(approveReportNode)) {
                        var maxLevelDto = _.max(approveReportNode.auditReportLevelList, function (m) {
                            return m.orderIndex
                        });
                        if ($scope.approveLevel > 0 && $scope.approveLevel < maxLevelDto.orderIndex) {
                            finishStatus = enums.FinishStatusEnum.NotFinished;
                        }
                    }

                    break;
                case enums.vatLogOperationTypeEnum.RejectReport:
                    status = constant.ProjectStatusEnum.ReportRejected;
                    dictKey = constant.DictionaryDictKey.WFReportApproval;
                    finishStatus = enums.FinishStatusEnum.NotFinished;
                    break;
            }

            setApproveMessageDto($scope.approveReportLogDto, dictKey);

            projectService.setProjectStatus(vatSessionService.project.id, getActualPeriod(), status)//设置项目状态
                .success(function (r1) {
                    if (!r1.result) {
                        return false;
                    }
                    $rootScope.$broadcast('statusChanged', {status: status});
                    vatSessionService.project.projectStatusList[getActualPeriod()] = status;
                    vatWorkflowService.setProcessLog(status, vatSessionService.project.id, getActualPeriod(), dictKey, finishStatus)//更新WorkflowTaskLog记录
                        .success(function (r2) {
                            if (!r2.result) {
                                return false;
                            }
                            vatWorkflowService.addApproveReportLog($scope.approveReportLogDto)//添加WorkflowApproveReportLog记录
                                .success(function (r3) {
                                    if (!r3.result) {
                                        return false;
                                    }
                                    vatWorkflowService.getWorkflowDetailByDb(vatSessionService.project.id, getActualPeriod())//获取Workflow详细内容
                                        .success(function (r4) {
                                            if (r4.result) {
                                                vatSessionService.project.workflow = r4.data;
                                                $log.debug("workflowData.result");
                                                $log.debug(vatSessionService.project.workflow);
                                                if ($scope.currentOperationTypeId === enums.vatLogOperationTypeEnum.SubmitReport ||
                                                    $scope.currentOperationTypeId === enums.vatLogOperationTypeEnum.WithdrawReport) {
                                                    setSubmitBtShow();
                                                    setApproveBtShow();
                                                    setCompleteBtShow();
                                                    setRemarkBtShowAndTitle();

                                                }

                                                if ($scope.currentOperationTypeId === enums.vatLogOperationTypeEnum.ApproveReport ||
                                                    $scope.currentOperationTypeId === enums.vatLogOperationTypeEnum.RejectReport) {
                                                    setSubmitBtShow();
                                                    setApproveBtShow();
                                                    setCompleteBtShow();
                                                    setRemarkBtShowAndTitle();
                                                }

                                                vatWorkflowService.addApproveMessage($scope.approveMessageDto)
                                                    .success(function (r5) {
                                                        if (r5.result) {
                                                            $log.debug("send report approve message successfully");
                                                        }
                                                    })
                                            }
                                        })
                                })
                        })
                })
            $scope.modalInstance.dismiss('cancel');
        }

        //显示上一步操作的备注
        $scope.showRemarks = function () {
            setPreviewsProcessData();
            $scope.modalInstance = $uibModal.open({
                animation: false,
                ariaLabelledBy: 'modal-title',
                ariaDescribedBy: 'modal-body',
                templateUrl: 'model-report-remarks.html',
                scope: $scope,
                windowClass: "model-report-remarks-popup"
            });

            $scope.closeModal = function () {
                $scope.modalInstance.dismiss('cancel');
            };

        }

        //显示提交审核还是撤销审核按钮
        function setSubmitBtShow() {
            vatCommonService.isViewReportBtShow(vatSessionService.project.workflow, $scope.buttonStatus);
            var wf = vatSessionService.project.workflow;
            var approveReportNode = _.where(wf.workflowNodeList, {dictionaryKey: constant.DictionaryDictKey.ApproveReport})[0];
            var maxLevelDto = _.max(approveReportNode.auditReportLevelList, function (m) {
                return m.orderIndex
            });
            var isPreviousNodeDone = vatCommonService.isPreviousNodeFinished(constant.DictionaryDictKey.ViewReport, wf);
            $scope.submitBtShow = $scope.buttonStatus.viewReportBtShow;
            $scope.showWithdrawBt = isPreviousNodeDone && !$scope.submitBtShow && !maxLevelDto.isDone;
        }

        //显示通过审核还是拒绝审核按钮
        function setApproveBtShow() {
            var wf = vatSessionService.project.workflow;
            var approveReportNode = _.where(wf.workflowNodeList, {dictionaryKey: constant.DictionaryDictKey.ApproveReport})[0];
            vatCommonService.isApproveReportBtShow(wf, $scope.buttonStatus);
            if (!_.isNull(approveReportNode) && !_.isUndefined(approveReportNode)) {
                var maxLevelDto = _.max(approveReportNode.auditReportLevelList, function (m) {
                    return m.orderIndex
                });
                var maxLevel = maxLevelDto.orderIndex;
                switch ($scope.currentOperationTypeId) {
                    case enums.vatLogOperationTypeEnum.ApproveReport:
                        $scope.approveBtShow = true && $scope.buttonStatus.approveReportBtShow;
                        $scope.showWithdrawBt = false; //只要审核过一级，就隐藏'撤销审核'按钮
                        //当审批通过最后一级后隐藏审核通过、拒绝审核按钮
                        if (approveReportNode.currentApproveLevel === maxLevel) {
                            var crtApproveLevelDto = _.where(approveReportNode.auditReportLevelList, {orderIndex: approveReportNode.currentApproveLevel})[0];
                            if (!_.isNull(crtApproveLevelDto) && !_.isUndefined(crtApproveLevelDto)) {
                                if (crtApproveLevelDto.isDone) {
                                    $scope.approveBtShow = !$scope.approveBtShow && $scope.buttonStatus.approveReportBtShow;
                                }
                            }
                        }

                        break;
                    case enums.vatLogOperationTypeEnum.RejectReport:
                        $scope.approveBtShow = false && $scope.buttonStatus.approveReportBtShow;
                        break;
                    default:
                        $scope.approveBtShow = true && $scope.buttonStatus.approveReportBtShow;

                }
            }

            $scope.approveLevel = vatCommonService.getLoginUserCurrentApproveReportLevel();
            $scope.approveLevelDesc = $translate.instant('ApproveLevelDescTitle').formatObj({level: $scope.approveLevel});
        }

        //显示、隐藏上传批准和完成申报按钮
        function setCompleteBtShow() {
            vatCommonService.isDeclarationComplete(vatSessionService.project.workflow, $scope.buttonStatus);
            $scope.declareCompleteBtShow = $scope.buttonStatus.decCompleteBtShow && !$scope.buttonStatus.approveReportBtShow;
            $scope.uploadCertifiBtShow = $scope.buttonStatus.uploadCerficateBtShow && !$scope.buttonStatus.approveReportBtShow;
        }

        //判断显示或者隐藏备注按钮，并且甚至弹出框的title
        function setRemarkBtShowAndTitle() {
            $scope.showRemarksBt = false;
            var wf = vatSessionService.project.workflow;
            switch (vatSessionService.project.projectStatusList[getActualPeriod()]) {
                case constant.ProjectStatusEnum.Generated:
                    var submitTask = getWorkflowTask(wf, constant.DictionaryDictKey.ViewReport, constant.DictionaryDictKey.WFSubmitReport);
                    if (!_.isNull(submitTask) && !_.isUndefined(submitTask)) {
                        vatWorkflowService.isExistApproveLogSubmitItem(vatSessionService.project.id, getActualPeriod(), $scope.approveLevel,
                            enums.vatLogOperationTypeEnum.SubmitReport, enums.ReportTemplateCode.TaxDeclarationCategory).success(function (or) {
                            if (or.result) {
                                if (!or.data) {//不存在任何'提交审核'的记录，说明还未做过任何报表审批操作。此时不显示'备注'按钮
                                    $scope.showRemarksBt = false;
                                } else {
                                    $scope.showRemarksBt = true;
                                }
                                setRemarkBtTitle($scope.showRemarksBt);
                            }
                        });
                    }
                    break;
                case constant.ProjectStatusEnum.ReportSubmitted:
                    $scope.showRemarksBt = true;
                    setRemarkBtTitle($scope.showRemarksBt);
                    break;
                case constant.ProjectStatusEnum.ReportApproved:
                    var approveReportNode = _.where(wf.workflowNodeList, {dictionaryKey: constant.DictionaryDictKey.ApproveReport})[0];
                    if (!_.isUndefined(approveReportNode)) {
                        var maxLevelDto = _.max(approveReportNode.auditReportLevelList, function (m) {
                            return m.orderIndex
                        });
                        if ($scope.approveLevel > 0 && $scope.approveLevel <= maxLevelDto.orderIndex && !maxLevelDto.isDone) {
                            $scope.showRemarksBt = true;
                        } else {//'审核通过'最后一级后，不显示'备注'
                            $scope.showRemarksBt = false;
                        }
                        setRemarkBtTitle($scope.showRemarksBt);
                    }
                    break;
                case constant.ProjectStatusEnum.ReportRejected:
                    $scope.showRemarksBt = true;
                    setRemarkBtTitle($scope.showRemarksBt);
                    break;
            }
        }

        function setRemarkBtTitle(isShowRemarksBt) {
            if (isShowRemarksBt) {
                switch (vatSessionService.project.projectStatusList[getActualPeriod()]) {
                    case constant.ProjectStatusEnum.Generated:
                        $scope.remarksTitle = $translate.instant('ReportWithdrawToApprove') + '-' + $translate.instant('Memo');
                        break;
                    case constant.ProjectStatusEnum.ReportSubmitted:
                        $scope.remarksTitle = $translate.instant('ReportSubmitToApprove') + '-' + $translate.instant('Memo');
                        break;
                    case constant.ProjectStatusEnum.ReportApproved:
                        $scope.remarksTitle = $translate.instant('ReportApprove') + $translate.instant('ApproveLevelDescTitle').formatObj({level: $scope.approveLevel - 1})
                            + '-' + $translate.instant('Memo');

                        break;
                    case constant.ProjectStatusEnum.ReportRejected:
                        $scope.remarksTitle = $translate.instant('ReportReject') + '-' + $translate.instant('Memo');
                        break;
                }
            }
        }

        //显示弹出框
        function showApproveLogPopUp(operationTypeId) {
            $scope.modalInstance = $uibModal.open({
                animation: false,
                //appendTo: angular.element(document.querySelector('#imgProjectStatus')),//把弹出框加到imgProjectStatus下面
                ariaLabelledBy: 'modal-title',
                ariaDescribedBy: 'modal-body',
                templateUrl: 'model-report-approve-log.html',
                scope: $scope,
                windowClass: "model-report-approve-popup"
            });

            $scope.closeModal = function () {
                $scope.modalInstance.dismiss('cancel');
            };

            setApproveValue(operationTypeId);
        }

        //设置弹出框页面的绑定数据
        //operationTypeId: 对应enums.vatLogOperationTypeEnum
        function setApproveValue(operationTypeId) {
            $scope.currentOperationTypeId = operationTypeId;
            var operationObjectVal = '';
            switch ($scope.moduleid) {
                case enums.vatModuleEnum.Report_TaxDeclaration:
                    operationObjectVal = $translate.instant('TaxReturn');
                    break;
                case enums.vatModuleEnum.Report_BSPL:
                    operationObjectVal = $translate.instant('FinancialReturn');
                    break;
                default:
                    operationObjectVal = '';
            }
            logDto.OperationObject = operationObjectVal;

            //构建$scope.approveReportLogDto数据
            $scope.approveReportLogDto.ID = PWC.newGuid();
            $scope.approveReportLogDto.OrderIndex = $scope.approveLevel;
            $scope.approveReportLogDto.CreateTime = new Date();
            $scope.approveReportLogDto.UpdateTime = new Date();
            $scope.approveReportLogDto.OperationTypeID = $scope.currentOperationTypeId;
            $scope.approveReportLogDto.OperationLog = logDto;


            //构建logDto数据
            logDto.ID = PWC.newGuid();
            logDto.CreateTime = new Date();
            logDto.UpdateTime = new Date();
            //logDto.OperationObject = $translate.instant('TaxReturn');


            switch (operationTypeId) {
                case enums.vatLogOperationTypeEnum.SubmitReport:
                    $scope.ApproveTitle = $translate.instant('IsConfirmToSubmitReportTitle');

                    $scope.approveReportLogDto.OperationTypeName = $translate.instant('ReportSubmitToApprove');

                    logDto.OperationName = $translate.instant('ReportSubmitToApprove') + operationObjectVal;
                    logDto.ModuleID = $scope.moduleid;
                    logDto.OperationType = $scope.currentOperationTypeId;
                    logDto.OperationContent = '';
                    logDto.OriginalState = '';
                    logDto.UpdateState = $translate.instant('ProjectStatusReportSubmitted');
                    logDto.Comment = '';

                    break;

                case enums.vatLogOperationTypeEnum.WithdrawReport:
                    $scope.ApproveTitle = $translate.instant('IsConfirmToWithdrawSubmitReportTitle');

                    $scope.approveReportLogDto.OperationTypeName = $translate.instant('ReportWithdrawToApprove');

                    logDto.OperationName = $translate.instant('ReportWithdrawToApprove') + operationObjectVal;
                    logDto.ModuleID = $scope.moduleid;
                    logDto.OperationType = $scope.currentOperationTypeId;
                    logDto.OperationContent = '';
                    logDto.OriginalState = '';
                    logDto.UpdateState = $translate.instant('ProjectStatusGenerated');
                    logDto.Comment = '';
                    break;

                case enums.vatLogOperationTypeEnum.ApproveReport:
                    $scope.ApproveTitle = $translate.instant('IsConfirmToApproveReportTitle');

                    $scope.approveReportLogDto.OperationTypeName = $translate.instant('ReportApprove');

                    logDto.OperationName = $translate.instant('ReportApprove') + operationObjectVal;
                    logDto.ModuleID = $scope.moduleid;
                    logDto.OperationType = $scope.currentOperationTypeId;
                    logDto.OperationContent = $translate.instant('OperationLogApproveContent').formatObj({level: $scope.approveLevel});
                    logDto.OriginalState = '';
                    logDto.UpdateState = $translate.instant('OperationLogApproveUpdateStauts').formatObj({level: $scope.approveLevel});
                    logDto.Comment = '';

                    break;

                case enums.vatLogOperationTypeEnum.RejectReport:
                    $scope.ApproveTitle = $translate.instant('IsConfirmToRejectReportTitle');

                    $scope.approveReportLogDto.OperationTypeName = $translate.instant('ReportReject');

                    logDto.OperationName = $translate.instant('ReportReject') + operationObjectVal;
                    logDto.ModuleID = $scope.moduleid;
                    logDto.OperationType = $scope.currentOperationTypeId;
                    logDto.OperationContent = $translate.instant('OperationLogApproveContent').formatObj({level: $scope.approveLevel});
                    logDto.OriginalState = '';
                    logDto.UpdateState = $translate.instant('ProjectStatusReportRejected');
                    logDto.Comment = '';
                    break;

                case enums.vatLogOperationTypeEnum.DeclarationComplete:

                    $scope.approveReportLogDto.OperationTypeName = $translate.instant('DeclarationComplete');

                    logDto.OperationName = $translate.instant('ReportReject') + operationObjectVal;
                    logDto.ModuleID = $scope.moduleid;
                    logDto.OperationType = $scope.currentOperationTypeId;
                    logDto.OperationContent = '',
                        logDto.OriginalState = '';
                    logDto.UpdateState = $translate.instant('ProejctStatusCompleted');
                    logDto.Comment = '';

                    break;
            }

        }

        function setApproveMessageDto(approveReportLogDto, taskDictkey) {
            $scope.approveMessageDto.LogOperationTypeId = approveReportLogDto.OperationTypeID;
            $scope.approveMessageDto.TaskDictKey = taskDictkey;
            $scope.approveMessageDto.MessageDetail = approveReportLogDto.Remark;

            switch (approveReportLogDto.OperationTypeID) {
                case enums.vatLogOperationTypeEnum.SubmitReport:
                    $scope.approveMessageDto.NodeDictKey = constant.DictionaryDictKey.ViewReport;
                    break;
                case enums.vatLogOperationTypeEnum.WithdrawReport:
                    $scope.approveMessageDto.NodeDictKey = constant.DictionaryDictKey.ViewReport;
                    break;
                case enums.vatLogOperationTypeEnum.ApproveReport:
                    $scope.approveMessageDto.NodeDictKey = constant.DictionaryDictKey.ApproveReport;
                    $scope.approveMessageDto.ApprovedLevel = $scope.approveLevel;
                    break;
                case enums.vatLogOperationTypeEnum.RejectReport:
                    $scope.approveMessageDto.NodeDictKey = constant.DictionaryDictKey.ApproveReport;
                    $scope.approveMessageDto.ApprovedLevel = $scope.approveLevel;
                    break;


            }
        }

        function setPreviewsProcessData() {
            $('#txtLastProcessRemark').val('');
            var wf = vatSessionService.project.workflow;

            switch (vatSessionService.project.projectStatusList[getActualPeriod()]) {
                case constant.ProjectStatusEnum.Generated: //1. 没有点击过'提交审核' 2. 点击过'撤销审核' 
                    //数据库中没有'提交审核'任何记录的话，就说明是情况1， 否则情况2                                                
                    var submitTask = getWorkflowTask(wf, constant.DictionaryDictKey.ViewReport, constant.DictionaryDictKey.WFSubmitReport);
                    if (!_.isUndefined(submitTask)) {
                        vatWorkflowService.isExistApproveLogSubmitItem(vatSessionService.project.id, getActualPeriod(), $scope.approveLevel,
                            enums.vatLogOperationTypeEnum.SubmitReport, enums.ReportTemplateCode.TaxDeclarationCategory).success(function (or) {
                            if (or.result) {
                                var operationTypeId = -1;
                                operationTypeId = or.data ? enums.vatLogOperationTypeEnum.WithdrawReport : enums.vatLogOperationTypeEnum.SubmitReport;
                                vatWorkflowService.getWorkflowApproveReportLog(vatSessionService.project.id, getActualPeriod()
                                    , $scope.approveLevel, operationTypeId, enums.ReportTemplateCode.TaxDeclarationCategory).success(function (or) {
                                    if (or.result) {
                                        $log.debug(or.data);
                                        if (!_.isNull(or.data)) {
                                            $('#txtLastProcessRemark').val(or.data.remark);
                                        }
                                    }
                                });
                            }
                        });
                    }
                    break;
                case constant.ProjectStatusEnum.ReportSubmitted: // 已经'提交审核'。
                    //获取'提交审核'最新的记录
                    vatWorkflowService.getWorkflowApproveReportLog(vatSessionService.project.id, getActualPeriod(), $scope.approveLevel
                        , enums.vatLogOperationTypeEnum.SubmitReport, enums.ReportTemplateCode.TaxDeclarationCategory)
                        .success(function (or) {
                            if (or.result) {
                                if (!_.isNull(or.data)) {
                                    $('#txtLastProcessRemark').val(or.data.remark);
                                }
                            }
                        });
                    break;
                case constant.ProjectStatusEnum.ReportApproved: // 已经'通过审核'，可能有多级
                    //获取'通过审核'最新记录, 层级参数为： $scope.approveLevel - 1
                    vatWorkflowService.getWorkflowApproveReportLog(vatSessionService.project.id, getActualPeriod(), $scope.approveLevel - 1
                        , enums.vatLogOperationTypeEnum.ApproveReport, enums.ReportTemplateCode.TaxDeclarationCategory)
                        .success(function (or) {
                            if (or.result) {
                                if (!_.isNull(or.data)) {
                                    $('#txtLastProcessRemark').val(or.data.remark);
                                }
                            }
                        });
                    break;
                case constant.ProjectStatusEnum.ReportRejected: //已经'拒绝审核'              
                    //获取最晚的''拒绝审核'记录，不用传递 $scope.approveLevel参数。因为1级通过，2级拒绝后，此时的 $scope.approveLevel = 1，返回值就为空了
                    vatWorkflowService.getWorkflowApproveReportLogNoLevel(vatSessionService.project.id, getActualPeriod()
                        , enums.vatLogOperationTypeEnum.RejectReport, enums.ReportTemplateCode.TaxDeclarationCategory)
                        .success(function (or) {
                            if (or.result) {
                                if (!_.isNull(or.data)) {
                                    $('#txtLastProcessRemark').val(or.data.remark);
                                }
                            }
                        });
                    break;
            }
        }

        function constructRemarksData() {
            setRemarkBtShowAndTitle();
            setPreviewsProcessData();
        }

        function getWorkflowTask(wf, nodeKey, taskKey) {
            if (!_.isNull(wf) && !_.isUndefined(wf)) {
                var node = _.where(wf.workflowNodeList, {dictionaryKey: nodeKey})[0];
                if (!_.isUndefined(node)) {
                    var task = _.where(node.workflowTaskList, {dictionaryKey: taskKey})[0];
                    return task;
                }
            } else {
                return null;
            }
        }

        var modelFillBackFunc = $scope.$on("ModelWriteBackHasChanged", function (event, data) {
            var dataSource = {
                id: data.dataSourceId,
                name: 'ModelBackFill',
                amount: _.reduce(data.currentWriteBackData, function (memo, item) {
                    return memo + item.value;
                }, 0),
                cellDataID: data.cellDataId,
                dataSourceType: enums.formulaDataSourceType.ModelDatasource,
                modelWriteBackData: data.currentWriteBackData
            };

            vatReportCacheService.modelBackFill(dataSource).then(function (manualData) {
                $scope.updateCellByModelFillBack(manualData);
                return $q.when();
            });
        });

        var reportRenderedFunc = $scope.$on(enums.vatEvent.reportRendered, function () {
            $scope.exportSheetNum = $scope.exportSheetNum - 1;
            if ($scope.exportSheetNum <= 0) {
                exportSpread();
            }
        });

        var reportDrillDownFunc = $scope.$on(enums.vatEvent.reportDrillDown, function (event, args) {
            $log.debug('report drill-down: ' + JSON.stringify(args));
            var range = PWC.excelCharToNum(args.cellName);
            if ($scope.reportId === args.reportID) {
                $scope.initRow = range.row;
                $scope.initCol = range.col;
            } else {
                var serviceTypeRouteStr = $state.current.name;
                serviceTypeRouteStr = serviceTypeRouteStr.split('.')[0];
                $state.go(serviceTypeRouteStr + '.generateReport.reportView', {
                    id: args.reportID,
                    templateid: args.reportTemplateID,
                    name: args.reportName,
                    templatecode: args.reportCode,
                    initrow: range.row,
                    initcol: range.col
                });
            }
        });

        var loadData = $rootScope.$on("loadData", function (event, args) {
            $log.debug('loadData: ' + JSON.stringify(args));
            loadCellData(vatSessionService.month);
        });

        $scope.$on('$destroy', function () {
            modelFillBackFunc();
            reportRenderedFunc();
            reportDrillDownFunc();
        });

        /****************************************证据文件function start*************************************************/
        /****************************************证据文件function start*************************************************/
        /****************************************证据文件function start*************************************************/

        $scope.showVoucherUploader = function (isVoucherModalShow) {
            if (isVoucherModalShow) {
                $("#showVoucherUploadPortal").modal('show');
                searchEvidence();
            }
        };

        var evidenceCount = 0;
        var allFilesCount = 0;
        var orderfileName = true;
        var orderperiodId = true;
        var orderfileType = true;
        $scope.chunkEvidenceSize = 10 * 1024 * 1024;
        var uploadEvidenceUrl = apiInterceptor.vatWebApiHostUrl + '/outputInvoiceImport/UploadVoucherFile';
        //上传文件
        var uploadEvidenceFile = function (files) {
            if (files && files.length) {
                allFilesCount = files.length;
                evidenceCount = 0;
                for (var i = 0; i < files.length; i++) {
                    var file = files[i];
                    if (!file.$error) {
                        var tempFileName = PWC.newGuid() + '.dat';
                        var token = $('input[name="__RequestVerificationToken"]').val();
                        Upload.upload({
                            url: uploadEvidenceUrl,
                            data: {
                                cancel: false,
                                filename: file.name,
                                tempFileName: tempFileName,
                                file: file,
                                period: getActualPeriod()
                                , dbName: vatSessionService.project.id

                            },
                            resumeChunkSize: resumable ? chunkSize : null,
                            headers: {
                                'Access-Control-Allow-Origin': '*',
                                Authorization: apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken(),
                                from: vatSessionService.project.id
                            },
                            __RequestVerificationToken: token,
                            withCredentials: true
                        }).then(uploadEvidenceSuccess);
                    }
                }
            }
        };
        //上传成功
        var uploadEvidenceSuccess = function (resp) {
            if (resp.data) {
                evidenceCount++;
                if (evidenceCount === allFilesCount) {
                    var projectId = vatSessionService.project.id;
                    var periodId = getActualPeriod();
                    var dicKeyUpload = "WFUploadCertificate";
                    var dicKeyFinish = "WFDeclarationComplete";
                    var isFinished = 1;
                    var promise = vatWorkflowService.setProcessLog(constant.ProjectStatusEnum.Completed, dbName, periodId, dicKeyUpload, isFinished);

                    promise.success(
                        projectService.setProjectStatus(vatSessionService.project.id, getActualPeriod(), constant.ProjectStatusEnum.Completed)
                            .success(function (r1) { //设置项目状态
                                if (!r1.result) {
                                    return false;
                                }
                                //  $rootScope.$broadcast('statusChanged', { status: constant.ProjectStatusEnum.Completed });
                                vatSessionService.project.projectStatusList[getActualPeriod()] = constant.ProjectStatusEnum.Completed;
                                vatWorkflowService.setProcessLog(constant.ProjectStatusEnum.Completed, vatSessionService.project.id, getActualPeriod(), constant.DictionaryDictKey.WFDeclarationComplete, enums.FinishStatusEnum.Finished)
                                    .success(function (r2) { //更新WorkflowTaskLog记录
                                        SweetAlert.success($translate.instant('UploadFileSuccess'));
                                        searchEvidence();
                                    })
                            }));

                }
            }
        };

        //验证导入文件数量和大小
        function validationEvidenceFile(files) {
            var isOk = true;

            $scope.selectedProject = vatSessionService.project;
            $scope.evidenceMsgMap = [];
            var sizeString = '';
            var exString = '';
            var sizeCount = 0;
            var exCount = 0;
            if (files.length > 5) {
                $scope.evidenceMsgMap.push(
                    {
                        'errorType': $translate.instant('OutOfAmountType'),
                        'errorCount': files.length,
                        'errorContent': $translate.instant('OutOfAmount')
                    });
            }
            if (files.length > 0) {
                for (var i = 0; i < files.length; i++) {
                    var file = files[i];
                    var fileSize = file.size;
                    var fileExtension = files[i].name.split('.').pop().toLowerCase();
                    //大小不能超过10M
                    if (fileSize > $scope.chunkEvidenceSize) {
                        sizeString += files[i].name + ',';
                        sizeCount++;
                    }
                    if (fileExtension === 'exe') {
                        exString += files[i].name + ',';
                        exCount++;
                    }
                }
                if (sizeString) {
                    $scope.evidenceMsgMap.push(
                        {
                            'errorType': $translate.instant('OutOfSizeType'),
                            'errorCount': sizeCount,
                            'errorContent': $translate.instant('OutOfSize').formatObj({rows: sizeString.substring(0, sizeString.length - 1)})
                        });
                }
                if (exString) {
                    $scope.evidenceMsgMap.push(
                        {
                            'errorType': $translate.instant('ErrorFileType'),
                            'errorCount': exCount,
                            'errorContent': $translate.instant('ErrorFileMsg'),
                        });
                }
            }
            if ($scope.evidenceMsgMap.length > 0) {
                $('#evidenceErrorListModal').modal('show');
                isOk = false;
            }
            return isOk;
        }

        //查询证据文件
        function searchEvidence() {
            $scope.voucherList = [];
            var queryParams = {
                queryType: $scope.queryType,
                queryString: $scope.queryString,
                isCurrentPeriod: false,
                periodId: getActualPeriod()
            };
            vatImportService.getVoucherList(queryParams).success(function (data) {
                if (data.length > 0) {
                    $scope.voucherList = data;
                    $scope.evidenceTotalCount = '(' + data.length + ')';
                } else if (data.length === 0) {
                    $scope.evidenceTotalCount = '';
                }
            }).error(function () {
                SweetAlert.error($translate.instant('PleaseContactAdministrator'));
            });
        }

        //查询文件数量
        function getEvidenceTitle() {
            searchEvidence();
            if ($scope.voucherList.length > 0) {
                return $translate.instant('EvidenceTab') + '(' + $scope.voucherList.length + ')';
            } else {
                return $translate.instant('EvidenceTab');
            }
        }

        //单击选中文件
        function selectEvidenceFile(evidence) {
            if ($scope.clicked) {
                $scope.cancelClick = true;
                return;
            }
            $scope.clicked = true;
            $timeout(function () {
                if ($scope.cancelClick) {
                    $scope.cancelClick = false;
                    $scope.clicked = false;
                    return;
                }
                evidence.isSelected = !evidence.isSelected;
                //clean up
                $scope.cancelClick = false;
                $scope.clicked = false;
            }, 300);
        }

        //下载
        $scope.openEvidenceFile = function (evidence) {
            var filePath = evidence.filePath;
            var fileName = evidence.fileName;
            $timeout(function (evidence) {
                var queryParams = {
                    filePath: filePath,
                    fileName: fileName
                };
                vatImportService.downloadEvidence(queryParams).success(function (data, status, headers) {
                    var octetStreamMime = 'application/octet-stream';
                    var contentType = headers('content-type') || octetStreamMime;
                    if (window.navigator.msSaveBlob) {

                        var blob = new Blob([data], {
                            type: contentType
                        });
                        navigator.msSaveBlob(blob, fileName);
                    } else {
                        var urlCreator = window.URL || window.webkitURL || window.mozURL || window.msURL;
                        if (urlCreator) {
                            var a = document.createElement('a');
                            var blob = new Blob([data], {
                                type: contentType
                            });
                            var url = urlCreator.createObjectURL(blob);
                            a.href = url;
                            a.target = '_blank';
                            a.download = fileName;
                            document.body.appendChild(a);
                            a.click();
                        }
                    }
                }).error(function () {
                    SweetAlert.error($translate.instant('PleaseContactAdministrator'));
                });
            });
        };

        //删除文件
        $scope.deleteEvidenceFiles = function (evidence) {
            var deleteList = [];

            deleteList.push(evidence.fileID);


            if (deleteList.length > 0) {
                SweetAlert.swal({
                        title: "warning!",
                        text: $translate.instant('IsDelete'),
                        type: "warning",
                        showCancelButton: true,
                        confirmButtonColor: "#DD6B55",
                        confirmButtonText: $translate.instant('Yes'),
                        cancelButtonText: $translate.instant('No'),
                        closeOnConfirm: true,
                        closeOnCancel: true
                    },
                    function (isConfirm) {
                        if (isConfirm) {
                            vatImportService.deleteEvidences(deleteList).success(function (data) {
                                if (data) {
                                    searchEvidence();
                                }
                            }).error(function () {
                                SweetAlert.error($translate.instant('PleaseContactAdministrator'));
                            });
                        }
                    });
            }
        };

        $scope.commitApprove = function () {
            if (vatSessionService.month)
                vatSessionService.project.period = vatSessionService.month;
            vatApproveService.approvalStatus(vatSessionService.project.id, vatSessionService.project.period).success(function (result) {
                if (result && result == 'committed') {
                    SweetAlert.error('报表提审中！');
                } else {
                    SweetAlert.swal({
                            title: "warning!",
                            text: $translate.instant('报表提审后不能手工录入和重新生成！'),
                            type: "warning",
                            showCancelButton: true,
                            confirmButtonColor: "#DD6B55",
                            confirmButtonText: $translate.instant('Yes'),
                            cancelButtonText: $translate.instant('No'),
                            closeOnConfirm: true,
                            closeOnCancel: true
                        },
                        function (isConfirm) {
                            if (isConfirm) {
                                var approveParam = {};
                                approveParam.projectId = vatSessionService.project.id;
                                approveParam.periodDate = vatSessionService.project.periodDate;
                                vatApproveService.commitNewApproval(approveParam);
                            }
                        });

                }

            });
        }

        $scope.doApprove = function () {
            if (vatSessionService.month)
                vatSessionService.project.period = vatSessionService.month;
            vatApproveService.approvalStatus(vatSessionService.project.id, vatSessionService.project.period).success(function (result) {
                if (result && result == 'committed') {
                    $("#ApprovalComment").modal('show')
                } else {
                    SweetAlert.warning('当前报表未提审，请先提审！');
                }
            });
            ;
        }

        $scope.rejectApproval = function () {
            if (vatSessionService.month)
                vatSessionService.project.period = vatSessionService.month;
            vatApproveService.checkTask(vatSessionService.project.id, vatSessionService.project.period, 'disagreed', $scope.comment).success(function () {
                $("#ApprovalComment").modal('hide');
            });
        }

        $scope.agreeApproval = function () {
            if (vatSessionService.month)
                vatSessionService.project.period = vatSessionService.month;
            vatApproveService.checkTask(vatSessionService.project.id, vatSessionService.project.period, 'agreed', $scope.comment).success(function () {
                vatCommonService.setProjectStatus(vatSessionService.project.id, vatSessionService.month, constant.ProjectStatusEnum.Completed
                    , constant.DictionaryDictKey.WFDataProcess, enums.FinishStatusEnum.Finished);
                $("#ApprovalComment").modal('hide');
            });
        }

        //排序
        function fileSort(i) {
            if ($scope.voucherList.length > 0) {
                if (i == 0) {
                    if (orderfileName) {
                        orderfileName = false;
                        var list = _.sortBy($scope.voucherList, 'fileName');
                        $scope.voucherList = list;
                    } else {
                        orderfileName = true;
                        var list = _.sortBy($scope.voucherList, 'fileName');
                        $scope.voucherList = list.reverse();
                    }
                }
                if (i == 1) {
                    if (orderperiodId) {
                        orderperiodId = false;
                        var list = _.sortBy($scope.voucherList, 'periodID');
                        $scope.voucherList = list;
                    } else {
                        orderperiodId = true;
                        var list = _.sortBy($scope.voucherList, 'periodID');
                        $scope.voucherList = list.reverse();
                    }
                }
                if (i == 2) {
                    if (orderfileType) {
                        orderfileType = false;
                        var list = _.sortBy($scope.voucherList, 'fileType');
                        $scope.voucherList = list;
                    } else {
                        orderfileType = true;
                        var list = _.sortBy($scope.voucherList, 'fileType');
                        $scope.voucherList = list.reverse();
                    }
                }
            }
        }

        //Update comments for file:
        $scope.updateEvidencesComments = function (fileId) {
            var comments = $scope.voucherComments;
            var fileID = fileId.trim();

            var commentsWithFileId = fileID + "||" + comments;

            var commentsWithFileIds = [];
            commentsWithFileIds.push(commentsWithFileId);

            if (fileID.length > 0) {
                vatImportService.updateEvidencesComments(commentsWithFileIds).success(function (data) {
                    if (data) {
                        $("#EditVoucherCommentsPortal").modal('hide');
                        searchEvidence();
                    }
                });
            }
        };
        $scope.showCommentsPopup = function (evidence) {
            $('#EditVoucherCommentsPortal').modal('show');
            $scope.voucherComments = evidence.comments;
            $scope.voucherFileName = evidence.fileName;
            $scope.voucherFileID = evidence.fileID;
        };

        $scope.upLoadManual = function () {
            if (vatSessionService.month)
                vatSessionService.project.period = vatSessionService.month;
            vatApproveService.approvalStatus(vatSessionService.project.id, vatSessionService.project.period).success(function (result) {
                if (result && result == 'committed') {
                    SweetAlert.error('报表提审中！');
                } else {
                    var text = $(".li").find(".active").text();
                    $log.debug(text);
                    if (text == '海关专用缴款书' || text == '代扣代缴税收通用缴款书取数') {
                        var excelIo = new GC.Spread.Excel.IO();
                        $("#importExcellFile").modal('show');
                    } else {
                        SweetAlert.warning("仅支持海关专用缴款书或代扣代缴税收通用缴款书取数");
                    }
                }

            });

        };

        $scope.loadMyExcel = function () {
            if ($scope.file) {
                if ($scope.hasManualLoad) {
                    SweetAlert.warning("已载入文件，如需重新载入请先清空！");
                } else {
                    $scope.manualSpread = new GC.Spread.Sheets.Workbook(document.getElementById("ss"), {sheetCount: 1});
                    var excelIo = new GC.Spread.Excel.IO();
                    var excelFile = $scope.file;
                    excelIo.open(excelFile, function (json) {
                        var workbookObj = json;
                        $scope.manualSpread.fromJSON(workbookObj);
                    }, function (e) {
                        alert(e.errorMessage);
                        if (e.errorCode === 2 /*noPassword*/ || e.errorCode === 3 /*invalidPassword*/) {
                            document.getElementById('password').onselect = null;
                        }
                    }, {});
                    $scope.hasManualLoad = true;
                }
            } else {
                SweetAlert.warning("请先选择文件！");
            }
        };


        $scope.multiWrite = function () {
            if ($scope.hasManualLoad) {
                var manualSheet = $scope.manualSpread.getActiveSheet();
                var currentSheet = $scope.spread.getActiveSheet()
                currentSheet.options.isProtected = false;
                var text = $(".li").find(".active").text();
                for (var rowIndex = 0; rowIndex < currentSheet.getRowCount(); rowIndex++) {
                    for (var columnIndex = 0; columnIndex < currentSheet.getColumnCount(); columnIndex++) {
                        var tagInfo = currentSheet.getTag(rowIndex, columnIndex);
                        if (tagInfo) {
                            var jsonTagInfo = JSON.parse(tagInfo);
                            if (jsonTagInfo.isReadOnly)
                                continue;
                            var cellValue = manualSheet.getValue(rowIndex, columnIndex);
                            var cellString = '';
                            if (typeof (cellValue) == "string") cellValue = cellValue.trim();

                            var r = /^00\d*|^\.\d+|\.$/;
                            if (typeof cellValue == "undefined" || cellValue == null || cellValue == "" || cellValue == "NaN") continue;
                            if (cellValue && (isNaN(cellValue) || r.test(cellValue)) || (text == '代扣代缴税收通用缴款书取数' && columnIndex == 0)) {
                                cellString = cellValue;
                            } else if (cellValue && parseFloat(cellValue).toFixed(2) > 9999999999999) {
                                SweetAlert.warning($translate.instant('CheckInputValueLength'));
                                return;
                            }

                            cellValue = jsonTagInfo.dataType === 5 ? parseInt(cellValue) : parseFloat(cellValue).toFixed(2);
                            var handInputModel = {};
                            if (cellString) {
                                handInputModel = {
                                    cellID: jsonTagInfo.cellID,
                                    cellTemplateID: jsonTagInfo.cellTemplateID,
                                    reportID: jsonTagInfo.reportID,
                                    keyinData: cellString
                                };
                            } else {
                                handInputModel = {
                                    cellID: jsonTagInfo.cellID,
                                    cellTemplateID: jsonTagInfo.cellTemplateID,
                                    reportID: jsonTagInfo.reportID,
                                    amount: cellValue
                                };
                            }

                            $scope.myConfirm(handInputModel);
                        }
                    }
                }
            } else {
                SweetAlert.warning("文件未载入！");
            }
        };


        $scope.myConfirm = function (handInputModel) {
            handInputModel.projectID = vatSessionService.project.id;
            handInputModel.serviceTypeID = vatSessionService.project.serviceTypeID;
            handInputModel.period = vatSessionService.month;

            if (handInputModel.amount || handInputModel.name || handInputModel.keyinData) {
                //日志对象
                logDto.ID = PWC.newGuid();
                logDto.CreateTime = new Date();
                logDto.UpdateTime = new Date();
                logDto.OperationContent = "Amount: " + handInputModel.amount
                    + "; Description: " + handInputModel.description;
                logDto.OperationName = $translate.instant('ManualInputDataSource');
                logDto.Comment = comment + "(Cell ID:" + handInputModel.cellID
                    + ", Cell Template ID:" + handInputModel.cellTemplateID + ")";
                logDto.OperationType = enums.vatLogOperationTypeEnum.RV_ManualInput;

                // 前端保存数据
                return vatReportService.addCellManualData(handInputModel, logDto).then(function (manualData) {
                    var obj = manualData.data.data;
                    obj.dataSourceType = manualData.dataSourceType;
                    $scope.updateCellByManualChange(obj);
                    return $q.when();
                });
            }

            return $q.reject();
        };

        $scope.cleanManual = function () {
            $scope.manualSpread = new GC.Spread.Sheets.Workbook(document.getElementById("ss"), {sheetCount: 1});
            $('#ss').html('');
            $scope.hasManualLoad = false;
            $scope.file = null;
        };

        $scope.closeManualModal = function () {
            loadCellData(getActualPeriod());
            $scope.cleanManual();
            $("#importExcellFile").modal('hide');
        };


        /****************************************证据文件function Ended*************************************************/
        /****************************************证据文件function Ended*************************************************/
        /****************************************证据文件function Ended*************************************************/

        (function initialize() {
            $log.debug('VatReportViewController.initialize()...');
            vatReportCacheService.initReportCache();
            $scope.reportApi = {
                onCellClick: $scope.isDocumentList ? null : cellClick,
                onCellDoubleClick: $scope.isDocumentList ? null : cellDoubleClick
            };
            $scope.cellDetail = {
                cellPosition: null,
                formulaDesc: null,
                dataSources: []
            };
            //单元格的详细信息
            $scope.taxCellDetail = {
                // 单元格类型，根据不同单元格的类型，展示不同的信息。
                // 1|进项数据源，2|进项明细数据源，3|销项发票数据源
                // 4|凭证数据源，5|报表数据源，6|条件判断数据源
                // 7|Sap日报数据源，8|土地出让金数据源，9|未开票数据源
                cellType: 1,
                cellTypeId: undefined,
                cellInfo: null,
                dataSourceCount: 0,
                items: [],
                dataGridSource: [],
                hasKeyIn: false,
                inputValue: '', //手动输入数值
                inputMemo: '' //手动输入备注
            };

            $scope.approveReportLogDto = {
                ID: '',
                ProjectId: vatSessionService.project.id,
                PeriodID: getActualPeriod(),
                OrderIndex: $scope.approveLevel,
                Remark: '',
                OperationTypeID: -1,
                OperationTypeName: '',
                TemplateCode: enums.ReportTemplateCode.TaxDeclarationCategory,
                TemplateName: $translate.instant('TaxReturn'),
                CreatorID: vatSessionService.logUser.id,
                UpdatorID: vatSessionService.logUser.id,
                OperationLog: null
            };

            $scope.approveMessageDto = {
                ProjectId: vatSessionService.project.id,
                PeriodId: getActualPeriod(),
                ProcessDateTime: null,
                NodeDictKey: null,
                TaskDictKey: null,
                MessageTitle: null,
                MessageCategoryId: null,
                MessageDetail: null,
                ApprovedLevel: "-1",
                LogOperationTypeId: -1
            };

            $scope.buttonStatus = {
                viewReportBtShow: false,
                approveReportBtShow: false,
                decCompleteBtShow: false,
                uploadCerficateBtShow: false
            };

            $scope.firstShowDetailForBSPL = true;
            $scope.approveLevel = 0;
            $scope.approveLevelArray = [];

            $scope.approveBtShow = false;
            $scope.submitBtShow = false;
            $scope.showWithdrawBt = false;
            $scope.declareCompleteBtShow = false;
            $scope.uploadCertifiBtShow = false;
            $scope.showRemarksBt = false;

            // vatWorkflowService.getWorkflowDetailByDb(vatSessionService.project.dbName, getActualPeriod())
            //     .success(function (or) {
            //         if (or.result) {
            //             vatSessionService.project.workflow = or.data;
            //             if (!_.isNull(vatSessionService.project.workflow) && !_.isUndefined(vatSessionService.project.workflow)) {
            //                 $scope.approveLevelArray = vatCommonService.getLoginUserApproveReportLevels();
            //                 setSubmitBtShow();
            //                 setApproveBtShow();
            //                 setCompleteBtShow();
            //                 constructRemarksData();
            //             }
            //         }
            //     });

            // if ($scope.templateCode === 'BS001' || $scope.templateCode === 'PL001') {
            //     //show cell detail panel
            //     $scope.isBSPL = true;
            //     $scope.moduleid = enums.vatModuleEnum.Report_BSPL;
            // } else {
            //     $scope.isBSPL = false;
            // }
            $scope.isBSPL = false;

            //load cell data
            loadCellData(getActualPeriod());

            /***************************************************************************************************************/
            /****************************************证据文件function start*************************************************/
            $scope.voucherList = [];
            $scope.isCurrentPeriod = true;
            $scope.queryType = '';
            $scope.queryString = '';
            $scope.searchEvidence = searchEvidence;
            //$scope.selectEvidenceFile = selectEvidenceFile;
            // $scope.deleteEvidenceFiles = deleteEvidenceFiles(evidence);
            $scope.getEvidenceTitle = getEvidenceTitle;
            //$scope.fileSort = fileSort;
            // searchEvidence();
            $scope.voucherFile = [];
            $scope.voucherFile.length = 1;
            $scope.$watch('voucherFile', function (newValue, oldValue) {
                if (newValue != null && newValue !== oldValue) {
                    //验证文件数量和大小
                    if (validationEvidenceFile($scope.voucherFile)) {
                        uploadEvidenceFile($scope.voucherFile);
                    }
                }
            });
            /****************************************证据文件function start*************************************************/
            /***************************************************************************************************************/

        })();
    }
]);