﻿commonModule.factory('spreadJsTipService', ['$rootScope', '$log', '$uibModal', '$translate',
    function ($rootScope, $log, $uibModal, $translate) {
        'use strict';

        var hasInvoiceImg;
        var hasVoucherImg;
        var hasKeyInImg;
        var hasModelImg;
        var hasExceptionImg;
        var hasValidationImg;

        var heightOffset = 150;
        var widthOffset = 250;

        var initialize = function (sheet) {

            sheet.suspendPaint();

            hasVoucherImg = new Image();
            hasVoucherImg.src = '/app-resources/images/vat/hasVoucher.png';
            hasVoucherImg.onload = function () {
                sheet.repaint();
            };

            hasInvoiceImg = new Image();
            hasInvoiceImg.src = '/app-resources/images/vat/hasInvoice.png';
            hasInvoiceImg.onload = function () {
                sheet.repaint();
            };

            hasKeyInImg = new Image();
            hasKeyInImg.src = '/app-resources/images/vat/hasKeyIn.png';
            hasKeyInImg.onload = function () {
                sheet.repaint();
            };

            hasModelImg = new Image();
            hasModelImg.src = '/app-resources/images/vat/hasModel.png';
            hasModelImg.onload = function () {
                sheet.repaint();
            };

            hasExceptionImg = new Image();
            hasExceptionImg.src = '/app-resources/images/vat/hasException.png';
            hasExceptionImg.onload = function () {
                sheet.repaint();
            };

            hasValidationImg = new Image();
            hasValidationImg.src = '/app-resources/images/vat/hasValidation.png';
            hasValidationImg.onload = function () {
                sheet.repaint();
            };

        };

        var paintSheet = function (sheet) {
            sheet.resumePaint();
        };

        var setCellTipString = function (cell, tip) {
            //cell.cellType(new IconCellType(null, null, 0, null, tip));
            cell.cellType(new IconsCellType([], [], tip));
        };

        var setCellTipByCellData = function (cell, cellData) {
            var tips = null;
            //BSPL
            if (cellData && cellData.formulaDescription) {
                tips = cellData.formulaDescription;
            }
            if (cellData && cellData.cellTemplateConfig && !_.isEmpty(cellData.cellTemplateConfig.formulaDescription)) {
                tips = cellData.cellTemplateConfig.formulaDescription;
            }

            var icons = [];
            var infos = [];

            var hasException = false;
            var exceptionMsg = '';
            if (cellData.hasModelError) {
                hasException = true;
                exceptionMsg = $translate.instant('ModelException');
            }

            if (!_.isEmpty(cellData.validationErrorList)) {
                hasException = true;
                if (_.isEmpty(exceptionMsg)) {
                    exceptionMsg = $translate.instant('ValidationException');
                }
                else {
                    exceptionMsg = exceptionMsg + '; ' + $translate.instant('ValidationException');
                }
            }

            if (hasException) {
                cell.value(cell.text() + '   '); // Edmond TODO: Temp move cell value for CIT demo
                icons.push(hasExceptionImg);
                infos.push(exceptionMsg);
            }

            try {
                cell.cellType(new IconsCellType(icons, infos, tips));
            }
            catch (e) {
            }
        };

        var setCellTip = function (cell, config) {
            var tips = null;
            if (config.formulaDescription) {
                tips = config.formulaDescription;
            }
            //if (config.hasVoucher && !config.hasInvoice) {
            //    cell.cellType(new IconCellType(hasVoucherImg, null, 1, [getHoverTip('voucher', config), null], tips)).hAlign(0);
            //}else if (config.hasInvoice && !config.hasVoucher) {
            //    cell.cellType(new IconCellType(hasInvoiceImg, null, 1, [getHoverTip('invoice', config), null], tips)).hAlign(0);
            //}else if (config.hasInvoice && config.hasVoucher) {
            //    cell.cellType(new IconCellType(hasVoucherImg, hasInvoiceImg, 2, [getHoverTip('voucher', config), getHoverTip('invoice', config)], tips)).hAlign(0);
            //} else if (config.formulaDescription) {
            //    cell.cellType(new IconCellType(null, null, 0, null, tips)).hAlign(0);
            //}

            var icons = [];
            var infos = [];
            if (config.validation) {
                icons.push(hasValidationImg);
                infos.push($translate.instant('ReportValidation'));
            }
            if (config.hasModel) {
                icons.push(hasModelImg);
                infos.push($translate.instant('LinkModel'));
            }
            if (config.hasKeyIn) {
                icons.push(hasKeyInImg);
                infos.push($translate.instant('ManualInput'));
            }
            if (config.hasVoucher) {
                icons.push(hasVoucherImg);
                infos.push(getHoverTip('voucher', config));
            }
            if (config.hasInvoice) {
                icons.push(hasInvoiceImg);
                infos.push(getHoverTip('invoice', config));
            }

            //--不需要左对齐对齐
            if (tips === null) {
                cell.cellType(new IconsCellType(icons, infos, tips));
            } else {
                cell.cellType(new IconsCellType(icons, infos, tips)).hAlign(0);
            }
        };

        var getHoverTip = function (tipType, config) {
            if (config) {
                if (tipType === 'voucher') {
                    return $translate.instant('VoucherTip') + getCodejoinString(config.accountCodes);
                } else if (tipType === 'invoice') {
                    return $translate.instant('InvoiceTip') + getInvoiceNameTip(config.invoiceType, config) + getInvoiceAmountName(config.invoiceAmountType);
                }
            }
        };

        var getCodejoinString = function (array) {
            if (array) {
                return array.join(',');
            } else {
                return '';
            }
        };

        var getInvoiceNameTip = function (invoiceType, config) {
            var taxRateStr = _.contains(config.taxRate, constant.selectAllValue) ? '' : ($translate.instant('TaxRate') + getCodejoinString(config.taxRate));
            if (invoiceType === 1) {
                return $translate.instant('Income') + '：' + taxRateStr;
            } else if (invoiceType === 2) {
                return $translate.instant('Output') + '：' + taxRateStr;
            } else if (invoiceType === 3) {
                return $translate.instant('CustomsAudit') + '：';
            } else {
                return '';
            }
        };

        var getInvoiceAmountName = function (invoiceAmountType) {
            if (invoiceAmountType === 1) {
                return $translate.instant('TotalAmount');
            } else if (invoiceAmountType === 2) {
                return $translate.instant('TaxAmount');
            } else if (invoiceAmountType === 3) {
                return $translate.instant('CopyNumber');
            } else {
                return '';
            }
        };

        var IconsCellType = function (icons, infos, formula) {
            if (icons && icons.length > 0) {
                this.Icons = icons;
                this.count = icons.length;
                this.Infos = infos;
                this.Formula = formula;
            } else {
                this.count = 0;
                this.Formula = formula;
            }
        };

        //var IconCellType = function (icon1, icon2, count, infos, formula) {

        //    this.icon1 = icon1;
        //    this.icon2 = icon2;

        //    this.count = count;
        //    this.Infos = infos;

        //    this.Formula = formula;
        //}


        IconsCellType.prototype = new GC.Spread.Sheets.CellTypes.Base();
        IconsCellType.prototype.paint = function (ctx, value, x, y, w, h, style, context) {
            GC.Spread.Sheets.CellTypes.Base.prototype.paint.call(this, ctx, value, x, y, w, h, style, context);
            for (var i = 1; i <= this.count; i++) {
                //距离当前单元格左侧距离，距离顶部距离 ， ICON宽度，ICON高度
                ctx.drawImage(this.Icons[i - 1], x + w - (h - 3) * i + (h - 6 - 18) * i, y + 3 + (h / 2 - 3 - 9), 18, 18);

                //if (i == 1 && this.icon1) {
                //    //距离当前单元格左侧距离，距离顶部距离 ， ICON宽度，ICON高度
                //    ctx.drawImage(this.icons[i], x + w - (h - 3) * i + (h - 6 - 18) * i, y + 3 + (h / 2 - 3 - 9), 18, 18);
                //}

                //if (i == 2 && this.icon2) {
                //    //第二个图标相对于第一个图标向左偏移一个单位的宽度加间隙
                //    ctx.drawImage(this.icon2, x + w - (h - 3) * i + (h - 6 - 18) * i, y + 3 + (h / 2 - 3 - 9), 18, 18);
                //}
            }
        };
        IconsCellType.prototype.getHitInfo = function (x, y, cellStyle, cellRect, context) {
            var index = x - (cellRect.x + cellRect.width - (cellRect.height - 3) * this.count) > 0 ?
                Math.floor(this.count - (x - (cellRect.x + cellRect.width - (cellRect.height - 3) * this.count)) / (cellRect.height - 3)) :
                -1;

            return {
                x: x,
                y: y,
                row: context.row,
                col: context.col,
                cellStyle: cellStyle,
                cellRect: cellRect,
                sheetArea: context.sheetArea,
                reservedLocationIndex: index
            };
        };
        IconsCellType.prototype.processMouseMove = function (hitinfo) {

            //console.log(hitinfo);
            //if (!this.Formula) {
            //    return;
            //}
            //console.log(hitinfo.reservedLocationIndex);
            var tipInfo = this.Formula;

            if (hitinfo.reservedLocationIndex >= 0) {
                tipInfo = this.Infos[hitinfo.reservedLocationIndex];
                //console.log(this.Infos[hitinfo.reservedLocationIndex]);
            }

            //如果没有Tip信息就不再做tip显示。
            if (!tipInfo) {
                return;
            }

            if (this._toolTipElement) {
                $(this._toolTipElement).text(tipInfo)
                    .css("top", hitinfo.y + 15 + heightOffset)
                    .css("left", hitinfo.x + 15 + widthOffset);
            }
            else {
                var div = document.createElement("div");
                $(div).css("position", "absolute")
                    .css("border", "1px #C0C0C0 solid")
                    .css("box-shadow", "1px 2px 5px rgba(0,0,0,0.4)")
                    .css("font", "9pt Arial")
                    .css("background", "white")
                    .css("max-width", "400px")
                    .css("word-wrap", "break-word")
                    .css("padding", 5);

                this._toolTipElement = div;
                $(this._toolTipElement).css("top", hitinfo.y + 15 + heightOffset)
                    .css("left", hitinfo.x + 15 + widthOffset);
                $(this._toolTipElement).hide();
                document.body.insertBefore(this._toolTipElement, null);
                $(this._toolTipElement).show("fast");
            }


        };
        IconsCellType.prototype.processMouseLeave = function (hitinfo) {
            if (this._toolTipElement) {
                document.body.removeChild(this._toolTipElement);
                this._toolTipElement = null;
            }
        };

        return {
            initialize: initialize,
            setCellTip: setCellTip,
            setCellTipString: setCellTipString,
            setCellTipByCellData: setCellTipByCellData,
            paintSheet: paintSheet
        };
    }]);