﻿vatModule.factory('dataTransformService', ['$translate', function ($translate) {
    'use strict';

    var prefixField = 'c_';

    _.groupByMulti = function (obj, values, context) {
        if (!values.length)
            return obj;
        var byFirst = _.groupBy(obj, values[0], context),
            rest = values.slice(1);
        for (var prop in byFirst) {
            byFirst[prop] = _.groupByMulti(byFirst[prop], rest, context);
        }
        return byFirst;
    };

    var computeValue = function (oneValue, oneGroup) {
        if (_.isEmpty(oneGroup)) {
            return null;
        }
        switch (oneValue.method) {
            case 'sum':
                var sum = _.reduce(_.pluck(oneGroup, oneValue.field),
                    function (memo, num) { return memo + num; }, 0);
                return _.isNumber(sum) ? sum : null;
            case 'avg':
                var sum = _.reduce(_.pluck(oneGroup, oneValue.field),
                    function (memo, num) { return memo + num; }, 0);
                return _.isNumber(sum) ? sum / oneGroup.length : 0;
            case 'max':
                var max = _.max(oneGroup, function (obj) { return obj[oneValue.field] })[oneValue.field];
                return max ? max : null;
            case 'min':
                var min = _.min(oneGroup, function (obj) { return obj[oneValue.field] })[oneValue.field];
                return min ? min : null;
            case 'count':
                return oneGroup.length;
            default:
                return null;
        }
    };

    //sourceData=[];
    //valueList: compute function
    //argumentFieldList: group by
    var computeGroup = function (sourceData, valueList, argumentFieldList) {
        //first group by argumentFieldList
        //groupData=[key:[],key1:[]]       
        var groupDicData = _.groupByMulti(sourceData, argumentFieldList);
        var newDataArray = [];
        var seriesArray = [];
        var argumentField = argumentFieldList[0];
        //dimension  index      data structure
        // 2          1         newData={'groupkey':value, 'groupkey1':''};seriesArray = [{'name':groupkey1, 'valueField':'groupkey1'}]
        // 1          n         newData={'month:'15/11','v0':number,'v1':number,...};seriesArray=[{name:'sum(xxxx)',valueField:'v0'},{name:'max(xxxx)',valueField:'v1'},...]

        if (argumentFieldList.length === 1) {
            var flag = true;
            _.each(groupDicData, function (groupData, key) {
                var newData = {};
                newData[argumentField] = key;
                for (var i = 0; i < valueList.length; i++) {
                    var oneValue = valueList[i];
                    var secondFieldKey = 'v' + i;                  
                    //fill data to series
                    if (flag) {
                        var fieldDisplayName = prefixField + valueList[i].field;
                        seriesArray.push({
                            valueField: secondFieldKey,
                            name: valueList[i].method + '(' + $translate.instant(fieldDisplayName) + ')'
                        });
                    }
                    newData[secondFieldKey] = computeValue(oneValue, groupData);
                }
                flag = false;
                newDataArray.push(newData);
            });

        } else if (argumentFieldList.length > 1) {
            _.each(groupDicData, function (groupData, key) {

                var newData = {};
                newData[argumentField] = key;
                var arName = _.first(_.keys(groupData));
                var group = groupData[arName];             
                addSeries(seriesArray, arName, arName);
                newData[arName] = computeValue(valueList[0], group);
                newDataArray.push(newData);

            });
        }
        return { dataSource: newDataArray, series: seriesArray, argumentField: argumentField };
    };

    var addSeries = function (seriesArray, sencondFieldName, sencondArgumentFieldValue) {
        var containIt = false;
        for (var i = 0; i < seriesArray.length ; i++) {
            if (seriesArray[i].valueField === sencondFieldName) {
                containIt = true;
                break;
            }
        }

        if (!containIt) {
            seriesArray.push({ valueField: sencondFieldName, name: sencondArgumentFieldValue });
        }
    };


    var getMeasuresValueList = function (valueField) {
        if (valueField.length === 0) {
            return new Array();
        }

        var valueList = new Array();

        var valueStrList = valueField.split(",");

        for (var i = 0 ; i < valueStrList.length; i++) {
            var methodArray = /sum|avg|count|max|min/g.exec(valueStrList[i]);
            var valuefieldArray = /\((.*?)\)/g.exec(valueStrList[i]);
            valueList.push({ method: methodArray[0], field: getPrefixedFieldName(valuefieldArray[1]) });
        }

        return valueList;
    };

    var getDimensionList = function (valueField) {
        if (valueField.length === 0) {
            return new Array();
        }

        var valueList = new Array();

        var valueStrList = valueField.split(",");

        valueStrList.forEach(function (e) {
            valueList.push(getPrefixedFieldName(e));
        });

        return valueList;
    };

    var fixFieldList = function (valueField) {
        var newValueField = new Array();
        for (var i = 0 ; i < valueField.length ; i++) {
            newValueField.push({ method: valueField[i].method, field: getFieldName(valueField[i].field) });
        }
        return newValueField;
    };

    var getFieldName = function (prefixedfiedName) {
        return prefixedfiedName;
        //return prefixedfiedName.length > 2 ? prefixedfiedName.substring(2) : '';
    }

    //lower fist letter
    var getPrefixedFieldName = function (fileName) {
        return fileName.length >= 1 ? fileName.substring(0, 1).toLowerCase() + fileName.substring(1) : '';
    };

    var getDimensionValue = function (argumentList) {
        if (typeof (argumentList) === 'undefined') {
            return '';
        }

        return argumentList.join(',');
    };

    var getMeasuresValue = function (measuresList) {
        if (typeof (measuresList) === 'undefined') {
            return '';
        }

        var valStr = '';

        measuresList.forEach(function (e) {
            valStr = valStr + e.method + '(' + e.field + '),';
        });

        if (valStr.length > 0) {
            valStr = valStr.substring(0, valStr.length - 1);
        }

        return valStr;
    };

    var getDisplayName = function (type, value, projectList) {
        for (var pindex = 0; pindex < projectList.length; pindex++) {
            if (type === 'regionID' && projectList[pindex]['regionId'] === value) {
                return projectList[pindex]['regionName'];
            };

            if (type === 'projectID' && projectList[pindex]['projectId'] === value) {
                return projectList[pindex]['projectName'];
            };
        }
    };

    var formatRegionProject = function (dataList, projectList) {
        for (var index = 0; index < dataList.length; index++) {
            dataList[index].vatType = $translate.instant('vattype_' + dataList[index].vatType);
            dataList[index].rateType = $translate.instant('ratetype_' + dataList[index].rateType);
            dataList[index].regionID = getDisplayName('regionID', dataList[index].regionID, projectList);
            dataList[index].projectID = getDisplayName('projectID', dataList[index].projectID, projectList);

            //month format
            var monthStr = dataList[index].month >= 10 ? dataList[index].month.toString() : '0' + dataList[index].month.toString();
            dataList[index].month = dataList[index].year.toString().substring(2, 4) + '/' + monthStr; //beacause number will be compressed, convert to string
        }
    };

    return {
        getTransformResult: function (dataSource, argumentFieldList, valueField) {
            //var valueList = getValueList(valueField);

            var fixedValueField = fixFieldList(valueField);
            var argumentField = getFieldName(argumentFieldList[0]);
            return computeGroup(dataSource, fixedValueField, argumentFieldList);
        },

        getFieldName: getFieldName,

        getPrefixedFieldName: getPrefixedFieldName,

        getShowName: function (fiedName) {
            return prefixField + fiedName;
        },

        getMeasuresValueList: getMeasuresValueList,

        getDimensionList: getDimensionList,

        getDimensionValue: getDimensionValue,

        getMeasuresValue: getMeasuresValue,

        formatRegionProject: formatRegionProject
    };
}]);