﻿vatModule.controller('VatImportIncomeInvoiceController', ['$scope', '$log', 'loginContext', '$translate', 'apiInterceptor', 'Upload', 'SweetAlert', 'vatImportService', '$timeout', '$q', 'uiGridConstants', '$interval', 'vatSessionService', 'enums', 'vatOperationLogService', 'projectService', 'vatCommonService',
function ($scope, $log, loginContext, $translate, apiInterceptor, Upload, SweetAlert, vatImportService, $timeout, $q, uiGridConstants, $interval, vatSessionService, enums, vatOperationLogService, projectService, vatCommonService) {
    'use strict';

    var uploadUrl = apiInterceptor.vatWebApiHostUrl + '/FileUpload/NewFile';
    var successCount = 0;
    $scope.chunkSize = 100000;
    var resumable = true;
    $scope.columnNum = 0;
    $scope.detailColumnNum = 0;
    var curPeriod = vatSessionService.month;
    var curYear = vatSessionService.project.year;
    var projectId = vatSessionService.project.id;

    $scope.showImportTable = false;//是否显示导入table
    $scope.showInitTable = false;//是否显示初始数据ui-grid
    $scope.showInputInvoiceErrorTable = false;//是否显示错误信息列表

    $scope.showImprotItemTable = false;
    $scope.showInitItemTable = false;
    $scope.showInputInvoiceItemErrorTable = false;//是否显示错误信息列表


    //写日志
    //导入方式
    $scope.importEnum = { Import: 0, CoverImport: 1, AddImport: 2 };
    var comment = vatSessionService.project.name + " " + vatSessionService.project.year + "年" + vatSessionService.month + "月";
    $scope.period = vatSessionService.month;
    $scope.moduleid = enums.vatModuleEnum.Import_InputInvoice;
    var logDto = {
        ID: '',
        OperationName: '',
        ModuleID: $scope.moduleid,
        //OperationObject: $translate.instant('IncomeInvoiceTitle'),
        OperationType: '',
        OperationContent: '',
        OriginalState: '',
        UpdateState: '',
        CreatorID: vatSessionService.logUser.ID,
        Comment: comment,
        IP: '',
        Period: $scope.period,
    };


    var validationErrorType = {
        ColumnsMapErrorType: $translate.instant('ColumnsMapErrorType'),
        InvoiceInfoEmptyCheckErrorType: $translate.instant('InvoiceInfoEmptyCheckErrorType'),
        InvoiceTypeEmptyCheckErrorType: $translate.instant('InvoiceTypeEmptyCheckErrorType'),
        CertificationDateEmptyCheckErrorType: $translate.instant('CertificationDateEmptyCheckErrorType'),
        InvoiceDateEmptyCheckErrorType: $translate.instant('InvoiceDateEmptyCheckErrorType'),
        AmountNumberCheckErrorType: $translate.instant('AmountNumberCheckErrorType'),
        TaxAmountNumberCheckErrorType: $translate.instant('TaxAmountNumberCheckErrorType'),
        TaxRateNumberCheckErrorType: $translate.instant('TaxRateNumberCheckErrorType'),
        InvoiceDateErrorType: $translate.instant('InvoiceDateErrorType'),
        CertificationDateFormatCheckErrorType: $translate.instant('CertificationDateFormatCheckErrorType'),
        CertificationDateRangeCheckErrorType: $translate.instant('CertificationDateRangeCheckErrorType'),
        OnlyInvoiceItemCheckErrorType: $translate.instant('OnlyInvoiceItemCheckErrorType'),
        StartRowError: $translate.instant('StartRowError'),
        NumOutRangeErrorType: $translate.instant('NumOutRangeErrorType'),
        OutputStringLengthType: $translate.instant('OutputStringLengthType'),
        InvoiceTypeErrorType: $translate.instant('InvoiceTypeErrorType'),
        InvoiceItemInfoEmptyCheckErrorType: $translate.instant('InvoiceItemInfoEmptyCheckErrorType')
    };

    var invoiceTypeEnum = {
        VATInvoice: $translate.instant('VATInvoice'),
        FreightTransport: $translate.instant('FreightTransport'),
        MotorVehicle: $translate.instant('MotorVehicle'),
        AgriculturalProduct: $translate.instant('AgriculturalProduct')
    };

    //定义进项发票类型数组
    var arrInvoiceTypeEnum = [
        $translate.instant('VATInvoice'),
        $translate.instant('FreightTransport'),
        $translate.instant('MotorVehicle'),
        $translate.instant('AgriculturalProduct')];

    //初始化ack-pagination
    $scope.pagingOptions = {
        pageIndex: 1, //当前页码
        totalItems: 0, //总数据
        totalPages: 10, //总页数
        maxSize: 5, //分页数字的限制。
        pageSize: constant.page.pageSizeArrary[3], //每页多少条数据,100条
        pageSizeString: constant.page.pageSizeArrary[3].toString(),
        firstPage: $translate.instant('PagingFirstPage'),
        previousPage: $translate.instant('PagingPreviousPage'),
        nextPage: $translate.instant('PagingNextPage'),
        lastPage: $translate.instant('PagingLastPage'),
    };

    $scope.totalData = {
        totalNumber: 0,
        amount: 0,
        taxAmount: 0
    };

    //根据当前的月份获取当月第一天日期
    var getCurrentMonthFirstDay = function () {
        var curMonth = curPeriod;
        if (curPeriod < 10) {
            curMonth = '0' + curPeriod;
        }
        var oneDay = 1000 * 60 * 60 * 8;
        return new Date(new Date(curYear + '-' + curMonth + '-' + '01') - oneDay)
    };

    //根据当前的月份获取当月的最后一天日期（实际获取的是下月的第一天00：00：00，因此在判断时应设置为小于而不是小于等于）
    var getCurrentMonthLastDay = function () {
        var curMonth = curPeriod;
        if (curPeriod < 10) {
            curMonth = '0' + curPeriod;
        }
        var date = new Date(curYear + '-' + curMonth + '-' + '01');
        var currentMonth = date.getMonth();
        var nextMonth = ++currentMonth;
        var nextMonthFirstDay = new Date(date.getFullYear(), nextMonth, 1);
        return new Date(nextMonthFirstDay);
    };


    //转换发票类型
    function translateInvoiceType(strType) {
        var type = 1;
        switch (strType) {
            case invoiceTypeEnum.VATInvoice:
                type = 1;
                break;
            case invoiceTypeEnum.FreightTransport:
                type = 2;
                break;
            case invoiceTypeEnum.MotorVehicle:
                type = 3;
                break;
            case invoiceTypeEnum.AgriculturalProduct:
                type = 4;
                break;
            default:
                type = 1;
        }
        return type;
    }


    /****************************************汇总表function start*************************************************/


    //初始化列数据
    var initColumns = function () {
        //初始化进项发票汇总表列名
        $scope.incomeInvoiceTotalColumns = [
            $translate.instant('PleaseSelectColumn'),
            $translate.instant('InvoiceFPDM'),
            $translate.instant('InvoiceFPHM'),
            $translate.instant('InvoiceKPRQ'),
            $translate.instant('InvoiceGHFSH'),
            $translate.instant('InvoiceJE'),
            $translate.instant('InvoiceSE'),
            $translate.instant('InvoiceRZRQ'),
            $translate.instant('InvoiceRZJG'),
            $translate.instant('InvoiceFPLX')
        ];


        $scope.initIncomeInvoiceTotalColumnsIndex = _.range(1, $scope.incomeInvoiceTotalColumns.length);

        //必须进行map的列
        $scope.mustMappingTotalColumns = [
            //$translate.instant('PleaseSelectColumn'),
            $translate.instant('InvoiceFPDM'),
            $translate.instant('InvoiceFPHM'),
            $translate.instant('InvoiceKPRQ'),
            $translate.instant('InvoiceGHFSH'),
            $translate.instant('InvoiceJE'),
            $translate.instant('InvoiceSE'),
            $translate.instant('InvoiceRZRQ'),
            $translate.instant('InvoiceRZJG'),
            $translate.instant('InvoiceFPLX')
        ];
    };

    //从数据库中load数据，如果有，则页面导入按钮为覆盖导入或追加导入，否则为导入按钮
    var loadIncomeInvoiceDataFromDB = function () {
        vatImportService.getInputInvoiceList(curPeriod).success(function (data) {
            if (data) {
                $scope.sheetData = { sheetNameList: [], dataList: [], selectedSheetIndex: 0 };
                var index = 1;
                var amountSum = 0;
                var taxAmountSum = 0;
                data.forEach(function (v) {
                    v.index = index++;
                    if (!_.isNaN(parseFloat(v.amount) && !_.isNaN(parseFloat(v.taxAmount)))) {
                        amountSum += v.amount;
                        taxAmountSum += v.taxAmount;
                    }
                    v.amount = PWC.round(v.amount, 2);
                    v.taxAmount = PWC.round(v.taxAmount, 2);
                });

                initPagingControl(data.length);
                $scope.totalData.totalNumber = data.length;
                $scope.totalData.amount = PWC.round(amountSum, 2);
                $scope.totalData.taxAmount = PWC.round(taxAmountSum, 2);
                var skipCount = ($scope.pagingOptions.pageIndex - 1) * $scope.pagingOptions.pageSize;
                var pagingData = _.chain(data).rest(skipCount).first($scope.pagingOptions.pageSize).value();

                $scope.initIncomeInvoiceList = pagingData;
                $scope.gridOptionsIncomeInvoiceTotal.data = pagingData;
                $scope.queryIncomeInvoiceResult = pagingData;
                if (pagingData.length >= 0) {
                    if (pagingData.length > 0) {
                        $scope.isShowImportTotalBtn = false;
                    }
                    $scope.showInitTable = true;
                    $scope.showImportTable = false;
                }

                //获取错误列表信息
                getInputInvoiceValidationList().then(function () {
                    if ($scope.gridOptionsInputInvoiceErrorMsg.data.length > 0) {
                        showInputInvoiceErrTab();
                        $scope.toggleInputInvoiceErrorTab();

                        //当有错误信息时，显示错误信息表
                        $scope.showInputInvoiceErrorTable = true;
                    }
                    else {
                        $scope.InputInvoiceImportErrorTab = false;
                        $scope.InputInvoiceImportErrorTag = false;
                        $scope.showInputInvoiceErrorTable = false;
                    }
                });
            }
        });

    };

    //发票信息汇总数据表选择列名事件
    //parentIndex：当前列号
    //index：下拉列表中选项的序号
    var mappingColumn = function (parentIndex, index, colName) {
        if (colName) {

            var oldColumn = $scope.selectedColumnMap[parentIndex];
            var newColumn = $scope.incomeInvoiceTotalColumns[index];

            //如果新的选择项与老的选择项一致，则不进行操作
            if (oldColumn === newColumn) {
                return;
            }
            else {
                $scope.selectedColumnMap[parentIndex] = colName;

                if (colName !== $translate.instant('PleaseSelectColumn')) {
                    //如果当前选择的不是"请选择列名",则将选项从数组中移除
                    $scope.incomeInvoiceTotalColumns.splice(index, 1);

                    if (oldColumn && oldColumn !== $translate.instant('PleaseSelectColumn')) {
                        //将老的选项追加到选项列表中
                        $scope.incomeInvoiceTotalColumns.push(oldColumn);
                    }
                }
                else {
                    //如果当前选中的是"请选择列名",则将该列原来的列名追加到选项列表中
                    if (oldColumn) {
                        $scope.incomeInvoiceTotalColumns.push(oldColumn);
                    }
                }
            }
        }
        $log.debug($scope.selectedColumnMap);
    };

    //设置上传文件的文件名
    var updateIncomeInvoceTotalFileName = function (newValue) {
        $scope.fileName = newValue.name;
    };

    var token = $('input[name="__RequestVerificationToken"]').val();
    //上传文件
    var doUploadIncomeInvoiceTotalFile = function (file) {
        if (file) {
            var arr = file.name.split('.');
            if (arr[arr.length - 1] !== 'xls' && arr[arr.length - 1] !== 'xlsx') {
                SweetAlert.warning($translate.instant('ImportFileInvalidType'));
                return;
            }
            $scope.initIncomeInvoiceList = [];
            $scope.gridOptionsIncomeInvoiceTotal.data = [];
            successCount = 0;
            if (!file.$error) {
                var tempFileName = PWC.newGuid() + '.dat';
                Upload.upload({
                    url: uploadUrl,
                    data: {
                        filename: file.name,
                        tempFileName: tempFileName,
                        file: file
                    },
                    resumeChunkSize: resumable ? $scope.chunkSize : null,
                    headers: {
                        'Access-Control-Allow-Origin': '*',
                        Authorization: apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken()
                    },
                    __RequestVerificationToken: token,
                    withCredentials: true
                }).then(uploadIncomeInvoiceTotalSuccess);
            }
        }
    };

    //上传成功后执行函数
    var uploadIncomeInvoiceTotalSuccess = function (resp) {
        successCount++;
        if (successCount === 1) {
            $scope.tempFileName = resp.data;
            //初始化到excel文件的第一个sheet
            changeIncomeInvoiceSheet(0);
        }
    };

    //初始化需要展示的sheet
    var changeIncomeInvoiceSheet = function (selectedSheet) {
        var sheetIndex = _.findIndex($scope.sheetData.sheetNameList, function (num) {
            return num.name === selectedSheet.name;
        })
        $scope.sheetInfo.selectedSheetIndex = sheetIndex === -1 ? 0 : sheetIndex;
        refreshIncomeInvoiceTable();
    };

    //重新load页面上的table
    var refreshIncomeInvoiceTable = function () {
        $scope.showImportTable = true;
        $scope.showInitTable = false;
        $scope.showInputInvoiceErrorTable = false;

        if (_.isString($scope.tempFileName)) {
            $scope.selectedColumnMap = [];
            $scope.columnNum = 0;
            $scope.sheetData = { sheetNameList: [], dataList: [], selectedSheetIndex: 0 };
            vatImportService.getFileContent($scope.tempFileName, $scope.sheetInfo.selectedSheetIndex, $scope.sheetInfo.topRowNumber).success(function (data) {
                //将导入文件的sheet赋值到数组
                $scope.sheetData = data;
                //加入index序号
                var index = 1;
                data.dataList.forEach(function (v) {
                    v.index = index++;
                });
                var sheetNameObjectArray = [];
                data.sheetNameList.forEach(function (v) {
                    sheetNameObjectArray.push({ 'name': v });
                });

                $scope.sheetData.sheetNameList = sheetNameObjectArray;

                $scope.columnNum = data.dataList[0] ? data.dataList[0].length : $scope.columnNum;
                $scope.firstDataRow = data.dataList[0];
                $scope.startRowList = _.range(data.lastRowIndex);
                $scope.sheetInfo.selectedSheetName = $scope.sheetData.sheetNameList[$scope.sheetInfo.selectedSheetIndex];
                initColumns();

                vatSessionService.dataChanged = true; //设置为true表示在切换页面前数据有变更, 需要提示

                if (data && data.result === false) {
                    SweetAlert.warning(data.resultMsg);
                    return;
                }
            });
        }
    };

    //对导入数据进行验证(前端)
    var FrontEndValidIncomeInvoiceData = function (inputInvoiceList) {

        //2.验证关键字段不为空（发票代码，发票号码，认证日期，开票日期）
        //3.验证数据格式是否正确（金额，税额，认证日期，开票日期）
        if (!KeyColumnsEmptyValidIncomeInvoice(inputInvoiceList)) {
            return;
        }
    };

    //1.判断是否有没有map的必须列和起始行输入格式是否正确
    var ColumnsMapCheckIncomeInvoice = function () {
        var result = true;
        //1：判断是否有没有map的必须列
        var notMappedColumns = '';
        if ($scope.selectedColumnMap.length >= 0) {
            var count1 = 0;
            $scope.mustMappingTotalColumns.forEach(function (m) {
                if (m !== $translate.instant('PleaseSelectColumn')) {
                    if ($.inArray(m, $scope.selectedColumnMap) < 0) {
                        notMappedColumns += m + ',';
                        count1++;
                    }
                }
            });
            if (count1) {
                result = false;
                //加入到错误列表中
                $scope.errorMsgMap.push(
                    {
                        'errorType': validationErrorType.ColumnsMapErrorType,
                        'errorCount': count1,
                        'errorContent': $translate.instant('ColumnsMapErrorMsg').formatObj({ columns: notMappedColumns.substring(0, notMappedColumns.length - 1) })
                    });
            }
        }
        //2:判断起始行输入框输入内容是否合理
        var startNumberCheckMsg = '';
        var startRowNum = $('#StartRowNum').val();
        if (startRowNum === '') {
            result = false;
            $scope.errorMsgMap.push(
                    {
                        'errorType': validationErrorType.StartRowError,
                        'errorCount': 1,
                        'errorContent': $translate.instant('StartRowNull')
                    });
        }
        else if (parseInt(startRowNum) > $scope.sheetData.dataList.length) {
            result = false;
            $scope.errorMsgMap.push(
                    {
                        'errorType': validationErrorType.StartRowError,
                        'errorCount': 1,
                        'errorContent': $translate.instant('StartRowNumberCheckMsg')
                    });
        }
        return result;
    };

    //2.验证关键字段不为空（发票代码，发票号码，认证日期，开票日期）
    var KeyColumnsEmptyValidIncomeInvoice = function (inputInvoiceList) {
        var result = true;
        //#region 定义错误信息以及错误数量

        //发票信息
        var invoiceInfoEmptyCheckMsg = '';
        var invoiceInfoEmptyCheckCount = 0;
        //发票类型
        var invoiceTypeEmptyCheckMsg = '';
        var invoiceTypeEmptyCheckCount = 0;

        //认证日期
        var certificationDateEmptyCheckMsg = '';
        var certificationDateEmptyCheckCount = 0;
        //开票日期
        var invoiceDateEmptyCheckMsg = '';
        var invoiceDateEmptyCheckCount = 0;

        //金额正确性验证
        var amountNumberCheckMsg = '';
        var amountNumberCheckCount = 0;

        //税额正确性验证
        var taxAmountNumberCheckMsg = '';
        var taxAmountNumberCheckCount = 0;

        //认证日期格式验证
        var certificationDateCheckMsg = '';
        var certificationDateCheckCount = 0;
        //开票日期格式验证
        var invoiceDateCheckMsg = '';
        var invoiceDateCheckCount = 0;
        //认证日期与选定区间不符
        var certificationDateRangeCheckMsg = '';
        var certificationDateRangeCheckCount = 0;

        //字符长度过长验证
        var stringOverLengthCheckMsg = '';
        var stringOverLengthCheckCount = 0;

        //数值范围验证
        var numOutRangeCheckMsg = '';
        var numOutRangeCheckCount = 0;

        //发票类型错误
        var invoiceTypeCheckMsg = '';
        var invoiceTypeCheckCount = 0;

        //#endregion

        //#region 验证错误

        inputInvoiceList.forEach(function (m) {

            //发票信息
            if (PWC.isNullOrEmpty(m.InvoiceCode) || PWC.isNullOrEmpty(m.InvoiceNumber) || PWC.isNullOrEmpty(m.InvoiceType)) {
                invoiceInfoEmptyCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceInfoEmptyCheckErrorMsg') + ';<br />';
                invoiceInfoEmptyCheckCount++;
            }
            else {
                //发票代码过长超过50个字符
                if (PWC.getStringLength(m.InvoiceCode) > constant.validateLength.StringLength_50) {
                    stringOverLengthCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceFPDM') + $translate.instant('OutputStringLengthMsg') + ';<br />';
                    stringOverLengthCheckCount++;
                }
                //发票号码过长超过50个字符
                if (PWC.getStringLength(m.InvoiceNumber) > constant.validateLength.StringLength_50) {
                    stringOverLengthCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceFPHM') + $translate.instant('OutputStringLengthMsg') + ';<br />';
                    stringOverLengthCheckCount++;
                }
            }


            //发票类型
            if (!PWC.isNullOrEmpty(m.InvoiceType)) {
                //发票类型过长超过50个字符
                if (PWC.getStringLength(m.InvoiceType) > constant.validateLength.StringLength_50) {
                    stringOverLengthCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceFPLX') + $translate.instant('OutputStringLengthMsg') + ';<br />';
                    stringOverLengthCheckCount++;
                }
                else {
                    //如果发票类型不在指定的发票类型数组中，则提示错误
                    if ($.inArray(m.InvoiceType, arrInvoiceTypeEnum) === -1) {
                        invoiceTypeCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceFPLX') + $translate.instant('InvoiceTypeErrorMsg') + ';<br />';
                        invoiceTypeCheckCount++;
                    }
                    else {
                        m.InvoiceType = translateInvoiceType(m.InvoiceType);
                    }
                }
            }

            //销方识别号过长超过50个字符
            if (PWC.getStringLength(m.SellerTaxNumber) > constant.validateLength.StringLength_50) {
                stringOverLengthCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceGHFSH') + $translate.instant('OutputStringLengthMsg') + ';<br />';
                stringOverLengthCheckCount++;
            }
            //认证结果过长超过50个字符
            if (PWC.getStringLength(m.CertificationResult) > constant.validateLength.StringLength_50) {
                stringOverLengthCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceRZJG') + $translate.instant('OutputStringLengthMsg') + ';<br />';
                stringOverLengthCheckCount++;
            }
            //认证日期
            if (PWC.isNullOrEmpty(m.CertificationDate)) {
                certificationDateEmptyCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('CertificationDateNullMsg') + ';<br />';
                certificationDateEmptyCheckCount++;
            }
            else {
                //判断认证日期格式是否正确
                if (!PWC.checkDate(m.CertificationDate)) {
                    certificationDateCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('CertificationDateFormatMsg') + ';<br />';
                    certificationDateCheckCount++;
                }
                //else {
                //    //如果认证日期不在选定期间
                //    if (!(new Date(m.CertificationDate.replace(/[\.-]/g, '/')) >= getCurrentMonthFirstDay() && new Date(m.CertificationDate.replace(/[\.-]/g, '/')) < getCurrentMonthLastDay())) {
                //        certificationDateRangeCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('CertificationDateCheckErrorMsg') + ';<br />';;
                //        certificationDateRangeCheckCount++;
                //    }
                //}
            }

            //开票日期
            if (PWC.isNullOrEmpty(m.InvoiceDate)) {
                invoiceDateEmptyCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceDateNullMsg') + ';<br />';
                invoiceDateEmptyCheckCount++;
            }
            else {
                //判断开票日期格式是否正确
                if (!PWC.checkDate(m.InvoiceDate)) {
                    invoiceDateCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceDateErrorMsg') + ';<br />';
                    invoiceDateCheckCount++;
                }
            }

            //判断金额是否为数字
            if (isNaN(parseFloat(m.Amount))) {
                amountNumberCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('AmountNumberCheckErrorMsg') + ';<br />';
                amountNumberCheckCount++;
            }
                //如果在规定的数值范围外，则提示错误
            else if (parseFloat(m.Amount) < constant.startNum || parseFloat(m.Amount) > constant.endNum) {
                numOutRangeCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceJE') + $translate.instant('NumOverLengthMsg') + ';<br />';
                numOutRangeCheckCount++;
            }
            else {
                m.Amount = PWC.round(m.Amount, 2);
            }

            //判断税额是否为数字
            if (isNaN(parseFloat(m.TaxAmount))) {
                taxAmountNumberCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('TaxAmountNumberCheckErrorMsg') + ';<br />';;
                taxAmountNumberCheckCount++;
            }
                //如果在规定的数值范围外，则提示错误
            else if (parseFloat(m.TaxAmount) < constant.startNum || parseFloat(m.TaxAmount) > constant.endNum) {
                numOutRangeCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceSE') + $translate.instant('NumOverLengthMsg') + ';<br />';
                numOutRangeCheckCount++;
            }
            else {
                m.TaxAmount = PWC.round(m.TaxAmount, 2);
            }
        });

        //#endregion

        //#region 将错误信息push到列表中

        //push发票信息空信息到错误列表
        if (invoiceInfoEmptyCheckCount > 0) {
            result = false;
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.InvoiceInfoEmptyCheckErrorType,
                    'errorCount': invoiceInfoEmptyCheckCount,
                    'errorContent': invoiceInfoEmptyCheckMsg
                });
        }
        //push发票类型空信息到错误列表
        if (invoiceTypeEmptyCheckCount > 0) {
            result = false;
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.InvoiceTypeEmptyCheckErrorType,
                    'errorCount': invoiceTypeEmptyCheckCount,
                    'errorContent': invoiceTypeEmptyCheckMsg
                });
        }
        //push发票类型错误信息到错误列表
        if (invoiceTypeCheckCount > 0) {
            result = false;
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.InvoiceTypeErrorType,
                    'errorCount': invoiceTypeCheckCount,
                    'errorContent': invoiceTypeCheckMsg
                });
        }
        //push认证日期空信息到错误列表
        if (certificationDateEmptyCheckCount > 0) {
            result = false;
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.CertificationDateEmptyCheckErrorType,
                    'errorCount': certificationDateEmptyCheckCount,
                    'errorContent': certificationDateEmptyCheckMsg
                });
        }
        //push开票日期空信息到错误列表
        if (invoiceDateEmptyCheckCount > 0) {
            result = false;
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.InvoiceDateEmptyCheckErrorType,
                    'errorCount': invoiceDateEmptyCheckCount,
                    'errorContent': invoiceDateEmptyCheckMsg
                });
        }
        //push金额错误信息到错误列表
        if (amountNumberCheckCount > 0) {
            result = false;
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.AmountNumberCheckErrorType,
                    'errorCount': amountNumberCheckCount,
                    'errorContent': amountNumberCheckMsg
                });
        }
        //push税额错误信息到错误列表
        if (taxAmountNumberCheckCount > 0) {
            result = false;
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.TaxAmountNumberCheckErrorType,
                    'errorCount': taxAmountNumberCheckCount,
                    'errorContent': taxAmountNumberCheckMsg
                });
        }
        //push认证日期错误信息到错误列表
        if (certificationDateCheckCount > 0) {
            result = false;
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.CertificationDateFormatCheckErrorType,
                    'errorCount': certificationDateCheckCount,
                    'errorContent': certificationDateCheckMsg
                });
        }
        //push开票日期错误信息到错误列表
        if (invoiceDateCheckCount > 0) {
            result = false;
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.InvoiceDateErrorType,
                    'errorCount': invoiceDateCheckCount,
                    'errorContent': invoiceDateCheckMsg
                });
        }
        //push认证日期不在导入期间的错误信息到错误列表
        if (certificationDateRangeCheckCount > 0) {
            result = false;
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.CertificationDateRangeCheckErrorType,
                    'errorCount': certificationDateRangeCheckCount,
                    'errorContent': certificationDateRangeCheckMsg
                });
        }
        //push字符串长度过长错误信息到错误列表
        if (stringOverLengthCheckCount > 0) {
            result = false;
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.OutputStringLengthType,
                    'errorCount': stringOverLengthCheckCount,
                    'errorContent': stringOverLengthCheckMsg
                });
        }
        //push数值超出范围信息到错误列表
        if (numOutRangeCheckCount > 0) {
            result = false;
            $scope.errorMsgMap.push(
                {
                    'errorType': validationErrorType.NumOutRangeErrorType,
                    'errorCount': numOutRangeCheckCount,
                    'errorContent': numOutRangeCheckMsg
                });
        }

        //#endregion

        return result;
    };

    function doImportInputInvoiceData(importType) {
        if (!$scope.fileName) {
            SweetAlert.warning($translate.instant('PleaseSelectFileFirst'));
            return;
        }

        var importType = importType;//0：第一次导入，1：覆盖导入，2：追加导入

        //设置数据导入时的起始行,默认设置从第一行开始
        var startRowNum = 1;
        if ($('#StartRowNum').val()) {
            startRowNum = $('#StartRowNum').val();
        }


        //验证前，将错误数据清空
        $scope.errorMsgMap = [];

        //判断是否有没有map的必须列和起始行输入格式是否正确
        if (!ColumnsMapCheckIncomeInvoice()) {
            $('#errorListModal').modal('show');
            return;
        }

        //将需要导入的数据组装到list中
        var inputInvoiceList = [];
        for (var i = startRowNum - 1; i < $scope.sheetData.dataList.length; i++) {
            var invoiceInfo = {
                'Index': $scope.sheetData.dataList[i].index,//行号
                'PeriodID': curPeriod,
                'InvoiceCode': $scope.sheetData.dataList[i][$.inArray($translate.instant('InvoiceFPDM'), $scope.selectedColumnMap)],
                'InvoiceNumber': $scope.sheetData.dataList[i][$.inArray($translate.instant('InvoiceFPHM'), $scope.selectedColumnMap)],
                'InvoiceDate': $scope.sheetData.dataList[i][$.inArray($translate.instant('InvoiceKPRQ'), $scope.selectedColumnMap)],
                'SellerTaxNumber': $scope.sheetData.dataList[i][$.inArray($translate.instant('InvoiceGHFSH'), $scope.selectedColumnMap)],
                'Amount': PWC.isNullOrEmpty($scope.sheetData.dataList[i][$.inArray($translate.instant('InvoiceJE'), $scope.selectedColumnMap)]) ?
                    0 : $scope.sheetData.dataList[i][$.inArray($translate.instant('InvoiceJE'), $scope.selectedColumnMap)],
                'TaxAmount': PWC.isNullOrEmpty($scope.sheetData.dataList[i][$.inArray($translate.instant('InvoiceSE'), $scope.selectedColumnMap)]) ?
                    0 : $scope.sheetData.dataList[i][$.inArray($translate.instant('InvoiceSE'), $scope.selectedColumnMap)],
                'CertificationDate': $scope.sheetData.dataList[i][$.inArray($translate.instant('InvoiceRZRQ'), $scope.selectedColumnMap)],
                'CertificationResult': $scope.sheetData.dataList[i][$.inArray($translate.instant('InvoiceRZJG'), $scope.selectedColumnMap)],
                'InvoiceType': $scope.sheetData.dataList[i][$.inArray($translate.instant('InvoiceFPLX'), $scope.selectedColumnMap)]
                //translateInvoiceType($scope.sheetData.dataList[i][$.inArray($translate.instant('InvoiceFPLX'), $scope.selectedColumnMap)])
            };
            inputInvoiceList.push(invoiceInfo);
        }



        //前台数据验证
//        FrontEndValidIncomeInvoiceData(inputInvoiceList); todo: to be open futrue(neo)

        if ($scope.errorMsgMap.length > 0) {
            $('#errorListModal').modal('show');
            return;
        }
        else {
            logDto.ID = PWC.newGuid();
            logDto.CreateTime = new Date();
            logDto.UpdateTime = new Date();
            logDto.OperationContent = $scope.fileName;
            logDto.OperationObject = $translate.instant('IncomeInvoiceTotal');
            if (importType === $scope.importEnum.Import) {
                logDto.OperationName = $translate.instant('ImportInputInvoice');
                logDto.OperationType = enums.vatLogOperationTypeEnum.Import;
            }
            else if (importType === $scope.importEnum.CoverImport) {
                logDto.OperationName = $translate.instant('CoverImportInputInvoice');
                logDto.OperationType = enums.vatLogOperationTypeEnum.CoverImport;
            }
            else if (importType === $scope.importEnum.AddImport) {
                logDto.OperationName = $translate.instant('AddImportInputInvoice');
                logDto.OperationType = enums.vatLogOperationTypeEnum.AddImport;
            }


            vatImportService.importInputInvoiceData(inputInvoiceList, importType).success(function (or) {
                if (or && or.result) {
                    logDto.UpdateState = $translate.instant('ImportSuccess');
                    vatOperationLogService.addOperationLog(logDto);
                    vatCommonService.importSetProjectStatus(projectDbName, curPeriod, constant.DictionaryDictKey.WFImportIncomeInvoice
                                                            , enums.FinishStatusEnum.Finished);
                    vatCommonService.setImportSubStatus(enums.VatImportSubStatus.isInputInvoiceImport, true);
                    $log.debug("import inputInvoice importInputInvoiceData: " + vatSessionService.project.importSubStatus.isInputInvoiceImport);

                    SweetAlert.success($translate.instant('ImportSuccess'));
                    loadIncomeInvoiceDataFromDB();
                    $scope.fileName = '';
                    vatSessionService.dataChanged = $scope.showInitItemTable ? false : true;//设置为false表示在切换页面前数据没有变更,不需要提示
                }
            }).error(function () {
                logDto.UpdateState = $translate.instant('ImportFail');
                vatOperationLogService.addOperationLog(logDto);
            });
        }
    }
    
    //点击导入按钮时，进行导入操作
    var importInputInvoiceData = function (importType) {
        if (vatSessionService.project.projectStatusList[curPeriod] >= constant.ProjectStatusEnum.Generated) {
            swal({
                title: "warning!",
                text: $translate.instant('IsConfirmToReImport').formatObj({ status: vatCommonService.getProjectStautsEnumDesc(vatSessionService.project.projectStatusList[curPeriod])}),
                type: "warning",
                showCancelButton: true,
                confirmButtonColor: "#DD6B55",
                confirmButtonText: $translate.instant('Yes'),
                cancelButtonText: $translate.instant('No'),
                closeOnConfirm: true,
                closeOnCancel: true
            },
            function (isConfirm) {
                if (isConfirm) {
                    doImportInputInvoiceData(importType);
                }
                else {
                    swal.close();
                }
            });
        }
        else {
            doImportInputInvoiceData(importType);
        }
    };

    //动态设置grid高度
    $scope.getGridTotalHeight = function () {
        if ($scope.isLoadInputInvoiceComplete) {
            return { height: ($('#input-invoice-grid-wrapper').height()) + "px" };
        }
        else {
            return { height: 0 + "px" };
        }
    };

    //刷新发票汇总信息（重新验证）
    var refreshInputInvoiceData = function () {
        logDto.ID = PWC.newGuid();
        logDto.CreateTime = new Date();
        logDto.UpdateTime = new Date();
        logDto.OperationContent = '';
        logDto.OperationName = $translate.instant('RefreshInputInvoiceValidationData');
        logDto.OperationType = enums.vatLogOperationTypeEnum.Refresh;
        logDto.OperationObject = $translate.instant('IncomeInvoiceTotal');

        vatImportService.validateInputInvoiceInfo(curPeriod).success(function () {
            logDto.UpdateState = $translate.instant('RefreshSuccess');
            vatOperationLogService.addOperationLog(logDto);

            $('#inputInvoiceDuplicateItemModal').modal('hide');
            SweetAlert.success($translate.instant('RefreshSuccess'));
            loadIncomeInvoiceDataFromDB();
        }).error(function () {
            logDto.UpdateState = $translate.instant('RefreshTbFail');
            vatOperationLogService.addOperationLog(logDto);

            SweetAlert.error($translate.instant('PleaseContactAdministrator'));
        });

        vatSessionService.dataChanged = $scope.showInitItemTable ? false : true;
    };

    //清除当期发票汇总信息（同时会清掉发票明细数据）
    var clearInputInvoiceData = function () {
        swal({
            title: "warning!",
            text: $translate.instant('InputInvoiceClearTips'),
            type: "warning",
            showCancelButton: true,
            confirmButtonColor: "#DD6B55",
            confirmButtonText: $translate.instant('Yes'),
            cancelButtonText: $translate.instant('No'),
            closeOnConfirm: true,
            closeOnCancel: true
        }, function (isConfirm) {
            if (isConfirm) {
                logDto.ID = PWC.newGuid();
                logDto.CreateTime = new Date();
                logDto.UpdateTime = new Date();
                logDto.OperationContent = '';
                logDto.OperationName = $translate.instant('ClearInputInvoice');
                logDto.OperationType = enums.vatLogOperationTypeEnum.ClearData;
                logDto.OperationObject = $translate.instant('IncomeInvoiceTotal');

                vatImportService.clearInputInvoiceData(curPeriod).success(function (data) {
                    if (data && data.result) {
                        logDto.UpdateState = $translate.instant('ClearSuccess');
                        vatOperationLogService.addOperationLog(logDto);

                        vatCommonService.clearProjectStatus(projectDbName, curPeriod, constant.DictionaryDictKey.WFImportIncomeInvoice
                                                            , enums.FinishStatusEnum.NotFinished);
                        vatCommonService.setImportSubStatus(enums.VatImportSubStatus.isInputInvoiceImport, false);
                        $log.debug("clear inputInvoice clearInputInvoiceData: " + vatSessionService.project.importSubStatus.isInputInvoiceImport);
                        SweetAlert.success($translate.instant('ClearSuccess'));
                        loadIncomeInvoiceDataFromDB();
                        loadIncomeInvoiceItemDataFromDB();
                        $scope.isShowImportTotalBtn = true;
                        $scope.isShowImportItemBtn = true;
                    }
                    else {
                        logDto.UpdateState = $translate.instant('ClearFailed');
                        vatOperationLogService.addOperationLog(logDto);

                        SweetAlert.warning($translate.instant('ClearFailed'));
                    }
                }).error(function () {
                    logDto.UpdateState = $translate.instant('ClearFailed');
                    vatOperationLogService.addOperationLog(logDto);

                    SweetAlert.error($translate.instant('PleaseContactAdministrator'));
                });
            }
        });
    };

    //获取发票汇总验证信息
    var getInputInvoiceValidationList = function () {
        var deferred = $q.defer();

        vatImportService.getValidationDtoInfo(enums.validationType.InputVoice, curPeriod).success(function (data) {
            if (data.length > 0) {
                var index = 1;
                data.forEach(function (v) {
                    v.index = index++;
                });
                $scope.gridOptionsInputInvoiceErrorMsg.data = data;
            }
            else {
                $scope.gridOptionsInputInvoiceErrorMsg.data = [];
            }
            deferred.resolve();
        });

        return deferred.promise;
    }

    var showInputInvoiceErrTab = function () {
        $scope.InputInvoiceImportErrorTab = true;//默认显示顶部block
        $scope.InputInvoiceImportErrorTag = true;//默认隐藏下方的错误信息列表
        $scope.inputInvoiceErrorMsg = $translate.instant('ImportErrorMsg').formatObj({ "NumberOfError": $scope.gridOptionsInputInvoiceErrorMsg.data.length });

    };

    $scope.toggleInputInvoiceErrorTab = function () {
        $scope.InputInvoiceImportErrorTag = !$scope.InputInvoiceImportErrorTag;

        // topIcon and content-resize gapBottom 15px
        if (!$scope.InputInvoiceImportErrorTag) {
            setInputInvoiceErrorWrapCssDefault();
        } else {

            if (parseInt($('#input-invoice-content-resizer').css('bottom')) < 100) {
                $('#input-invoice-content-resizer').css('bottom', '130px');
                $('#input-invoice-error-info-wrapper').css('height', '130px');
            }
        }
    };

    var setInputInvoiceErrorWrapCssDefault = function () {
        $('#input-invoice-content-resizer').css('bottom', '20px');
        $('#input-invoice-error-info-wrapper').css('height', '0px');
    };

    //获取发票汇总错误信息grid高度
    $scope.getInputInvoiceErrorGridHeight = function () {
        if ($scope.isLoadInputInvoiceComplete) {
            var y = $("#input-invoice-error-info-wrapper").height();

            // Enough space
            if (y > constant.UIGrid.gapHeight) {
                y = y - constant.UIGrid.gapHeight;
                return { height: y + "px" };
            } else {

                return { height: '0px' };
            }
        }

        return {};
    };

    //查看汇总发票日志
    var showInputInvoiceOperateLogPop = function () {
        $scope.isShowInputInvoiceLog = true;
    };

    /****************************************汇总表function end*************************************************/


    /****************************************明细表function start*************************************************/

    //初始化明细表列数据
    var initDetailColumns = function () {
        //初始化进项发票汇总表列名
        $scope.incomeInvoiceDetailColumns = [
            $translate.instant('PleaseSelectColumn'),
            $translate.instant('InvoiceFPDM'),
            $translate.instant('InvoiceFPHM'),
            $translate.instant('InvoiceJE'),
            $translate.instant('InvoiceSE'),
            $translate.instant('InvoiceSL'),
            $translate.instant('InvoiceHWMC')
        ];

        $scope.initIncomeInvoiceDetailColumnsIndex = _.range(1, $scope.incomeInvoiceDetailColumns.length);

        //必须进行map的列
        $scope.mustMappingItemColumns = [
            $translate.instant('InvoiceFPDM'),
            $translate.instant('InvoiceFPHM'),
            $translate.instant('InvoiceJE'),
            $translate.instant('InvoiceSE'),
            $translate.instant('InvoiceSL'),
            $translate.instant('InvoiceHWMC')
        ];
    };

    //从数据库中load数据，如果有，则页面导入按钮为覆盖导入或追加导入，否则为导入按钮
    var loadIncomeInvoiceItemDataFromDB = function () {
        vatImportService.getInputInvoiceItemList(curPeriod).success(function (data) {
            if (data) {
                $scope.sheetDataItem = { sheetNameList: [], dataList: [], selectedSheetIndex: 0 };
                var index = 1;
                data.forEach(function (v) {
                    v.index = index++;
                    v.amount = PWC.round(v.amount, 2);
                    v.taxAmount = PWC.round(v.taxAmount, 2);
                });
                $scope.initIncomeInvoiceDetailList = data;
                $scope.gridOptionsIncomeInvoiceItem.data = data;
                $scope.queryIncomeInvoiceItemResult = data;
                if (data.length >= 0) {
                    if (data.length > 0) {
                        $scope.isShowImportItemBtn = false;
                    }
                    $scope.showInitItemTable = true;
                    $scope.showImportItemTable = false;
                }

                //获取错误列表信息
                getInputInvoiceItemValidationList().then(function () {
                    if ($scope.gridOptionsInputInvoiceItemErrorMsg.data.length > 0) {
                        showInputInvoiceItemErrTab();
                        $scope.toggleInputInvoiceItemErrorTab();

                        //当有错误信息时，显示错误信息表
                        $scope.showInputInvoiceItemErrorTable = true;
                    }
                    else {
                        $scope.InputInvoiceItemImportErrorTab = false;
                        $scope.InputInvoiceItemImportErrorTag = false;
                        $scope.showInputInvoiceItemErrorTable = false;
                    }
                });
            }
        });
    };

    //发票信息汇总数据表选择列名事件
    //parentIndex：当前列号
    //index：下拉列表中选项的序号
    var mappingDetailColumn = function (parentIndex, index, colName) {
        if (colName) {

            var oldColumn = $scope.selectedColumnMapItem[parentIndex];
            var newColumn = $scope.incomeInvoiceDetailColumns[index];

            //如果新的选择项与老的选择项一致，则不进行操作
            if (oldColumn === newColumn) {
                return;
            }
            else {
                $scope.selectedColumnMapItem[parentIndex] = colName;

                if (colName !== $translate.instant('PleaseSelectColumn')) {
                    //如果当前选择的不是"请选择列名",则将选项从数组中移除
                    $scope.incomeInvoiceDetailColumns.splice(index, 1);

                    if (oldColumn && oldColumn !== $translate.instant('PleaseSelectColumn')) {
                        //将老的选项追加到选项列表中
                        $scope.incomeInvoiceDetailColumns.push(oldColumn);
                    }
                }
                else {
                    //如果当前选中的是"请选择列名",则将该列原来的列名追加到选项列表中
                    if (oldColumn) {
                        $scope.incomeInvoiceDetailColumns.push(oldColumn);
                    }
                }
            }
        }
        $log.debug($scope.selectedColumnMapItem);
    };

    //设置上传文件的文件名
    var updateIncomeInvoceDetailFileName = function (newValue) {
        $scope.detailFileName = newValue.name;
    };

    //上传文件
    var doUploadIncomeInvoiceDetailFile = function (file) {
        if (file) {
            var arr = file.name.split('.');
            if (arr[arr.length - 1] !== 'xls' && arr[arr.length - 1] !== 'xlsx') {
                SweetAlert.warning($translate.instant('ImportFileInvalidType'));
                return;
            }
            $scope.initIncomeInvoiceDetailList = [];
            $scope.gridOptionsIncomeInvoiceItem.data = [];
            successCount = 0;
            if (!file.$error) {
                var tempFileName = PWC.newGuid() + '.dat';
                Upload.upload({
                    url: uploadUrl,
                    data: {
                        filename: file.name,
                        tempFileName: tempFileName,
                        file: file
                    },
                    resumeChunkSize: resumable ? $scope.chunkSize : null,
                    headers: {
                        'Access-Control-Allow-Origin': '*',
                        Authorization: apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken()
                    },
                    __RequestVerificationToken: token,
                    withCredentials: true
                }).then(uploadIncomeInvoiceDetailSuccess);
            }
        }
    };

    //上传成功后执行函数
    var uploadIncomeInvoiceDetailSuccess = function (resp) {
        successCount++;
        if (successCount === 1) {
            $scope.tempDetailFileName = resp.data;
            //初始化到excel文件的第一个sheet
            changeIncomeInvoiceDetailSheet(0);
        }
    };

    //初始化需要展示的sheet
    var changeIncomeInvoiceDetailSheet = function (selectedSheet) {
        var sheetIndex = _.findIndex($scope.sheetDataItem.sheetNameList, function (num) {
            return num.name === selectedSheet.name;
        })
        $scope.sheetInfoDetail.selectedSheetIndex = sheetIndex === -1 ? 0 : sheetIndex;
        refreshIncomeInvoiceDetailTable();
    };

    //重新load页面上的table
    var refreshIncomeInvoiceDetailTable = function () {
        $scope.showImportItemTable = true;
        $scope.showInitItemTable = false;
        $scope.showInputInvoiceItemErrorTable = false;

        if (_.isString($scope.tempDetailFileName)) {
            $scope.selectedColumnMapItem = [];
            $scope.detailColumnNum = 0;
            $scope.sheetDataItem = { sheetNameList: [], dataList: [], selectedSheetIndex: 0 };
            vatImportService.getFileContent($scope.tempDetailFileName, $scope.sheetInfoDetail.selectedSheetIndex, $scope.sheetInfoDetail.topRowNumber).success(function (data) {

                //将导入文件的sheet赋值到数组
                $scope.sheetDataItem = data;
                //加入index序号
                var index = 1;
                data.dataList.forEach(function (v) {
                    v.index = index++;
                });
                var sheetNameObjectArray = [];
                data.sheetNameList.forEach(function (v) {
                    sheetNameObjectArray.push({ 'name': v });
                });
                $scope.sheetDataItem.sheetNameList = sheetNameObjectArray;

                $scope.detailColumnNum = data.dataList[0] ? data.dataList[0].length : $scope.detailColumnNum;;
                $scope.firstDetailDataRow = data.dataList[0];
                $scope.startRowListDetail = _.range(data.lastRowIndex);
                $scope.sheetInfoDetail.selectedSheetName = $scope.sheetDataItem.sheetNameList[$scope.sheetInfoDetail.selectedSheetIndex];
                initDetailColumns();

                vatSessionService.dataChanged = true; //设置为true表示在切换页面前数据有变更, 需要提示

                if (data && data.result === false) {
                    SweetAlert.warning(data.resultMsg);
                    return;
                }
            });
        }
    };

    //对导入数据进行验证(前端)
    var FrontEndValidIncomeInvoiceItemData = function (inputInvoiceItemList) {

        //3.验证关键字段不为空（发票代码，发票号码）
        //4.验证数据格式是否正确（金额，税额，税率）
        if (!KeyColumnsEmptyValidIncomeInvoiceItem(inputInvoiceItemList)) {
            return;
        }
    };

    //1.单独导入进项明细表check
    var OnlyInvoiceItemCheck = function () {
        var result = true;
        //如果进项汇总表没有数据，则提示不允许导入
        if ($scope.gridOptionsIncomeInvoiceTotal.data.length === 0) {
            result = false;
            $scope.itemErrorMsgMap.push(
               {
                   'errorType': validationErrorType.OnlyInvoiceItemCheckErrorType,
                   'errorCount': 1,
                   'errorContent': $translate.instant('OnlyInvoiceItemCheckErrorMsg')
               });
        }
        return result;
    };

    //2.判断是否有没有map的必须列和起始行输入格式是否正确
    var ColumnsMapCheckIncomeInvoiceItem = function () {
        var result = true;
        //1：判断是否有没有map的必须列
        var notMappedColumns = '';
        if ($scope.selectedColumnMapItem.length >= 0) {
            var count1 = 0;
            $scope.mustMappingItemColumns.forEach(function (m) {
                if (m !== $translate.instant('PleaseSelectColumn')) {
                    if ($.inArray(m, $scope.selectedColumnMapItem) < 0) {
                        notMappedColumns += m + ',';
                        count1++;
                    }
                }
            });
            if (count1) {
                result = false;
                //加入到错误列表中
                $scope.itemErrorMsgMap.push(
                    {
                        'errorType': validationErrorType.ColumnsMapErrorType,
                        'errorCount': count1,
                        'errorContent': $translate.instant('ColumnsMapErrorMsg').formatObj({ columns: notMappedColumns.substring(0, notMappedColumns.length - 1) })
                    });
            }
        }

        //2:判断起始行输入框输入内容是否合理
        var startNumberCheckMsg = '';
        var startRowNum = $('#ItemStartRowNum').val();
        if (startRowNum === '') {
            result = false;
            $scope.itemErrorMsgMap.push(
                    {
                        'errorType': validationErrorType.StartRowError,
                        'errorCount': 1,
                        'errorContent': $translate.instant('StartRowNull')
                    });
        }
        else if (parseInt(startRowNum) > $scope.sheetDataItem.dataList.length) {
            result = false;
            $scope.itemErrorMsgMap.push(
                    {
                        'errorType': validationErrorType.StartRowError,
                        'errorCount': 1,
                        'errorContent': $translate.instant('StartRowNumberCheckMsg')
                    });
        }

        return result;
    };

    //3.验证关键字段不为空（发票代码，发票号码）
    var KeyColumnsEmptyValidIncomeInvoiceItem = function (inputInvoiceItemList) {
        var result = true;
        //#region 定义错误信息

        //发票信息
        var invoiceInfoEmptyCheckMsg = '';
        var invoiceInfoEmptyCheckCount = 0;

        //金额
        var amountNumberCheckMsg = '';
        var amountNumberCheckCount = 0;
        //税额
        var taxAmountNumberCheckMsg = '';
        var taxAmountNumberCheckCount = 0;
        //税率
        var taxRateNumberCheckMsg = '';
        var taxRateNumberCheckCount = 0;

        //数据范围超出范围
        var numOutRangeCheckMsg = '';
        var numOutRangeCheckCount = 0;

        //字符长度过长验证
        var stringOverLengthCheckMsg = '';
        var stringOverLengthCheckCount = 0;
        //#endregion

        //#region 进行验证操作

        inputInvoiceItemList.forEach(function (m) {
            //发票信息
            if (PWC.isNullOrEmpty(m.InvoiceCode) || PWC.isNullOrEmpty(m.InvoiceNumber)) {
                invoiceInfoEmptyCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceItemInfoEmptyCheckErrorMsg') + ';<br />';
                invoiceInfoEmptyCheckCount++;
            }
            else {
                //发票代码过长超过50个字符
                if (PWC.getStringLength(m.InvoiceCode) > constant.validateLength.StringLength_50) {
                    stringOverLengthCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceFPDM') + $translate.instant('OutputStringLengthMsg') + ';<br />';
                    stringOverLengthCheckCount++;
                }
                //发票号码过长超过50个字符
                if (PWC.getStringLength(m.InvoiceNumber) > constant.validateLength.StringLength_50) {
                    stringOverLengthCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceFPHM') + $translate.instant('OutputStringLengthMsg') + ';<br />';
                    stringOverLengthCheckCount++;
                }
            }

            //判断金额是否为数字
            if (isNaN(parseFloat(m.Amount))) {
                amountNumberCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('AmountNumberCheckErrorMsg') + ';<br />';
                amountNumberCheckCount++;
            }
                //如果在规定的数值范围外，则提示错误
            else if (parseFloat(m.Amount) < constant.startNum || parseFloat(m.Amount) > constant.endNum) {
                numOutRangeCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceJE') + $translate.instant('NumOverLengthMsg') + ';<br />';
                numOutRangeCheckCount++;
            }
            else {
                m.Amount = PWC.round(m.Amount, 2);
            }

            //判断税额是否为数字
            if (isNaN(parseFloat(m.TaxAmount))) {
                taxAmountNumberCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('TaxAmountNumberCheckErrorMsg') + ';<br />';
                taxAmountNumberCheckCount++;
            }
                //如果在规定的数值范围外，则提示错误
            else if (parseFloat(m.TaxAmount) < constant.startNum || parseFloat(m.TaxAmount) > constant.endNum) {
                numOutRangeCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceSE') + $translate.instant('NumOverLengthMsg') + ';<br />';
                numOutRangeCheckCount++;
            }
            else {
                m.TaxAmount = PWC.round(m.TaxAmount, 2);
            }


            //判断税率是否为固定的数字
            if ($.inArray(m.TaxRate, constant.effectiveTaxRate) < 0) {
                taxRateNumberCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('TaxRateNumberCheckErrorMsg') + ';<br />';;
                taxRateNumberCheckCount++;
            }
            else {
                if (m.TaxRate.indexOf('%') > 0) {
                    m.TaxRate = m.TaxRate.substring(0, m.TaxRate.length - 1) * 0.01;
                }
            }

            //判断货物名称长度是否超过50个字符
            if (PWC.getStringLength(m.ProductionName) > constant.validateLength.StringLength_50) {
                stringOverLengthCheckMsg += $translate.instant('RowIndex').formatObj({ rowIndex: m.Index }) + $translate.instant('InvoiceHWMC') + $translate.instant('OutputStringLengthMsg') + ';<br />';
                stringOverLengthCheckCount++;
            }

        });

        //#endregion

        //#region 组装错误信息

        //push发票信息空信息到错误列表
        if (invoiceInfoEmptyCheckCount > 0) {
            result = false;
            $scope.itemErrorMsgMap.push(
                {
                    'errorType': validationErrorType.InvoiceItemInfoEmptyCheckErrorType,
                    'errorCount': invoiceInfoEmptyCheckCount,
                    'errorContent': invoiceInfoEmptyCheckMsg
                });
        }
        //push金额错误信息到错误列表
        if (amountNumberCheckCount > 0) {
            result = false;
            $scope.itemErrorMsgMap.push(
                {
                    'errorType': validationErrorType.AmountNumberCheckErrorType,
                    'errorCount': amountNumberCheckCount,
                    'errorContent': amountNumberCheckMsg
                });
        }
        //push税额错误信息到错误列表
        if (taxAmountNumberCheckCount > 0) {
            result = false;
            $scope.itemErrorMsgMap.push(
                {
                    'errorType': validationErrorType.TaxAmountNumberCheckErrorType,
                    'errorCount': taxAmountNumberCheckCount,
                    'errorContent': taxAmountNumberCheckMsg
                });
        }
        //push税率错误信息到错误列表
        if (taxRateNumberCheckCount > 0) {
            result = false;
            $scope.itemErrorMsgMap.push(
                {
                    'errorType': validationErrorType.TaxRateNumberCheckErrorType,
                    'errorCount': taxRateNumberCheckCount,
                    'errorContent': taxRateNumberCheckMsg
                });
        }
        //push数值超出范围的错误信息到错误列表中
        if (numOutRangeCheckCount > 0) {
            result = false;
            $scope.itemErrorMsgMap.push(
                {
                    'errorType': validationErrorType.NumOutRangeErrorType,
                    'errorCount': numOutRangeCheckCount,
                    'errorContent': numOutRangeCheckMsg
                });
        }
        //push字符串长度超过50个字符的错误信息到错误列表中
        if (stringOverLengthCheckCount > 0) {
            result = false;
            $scope.itemErrorMsgMap.push(
                {
                    'errorType': validationErrorType.OutputStringLengthType,
                    'errorCount': stringOverLengthCheckCount,
                    'errorContent': stringOverLengthCheckMsg
                });
        }

        //#endregion
        return result;
    };

    //点击导入按钮时，进行导入操作
    var importInputInvoiceItemData = function (importType) {
        if (!$scope.detailFileName) {
            SweetAlert.warning($translate.instant('PleaseSelectFileFirst'));
            return;
        }

        var importType = importType;//0：第一次导入，1：覆盖导入，2：追加导入

        //设置数据导入时的起始行,默认设置从第一行开始
        var startRowNum = 1;
        if ($('#ItemStartRowNum').val()) {
            startRowNum = $('#ItemStartRowNum').val();
        }

        //验证前，将错误数据清空
        $scope.itemErrorMsgMap = [];

//        //1：单独导入进项明细表check //todo: to be open futrue (neo)
//        if (!OnlyInvoiceItemCheck()) {
//            $('#errorItemListModal').modal('show');
//            return;
//        }
//        //2：判断是否有没有map的必须列和起始行输入格式是否正确
//        if (!ColumnsMapCheckIncomeInvoiceItem()) {
//            $('#errorItemListModal').modal('show');
//            return;
//        }


        //将需要导入的数据组装到list中
        var inputInvoiceItemList = [];
        for (var i = startRowNum - 1; i < $scope.sheetDataItem.dataList.length; i++) {
            var invoiceItemInfo = {
                'Index': $scope.sheetDataItem.dataList[i].index,//行号
                'PeriodID': curPeriod,
                'InvoiceCode': $scope.sheetDataItem.dataList[i][$.inArray($translate.instant('InvoiceFPDM'), $scope.selectedColumnMapItem)],
                'InvoiceNumber': $scope.sheetDataItem.dataList[i][$.inArray($translate.instant('InvoiceFPHM'), $scope.selectedColumnMapItem)],
                'Amount': PWC.isNullOrEmpty($scope.sheetDataItem.dataList[i][$.inArray($translate.instant('InvoiceJE'), $scope.selectedColumnMapItem)]) ?
                    0 : $scope.sheetDataItem.dataList[i][$.inArray($translate.instant('InvoiceJE'), $scope.selectedColumnMapItem)],
                'TaxAmount': PWC.isNullOrEmpty($scope.sheetDataItem.dataList[i][$.inArray($translate.instant('InvoiceSE'), $scope.selectedColumnMapItem)]) ?
                    0 : $scope.sheetDataItem.dataList[i][$.inArray($translate.instant('InvoiceSE'), $scope.selectedColumnMapItem)],
                'TaxRate': PWC.isNullOrEmpty($scope.sheetDataItem.dataList[i][$.inArray($translate.instant('InvoiceSL'), $scope.selectedColumnMapItem)]) ?
                    0 : $scope.sheetDataItem.dataList[i][$.inArray($translate.instant('InvoiceSL'), $scope.selectedColumnMapItem)],
                'ProductionName': $scope.sheetDataItem.dataList[i][$.inArray($translate.instant('InvoiceHWMC'), $scope.selectedColumnMapItem)],
            };
            inputInvoiceItemList.push(invoiceItemInfo);
        }



        //前台数据验证
//        FrontEndValidIncomeInvoiceItemData(inputInvoiceItemList); todo: skip to open futrue (neo)

        if ($scope.itemErrorMsgMap.length > 0) {
            $('#errorItemListModal').modal('show');
            return;
        }
        else {
            logDto.ID = PWC.newGuid();
            logDto.CreateTime = new Date();
            logDto.UpdateTime = new Date();
            logDto.OperationContent = $scope.detailFileName;
            logDto.OperationObject = $translate.instant('IncomeInvoiceDetail');

            if (importType === $scope.importEnum.Import) {
                logDto.OperationName = $translate.instant('ImportInputInvoiceItem');
                logDto.OperationType = enums.vatLogOperationTypeEnum.Import;
            }
            else if (importType === $scope.importEnum.CoverImport) {
                logDto.OperationName = $translate.instant('CoverImportInputInvoiceItem');
                logDto.OperationType = enums.vatLogOperationTypeEnum.CoverImport;
            }
            else if (importType === $scope.importEnum.AddImport) {
                logDto.OperationName = $translate.instant('AddImportInputInvoiceItem');
                logDto.OperationType = enums.vatLogOperationTypeEnum.AddImport;
            }

            //如果是覆盖导入,则提示，否则直接导入
            if (importType === enums.vatImportBtnType.CoverImport) {
                swal({
                    title: "warning!",
                    text: $translate.instant('InputInvoiceCoverImportTips'),
                    type: "warning",
                    showCancelButton: true,
                    confirmButtonColor: "#DD6B55",
                    confirmButtonText: $translate.instant('Yes'),
                    cancelButtonText: $translate.instant('No'),
                    closeOnConfirm: true,
                    closeOnCancel: true
                }, function (isConfirm) {
                    if (isConfirm) {
                        //数据进库
                        vatImportService.importInputInvoiceItemData(inputInvoiceItemList, importType).success(function (or) {
                            if (or && or.result) {
                                logDto.UpdateState = $translate.instant('ImportSuccess');
                                vatOperationLogService.addOperationLog(logDto);

                                SweetAlert.success($translate.instant('ImportSuccess'));
                                loadIncomeInvoiceItemDataFromDB();
                                $scope.detailFileName = '';
                                vatSessionService.dataChanged = $scope.showInitTable ? false : true;//设置为false表示在切换页面前数据没有变更,不需要提示
                            }
                        }).error(function () {
                            logDto.UpdateState = $translate.instant('ImportFail');
                            vatOperationLogService.addOperationLog(logDto);
                        });
                    }
                });
            }
            else {
                //数据进库
                vatImportService.importInputInvoiceItemData(inputInvoiceItemList, importType).success(function (or) {
                    if (or && or.result) {
                        logDto.UpdateState = $translate.instant('ImportSuccess');
                        vatOperationLogService.addOperationLog(logDto);

                        SweetAlert.success($translate.instant('ImportSuccess'));
                        loadIncomeInvoiceItemDataFromDB();
                        $scope.detailFileName = '';
                        vatSessionService.dataChanged = $scope.showInitTable ? false : true;//设置为false表示在切换页面前数据没有变更,不需要提示
                    }
                }).error(function () {
                    logDto.UpdateState = $translate.instant('ImportFail');
                    vatOperationLogService.addOperationLog(logDto);
                });
            }
        }
    };

    //动态设置grid高度
    $scope.getGridItemHeight = function () {
        if ($scope.isLoadInputInvoiceItemComplete) {
            return { height: ($('#input-invoice-item-grid-wrapper').height()) + "px" };
        }
        else {
            return { height: 0 + "px" };
        }
    };

    //刷新发票明细信息（重新验证）
    var refreshInputInvoiceItemData = function () {
        logDto.ID = PWC.newGuid();
        logDto.CreateTime = new Date();
        logDto.UpdateTime = new Date();
        logDto.OperationContent = '';
        logDto.OperationName = $translate.instant('RefreshInputInvoiceItemValidationData');
        logDto.OperationType = enums.vatLogOperationTypeEnum.Refresh;
        logDto.OperationObject = $translate.instant('IncomeInvoiceDetail');

        vatImportService.validateInputInvoiceItemInfo(curPeriod).success(function () {
            logDto.UpdateState = $translate.instant('RefreshSuccess');
            vatOperationLogService.addOperationLog(logDto);

            SweetAlert.success($translate.instant('RefreshSuccess'));
            loadIncomeInvoiceItemDataFromDB();
        }).error(function () {
            logDto.UpdateState = $translate.instant('RefreshTbFail');
            vatOperationLogService.addOperationLog(logDto);

            SweetAlert.error($translate.instant('PleaseContactAdministrator'));
        });

        vatSessionService.dataChanged = $scope.showInitTable ? false : true;
    };

    //清除当期发票明细信息
    var clearInputInvoiceItemData = function () {
        swal({
            title: "warning!",
            text: $translate.instant('InputInvoiceItemClearTips'),
            type: "warning",
            showCancelButton: true,
            confirmButtonColor: "#DD6B55",
            confirmButtonText: $translate.instant('Yes'),
            cancelButtonText: $translate.instant('No'),
            closeOnConfirm: true,
            closeOnCancel: true
        }, function (isConfirm) {
            if (isConfirm) {
                logDto.ID = PWC.newGuid();
                logDto.CreateTime = new Date();
                logDto.UpdateTime = new Date();
                logDto.OperationContent = '';
                logDto.OperationName = $translate.instant('ClearInputInvoiceItem');
                logDto.OperationType = enums.vatLogOperationTypeEnum.ClearData;
                logDto.OperationObject = $translate.instant('IncomeInvoiceDetail');

                vatImportService.clearInputInvoiceItemData(curPeriod).success(function (data) {
                    if (data && data.result) {
                        logDto.UpdateState = $translate.instant('ClearSuccess');
                        vatOperationLogService.addOperationLog(logDto);

                        SweetAlert.success($translate.instant('ClearSuccess'));
                        loadIncomeInvoiceItemDataFromDB();
                        $scope.isShowImportItemBtn = true;
                    }
                    else {
                        logDto.UpdateState = $translate.instant('ClearFailed');
                        vatOperationLogService.addOperationLog(logDto);

                        SweetAlert.warning($translate.instant('ClearFailed'));
                    }
                }).error(function () {
                    logDto.UpdateState = $translate.instant('ClearFailed');
                    vatOperationLogService.addOperationLog(logDto);

                    SweetAlert.error($translate.instant('PleaseContactAdministrator'));
                });
            }
        });
    };

    //获取发票明细验证信息
    var getInputInvoiceItemValidationList = function () {
        var deferred = $q.defer();

        vatImportService.getValidationDtoInfo(enums.validationType.InputInvoiceItem, curPeriod).success(function (data) {
            if (data.length > 0) {
                var index = 1;
                data.forEach(function (v) {
                    v.index = index++;
                });
                $scope.gridOptionsInputInvoiceItemErrorMsg.data = data;
            }
            else {
                $scope.gridOptionsInputInvoiceItemErrorMsg.data = [];
            }
            deferred.resolve();
        });

        return deferred.promise;
    }

    var showInputInvoiceItemErrTab = function () {
        $scope.InputInvoiceItemImportErrorTab = true;//默认显示顶部block
        $scope.InputInvoiceItemImportErrorTag = true;//默认隐藏下方的错误信息列表
        $scope.inputInvoiceItemErrorMsg = $translate.instant('ImportErrorMsg').formatObj({ "NumberOfError": $scope.gridOptionsInputInvoiceItemErrorMsg.data.length });

    };

    $scope.toggleInputInvoiceItemErrorTab = function () {
        $scope.InputInvoiceItemImportErrorTag = !$scope.InputInvoiceItemImportErrorTag;

        // topIcon and content-resize gapBottom 15px
        if (!$scope.InputInvoiceItemImportErrorTag) {
            setInputInvoiceItemErrorWrapCssDefault();
        } else {

            if (parseInt($('#input-invoice-item-content-resizer').css('bottom')) < 100) {
                $('#input-invoice-item-content-resizer').css('bottom', '130px');

                $('#input-invoice-item-error-info-wrapper').css('height', '130px');
            }
        }
    };

    var setInputInvoiceItemErrorWrapCssDefault = function () {
        $('#input-invoice-item-content-resizer').css('bottom', '20px');

        $('#input-invoice-item-error-info-wrapper').css('height', '0px');
    };

    //获取发票明细错误信息grid高度
    $scope.getInputInvoiceItemErrorGridHeight = function () {
        if ($scope.isLoadInputInvoiceItemComplete) {
            var y = $("#input-invoice-item-error-info-wrapper").height();

            // Enough space
            if (y > constant.UIGrid.gapHeight) {
                y = y - constant.UIGrid.gapHeight;
                return { height: y + "px" };
            } else {

                return { height: '0px' };
            }
        }

        return {};
    };

    //显示图标
    var errorLevelToString = function (errorLevel) {
        if (errorLevel === 0) {
            return '/app-resources/images/vat/error.png';
        }
        if (errorLevel === 1) {
            return '/app-resources/images/vat/warning.png';
        }
        if (errorLevel === 2) {
            return '/app-resources/images/vat/tips.png';
        }
        else {
            return '/app-resources/images/vat/tips.png';
        }
    };

    //显示重复发票明细
    var showInputInvoiceModal = function (typeid, items) {
        var data = JSON.parse(items);
        $scope.devgridDataSource = data;
        //组装列
        var columns = [{ caption: $translate.instant('InvoiceFPDM'), dataField: "InvoiceCode", width: 200 },
                     { caption: $translate.instant('InvoiceFPHM'), dataField: "InvoiceNumber", width: 200 },
                     { caption: $translate.instant('InvoiceKPRQ'), dataField: "InvoiceDate", dataType: "date", width: 120 },
                     { caption: $translate.instant('InvoiceGHFSH'), dataField: "SellerTaxNumber", width: 100 },
                     { caption: $translate.instant('InvoiceJE'), dataField: "Amount", format: { type: 'fixedPoint', precision: 2 }, width: 100 },
                     { caption: $translate.instant('InvoiceSE'), dataField: "TaxAmount", format: { type: 'fixedPoint', precision: 2 }, width: 100 },
                     { caption: $translate.instant('InvoiceRZRQ'), dataField: "CertificationDate", dataType: "date", width: 120 },
                     { caption: $translate.instant('InvoiceRZJG'), dataField: "CertificationResult", width: 100 },
                     { caption: $translate.instant('InvoiceFPLX'), dataField: "InvoiceTypeName", cellTemplate: "translateCellTemplate", width: 100 }];


        $scope.deferStatus = true;
        $scope.filterCondition = ["IsDuplicate", "=", true];

        $scope.gridInputInvoiceDuplicateItemGridOptions = {
            columns: columns,
            bindingOptions: {
                "dataSource.store.data": "devgridDataSource",
                "selection.deferred": "deferStatus",
                "selectionFilter": "filterCondition"
            },
            dataSource: {
                store: {
                    type: "array",
                    key: 'ID'
                }
            },
            selection: {
                selectAllMode: "allPages",
                showCheckBoxesMode: "always",
                mode: "multiple"
            },
            hoverStateEnabled: true,
            paging: {
                pageSize: 10
            }
        }

        $('#inputInvoiceDuplicateItemModal').modal('show');

    };

    //删除重复数据
    var deleteDuplicateData = function () {
        var dataGrid = $('#gridInputInvoiceDuplicateItemGrid').dxDataGrid("instance");
        var deleteList = [];
        dataGrid.getSelectedRowsData().done(function (rowData) {
            for (var i = 0; i < rowData.length; i++) {
                deleteList.push(rowData[i].AcctCode);
            }
        });

        dataGrid.getSelectedRowKeys().done(function (keys) {
            if (keys && keys.length === 0) {
                SweetAlert.warning($translate.instant('DeleteEmptyWarning'));
                return;
            }

            logDto.ID = PWC.newGuid();
            logDto.CreateTime = new Date();
            logDto.UpdateTime = new Date();
            logDto.OperationContent = deleteList.join(",");
            logDto.OperationName = $translate.instant('DeleteDuplicateData');
            logDto.OperationType = enums.vatLogOperationTypeEnum.DeleteDuplicate;
            logDto.OperationObject = $translate.instant('IncomeInvoiceTotal');

            vatImportService.deleteInputInvoiceDuplicateData(keys).success(function (data) {
                if (data && data.result) {
                    //添加删除重复数据日志
                    logDto.UpdateState = $translate.instant('DeleteSuccess');
                    vatOperationLogService.addOperationLog(logDto);

                    //删除数据后重新校验
                    loadIncomeInvoiceDataFromDB();
                    vatImportService.validateInputInvoiceInfo(curPeriod).success(function () {
                        $('#inputInvoiceDuplicateItemModal').modal('hide');
                        SweetAlert.success($translate.instant('DeleteSuccess'));
                        //获取错误列表信息
                        getInputInvoiceValidationList().then(function () {
                            if ($scope.gridOptionsInputInvoiceErrorMsg.data.length > 0) {
                                showInputInvoiceErrTab();
                                $scope.toggleInputInvoiceErrorTab();

                                //当有错误信息时，显示错误信息表
                                $scope.showInputInvoiceErrorTable = true;
                            }
                            else {
                                $scope.InputInvoiceImportErrorTab = false;
                                $scope.InputInvoiceImportErrorTag = false;
                                $scope.showInputInvoiceErrorTable = false;
                            }
                        });

                    });
                }
                else {
                    logDto.UpdateState = $translate.instant('DeleteFailed');
                    vatOperationLogService.addOperationLog(logDto);
                }
            }).error(function () {
                logDto.UpdateState = $translate.instant('DeleteFailed');
                vatOperationLogService.addOperationLog(logDto);
                SweetAlert.error($translate.instant('PleaseContactAdministrator'));
            });
        });


    };

    //显示错误税率明细
    var showInputInvoiceItemModal = function (typeid, items) {
        var data = JSON.parse(items);
        $scope.devgridDataSource = data;

        //如果类型为显示汇总和明细金额、税额对比结果
        if (typeid === 66) {
            //组装列
            var columns = [{ caption: $translate.instant('InvoiceFPDM'), dataField: "InvoiceCode", width: '15%' },
                         { caption: $translate.instant('InvoiceFPHM'), dataField: "InvoiceNumber", width: '15%' },
                         { caption: $translate.instant('InvoiceTotalAmount'), dataField: "TotalAmount", format: { type: 'fixedPoint', precision: 2 }, width: '17.5%' },
                         { caption: $translate.instant('InvoiceTotalTaxAmount'), dataField: "TotalTaxAmount", format: { type: 'fixedPoint', precision: 2 }, width: '17.5%' },
                         { caption: $translate.instant('InvoiceItemAmount'), dataField: "ItemAmount", format: { type: 'fixedPoint', precision: 2 }, width: '17.5%' },
                         { caption: $translate.instant('InvoiceItemTaxAmount'), dataField: "ItemTaxAmount", format: { type: 'fixedPoint', precision: 2 }, width: '17.5%' }];


            $scope.gridInputInvoiceAmountCompareGridOptions = {
                columns: columns,
                selection: {
                    mode: "single"
                },
                bindingOptions: {
                    "dataSource": "devgridDataSource"
                },
                hoverStateEnabled: true,
                paging: {
                    pageSize: 10
                }
            }

            $('#inputInvoiceAmountCompareModal').modal('show');
        }
        else {
            //组装列
            var columns = [{ caption: $translate.instant('InvoiceFPDM'), dataField: "InvoiceCode", width: '15%' },
                         { caption: $translate.instant('InvoiceFPHM'), dataField: "InvoiceNumber", width: '15%' },
                         { caption: $translate.instant('InvoiceJE'), dataField: "Amount", format: { type: 'fixedPoint', precision: 2 }, width: '15%' },
                         { caption: $translate.instant('InvoiceSE'), dataField: "TaxAmount", format: { type: 'fixedPoint', precision: 2 }, width: '15%' },
                         { caption: $translate.instant('InvoiceSL'), dataField: "TaxRate", cellTemplate: "taxRateCellTemplate", width: '15%' },
                         { caption: $translate.instant('InvoiceHWMC'), dataField: "ProductionName", width: '25%' }];


            $scope.gridInputInvoiceItemGridOptions = {
                columns: columns,
                selection: {
                    mode: "single"
                },
                bindingOptions: {
                    "dataSource": "devgridDataSource"
                },
                hoverStateEnabled: true,
                paging: {
                    pageSize: 10
                }
            }

            $('#inputInvoiceItemModal').modal('show');
        }
    };

    //查看发票日志
    var showOperateLogPop = function () {
        $scope.isShowLog = true;
    };

    //控制明细信息中删除权限
    var getUserPermission = function () {
        var list = [];
        var code = constant.vatPermission.dataImport.inputInvoice.importCode;
        list.push(code);

        $scope.$root.checkUserOrganizationPermissionList(list).success(function (data) {
            $scope.hasEditPermission = data[code];
        });
    };

    var setButtonWrapStyle = function () {
        if ($scope.showImportTable) {
            return { width: "100%" };
        }
    };

    var initPagingControl = function (totalItemsCount) {
        $scope.pagingOptions.totalItems = totalItemsCount;
    };

    $scope.pagingService = {
        refreshInvoiceDataGrid: function () {
            $log.debug("refreshInvoiceDataGrid");
            loadIncomeInvoiceItemDataFromDB();
        },
    };
    /****************************************明细表function end*************************************************/


    (function initialize() {
        $log.debug('VatImportIncomeInvoiceController.ctor()...');

        /****************************************汇总表function start***********************************************/
        $scope.errorMsgMap = [];
        $scope.incomeInvoiceTotalFileName = '';
        $scope.selectedColumnMap = [];
        $scope.incomeInvoiceTotalColumns = [];
        $scope.initIncomeInvoiceTotalColumnsIndex = [];
        $scope.startRowList = [];
        $scope.sheetData = { sheetNameList: [], dataList: [], selectedSheetIndex: 0 };
        $scope.sheetInfo = { selectedSheetName: '', selectedSheetIndex: 0, topRowNumber: 0 };
        $scope.isShowImportTotalBtn = true;
        $scope.isLoadInputInvoiceComplete = false;
        

        initColumns();
        loadIncomeInvoiceDataFromDB();
        getUserPermission();

        $scope.changeIncomeInvoiceSheet = changeIncomeInvoiceSheet;

        $scope.mappingColumn = mappingColumn;

        $scope.importInputInvoiceData = importInputInvoiceData;

        $scope.clearInputInvoiceData = clearInputInvoiceData;

        $scope.refreshInputInvoiceData = refreshInputInvoiceData;

        $scope.errorLevelToString = errorLevelToString;

        $scope.showInputInvoiceModal = showInputInvoiceModal;

        $scope.deleteDuplicateData = deleteDuplicateData;

        $scope.loadIncomeInvoiceDataFromDB = loadIncomeInvoiceDataFromDB;

        $scope.showOperateLogPop = showOperateLogPop;

        $scope.setButtonWrapStyle = setButtonWrapStyle;

        $scope.$watch('incomeInvoiceTotalFileName', function (newValue, oldValue) {
            if (newValue && newValue !== oldValue) {
                updateIncomeInvoceTotalFileName(newValue);
                doUploadIncomeInvoiceTotalFile($scope.incomeInvoiceTotalFileName);
                $scope.InputInvoiceImportErrorTab = false;
                $scope.InputInvoiceImportErrorTag = false;
                $('#StartRowNum').val('2');
            }
        });

        $scope.gridOptionsIncomeInvoiceTotal = {
            rowHeight: constant.UIGrid.rowHeight,
            selectionRowHeaderWidth: constant.UIGrid.rowHeight,
            enableSorting: false,
            enableColumnMenus: false,
            virtualizationThreshold: 50,//默认加载50条数据，避免在数据展示时，只显示前面4条
            enableHorizontalScrollbar: uiGridConstants.scrollbars.enableHorizontalScrollbar,
            enableVerticalScrollbar: uiGridConstants.scrollbars.enableVerticalScrollbar,
            columnDefs: [
              {
                  name: $translate.instant('ImportErrorPopUpNoCol'), width: 100, headerCellClass: '',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.index}}<span></div>'
              },
              {
                  name: $translate.instant('InvoiceFPDM'), width: 150, headerCellClass: '',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.invoiceCode}}<span></div>'
              },
              {
                  name: $translate.instant('InvoiceFPHM'), width: 150, headerCellClass: 'right',
                  cellTemplate: '<div class="ui-grid-cell-contents right"><span>{{row.entity.invoiceNumber}}<span></div>'
              },
              {
                  name: $translate.instant('InvoiceKPRQ'), width: 150, headerCellClass: '',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.invoiceDate | date:"yyyy-MM-dd"}}</span></div>'
              },
              {
                  name: $translate.instant('InvoiceGHFSH'), width: 200, headerCellClass: 'right',
                  cellTemplate: '<div class="ui-grid-cell-contents right"><span>{{row.entity.sellerTaxNumber}}</span></div>'
              },
              {
                  name: $translate.instant('InvoiceJE'), width: 150, headerCellClass: 'right',
                  cellTemplate: '<div class="ui-grid-cell-contents right"><span>{{row.entity.amount}}</span></div>'
              },
              {
                  name: $translate.instant('InvoiceSE'), width: 150, headerCellClass: 'right',
                  cellTemplate: '<div class="ui-grid-cell-contents right"><span>{{row.entity.taxAmount}}</span></div>'
              },
              {
                  name: $translate.instant('InvoiceRZRQ'), width: 150, headerCellClass: '',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.certificationDate | date:"yyyy-MM-dd"}}</span></div>'
              },
              {
                  name: $translate.instant('InvoiceRZJG'), width: 200, headerCellClass: '',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.certificationResult}}</span></div>'
              },
              {
                  name: $translate.instant('InvoiceFPLX'), width: 150, headerCellClass: '',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.invoiceTypeName | translate}}</span></div>'
              }
            ],
            onRegisterApi: function (gridApi) {
                $scope.gridApiTotal = gridApi;

                $interval(function () {
                    $scope.gridApiTotal.core.handleWindowResize();
                }, 500, 60 * 60 * 8);
            }
        };


        $scope.gridOptionsInputInvoiceErrorMsg = {
            rowHeight: constant.UIGrid.rowHeight,
            selectionRowHeaderWidth: constant.UIGrid.rowHeight,
            enableSorting: false,
            enableColumnMenus: false,
            virtualizationThreshold: 50,//默认加载50条数据，避免在数据展示时，只显示前面4条
            enableHorizontalScrollbar: uiGridConstants.scrollbars.enableHorizontalScrollbar,
            enableVerticalScrollbar: uiGridConstants.scrollbars.enableVerticalScrollbar,
            columnDefs: [
              {
                  name: ' ', width: '5%',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span><img data-ng-src="{{grid.appScope.errorLevelToString(row.entity.errorLevel)}}"><span></div>'
              },
              {
                  name: $translate.instant('ImportErrorPopUpNoCol'), width: '5%',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.index}}<span></div>'
              },
              {
                  name: $translate.instant('ErrorResult'), width: '35%',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.validationResult}}<span></div>'
              },
              {
                  name: $translate.instant('ImportErrorPopUpErrorCountCol'), width: '10%',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.number}}<span></div>'
              },
              {
                  name: $translate.instant('ErrorDetail'), width: '20%',
                  cellTemplate: '<div class="ui-grid-cell-contents">' +
                      '<a ng-if="row.entity.isShowDetail" ng-click="grid.appScope.showInputInvoiceModal(row.entity.erpCheckTypeId, row.entity.validationDetails)">{{"ViewDetails" | translate}}</a>' +
                      '<span ng-if="!row.entity.isShowDetail" title="{{row.entity.validationTips}}" data-ng-bind-html="row.entity.validationTips"><span>' +
                      '</div>'
              },
              {
                  name: $translate.instant('ValidationTips'), width: '25%',
                  cellTemplate: '<div class="ui-grid-cell-contents">' +
                      '<span ng-if="row.entity.isShowDetail" title="{{row.entity.validationTips}}" data-ng-bind-html="row.entity.validationTips"></span>' +
                      '<span ng-if="!row.entity.isShowDetail"></span>' +
                      '</div>'
              }
            ],
            onRegisterApi: function (gridApi) {
                $scope.inputInvoiceErrorGridApi = gridApi;

                $interval(function () {
                    $scope.inputInvoiceErrorGridApi.core.handleWindowResize();
                }, 500, 60 * 60 * 8);
            }
        };

        $timeout(function () {
            $scope.isLoadInputInvoiceComplete = true;
        }, 500);
        /****************************************汇总表function end***********************************************/

        $scope.itemErrorMsgMap = [];
        $scope.incomeInvoiceDetailFileName = '';
        $scope.selectedColumnMapItem = [];
        $scope.incomeInvoiceDetailColumns = [];
        $scope.initIncomeInvoiceDetailColumnsIndex = [];
        $scope.startRowListDetail = [];
        $scope.sheetDataItem = { sheetNameList: [], dataList: [], selectedSheetIndex: 0 };
        $scope.sheetInfoDetail = { selectedSheetName: '', selectedSheetIndex: 0, topRowNumber: 0 };
        $scope.isShowImportItemBtn = true;


        initDetailColumns();
        loadIncomeInvoiceItemDataFromDB();

        $scope.changeIncomeInvoiceDetailSheet = changeIncomeInvoiceDetailSheet;

        $scope.mappingDetailColumn = mappingDetailColumn;

        $scope.importInputInvoiceItemData = importInputInvoiceItemData;

        $scope.clearInputInvoiceItemData = clearInputInvoiceItemData;

        $scope.refreshInputInvoiceItemData = refreshInputInvoiceItemData;

        $scope.showInputInvoiceItemModal = showInputInvoiceItemModal;

        $scope.$watch('incomeInvoiceDetailFileName', function (newValue, oldValue) {
            if (newValue && newValue !== oldValue) {
                updateIncomeInvoceDetailFileName(newValue);
                doUploadIncomeInvoiceDetailFile($scope.incomeInvoiceDetailFileName);
                $scope.InputInvoiceItemImportErrorTab = false;
                $scope.InputInvoiceItemImportErrorTag = false;
                $('#ItemStartRowNum').val('2');
            }
        });

        $scope.gridOptionsIncomeInvoiceItem = {
            rowHeight: constant.UIGrid.rowHeight,
            selectionRowHeaderWidth: constant.UIGrid.rowHeight,
            enableSorting: false,
            enableColumnMenus: false,
            virtualizationThreshold: 50,//默认加载50条数据，避免在数据展示时，只显示前面4条
            enableHorizontalScrollbar: uiGridConstants.scrollbars.enableHorizontalScrollbar,
            enableVerticalScrollbar: uiGridConstants.scrollbars.enableVerticalScrollbar,
            columnDefs: [
              {
                  name: $translate.instant('ImportErrorPopUpNoCol'), width: '10%', headerCellClass: '',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.index}}<span></div>'
              },
              {
                  name: $translate.instant('InvoiceFPDM'), width: '15%', headerCellClass: '',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.invoiceCode}}<span></div>'
              },
              {
                  name: $translate.instant('InvoiceFPHM'), width: '15%', headerCellClass: '',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.invoiceNumber}}<span></div>'
              },
              {
                  name: $translate.instant('InvoiceJE'), width: '15%', headerCellClass: 'right',
                  cellTemplate: '<div class="ui-grid-cell-contents right"><span>{{row.entity.amount}}</span></div>'
              },
              {
                  name: $translate.instant('InvoiceSE'), width: '15%', headerCellClass: 'right',
                  cellTemplate: '<div class="ui-grid-cell-contents right"><span>{{row.entity.taxAmount}}</span></div>'
              },
              {
                  name: $translate.instant('InvoiceSL'), width: '15%', headerCellClass: 'right',
                  cellTemplate: '<div class="ui-grid-cell-contents right"><span>{{(row.entity.taxRate * 100) + "%"}}</span></div>'
              },
              {
                  name: $translate.instant('InvoiceHWMC'), width: '20%', headerCellClass: '',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.productionName}}</span></div>'
              }
            ],
            onRegisterApi: function (gridApi) {
                $scope.gridApiItem = gridApi;

                $interval(function () {
                    $scope.gridApiItem.core.handleWindowResize();
                }, 500, 60 * 60 * 8);
            }
        };

        $scope.gridOptionsInputInvoiceItemErrorMsg = {
            rowHeight: constant.UIGrid.rowHeight,
            selectionRowHeaderWidth: constant.UIGrid.rowHeight,
            enableSorting: false,
            enableColumnMenus: false,
            virtualizationThreshold: 50,//默认加载50条数据，避免在数据展示时，只显示前面4条
            enableHorizontalScrollbar: uiGridConstants.scrollbars.enableHorizontalScrollbar,
            enableVerticalScrollbar: uiGridConstants.scrollbars.enableVerticalScrollbar,
            columnDefs: [
              {
                  name: ' ', width: '5%',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span><img data-ng-src="{{grid.appScope.errorLevelToString(row.entity.errorLevel)}}"><span></div>'
              },
              {
                  name: $translate.instant('ImportErrorPopUpNoCol'), width: '5%',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.index}}<span></div>'
              },
              {
                  name: $translate.instant('ErrorResult'), width: '35%',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.validationResult}}<span></div>'
              },
              {
                  name: $translate.instant('ImportErrorPopUpErrorCountCol'), width: '10%',
                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.number}}<span></div>'
              },
              {
                  name: $translate.instant('ErrorDetail'), width: '20%',
                  cellTemplate: '<div class="ui-grid-cell-contents">' +
                      '<a ng-if="row.entity.isShowDetail" ng-click="grid.appScope.showInputInvoiceItemModal(row.entity.erpCheckTypeId, row.entity.validationDetails)">{{"ViewDetails" | translate}}</a>' +
                      '<span ng-if="!row.entity.isShowDetail" title="{{row.entity.validationTips}}" data-ng-bind-html="row.entity.validationTips"><span>' +
                      '</div>'
              },
              {
                  name: $translate.instant('ValidationTips'), width: '25%',
                  cellTemplate: '<div class="ui-grid-cell-contents">' +
                      '<span ng-if="row.entity.isShowDetail" title="{{row.entity.validationTips}}" data-ng-bind-html="row.entity.validationTips"></span>' +
                      '<span ng-if="!row.entity.isShowDetail"></span>' +
                      '</div>'
              }
            ],
            onRegisterApi: function (gridApi) {
                $scope.inputInvoiceItemErrorGridApi = gridApi;

                $interval(function () {
                    $scope.inputInvoiceItemErrorGridApi.core.handleWindowResize();
                }, 500, 60 * 60 * 8);
            }
        };


        $timeout(function () {
            $scope.isLoadInputInvoiceItemComplete = true;
        }, 500);

    })();
}]);