﻿invoiceModule.controller('invoiceManageMainUploadScanInvoiceController', ['$scope', '$log', '$document', '$uibModal', 'SweetAlert', '$translate', '$q', 'apiInterceptor', '$interval', 'region', '$timeout', 'InvoiceManageService',
        function ($scope, $log, $document, $uibModal, SweetAlert, $translate, $q, apiInterceptor, $interval, region, $timeout, InvoiceManageService) {
            'use strict';

            //分页的设置
            $scope.pagingOptions = {
                pageIndex: 1,  //当前页码
                totalItems: 100,  //总数据
                totalPages: 10,//总页数
                maxSize: 10, //分页数字的限制。
                pageSize: constant.page.pageSizeArrary[1],  //每页多少条数据 
                pageSizeString: constant.page.pageSizeArrary[1].toString(),
                firstPage: $translate.instant('PagingFirstPage'),
                previousPage: $translate.instant('PagingPreviousPage'),
                nextPage: $translate.instant('PagingNextPage'),
                lastPage: $translate.instant('PagingLastPage'),
            };

            //翻译
            $scope.translated = {
                NoData: $translate.instant('NoDataText'),
                pleaseSelect: $translate.instant("ChoosePlaceholder"),
                pleaseInput: $translate.instant('InputPlaceholder'),
                deleteBtn: $translate.instant('BtnDelete'),
                editBtn: $translate.instant('BtnEdit'),
                confirmDeleteMsg: $translate.instant('IsDelete'),
            };

            $scope.searchFilterOptions = {};
            $scope.invoiceEntity = {};
            $scope.datagrid = {};
            $scope.isShowModal = false;
            $scope.uploadInvoiceTotalCount = 0;
            //从系统读取来的invoice 列表
            $scope.existSystemInvoiceList = {};
            $scope.batchDeletedDisabled = false;

            //上传的状态
            var uploadStatusEnum = {
                success: {
                    id: constant.inputInvoice.invoiceUploadStatus.success.id,
                    value: $translate.instant(constant.inputInvoice.invoiceUploadStatus.success.value)
                },
                failure: {
                    id: constant.inputInvoice.invoiceUploadStatus.failure.id,
                    value: $translate.instant(constant.inputInvoice.invoiceUploadStatus.failure.value)
                },
                duplicate: {
                    id: constant.inputInvoice.invoiceUploadStatus.duplicate.id,
                    value: $translate.instant(constant.inputInvoice.invoiceUploadStatus.duplicate.value)
                }
            }



            //获取数据服务
            var getService = {
                //筛选invoice list
                getInputInvoiceList: function () {
                    var deferred = $q.defer();

                    var invoiceParams = {
                        InvoiceQuery: $scope.searchEntity,
                        pagingDto: $scope.pagingOptions,
                    };

                    InvoiceManageService.getInputInvoiceList(invoiceParams).success(function (data) {
                        if (data) {
                            $scope.pagingOptions.totalItems = data.pageInfo && data.pageInfo.totalCount;
                            if (data && data.list) {
                                data.list.forEach(function (item) {
                                    item.statusString = $translate.instant(item.statusString);
                                    item.invoiceEntityTypeString = $translate.instant(item.invoiceEntityTypeString);
                                    item.invoiceSourceType = -1; //$translate.instant(item.invoiceSourceTypeString);
                                    item.invoiceSourceTypeString ='', //$translate.instant(item.invoiceSourceTypeString);
                                    item.invoiceTypeString = $translate.instant(item.invoiceTypeString);
                                    item.uploadTypeString = $translate.instant(item.uploadTypeString);
                                    item.uploadDate = moment(item.uploadDate).format(constant.date.dateFormatUppercase);
                                    item.invoiceDate = moment(item.invoiceDate).format(constant.date.dateFormatUppercase);
                                });
                            }
                            $scope.invoiceGridDataSource = data.list;
                            $scope.invoiceEntity = data.list;
                            deferred.resolve(data);
                        };
                    });
                    return deferred.promise;
                },

                //除去已经失效的invoice，用来做重复验证
                getAllValidInvoiceList: function () {
                    var deferred = $q.defer();
                    InvoiceManageService.getInputInvoiceList(invoiceParams).success(function (data) {
                        if (data)
                        {
                            $scope.existSystemInvoiceList = data;

                            deferred.resolve(data);
                        }
                    });
                    return deferred.promise;
                },
            }

            //添加数据服务
            var addService = {
                //上传invoice
                invoiceUploadFromScan: function (invoiceList) {
                    var deferred = $q.defer();
                    InvoiceManageService.invoiceUploadFromScan(invoiceList).success(function (data) {
                        if (data) {
                            deferred.resolve(data);
                        }
                    });
                    return deferred.promise;
                },
            };
            //消息弹框
            var messageBox = {
                getMsg: function (resultMsg, hasTranslated) {
                    hasTranslated = hasTranslated || false;
                    var msg = resultMsg;
                    if (!hasTranslated) {
                        msg = $translate.instant(resultMsg);
                    }
                    return msg;
                },

                success: function (resultMsg, hasTranslated) {
                    var msg = this.getMsg(resultMsg, hasTranslated)
                    SweetAlert.success(msg);
                },

                warning: function (resultMsg, hasTranslated) {
                    var msg = this.getMsg(resultMsg, hasTranslated)
                    SweetAlert.warning(msg);
                },

                //有确认和取消按钮
                confirm: function (title, text, customClass) {
                    var deferred = $q.defer();
                    SweetAlert.swal({
                        title: title,
                        text: text,
                        html: true,
                        type: "warning",
                        customClass: customClass || '',
                        showCancelButton: true,
                        confirmButtonColor: "#DD6B55",
                        allowOutsideClick: false,
                        confirmButtonText: $translate.instant('Confirm'),
                        cancelButtonText: $translate.instant('Cancel'),
                        closeOnConfirm: true,
                        closeOnCancel: true
                    },
                      function (isConfirm) {
                          deferred.resolve(isConfirm);
                      });

                    return deferred.promise;
                },

                //没有取消按钮
                info: function (title, text, customClass) {
                    var deferred = $q.defer();

                    SweetAlert.swal({
                        title: title,
                        text: text,
                        html: true,
                        type: "warning",
                        customClass: customClass || '',
                        showCancelButton: false,
                        confirmButtonColor: "#DD6B55",
                        allowOutsideClick: false,
                        confirmButtonText: $translate.instant('Confirm'),
                        cancelButtonText: $translate.instant('Cancel'),
                        closeOnConfirm: true,
                        closeOnCancel: false
                    },
                      function (isConfirm) {
                          deferred.resolve(isConfirm);
                      });

                    return deferred.promise;
                },
            };

            //内部处理
            var memoryService = {
                //发票来源列表构造
                populateInvoiceSourceTypList: function () {
                    $scope.searchFilterOptions.invoiceSourceList = memoryService.translateEnumValue(constant.inputInvoice.invoiceSourceTypeArray);
                },

                repaintInvoiceGrid: function () {
                    $scope.setInvoiceGridHeight();
                },
                //翻译查询的数据源
                translateEnumValue: function (data) {
                    if (!data || data.length == 0) return null;

                    data.forEach(function (item) {
                        item.value = $translate.instant(item.value);
                    });
                    return data;
                },
                //分页下拉
                populatePagingSelection: function () {
                    var pagingSelection = [];
                    var pageArray = constant.page.pageSizeArrary;
                    for (var i = 0 ; i < pageArray.length; i++) {
                        var selection = { id: pageArray[i], value: pageArray[i] };
                        pagingSelection.push(selection);
                    }
                    $scope.pagingOptions.pagingSelection = pagingSelection;
                },
            };

            $scope.validationService = {

                duplicateValidation: function (invoiceCode, invoiceNumber) {
                    //当前table验证
                    var data = angular.copy($scope.invoiceGridDataSource);
                    var filterList = _.filter(data, function (item) {
                        return item.invoiceCode == invoiceCode && item.invoiceNumber == invoiceNumber;
                    });
                    if (filterList.length > 0) {
                        return false;//重复
                    }
                    //系统已经存在验证
                    data = angular.copy($scope.existSystemInvoiceList);
                    var filterList = _.filter(data, function (item) {
                        return item.invoiceCode == invoiceCode && item.invoiceNumber == invoiceNumber;
                    });
                    if (filterList.length > 0) {
                        return false;//重复
                    }
                    return true ;//成功
                },

                lengthValidation: function (invoiceCode, invoiceNumber) {
                    if (invoiceCode.length != 8 || invoiceNumber.length != 8) {
                        return false;
                    }
                    return true;
                },

            };
            $scope.eventServie = {
                //手动添加新的invoice弹框
                addNewInvoiceModal: function () {
                    $scope.newInvoiceEntity = {};
                    $scope.newInvoiceUpdated = false;
                    $scope.newInvoiceOperateType = constant.Operation.Add;
                },

                //编辑invoice弹框
                editInvoiceModal: function (data) {
                    $scope.newInvoiceEntity = data;
                    $scope.newInvoiceUpdated = false;
                    $scope.newInvoiceOperateType = constant.Operation.Edit;
                },
                //双向绑定，自动会更新datagrid
                updateInvoiceRow: function (row) {
                
                },
                //datagrid 新增一行数据
                addInvoiceRow: function (row) {
                    var newInvoiceEntity = {
                        id: PWC.newGuid(),
                        invoiceCode: row && row.invoiceCode || '',
                        invoiceNumber: row && row.invoiceNumber || '',
                       // buyerName: row && row.buyerName || '',
                        //sellerTaxNumber: row && row.sellerTaxNumber || '',
                        amount: row && row.amount || 0,
                       // taxAmount: row && row.taxAmount || 0,
                        invoiceDate: row && row.invoiceDate || '',
                        uploadStatus: uploadStatusEnum.success.id,
                    };

                    //长度验证
                    var lengthStatus = $scope.validationService.lengthValidation(newInvoiceEntity.invoiceCode, newInvoiceEntity.invoiceNumber);
                    if (!lengthStatus) {
                         newInvoiceEntity.uploadStatus = uploadStatusEnum.failure.id;
                    }
                    else {
                        //做重复性验证
                        var duplicateStatus = $scope.validationService.duplicateValidation(newInvoiceEntity.invoiceCode, newInvoiceEntity.invoiceNumber);
                        if (!duplicateStatus) {
                            newInvoiceEntity.uploadStatus = uploadStatusEnum.duplicate.id;
                        }
                    }

                    if (!$scope.invoiceGridDataSource) $scope.invoiceGridDataSource = [];
                    $scope.invoiceGridDataSource.push(newInvoiceEntity);
                },


                //解析扫描的七个字符串
                parseScanCode: function (codeData) {
                    //01,10,031001700111,65285505,88.0,20170811,09307379165572922287
                    if (!codeData || codeData.length == 0) return;

                    var arr = codeData.split(',');
                    var dateStr = arr[5];
                    var year = dateStr.substr(0, 4);
                    var month = dateStr.substr(4, 2);
                    var day = dateStr.substr(6, 2);
                    var dateSplit = '-';
                    var invoiceDate = year + dateSplit + month + dateSplit + day;

                    //小数点位数
                    var decimalNumber = 2;

                    var invoiceEntity = {
                        invoiceCode: arr[2],
                        invoiceNumber: arr[3],
                        amount: parseFloat(arr[4]).toFixed(decimalNumber),
                     //  taxAmount: parseFloat(arr[4] * 0.06).toFixed(decimalNumber),
                        invoiceDate: invoiceDate
                    };

                    //添加一行
                    $scope.eventServie.addInvoiceRow(invoiceEntity);
                },

                //完成上传，上传到服务器
                uploadToServer: function () {
                    //到国税局做识别，要么识别成功，识别失败，或者无法识别（假票）
                    //识别成功的直接更新为待匹配（只有扫描枪上传的方式下）
                    var data = $scope.invoiceGridDataSource;
                    var filterFailureList =_.filter(data,function(item){
                        return item.uploadStatus == uploadStatusEnum.failure.id;
                    });
                    if (filterFailureList.length > 0)
                    {
                        messageBox.warning("请修改上传状态失败的数据", true);
                        return false;
                    }

                    var filterDuplicateList = _.filter(data, function (item) {
                        return item.uploadStatus == uploadStatusEnum.duplicate.id;
                    });
                    if (filterDuplicateList.length > 0) {
                        messageBox.warning("请修改上传状态重复的数据", true);
                        return false;
                    }

                    addService.invoiceUploadFromScan($scope.invoiceGridDataSource).then(function (data) {
                        if (data) {
                            messageBox.success("上传成功", true);
                            //清空当前数据源
                            $scope.invoiceGridDataSource = [];
                        }
                    });
                },

                //删除行
                deleteRow:function(row){
                    var tempData = $scope.invoiceGridDataSource;
                    for (var i = 0; i < tempData.length; i++) {
                        if (tempData[i].id === row.id) {
                            tempData.splice(i, 1);
                            break;
                        }
                    }
                    $scope.invoiceGridDataSource = tempData;
                },

                //批量删除
                batchDelete: function () {
                    if (!$scope.invoiceGridDataSource) return;
                    if (!$scope.datagrid.selectedItems || $scope.datagrid.selectedItems.length == 0)
                    {
                        return;
                    }

                    messageBox.confirm($scope.translated.confirmDeleteMsg, '').then(function (isConfirm) {
                        var data = $scope.datagrid.selectedItems;
                        data.forEach(function (item) {
                            $scope.eventServie.deleteRow(item);
                        });
                    });
                },
            };

            //检测弹框有新的invoice添加或者编辑，这里的状态都应该是补录，而不是上传
            $scope.$watch('newInvoiceUpdated', function (newValue, oldValue) {
                if (newValue) {
                    console.log("newInvoiceEntity: " + JSON.stringify($scope.newInvoiceEntity));
                    //$scope.eventServie.addInvoiceRow($scope.newInvoiceEntity);
                    
                    //编辑信息过后，更新状态为已经补录
                    if ($scope.newInvoiceEntity.remarks && $scope.newInvoiceEntity.remarks.length > 0) {
                        $scope.newInvoiceEntity.status = invoiceStatusType.InvoiceHasAddRecord.id;
                    }

                    if ($scope.newInvoiceEntity.operationAction == constant.Operation.Edit) {
                     
                        $scope.eventServie.updateInvoiceRow($scope.newInvoiceEntity);
                    }
                    else if ($scope.newInvoiceEntity.operationAction == constant.Operation.Add) {
                        $scope.eventServie.addInvoiceRow($scope.newInvoiceEntity);
                    }
                
                }
            });
            
            var txtInvoiceSelector = '#txtInvoiceOptions';

            //var e = jQuery.Event("keydown", { keyCode: 13 });
           
            //$(txtInvoiceSelector).on('keydown', function (e) {
            //    if (e.keyCode === 13) {
            //         alert('enter' + $scope.invoiceString);
            //        console.log("$scope.invoiceString:" + $scope.invoiceString);

            //        var invoiceEntity = {
            //            invoiceCode: $scope.invoiceString
            //        }
            //        $scope.eventServie.addInvoiceRow(invoiceEntity);
            //    }
            //});
            //扫描控件初始化
            var initTextBoxOptions = function () {

                var isShowClearButton = true;
                $(txtInvoiceSelector).dxTextBox({
                    bindingOptions: {
                        value: 'invoiceString'
                    },
                    valueChangeEvent: "change",
                    onValueChanged: function (e) {
                        console.log("e:" + e.value);
                        if (!e.value || e.value == '') return;
                        if (e.previousValue !== '') return;
                        try {
                                $scope.invoiceString = e.value;
                                $scope.eventServie.parseScanCode(e.value);

                                $(txtInvoiceSelector).focus();
                        }
                        catch (ex) {

                        }
                        finally {
                            $timeout(function () {
                                e.component.option('value', '');
                                $scope.invoiceString = '';
                            }, 0);
                        }
                    },
  
                    placeholder: '',
                    showClearButton: isShowClearButton,
                });

            };

            //datagrid初始化控件
            var initInvoiceDataGridControl = function () {
                var valueExpr = "id";
                var displayExpr = "value";

                var isShowClearButton = false;

                $scope.scanInvoiceGridOptions = {
                    bindingOptions: {
                        dataSource: 'invoiceGridDataSource'
                    },
                    // keyExpr: "ID",
                    selection: {
                        mode: "multiple",
                        showCheckBoxesMode: 'always'
                    },
                    paging: {
                        enabled: false,
                    },
                    allowColumnResizing: true,
                    columnAutoWidth: true,
                    showRowLines: true,
                    showColumnLines: true,
                    rowAlternationEnabled: true,
                    showBorders: true,
                    noDataText: $translate.instant('NoDataText'),
                    selectAllText: $translate.instant('SelectAll'),
                    onContentReady: function (e) {
                        $scope.scanInvoiceGridInstance = e.component;
                  
                         $scope.uploadInvoiceTotalCount = e.component.totalCount();
                        
                    },
                    onSelectionChanged: function (data) {
                        $scope.datagrid.selectedItems = data.selectedRowsData;
                        $scope.datagrid.selectedRecourdCount = data.selectedRowsData.length;
                        $scope.selectedItemIDs = _.pluck(data.selectedRowsData, 'id');
                        $scope.batchDeletedDisabled = $scope.selectedItemIDs.length == 0;
                    },
                    columns: [
                            { dataField: 'invoiceCode', caption: $translate.instant('InvoiceFPDM'), },
                            { dataField: 'invoiceNumber', caption: $translate.instant('InvoiceFPHM') },
                            //{ dataField: 'buyerName', caption: $translate.instant('BuyerName') },
                            //{ dataField: 'buyerTaxNumber', caption: $translate.instant('BuyerTaxNumber') },
                            //{ dataField: 'sellerName', caption: $translate.instant('SellerName') },
                           // { dataField: 'sellerTaxNumber', caption: $translate.instant('SellerTaxNumber') },
                            { dataField: 'amount', caption: $translate.instant('FaceTaxAmount') }, //税额合计小写
                          //  { dataField: 'taxAmount', caption: $translate.instant('InvoiceSE') },  //税额
                            { dataField: 'invoiceDate', caption: $translate.instant('InvoiceDate') }, //开票日期
                            {
                                dataField: 'invoiceSourceType', caption: $translate.instant('InvoiceSource'),  //发票来源
                                width: 150,
                                cellTemplate: function (container, options) {
                                    try {

                                        $('<div />').dxSelectBox({
                                            displayExpr: displayExpr,
                                            valueExpr: valueExpr,
                                            dataSource: $scope.searchFilterOptions.invoiceSourceList,
                                            value: options.data.invoiceSourceType,
                                            //bindingOptions: {
                                            //    value: 'invoiceEntity.invoiceSourceType',
                                            //},
                                            onValueChanged: function (data) {
                                                options.data.invoiceSourceType = data.value;
                                                options.component.repaintRows(options.rowIndex);
                                            },
                                            height: 25,
                                            placeholder: $scope.pleaseSelect,
                                            showClearButton: isShowClearButton,
                                            noDataText: $scope.translated.NoData,
                                        }).appendTo(container);
                                    }
                                    catch (e) {
                                        $log.error(e);
                                    }
                                }
                            },
                            {
                                dataField: 'PoOrStaff', caption: '',   //根据发票来源，补充不同的信息
                                width: 150,
                                cellTemplate: function (container, options) {
                                    try {
                                        container.empty();
                                        if (options.data.invoiceSourceType == constant.inputInvoice.invoiceSourceType.InvoiceClaim.id) {
                                            
                                            $('<div class="staff-id" />').dxTextBox({

                                                //bindingOptions: {
                                                //    value: 'invoiceEntity.invoiceSourceType11',
                                                //},
                                                onValueChanged: function (data) {
                                                    options.data.staffID = data.value;
                                                },
                                                height: 25,
                                                placeholder: "输入员工编号",
                                                showClearButton: isShowClearButton,
                                            }).appendTo(container);

                                        }
                                        else if (options.data.invoiceSourceType == constant.inputInvoice.invoiceSourceType.InvoiceNonRawMaterialPurchase.id) {


                                            $('<div class="po-id" />').dxTextBox({

                                                //bindingOptions: {
                                                //    value: 'invoiceEntity.invoiceSourceType11',
                                                //},
                                                onValueChanged: function (data) {
                                                    options.data.poNumber = data.value;
                                                },
                                                height: 25,
                                                placeholder: "输入PO编号",
                                                showClearButton: isShowClearButton,
                                            }).appendTo(container);
                                        }
                                    }
                                    catch (e) {
                                        $log.error(e);
                                    }
                                }
                            },

                            {
                                dataField: 'uploadStatus', caption: $translate.instant('UploadStatus'),  //上传状态
                                alignment: 'left',
                                cellTemplate: function (container, options) {
                                    try {
                                        if (options.data.uploadStatus === uploadStatusEnum.failure.id) {
                                            $("<span class='status-fail span-btn' />")
                                                  .text(uploadStatusEnum.failure.value)
                                                  //.on('click', function () {
                                                  //    //弹框补录信息
                                                  //    $scope.eventServie.editInvoiceModal(options.data);
                                                  //})
                                              .appendTo(container);
                                        } else if (options.data.uploadStatus === uploadStatusEnum.success.id) {
                                            $("<span class='status-nomal' />")
                                                  .text(uploadStatusEnum.success.value)
                                              .appendTo(container);
                                        }
                                        else if (options.data.uploadStatus === uploadStatusEnum.duplicate.id) {
                                            $("<span class='status-duplicate' />")
                                                  .text(uploadStatusEnum.duplicate.value)
                                              .appendTo(container);
                                        }
                                    }
                                    catch (e) {
                                        $log.error(e);
                                    }
                                }
                            },
                            {
                                dataField: 'id', caption: $translate.instant('Operation'), //操作
                                cellTemplate: function (container, options) {
                                    try {
                                        $("<span class='span-btn' />")
                                            .text($scope.translated.editBtn)
                                            .on('click', function () {
                                                //弹框编辑信息
                                                $scope.eventServie.editInvoiceModal(options.data);
                                            })
                                        .appendTo(container);

                                        $("<span class='span-btn' />")
                                            .text($scope.translated.deleteBtn)
                                            .on('click', function () {
                                                //确认删除？
                                                messageBox.confirm($scope.translated.confirmDeleteMsg, '').then(function (isConfirm) {
                                                    $scope.eventServie.deleteRow(options.data);
                                                });
                                            })
                                        .appendTo(container);
                                    }
                                    catch (e) {
                                        $log.error(e);
                                    }
                                }
                            },

                    ],

                };
            };

            var initControl = function () {
                memoryService.populateInvoiceSourceTypList();
                initInvoiceDataGridControl();
                initTextBoxOptions();
            };

            var initData = function () {
                //getService.getInputInvoiceList();
            };

            (function initialize() {
                $log.debug('invoiceManageMainUploadScanInvoiceController.ctor()...');
                initControl();
                initData();
            })();
        }
]);
